#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Esta classe eh responsavel pela comunicacao com a lib TEC_Linux.so que comunica com os
teclados 'GERTEC TEC 44(PS2 e USB)', 'GERTEC TEC 55(PS2 e USB)', 'GERTEC TEC 65(PS2, USB, Unisys)'

OBS: Eh necessario ter a libusb-1.0 no sistema (disponivel via apt-get) para usar os telcados via usb(sem adaptador)
"""

import os
from ctypes import cdll
from stoqdrivers.logs.log import traced as log
path = '/tmp/TEC_Linux.so'
# gertec_lib = cdll.LoadLibrary(path)
gertec_lib = ''

# noinspection PyPep8Naming
class Gertec(object):

    def __init__(self, parent):
        self.parent = parent

    # TEC_Close - Fecha o acesso para o TEC
    @staticmethod
    def TEC_Close():
        return gertec_lib.TEC_Close()

    # TEC_Reset - Reseta o teclado
    @staticmethod
    def TEC_Reset():
        return gertec_lib.TEC_Reset()

    # KBD_GetEnabled - Retorna o status do Teclado
    @staticmethod
    def KBD_GetEnabled():
        return gertec_lib.KBD_GetEnabled()

    # KBD_SetEnabled- Habilita ou Desabilita o Teclado
    @staticmethod
    def KBD_SetEnabled(OnOff):
        """ :param OnOff: booleano
        """
        return gertec_lib.KBD_SetEnabled(OnOff)

    # KBD_Beep - Habilita ou Desabilita beep continuo
    @staticmethod
    def KBD_Beep(OnOff):
        """ :param OnOff: booleano
        """
        return gertec_lib.KBD_Beep(OnOff)

    # KBD_BeepKey - Habilita ou Desabilita beep por tecla
    @staticmethod
    def KBD_BeepKey(OnOff):
        """ :param OnOff: booleano
        """
        return gertec_lib.KBD_BeepKey(OnOff)

    # DIS_GetEnabled - Pega o estado do display.
    @staticmethod
    def DIS_GetEnabled():
        return gertec_lib.DIS_GetEnabled()

    # DIS_SetEnabled - Pega o estado do display.
    @staticmethod
    def DIS_SetEnabled(OnOff):
        """ :param OnOff: booleano
        """
        return gertec_lib.DIS_SetEnabled(OnOff)

    # DIS_Backspace - Envia um BackSpace para o display
    @staticmethod
    def DIS_Backspace():
        return gertec_lib.DIS_Backspace()

    # DIS_CarRet - Envia um Carry Return para o display
    @staticmethod
    def DIS_CarRet():
        return gertec_lib.DIS_CarRet()

    # DIS_LineFeed - Envia um LineFeed para o display
    @staticmethod
    def DIS_LineFeed():
        return gertec_lib.DIS_LineFeed()

    # DIS_FormFeed - Envia um FormFeed para o display
    @staticmethod
    def DIS_FormFeed():
        return gertec_lib.DIS_FormFeed()

    # DIS_ClrLn - Apaga uma linha do display
    @staticmethod
    def DIS_ClrLn(Lin):
        return gertec_lib.DIS_ClrLn(Lin)

    # DIS_PrintCh - Envia um caracter para o display.
    @staticmethod
    def DIS_PrintCh(c):
        return gertec_lib.DIS_PrintCh(c)

    # DIS_PrintStr - Envia uma string para o display
    @staticmethod
    def DIS_PrintStr(s):
        return gertec_lib.DIS_PrintStr(s)

    # DIS_CursorOff
    @staticmethod
    def DIS_CursorOff(OnOff):
        """ :param OnOff: booleano
        """
        return gertec_lib.DIS_CursorOff(OnOff)

    # DIS_Contraste
    @staticmethod
    def DIS_Contraste(c):
        """ :param c: inteiro de 1 a 9
        """
        return gertec_lib.DIS_CursorOff(c)

    # DIS_FonteExp
    @staticmethod
    def DIS_FonteExp(OnOff):
        """ :param OnOff: booleano
        """
        return gertec_lib.DIS_FonteExp(OnOff)

    # DIS_GoToXY - Posiciona o cursor do display em (LIN, COL)
    @staticmethod
    def DIS_GoToXY(Lin, Col):
        """ :param Lin: char
            :param Col: char
        """
        return gertec_lib.DIS_GoToXY(Lin, Col)

    # DIS_GoToXYRef - Posiciona o cursor do display em (LIN++DeltaLin, COL++DeltaCol)
    @staticmethod
    def DIS_GoToXYRef(DeltaLin, DeltaCol):
        """ :param DeltaLin: char
            :param DeltaCol: char
        """
        return gertec_lib.DIS_GoToXY(DeltaLin, DeltaCol)

    # LEC_GetEnabled - Pega o estado do LEC.
    @staticmethod
    def LEC_GetEnabled():
        return gertec_lib.LEC_GetEnabled()

    # LEC_SetEnabled
    @staticmethod
    def LEC_SetEnabled(OnOff):
        """ :param OnOff: bool
        """
        return gertec_lib.LEC_GetEnabled(OnOff)

    # LEC_Leitura
    @staticmethod
    def LEC_Leitura(TimeOut, buf1, buf2):
        """ :param TimeOut: int
            :param buf1: char*
            :param buf2: char*
        """
        return gertec_lib.LEC_GetEnabled(TimeOut, buf1, buf2)

    # CHV_Leitura
    @staticmethod
    def CHV_Leitura(buf):
        """ :param buf: char*
        """
        return gertec_lib.CHV_Leitura(buf)

    @staticmethod
    def formata_texto(texto):
        '''
        Formata as quebras de linha do texto
        '''
        aux = ''
        cont = 0
        texto = texto.replace('\n', ' ')
        tam_texto = len(texto)
        for i in range(0, tam_texto / 33 +1):
            aux += texto[cont: cont+33] + '\n'
            cont += 33
            if cont > tam_texto:
                return aux

    def fechar_teclado(self):
        self.TEC_Close()

    def limpar_display(self):
        """
        Limpa completamente o display
        :return:
        """
        self.DIS_FormFeed()

    def limpar_linha(self, nro_linha):
        self.DIS_ClrLn(nro_linha)

    def envia_display(self, texto, limpa_display=True):
        '''
        Limpa e envia um texto para o display
        '''
        try:
            texto = self.parent.remove_accents(texto)
            texto = self.formata_texto(texto)
            if limpa_display:
                self.limpar_display() # limpa o diplay

            self.DIS_PrintStr(str(texto))
        except Exception as e:
            log.error('Erro ao enviar texto para o display do teclado! \n%s' % str(e))
            return False

        return True

    def envia_linha_display(self, texto, linha):
        '''
        Envia o texto para a linha escolhida
        '''
        try:
            texto = self.parent.remove_accents(texto)
            texto = self.formata_texto(texto)
            self.limpar_linha(linha) # limpa linha
            self.DIS_GoToXY(linha,1)
            self.DIS_PrintStr(str(texto))
        except Exception as e:
            log.error('Erro ao enviar a linha para display do teclado! \n%s' % str(e))
            return False

        return True

    def envia_botoes_display(self, texto):
        '''
        Envia o texto dos botoes da caixa de mensagens para a quarta linha do display
        '''
        try:
            texto = self.parent.remove_accents(texto.replace('&',''))
            texto = self.formata_texto(texto)
            self.limpar_linha(4)
            self.DIS_GoToXY(4,1)
            self.DIS_PrintStr(str(texto))
        except Exception as e:
            log.error('Erro ao enviar botoes para o display do teclado! \n%s' % str(e))
            return False

        return True

    def go_to(self, Lin, Col):
        self.DIS_GoToXY(Lin, Col)