#!/usr/bin/python
# -*- coding: utf-8 -*-

from sigpdv.models.db_utils import  exec_sql, testa_conexao_pg
from sigpdv.tools.log import traced as Log
from sigpdv.ui.messagebox import MSG_INFO, message_dlg
from datetime import datetime
from decimal import Decimal
from sigpdv.tools.config import config
from psycopg2.extensions import QueryCanceledError

import psycopg2
TWOPLACES = Decimal(10) ** -2


class Cadfuncionarios(object):
    def __init__(self, parent):
        self.parent = parent
        self.corpos = []

    @Log
    def get_connection_sup_master(self, time_out=0):
        '''
        Pega a conexao com o supervisor_master para consulta de desconto do funcionario 
        '''
        db_name = config['db_sup_name_master']
        db_user = config['db_sup_user_master']
        db_host = config['db_sup_host_master']
        db_password = config['db_sup_pass_master']
        if '' in [db_name, db_user, db_host]:
            return False

        if not testa_conexao_pg(db_name, db_user, db_host, db_password):
            return False

        connection = psycopg2.connect(database=db_name,
                                      user=db_user,
                                      password=db_password,
                                      host=db_host,
                                      connect_timeout=5,
                                      options='-c statement_timeout=%ss' % time_out)
        return connection

    @Log
    def get_desconto_ja_concedido_funcionario(self, codigo_cliente):
        '''
        retorna o desconto ja concedido para o funcionario apenas da Promocao(desconto) de
        funcionario
        :param codigo_cliente: cpf/cnpj do cliente
        :return:
        '''

        mes = '%02d' % datetime.now().date().month
        ano = '%s' % datetime.now().date().year
        sql = """
            SELECT smcor.supcor_desconto_item_sem_rat AS desconto, -- apenas desoconto do item
                   smcor.supcor_venda_cancelamento    AS venda_cancelado,
                   smc.supcap_tipo
            FROM supervisor_mov_corpos AS smcor
            INNER JOIN supervisor_mov_capa AS smc ON smc.id = smcor.supcor_capa_id
            WHERE smc.supcap_data_cupom >= '%(ano)s-%(mes)s-01'
              AND smc.supcap_data_cupom <= CURRENT_DATE
              AND smc.supcap_cliente = '%(codigo_cliente)s'
              and smcor.supcor_tipo_promo = '3' -- Desconto de funcionario
              AND smcor.supcor_desconto_item != 0
              AND smc.supcap_tipo in ('C', 'V')
        """ % locals()
        try:
            connection = self.get_connection_sup_master()
            res = exec_sql(sql, 'S', connection=connection)
        except Exception as e:
            Log.error(str(e))
            return 'ERRO'

        desconto_total = 0
        for dado in res:
            # cupom de venda sem item cancelado
            if dado.get('supcap_tipo') == 'V' and dado.get('venda_cancelado') == 'V':
                desconto_total += dado.get('desconto')

            elif dado.get('supcap_tipo') == 'C': # cupom de cancelamento
                desconto_total -= dado.get('desconto')

        return desconto_total

    def get_desconto_maximo(self, dados, desconto_dado):
        '''
        Recupera o valor maximo de desconto que poderá ser concedido
        :param dados: dados do funcionario
        :param dados: desconto dado no periodo de 1 mes
        :return:
        '''
        salario = dados.get('salario')
        # Percentual do limite de desconto sobre o salario
        percent_salario = dados.get('percent_salario')
        maximo = Decimal(salario * percent_salario/100).quantize(TWOPLACES) - desconto_dado
        if maximo < 0.01:
            maximo = 0

        return maximo

    @Log
    def get_dados_funcionario(self, codigo_cliente):
        """
        Retorna os dados do funcionario
        :param codigo_cliente:
        :return:
        """
        sql = """
            SELECT supfun_salario                    AS salario,
                   supfun_percentual_limite_desconto AS percent_salario,
                   supfun_percentual_desconto        AS percent_desconto
            FROM supervisor_cad_funcionarios    AS scf
            INNER JOIN supervisor_cad_clientes AS scc ON scc.supcli_codigo_interno = scf.supfun_codigo_cliente
            WHERE scc.supcli_cnpj_cpf = '%(codigo_cliente)s'
              """ % locals()
        try:
            connection = self.get_connection_sup_master()
            res = exec_sql(sql, 'S', connection=connection)
        except QueryCanceledError as e:
            Log.error('Erro ao consultar funcionario(cliente: %s): %s' % (str(codigo_cliente),
                                                                          str(e)))
            message_dlg(self.parent, MSG_INFO, 'Atenção! \nErro ao consultar tabela de funcionários, informe o Supervisor!')
            res = False

        except Exception as e:
            Log.error('Erro ao consultar funcionario(cliente: %s): %s' % (str(codigo_cliente),
                                                                          str(e)))
            res = False

        return res

    def calcula_desconto_funcionario(self, dados_funcionario, desconto_maximo):
        """
        Calcula, grava na ECF e no banco o valor do desconto nos itens e retorna o desconto total
        concedido
        :param dados_funcionario:
        :param corpos:
        :param desconto_maximo:
        :return:
        """
        self.parent.dict_itens_desconto_funcionario = {}
        self.parent.desconto_funcionario = Decimal(0)
        # lista de itens que irao participar de outro tipo de promocao/desconto
        item_com_desconto = map(lambda x: x.get('sequencia_item'),
                               self.parent.list_itens_desconto_promo)
        item_com_desconto.extend(map(lambda x: int(x),
                      filter(lambda x: self.parent.dict_mov_corpos_socio.get(x).get('desconto') > 0,
                             self.parent.dict_mov_corpos_socio)))
        # pega apenas os corpos que os itens nao receberao desconto de outras promocoes e nao sao
        # itens cancelados
        self.corpos = filter(lambda x: x.supcor_sequencia not in item_com_desconto and
                                       x.supcor_venda_cancelamento == 'V' and
                                       x.supcor_desconto_item == 0,
                             self.parent.mov_corpos)
        total_itens = Decimal(sum(map(lambda x: x.supcor_preco_total, self.corpos)))
        # Percentual de desconto
        percent_desconto = Decimal(dados_funcionario.get('percent_desconto'))
        valor_desconto = Decimal(total_itens * percent_desconto / Decimal(100)).quantize(TWOPLACES)
        if valor_desconto > desconto_maximo:
            valor_desconto = desconto_maximo

        # calcula novo percentual de desconto pois o desconto pode ser menor por causa dos descontos
        # ja concedidos
        novo_percent_desconto = total_itens and valor_desconto/total_itens or 0
        desconto_dado_total = 0
        maior_item = (0,0)
        piso_sequencia = 0
        if self.parent._printer.brand == 'bematech':
            piso_sequencia = len(self.parent.mov_corpos) > 300 and len(self.parent.mov_corpos) - \
                                                                   300 or 0

        for corpo in self.corpos:
            desconto = Decimal(corpo.supcor_preco_total * novo_percent_desconto).quantize(TWOPLACES)
            item = corpo.supcor_sequencia
            # pega apenas os ultimos 300 itens, limitacao da ecf
            if item < piso_sequencia:
                continue
            # nao da desconto em itens que estao parcitipando de outras promocoes
            if item in item_com_desconto:
                continue

            desconto_dado_total += desconto
            if corpo.supcor_preco_total > maior_item[1]:
                maior_item = (item, corpo.supcor_preco_total)

            self.parent.dict_itens_desconto_funcionario[item] = desconto

        dif = valor_desconto - desconto_dado_total
        if dif <= -0.01 or 0.01 <= dif:
            self.parent.dict_itens_desconto_funcionario[maior_item[0]] += dif
            desconto_dado_total += dif

        return desconto_dado_total

    @Log
    def get_desconto_funcionario(self):
        if self.parent._coupon_totalize: # cupom ja totalizado, nao precisa calcular o desconto
            return 'Cupom totalizado'

        codigo_cliente = self.parent.mov_capa.supcap_cliente # cpf/cnpj
        if not codigo_cliente:
            return 'Sem cliente'

        try:
            # Verifica se o modulo supervisor_funcionario esta instalado
            connection = self.get_connection_sup_master()
            res = exec_sql("""SELECT relname
                              FROM pg_class
                              WHERE relname = 'supervisor_cad_funcionarios'
                                AND relkind='r';""",'S', connection=connection)
        except Exception as e:
            Log.error(str(e))
            return '1-Sem conexao com o supervisor'

        if not res:
            return 'Módulo supervisor_funcionario não instalado'

        dados_funcionario = self.get_dados_funcionario(codigo_cliente)
        if not dados_funcionario:
            return 'Cliente nao e funcionario'

        dados_funcionario = dados_funcionario[0]
        desconto_dado = self.get_desconto_ja_concedido_funcionario(codigo_cliente)
        if desconto_dado == 'ERRO':
            message_dlg(self.parent, MSG_INFO, u"Sem conexão com o supervisor")
            return '2-Sem conexao com o supervisor'

        desconto_maximo = self.get_desconto_maximo(dados_funcionario, desconto_dado)
        if desconto_maximo == 0: # limite de desconto ja atingido
            message_dlg(self.parent, MSG_INFO, u"Desconto para funcionário não concedido!\n\n"
                                               u"Limite de desconto já foi atingido!")
            return 'Limite atingido'

        valor_desconto = self.calcula_desconto_funcionario(dados_funcionario, desconto_maximo)
        self.parent.desconto_funcionario = valor_desconto

        self.parent.atualiza_valor_a_pagar()
        return "Desconto: %s" % valor_desconto

    @Log
    def aplica_desc_funcionario(self):
        # Aplica desconto na politica de funcionario
        desconto_total = 0
        for corpo in self.corpos:
            item = corpo.supcor_sequencia
            desconto = self.parent.dict_itens_desconto_funcionario.get(item) or 0
            if desconto == 0: # desconto igual a 0, nao eh concedido.
                continue

            self.parent.send_command("desc_acres_item('D', %s, %s)" % (item, desconto))
            corpo.supcor_desconto_item = desconto
            corpo.supcor_desconto_item_sem_rat = desconto
            corpo.supcor_tipo_promo = '3' # tipo de desconto para Funcionario
            desconto_total += desconto

        # limpa o desconto para a proxima venda
        self.parent.dict_itens_desconto_funcionario = {}
        self.corpos = []
        return "Desconto Total: %s" % desconto_total

