#!/usr/bin/python
# -*- encoding: utf-8 -*-

from decimal import Decimal, ROUND_DOWN, ROUND_UP
from sigpdv.models.cad_kits_data import CaixaCadKits
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.tools.log import traced as Log


TWOPLACES = Decimal(10) ** -2
TREPLACES = Decimal(10) ** -3


class CadKits(object):
    def __init__(self, parent=None):
        self.parent = parent

    @Log
    def get_itens_kit(self, cod_barra):
        """
        Adiciona numa lista todos os itens do kit.
        :param cod_barra: codigo de barra do kit.
        :type cod_barra: str
        :rtype: list
        """
        res = []
        quantidades = []
        vr_unitario = []
        descontos = []
        apenas_um = False
        kits = CaixaCadKits.query.filter("supkit_codigo_barra_kit = '%s' " % cod_barra).all()
        if kits and len(kits) == 1 and kits[0].supkit_quantidade == 1:
            apenas_um = True

        for kit in CaixaCadKits.query.filter("supkit_codigo_barra_kit = '%s' " % cod_barra).all():
            item = CaixaCadItens.get_by(supite_codigo_item=kit.supkit_codigo_barra_item,
                                        supite_loja=self.parent.loja.supcfc_loja)
            res.append(item)
            quantidades.append(kit.supkit_quantidade)
            # resolve o problema do valor do kit ser muito maior que o dobro do produto, no caso de
            # carcaca de carne.
            if not apenas_um:
                vr_unitario.append(item.supite_preco)
                desconto = item.supite_preco * kit.supkit_quantidade - kit.supkit_preco_venda
                descontos.append(desconto)
            else:
                vr_unitario.append(kit.supkit_preco_venda)
                descontos.append(0)

        return res, quantidades, vr_unitario, descontos

    @Log
    def rateia_desc_acres_kit(self, quantidades, vr_unitario, qtd_kit, desc_acres):
        '''
        Rateia o desconto/acrescimo dado no item kit para os itens que compoem o kit de forma
        proporcional ao valor total do item
        :param qtd_kit: lista dos itens a receberem o desconto ou acrescimo
        :param vr_unitario: lista dos itens a receberem o desconto ou acrescimo
        :param desc_acres: valor do desconto/acrescimo dado ao item kit
        :return: retorno dicionario com codigo do item como chave e o valor é outro dicionario onde
                a quantidade do item é a chave e o valor é o valor rateado do desc/acres
        '''
        res = []

        # # quando o desconto eh percentual o calculo eh feito na funcao 'get_desconto_acrescimo'
        # if self.parent.ui.cbTipoDesc.currentIndex() == 1:
        #     return res

        indice_maior_valor = 0
        maior_valor = 0
        total_kit = 0
        for i in range(0, len(quantidades)):
            total_kit += vr_unitario[i] * quantidades[i] * qtd_kit

        percentual = Decimal(desc_acres / total_kit * 100).quantize(TWOPLACES)
        total_registrado = 0
        for i in range(0, len(quantidades)):
            qtd_item = quantidades[i] * qtd_kit
            total_desc_acres = Decimal(vr_unitario[i] * qtd_item * percentual
                                 / Decimal(100)).quantize(TWOPLACES)
            valor_tot_item = vr_unitario[i] * qtd_item
            if valor_tot_item > maior_valor:
                indice_maior_valor = i
                maior_valor = valor_tot_item

            desc_acres_calc = Decimal(total_desc_acres).quantize(TWOPLACES)
            res.append(desc_acres_calc)
            total_registrado += desc_acres_calc

        dif = total_registrado - desc_acres
        # joga a diferenca do desc/acres dado com o total rateado no item de maior valor total
        if dif <= -0.01 or 0.01 <= dif:
            res[indice_maior_valor] += dif

        return res

