#!/usr/bin/python
# -*- coding: utf-8 -*-

from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from sigpdv.control.mov_cupom_servidor import MovCupomServidor
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.ui.messagebox import MSG_ERROR, MSG_INFO, MSG_NONE, message_dlg
from SigUtils.numero import getNumeroEditado
in_coupon = 3
in_end_sale = 12


class Doacao():

    def __init__(self, parent):
        self.parent = parent

    @Log
    def efetua_doacao(self, cod_totalizador, cod_finalizadora, valor):
        self.parent.bloqueia_botoes()
        self.parent._set_screen_state(in_end_sale)
        self.parent.show_info_proccess(True, 'Efetuando Doação. Aguarde...')
        try:
            tots = self.parent.send_command("get_totalizadores_nao_fiscais(%s)" % cod_totalizador)
            if int(cod_totalizador) not in tots:
                self.parent.show_info_proccess(False)
                message_dlg(self.parent, MSG_INFO,
                            "Atenção!\nO totalizador não fiscal '%02d' não está cadastrado na ECF!"
                            % int(cod_totalizador))
                return False

            self.parent.send_command("_add_voucher(%r, Decimal('%s'), %r)" %
                                                                  ('%02d' % cod_totalizador,
                                                                   valor,
                                                                   cod_finalizadora))
        except Exception as e:
            self.parent.show_info_proccess(False)
            return False

        self.cria_capa_cupom(cod_totalizador, valor)
        self.cria_cupom_vinculado(valor)
        return True

    @Log
    def cria_capa_cupom(self, cod_totalizador, valor):
        data_ini = self.parent.obj_fiscal_printer.get_data_ecf()
        data_cupom = self.parent.send_command('get_data_mov()')
        nro_cupom = self.parent.send_command('get_coo()')
        self.parent.ui.lblNumCaixa.setText(u'%03d <b>Cupom #: %06d' % (self.parent.caixa_id,
                                                                       nro_cupom))
        self.parent._coupon_id = nro_cupom

        # --------------------------------- Cria a capa do cupom --------------------------------- #
        vals_capa = dict(supcap_data_cupom=data_cupom,
                         supcap_loja=self.parent.loja.supcfc_loja,
                         supcap_caixa=self.parent.caixa_id,
                         supcap_nro_cupom=nro_cupom,
                         supcap_operador_id=self.parent.user_logged.id,
                         supcap_cliente=self.parent.cliente.supcli_cnpj_cpf,
                         supcap_data_inicio=data_ini,
                         supcap_data_fim=data_ini,
                         supcap_tipo='W',
                         supcap_valor_desc_acres=0,
                         supcap_total_cupom=valor,
                         supcap_mf_adicional=False,
                         supcap_nro_proprietario=1,
                         supcap_tipo_comprovante='CN',
                         supcap_subtotal=valor,
                         supcap_indicador_cancelamento='N',
                         supcap_cancelamento_acrescimo=0,
                         supcap_cancelado_posteriormente=False,
                         supcap_nome_cliente=self.parent.cliente.name,
                         supcap_cod_vendedor=self.parent.vendedor_codigo,
                         supcap_cod_tot_nao_fiscal=cod_totalizador)

        res = self.parent.obj_mov_capa.create(vals_capa)
        self.parent._coupon = self.parent.mov_capa = res
        #cria cupom servidor
        MovCupomServidor().create({'supcse_capa_id': self.parent.mov_capa.id})
        return True

    @Log
    def cria_cupom_vinculado(self, valor):
        if not config['imprimir_2via_doacao']:
            return "Doacao sem a via do cliente!"

        parametro = self.parent.obj_parametros.get_by()
        msg = '%s' % self.parent.remove_accents(parametro.supcfp_mensagem_doacao or '')
        text = '%s%s' %('Valor da doacao: R$%s\n\n' % (getNumeroEditado(valor)), msg)
        GerencialReport(report_id=config['rg_relatorio_gerencial'],
                        text=str(text),
                        loja_id=self.parent.loja.supcfc_loja,
                        caixa_id=self.parent.caixa_id,
                        operador_id=self.parent.user_logged.id,
                        nro_cupom_vinculado=self.parent.mov_capa.supcap_nro_cupom,
                        tipo='w',
                        tipo_c='CC',
                        total=valor,
                        parent=self.parent)