# -*- coding: utf-8 -*-

import os
from psycopg2 import IntegrityError
from sigpdv.models.db_utils import dict_2_sql
from sigpdv.models.cfg_versao_data import CaixaCfgVersao
from sigpdv.models.cad_ficha_data import CaixaCadFicha
from sigpdv.models.mov_corpo_xz_data import CaixaMovCorpoXZ
from sigpdv.models.mov_totalizadores_xz_data import CaixaMovTotalizadoresXZ
from sigpdv.models.mov_vencimentos_data import CaixaMovVencimentos
from sigpdv.models.mov_autorizacoes_data import CaixaMovAutorizacoes
from sigpdv.models.mov_caixa_data import CaixaMovCaixa
from sigpdv.models.mov_meios_pagamentos_data import CaixaMovMeiosPagamentos
from sigpdv.models.mov_cupom_servidor_data import CaixaMovCupomServidor
from sigpdv.models.mov_capa_data import CaixaMovCapa
from sigpdv.models.mov_corpos_data import CaixaMovCorpos
from sigpdv.models.cad_tributacao_data import CaixaCadTributacao
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.mov_kits_vendidos_data import CaixaMovKitsVendidos
from sigpdv.models.mov_fechamentos_data import CaixaMovFechamentos
from sigpdv.models.nfe_capa_data import CaixaNfeCapa
from sigpdv.models.supervisor import SupervisorMovCapa, SupervisorMovCorpos, \
    SupervisorMovFechamentos, SupervisorMovCupomServidor
from sigpdv.models.db_utils import sess, exec_sql, testa_conexao_rpc
from sigpdv.tools.rpc_timeout import ServerProxy
from sigpdv.tools.config import config
from sigpdv.tools.log_exportar import logExportar as traced
from sqlalchemy import func
from datetime import datetime, timedelta
import glob
from sigpdv import root_path

ROOT_PATH = os.path.dirname(os.path.abspath(__file__))


def get_info():
    uname = os.uname()
    versao_db = sess.query(func.max(CaixaCfgVersao.supver_versao)).one()
    info = {'so_sysname': uname[0],
            'so_nodename': uname[1],
            'so_release': uname[2],
            'so_version': uname[3],
            'so_machine': uname[4],
            'db_version': CaixaCfgVersao.get_by(supver_versao=versao_db[0]).supver_versao if versao_db[0] else ''}
    return info


class Exportacao(object):
    def __init__(self, parent=None):
        self._ocupado = False
        self._parent = parent
        self._run = False

    def __repr__(self):
        return '<class Exportacao()>'

    def trata_erro(self, retorno, sess):
        '''
        Se nao houver retonor a funcao retorna True(Tem erro) se nao retorna False(Sem erro)
        '''
        if not retorno:
            sess.rollback()
            return True
        return False

    def cria_acao_pendente(self, tabela):
        try:
            sql = """
                SELECT *
                FROM supervisor_cad_pdvs"""
            dados_pdvs = exec_sql(sql, 'S', ret=True, sta_tout=0)
            for pdv in dados_pdvs:
                sql_insert = """
                INSERT INTO supervisor_mov_acao_pendente(
                            supape_tabela, supeape_pdv, supape_total_em_aberto_cliente,
                            supeape_loja, supape_situacao)
                VALUES ('{supape_tabela}', {supeape_pdv}, {supape_total_em_aberto_cliente},
                            {supeape_loja}, '{supape_situacao}');""".format(
                            supape_tabela=tabela,
                            supeape_pdv=pdv.get('suppdv_caixa'),
                            supape_total_em_aberto_cliente=0.0,
                            supeape_loja=pdv.get('suppdv_loja'),
                            supape_situacao='A')
                exec_sql(sql_insert, 'S', ret=True, sta_tout=0)

        except Exception as e:
            traced.error('Erro ao criar ação pendente: %s' % str(e))

        return True

    def altera_data_atualizacao(self):
        '''Atualiza no Supervisor a data de conexao com o mesmo para saber se o caixa esta online'''
        # parametros da aba 'SigERP'
        # server = config.get('db_ges_host')
        # bd = config.get('db_ges_name')
        # porta = config.get('db_ges_port')
        # pwd_sup = config.get('db_ges_pass')
        # username_sup = config.get('db_ges_user')
        caixa = config.get('caixa')
        loja = config.get('loja')
        #
        # check = testa_conexao_rpc(server, username_sup, pwd_sup, bd, porta)
        # if not check.get('res'):
        #     return False
        #
        # sock_common = ServerProxy('http://%s:%s/xmlrpc/common' % (server, porta))
        # uid = sock_common.login(bd, username_sup, pwd_sup)
        # sock = ServerProxy('http://%s:%s/xmlrpc/object' % (server, porta))
        # try:
        #     res = sock.execute(bd, uid, pwd_sup, 'supervisor.cad.pdvs', 'altera_data_atualizacao',
        #                        loja, caixa, {})
        # except Exception as e:
        #     traced.error("Erro ao Alterar data de atualizacao ")
        #     res = False
        #
        # return res

        res = exec_sql("""SELECT id 
                          FROM supervisor_cad_pdvs 
                          WHERE suppdv_loja = %s
                            AND suppdv_caixa = %s """ % (loja, caixa), 'S')
        if not res or not res[0].get('id'):
            res = exec_sql("""INSERT INTO supervisor_cad_pdvs
                              (suppdv_loja, suppdv_caixa, suppdv_data_pdv_atualizacao) 
                              VALUES (%s, %s, '%s' at time zone 'UTC') """ % (loja, caixa, datetime.now()), 'S')
            return res

        res = exec_sql("""UPDATE supervisor_cad_pdvs 
                          SET suppdv_data_pdv_atualizacao = '%s' at time zone 'UTC' 
                          WHERE id = %s""" % (datetime.now(),res[0].get('id')),  'S')

        return res

    def transacaoDB(self, campos, oldObject, nameTabela, optDelete=False, nro_cupom=False, data_cupom=False, default={}):
        """
        Executa o insert no banco de dados do Supervisor.
        """
        id_insert = None

        try:
            data = oldObject.to_dict()
            if data.get('id'):
                del data['id']

            # caso o campo scanntech esteja vazio, nao envia pois no erp pode nao ter o modulo 'supervisor_promocoes'
            if 'scanntech' in data and not data.get('scanntech'):
                del data['scanntech']

            if 'supcor_codigo_ficha' in data: # no supervisor nao existe este campo
                del data['supcor_codigo_ficha']

            if optDelete:
                delete = dict(filter(lambda i: i[0] in campos, data.items()))
                sqlDelete = dict_2_sql(nameTabela, delete, 'D')
                exec_sql(sqlDelete, 'S', sta_tout=0)
            for key in default:
                data[key] = default.get(key)
            # para a tabela supervisor_mov_caixa a data esta sendo ajustada pois no insert no erp esta sendo gravada
            # em UTC mas no frente esta gravada no horario local no bando, esse ajuste eh apenas no Update
            if nameTabela == 'supervisor_mov_caixa' and data.get('supcax_fechado') == 'F':
                condicao = data.copy()
                data.pop('supcax_data_hora_abertura')
                data.pop('supcax_data_hora_fechamento')
                hora_dif = datetime.utcnow() - datetime.now()
                hora_dif = timedelta(hours=round(hora_dif.seconds/3600.0)) # dif. da hora local com a hora UTC
                data['supcax_data_hora_abertura'] = str(condicao.get('supcax_data_hora_abertura') + hora_dif)
                data['supcax_data_hora_fechamento'] = str(condicao.get('supcax_data_hora_fechamento') + hora_dif)
                condicao.pop('supcax_data_hora_fechamento')
                condicao['supcax_fechado'] = 'A'
                condicao['supcax_data_hora_abertura'] = str(condicao.get('supcax_data_hora_abertura'))
                sql = dict_2_sql(nameTabela, data, 'U', where=condicao, tz_utc=True)
            elif nameTabela == 'supervisor_mov_kits_vendidos':
                sql = dict_2_sql(nameTabela, data, 'I', tz_utc=True)
                sql = sql.replace("%", '')
            else:
                sql = dict_2_sql(nameTabela, data, 'IU', tz_utc=True)
                sql = sql.replace("%", '')
            traced.info('EXPORTANDO %s\n%s\n' % (nameTabela.replace('supervisor_', ''), sql))
            id_insert = exec_sql(sql, 'S', ret=True, sta_tout=0)
            if id_insert and isinstance(id_insert, list):
                id_insert = id_insert[0]
            if id_insert and isinstance(id_insert, dict):
                id_insert = id_insert.get('id')
            traced.info('%s.id: %s\n' % (nameTabela, str(id_insert)))
        except Exception as e:
            if nro_cupom and data_cupom:
                traced.error('EXPORTACAO CUPOM: erro no envio cupom %s do dia %s :\n(%s)' % (nro_cupom, data_cupom,
                                                                                             str(e)))
            else:
                traced.error('EXPORTACAO CUPOM: erro(%s)' % str(e))

        return id_insert

    def exportar_arq_mfd(self):
        '''
        Exporta todos os arquivos que estiverem na pasta sigpdv/MFD/arquivo e ainda nao foram exportados
        :return:
        '''
        arquivos = glob.glob(str(root_path+'/MFD/arquivo/*.txt').replace('//', '/'))
        for arq in arquivos:
            nome_arquivo = arq.split(str(root_path+'/MFD/arquivo/').replace('//', '/'))[1]
            if nome_arquivo == 'controle.txt':
                continue
            try:
                f = open(str(root_path+'/MFD/arquivo/controle.txt').replace('//', '/'), 'r')
                controle = f.read()
                f.close()
                if nome_arquivo in controle:
                    continue
            except:
                controle=''

            f = open(str(root_path+'/MFD/arquivo/controle.txt').replace('//', '/'), 'w')
            f.write(controle+nome_arquivo+'\n')
            f.close()
            sql = '''SELECT id
                     FROM supervisor_arquivo_mfd
                     WHERE supmfd_nome_arquivo = '{nome_arquivo}'
                       AND supmfd_loja = {loja}
                       AND supmfd_caixa = {caixa}
                  '''.format(loja=config['loja'],
                             caixa=config['caixa'],
                             nome_arquivo=nome_arquivo)
            if exec_sql(sql, 'S', ret=True, sta_tout=0):
                continue

            f = open(arq, 'r')
            conteudo = f.read()
            f.close()
            sql = ''' INSERT INTO supervisor_arquivo_mfd
                             (supmfd_loja, supmfd_caixa, supmfd_nome_arquivo, supmfd_conteudo)
                      VALUES ({loja}, {caixa}, '{nome_arquivo}', '{conteudo}')
                      RETURNING id;
                  '''.format(loja=config['loja'],
                             caixa=config['caixa'],
                             nome_arquivo=nome_arquivo,
                             conteudo=str(self._parent.remove_accents(conteudo)))

            try:
                traced.info('EXPORTANDO %s\n%s\n' % ('supervisor_arquivo_mfd', sql))
                id_insert = exec_sql(sql, 'S', ret=True, sta_tout=0)
                traced.info('%s.id: %s\n' % ('supervisor_arquivo_mfd', str(id_insert)))
            except Exception as e:
                traced.error('Erro ao exportar arquivo MFD: ' + str(e))

    def exportar_fichas(self):
        if not config['usar_ficha']:
            return

        list_fichas = exec_sql("""SELECT * 
                                  FROM caixa_cad_ficha
                                  WHERE supfch_exportado = FALSE""", 'C', sta_tout=0)
        if not list_fichas:
            return

        values_itens = []
        msg = []
        ids = []
        for i in list_fichas:
            msg.append('Ficha %s exportada!' % i.get('supfch_codigo_ficha'))
            ids.append('%s' % i.get('id'))
            values_itens.append("""({cod_ficha}, '{state}', '{create_date}' at time zone 'UTC', 
                                    {valor}, {desconto}, {loja}, {caixa}, '{item}', {exportado}, 
                                    {caixa_fat}, {coo}, '{data_cupom}' at time zone 'UTC', 
                                    '{cliente}', '{data_entrega}', '{obs}', '{state_impressao}')""".
                                format(cod_ficha=i.get('supfch_codigo_ficha'),
                                       state=i.get('state'),
                                       create_date=i.get('create_date'),
                                       valor=i.get('supfch_valor'),
                                       desconto=i.get('supfch_desconto') or 0,
                                       loja=i.get('supfch_loja'),
                                       caixa=i.get('supfch_caixa'),
                                       item=i.get('supfch_item'),
                                       exportado=i.get('supfch_exportado'),
                                       caixa_fat=i.get('supfch_caixa_faturado') or 'null',
                                       coo=i.get('supfch_cupom_coo') or 'null',
                                       data_cupom=i.get('supfch_cupom_data') or 'null',
                                       cliente=i.get('supfch_cz_nome_cliente') or 'null',
                                       data_entrega=i.get('supfch_cz_data_hora_entrega') or 'null',
                                       obs=i.get('supfch_cz_observacoes') or 'null',
                                       state_impressao=i.get('supfch_cz_state_impressao'))
                                )
        try:
            sql = """INSERT INTO supervisor_cad_ficha
                           (supfch_codigo_ficha, state, create_date, supfch_valor, supfch_desconto,
                            supfch_loja, supfch_caixa, supfch_item, supfch_exportado,
                            supfch_caixa_faturado, supfch_cupom_coo, supfch_cupom_data,
                            supfch_cz_nome_cliente, supfch_cz_data_hora_entrega,
                            supfch_cz_observacoes, supfch_cz_state_impressao
                            )
                     VALUES %s
                """ % ',\n'.join(values_itens)
            sql = sql.replace("'null'", 'null')
            exec_sql(sql, 'S', sta_tout=0)
            exec_sql('UPDATE caixa_cad_ficha SET supfch_exportado = True WHERE id IN (%s)' %
                     ','.join(ids), 'C', sta_tout=0)
            traced.info('Exportacao de fichas....\n%s' % '\n'.join(msg))
        except Exception as e:
            traced.error('Erro ao exportar fichas: %s' % e)

    def exportar(self):
        if not self._parent.conectado:
            traced.info('EXPORTACAO: sem conexao!')
            return False

        try:  # TESTA CONEXÃO
            sql = """SELECT id
                     FROM   supervisor_cad_pdvs
                     WHERE  suppdv_loja = %d
                       AND suppdv_caixa = %d LIMIT 1 """ % (config['loja'], config['caixa'])
            id_pdv = exec_sql(sql, 'S', ret=True, sta_tout=0)
        except Exception as e:
            traced.error('EXPORTACAO CUPOM: erro(%s)' % str(e))

        if self._ocupado:
            return False

        try:
            self.altera_data_atualizacao()
        except Exception as e:
            traced.error('Erro ao atualizar data no supervisor%s' % e)

        self._ocupado = True
        self.exportar_arq_mfd()
        # ------------------------------------ Exporta turnos ------------------------------------ #
        list_turnos = CaixaMovCaixa.query.filter(
            CaixaMovCaixa.supcax_loja == config['loja']).filter(
            CaixaMovCaixa.supcax_enviado == 'N')

        if list_turnos.count():
            traced.info('Exportando Caixa:')
            for turno in list_turnos:
                try:
                    sess.begin()
                    supervisor_mov_caixa_id = self.transacaoDB(['supcax_loja', 'supcax_caixa',
                                                                'supcax_data_hora_abertura',
                                                                'supcax_operador_id'],
                                                               turno, 'supervisor_mov_caixa',
                                                               optDelete=True)
                    if self.trata_erro(supervisor_mov_caixa_id, sess):
                        continue
                    turno.supcax_enviado = 'S'
                    sess.commit()
                except Exception as e:
                    traced.error(e)
                    sess.rollback()

        # ------------------------------------ Exporta cupons ------------------------------------ #
        list_cupons = CaixaMovCapa.query.join(CaixaMovCupomServidor).filter(
            CaixaMovCupomServidor.supcse_enviado == False).order_by('supcap_nro_cupom').all()
        self.exportar_fichas()

        if not list_cupons:
            self._ocupado = False
            return False

        qtd_cupoms = 0
        for cupom in list_cupons:
            extornar_cupom = False
            try:
                if not cupom:
                    traced.warning('EXPORTACAO CUPOM: NaoExiste'
                                   '(No. %d, Data: %s, Loja: %.4d, Caixa: %.3d)' %
                                   (cupom.supcap_nro_cupom,
                                    cupom.supcap_data_cupom.strftime('%d-%m-%Y'),
                                    cupom.supcap_loja, cupom.supcap_caixa))

                elif cupom.supcap_caixa == 999:  # NOTA FISCAL
                    self.exportar_nf(cupom)

                else:  # Cupom existe
                    traced.info(
                        'EXPORTACAO CUPOM: (No. %d, Data: %s, Loja: %.4d, Caixa: %.3d)' %
                        (cupom.supcap_nro_cupom, cupom.supcap_data_cupom.strftime('%d-%m-%Y'),
                         cupom.supcap_loja, cupom.supcap_caixa))

                    sess.begin()
                    try:
                        supervisor_capa_id = self.transacaoDB(['supcap_nro_cupom', 'supcap_loja',
                                                               'supcap_caixa', 'supcap_data_cupom'],
                                                              cupom, 'supervisor_mov_capa',
                                                              optDelete=True, nro_cupom=cupom.supcap_nro_cupom,
                                                              data_cupom=cupom.supcap_data_cupom)
                        if self.trata_erro(supervisor_capa_id, sess):
                            continue
                    except Exception as e:
                        sess.rollback()
                        traced.warning('EXPORTACAO CUPOM: {0:s}'.format(str(e)))
                        continue

                    mov_corpos = CaixaMovCorpos.query.join(CaixaMovCapa).filter(
                        CaixaMovCorpos.supcor_capa_id == cupom.id).all()

                    mov_corpo_xz = CaixaMovCorpoXZ.query.join(CaixaMovCapa).filter(
                        CaixaMovCorpoXZ.supcxz_capa_id == cupom.id).all()

                    mov_totalizadores_xz = CaixaMovTotalizadoresXZ.query.join(CaixaMovCapa).filter(
                        CaixaMovTotalizadoresXZ.suptxz_capa_id == cupom.id).all()

                    mov_fechamentos = CaixaMovFechamentos.query.join(CaixaMovCapa).filter(
                        CaixaMovFechamentos.supfec_capa_id == cupom.id).all()

                    mov_cupom_servidor = CaixaMovCupomServidor.query.join(CaixaMovCapa).filter(
                        CaixaMovCupomServidor.supcse_capa_id == cupom.id).all()

                    mov_autorizacoes = CaixaMovAutorizacoes.query.join(CaixaMovCapa).filter(
                        CaixaMovAutorizacoes.supaut_capa_id == cupom.id).all()

                    mov_meios_pagamentos = CaixaMovMeiosPagamentos.query.filter(
                        CaixaMovMeiosPagamentos.supmpa_data_movimento == cupom.supcap_data_cupom).filter(
                        CaixaMovMeiosPagamentos.supmpa_loja == cupom.supcap_loja).filter(
                        CaixaMovMeiosPagamentos.supmpa_caixa == cupom.supcap_caixa).filter(
                        CaixaMovMeiosPagamentos.supmpa_enviado == 'N').all()

                    kits_vendidos_ids = {}
                    for corpo in mov_corpos:
                        old_kit_vendido_id = False
                        if corpo.supcor_kit_vendido_id:
                            old_kit_vendido_id = corpo.supcor_kit_vendido_id
                            if not kits_vendidos_ids.get(corpo.supcor_kit_vendido_id):
                                kits_vendidos = CaixaMovKitsVendidos.query.filter(
                                              CaixaMovKitsVendidos.id == corpo.supcor_kit_vendido_id).all()
                                kit_vendido_id = self.transacaoDB('kits_vendidos()', kits_vendidos[0],
                                                                  'supervisor_mov_kits_vendidos',
                                                                  nro_cupom=cupom.supcap_nro_cupom,
                                                                  data_cupom=cupom.supcap_data_cupom)
                                kits_vendidos_ids[corpo.supcor_kit_vendido_id] = kit_vendido_id
                            else:
                                kit_vendido_id = kits_vendidos_ids.get(corpo.supcor_kit_vendido_id)

                            corpo.supcor_kit_vendido_id = kit_vendido_id

                        idOld = corpo.supcor_capa_id
                        corpo.supcor_capa_id = supervisor_capa_id
                        supervisor_mov_corpos_id = self.transacaoDB('corpos()', corpo,
                                                                    'supervisor_mov_corpos',
                                                                    nro_cupom=cupom.supcap_nro_cupom,
                                                                    data_cupom=cupom.supcap_data_cupom)
                        if self.trata_erro(supervisor_mov_corpos_id, sess):
                            extornar_cupom = True
                            break
                        corpo.supcor_capa_id = idOld
                        if old_kit_vendido_id:
                            corpo.supcor_kit_vendido_id = old_kit_vendido_id

                    if extornar_cupom:
                        continue

                    for corpo_xz in mov_corpo_xz:
                        idOld = corpo_xz.supcxz_capa_id
                        corpo_xz.supcxz_capa_id = supervisor_capa_id
                        supervisor_mov_corpo_xz_id = self.transacaoDB('corpo_xz()', corpo_xz,
                                                                      'supervisor_mov_corpo_xz',
                                                                      nro_cupom=cupom.supcap_nro_cupom,
                                                                      data_cupom=cupom.supcap_data_cupom)
                        if self.trata_erro(supervisor_mov_corpo_xz_id, sess):
                            extornar_cupom = True
                            break
                        corpo_xz.supcxz_capa_id = idOld

                    if extornar_cupom:
                        continue

                    for totalizadores_xz in mov_totalizadores_xz:
                        idOld = totalizadores_xz.suptxz_capa_id
                        totalizadores_xz.suptxz_capa_id = supervisor_capa_id
                        supervisor_mov_totalizadores_xz_id = self.transacaoDB('totalizadores_xz()', totalizadores_xz,
                                                                              'supervisor_mov_totalizadores_xz',
                                                                              nro_cupom=cupom.supcap_nro_cupom,
                                                                              data_cupom=cupom.supcap_data_cupom)
                        if self.trata_erro(supervisor_mov_totalizadores_xz_id, sess):
                            extornar_cupom = True
                            break
                        totalizadores_xz.suptxz_capa_id = idOld

                    if extornar_cupom:
                        continue

                    for fechamentos in mov_fechamentos:
                        mov_vencimentos = CaixaMovVencimentos.query.join(CaixaMovFechamentos).filter(
                            CaixaMovVencimentos.supfve_fechamentos_id == fechamentos.id).all()
                        idOld = fechamentos.supfec_capa_id
                        fechamentos.supfec_capa_id = supervisor_capa_id
                        supervisor_mov_fechamentos_id = self.transacaoDB('fechamentos()',
                                                                         fechamentos,
                                                                         'supervisor_mov_fechamentos',
                                                                         nro_cupom=cupom.supcap_nro_cupom,
                                                                         data_cupom=cupom.supcap_data_cupom)
                        if self.trata_erro(supervisor_mov_fechamentos_id, sess):
                            extornar_cupom = True
                            break
                        fechamentos.supfec_capa_id = idOld

                        if mov_vencimentos:
                            for vencimentos in mov_vencimentos:
                                idOld = vencimentos.supfve_fechamentos_id
                                vencimentos.supfve_fechamentos_id = supervisor_mov_fechamentos_id
                                supervisor_mov_vencimentos_id = self.transacaoDB('vencimentos()', vencimentos,
                                                                                 'supervisor_mov_vencimentos',
                                                                                  nro_cupom=cupom.supcap_nro_cupom,
                                                                                  data_cupom=cupom.supcap_data_cupom)
                                if self.trata_erro(supervisor_mov_vencimentos_id, sess):
                                    extornar_cupom = True
                                    break
                                vencimentos.supfve_fechamentos_id = idOld

                    if extornar_cupom:
                        continue

                    for autorizacoes in mov_autorizacoes:
                        idOld = autorizacoes.supaut_capa_id
                        autorizacoes.supaut_capa_id = supervisor_capa_id
                        supervisor_mov_autorizacoes_id = self.transacaoDB('autorizacoes()', autorizacoes,
                                                                          'supervisor_autorizacoes',
                                                                          nro_cupom=cupom.supcap_nro_cupom,
                                                                          data_cupom=cupom.supcap_data_cupom)
                        if self.trata_erro(supervisor_mov_autorizacoes_id, sess):
                            extornar_cupom = True
                            break
                        autorizacoes.supaut_capa_id = idOld

                    if extornar_cupom:
                        continue

                    for meios_pagamentos in mov_meios_pagamentos:
                        supervisor_mov_meios_pagamentos_id = self.transacaoDB(
                            ['supmpa_loja', 'supmpa_caixa', 'supmpa_data_movimento',
                             'supmpa_finalizadora', 'supmpa_tipo_documento'],
                            meios_pagamentos, 'supervisor_mov_meios_pagamentos', optDelete=True,
                            nro_cupom=cupom.supcap_nro_cupom, data_cupom=cupom.supcap_data_cupom)
                        if self.trata_erro(supervisor_mov_meios_pagamentos_id, sess):
                            extornar_cupom = True
                            break
                        meios_pagamentos.supmpa_enviado = 'S'

                    if extornar_cupom:
                        continue

                    for cupom_servidor in mov_cupom_servidor:
                        idOld = cupom_servidor.supcse_capa_id
                        cupom_servidor.supcse_capa_id = supervisor_capa_id
                        supervisor_mov_cupom_servidor_id = self.transacaoDB('cupom_servidor()', cupom_servidor,
                                                                            'supervisor_mov_cupom_servidor',
                                                                            nro_cupom=cupom.supcap_nro_cupom,
                                                                            data_cupom=cupom.supcap_data_cupom,
                                                                            default={'supcse_enviado': False})
                        if self.trata_erro(supervisor_mov_cupom_servidor_id, sess):
                            extornar_cupom = True
                            break
                        cupom_servidor.supcse_capa_id = idOld
                        cupom_servidor.supcse_enviado = 'T'

                    if extornar_cupom:
                        continue

                    if not mov_cupom_servidor:
                        CaixaMovCupomServidor.update_or_create({'supcse_enviado': 'T',
                                                                'supcse_capa_id': cupom.id})

                    sess.commit()

                    traced.info(
                        'EXPORTACAO CUPOM: Enviado(Cupom No. %d, Data: %s, Loja: %.4d, Caixa: %.3d'
                        % (cupom.supcap_nro_cupom, cupom.supcap_data_cupom.strftime('%d-%m-%Y'),
                           cupom.supcap_loja, cupom.supcap_caixa))

                    qtd_cupoms += 1
            except Exception as e:
                sess.rollback()
                if cupom.supcap_nro_cupom and cupom.supcap_data_cupom:
                    traced.error('EXPORTACAO CUPOM: erro no envio cupom %s do dia %s :\n(%s)' %(cupom.supcap_nro_cupom,
                                                                                                cupom.supcap_data_cupom,
                                                                                                str(e)))
                else:
                    traced.error('EXPORTACAO CUPOM: erro(%s)' % str(e))
                self._ocupado = False

        traced.info('*** Total  Enviados %.6d' % qtd_cupoms)

        self._ocupado = False
        return {'*** Total  Enviados ': '%.6d' % qtd_cupoms}

    def exportar_nf(self, cupom):
        nfe_capa = CaixaNfeCapa.get_by(id=cupom.supcap_nro_cupom)
        if nfe_capa:
            t_tc = ('V', 'RG')
            if nfe_capa.state == 0:
                t_tc = ('C', 'NC')
            cliente = ''
            if nfe_capa.destinatario_doc:
                cliente = nfe_capa.destinatario_doc.rjust(15, '0')
            mov_capa = SupervisorMovCapa()
            mov_capa.supcap_data_cupom = nfe_capa.data_emissao
            mov_capa.supcap_loja = 1
            mov_capa.supcap_caixa = 999
            mov_capa.supcap_nro_cupom = nfe_capa.id
            mov_capa.supcap_operador_id = 1
            mov_capa.supcap_data_inicio = nfe_capa.data_emissao
            mov_capa.supcap_data_fim = nfe_capa.data_emissao
            mov_capa.supcap_tipo = t_tc[0]
            mov_capa.supcap_tipo_comprovante = t_tc[1]
            mov_capa.supcap_total_cupom = nfe_capa.vr_total_nf
            mov_capa.supcap_cliente = cliente
            mov_capa.supcap_nome_cliente = nfe_capa.destinatario_nome
            data = mov_capa.to_dict()
            sqlDelete = dict_2_sql('supervisor_mov_capa', data, 'D')
            exec_sql(sqlDelete, 'S', ret=True, sta_tout=0)
            if data.get('id'):
                del data['id']
            s = dict_2_sql('supervisor_mov_capa', data, 'I')
            idDbCapa = exec_sql(s, 'S', ret=True, sta_tout=0)
            for nfe in nfe_capa.item_ids:
                cod_ret = trib_ecf = ''
                departamento = 0
                cad_itens = CaixaCadItens.get_by(CodigoItem=nfe.codigo_item,
                                                 supite_loja=self._parent.loja.supcfc_loja)
                if cad_itens:
                    departamento = cad_itens.Departamento
                    trib_ecf = cad_itens.Tributacao
                    trib = CaixaCadTributacao.get_by(CodigoTribEcf=trib_ecf)
                    if trib:
                        cod_ret = trib.TribRetag
                mov_corpos = SupervisorMovCorpos()
                mov_corpos.supcor_capa_id = idDbCapa[0][0]
                mov_corpos.supcor_sequencia = nfe.sequencia
                mov_corpos.supcor_item = nfe.codigo_item
                mov_corpos.supcor_quantidade = nfe.quantidade
                mov_corpos.supcor_preco_total = nfe.vr_total_item
                mov_corpos.supcor_desconto_item = nfe.vr_desconto
                mov_corpos.supcor_acrescimo_item = nfe.vr_acrescimos
                mov_corpos.supcor_venda_cancelamento = t_tc[0]
                mov_corpos.supcor_tributacao_ecf = trib_ecf
                mov_corpos.supcor_tributacao_retag = cod_ret
                mov_corpos.supcor_aliquota = 0
                mov_corpos.supcor_base = 0
                mov_corpos.supcor_nome_item = nfe.descricao
                mov_corpos.supcor_unidade = nfe.unidade
                mov_corpos.supcor_preco_unitario = nfe.vr_unitario
                mov_corpos.supcor_indicador_arr_trunc = 'T'
                mov_corpos.supcor_indicador_ppt = 'T'
                data = mov_corpos.to_dict()
                if data.get('id'):
                    del data['id']
                s = dict_2_sql('supervisor_mov_corpos', data, 'I')
                idDbCorpos = exec_sql(s, 'S', ret=True, sta_tout=0)

            mov_fechamentos = SupervisorMovFechamentos()
            mov_fechamentos.supfec_loja = 1
            mov_fechamentos.supfec_caixa = 999
            mov_fechamentos.supfec_capa_id = idDbCapa[0][0]
            mov_fechamentos.supfec_sequencia = 1
            mov_fechamentos.supfec_finalizadora = 1
            mov_fechamentos.supfec_tipo_finalizadora = 'D'
            mov_fechamentos.supfec_valor = nfe_capa.vr_total_nf
            mov_fechamentos.supfec_cnpj = nfe_capa.destinatario_doc
            mov_fechamentos.supfec_nome_finalizadora = 'DINHEIRO'
            data = mov_fechamentos.to_dict()
            if data.get('id'):
                del data['id']
            s = dict_2_sql('supervisor_mov_fechamentos', data, 'I')
            idDbFechamentos = exec_sql(s, 'S', ret=True, sta_tout=0)
            sess.begin()
            cupom.Enviado = 'S'
            sess.commit()
            mov_cupom_servidor = SupervisorMovCupomServidor()
            mov_cupom_servidor.supcse_capa_id = idDbCapa
            mov_cupom_servidor.supcse_enviado = False
            data = mov_cupom_servidor.to_dict()
            if data.get('id'):
                del data['id']
            s = dict_2_sql('supervisor_mov_cupom_servidor', data, 'I')
            idDbCupomServior = exec_sql(s, 'S', ret=True, sta_tout=0)


if __name__ == '__main__':
    e = Exportacao()
    e.exportar()
