#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8 -*-


from decimal import Decimal
from time import sleep
import os
from datetime import datetime

from stoqdrivers.exceptions import CouponOpenError
from stoqdrivers.printers.fiscal import FiscalPrinter
from stoqdrivers.serialbase import SerialPort
from stoqdrivers.enum import UnitType, TaxType
from kiwi.log import Logger, set_log_file

from tools.config import config, LOG_FILE
from models import tbl_cad, db_utils
from models.db_utils import sess


set_log_file(LOG_FILE)
log = Logger('sigpdv.teste')


#ECF = get_printer(config['loja'], config['caixa'])

USER_C = config['db_pdv_user']
PWD_C = config['db_pdv_pass']
HOST_C = config['db_pdv_host']
DB_C = config['db_pdv_name']

USER_P = config['db_sup_user']
PWD_P = config['db_sup_pass']
HOST_P = config['db_sup_host']
DB_P = config['db_sup_name']

SQL = db_utils.NativeSql

class sig(object):
    def __init__(self):
        pass


    def inserir_iten_tabela(self, cod, desc, valor):
        i = False
        try:
            sess.begin() # Inicia Transação

            i = tbl_cad.itens(

                Loja = config['loja'],
                CodigoItem = cod,
                Descricao = desc,
                Departamento = 0,
                Tributacao = 'F',
                Pesavel = 'N',
                Fracionado = 'N',
                Preco = valor,
                Unidade = 'UN',
                CodigoRetaguarda = '0' * 20,
                FatorMultiplicador = 0,
                Producao = 'P',
                IAT = 'P'
            )
            sess.commit()
        except Exception as e:
            print 'ERRO:', e
            sess.rollback()
        return i


    def vender(self):
        # Identifica o cliente na aberttura do cupom'
        ECF.identify_customer('Christian Douglas', 'Rua Terenas, 47 - Carmo', '888.159.351-34')
        # Envia o camando para a abertura de cupom fiscal
        ECF.open()
        # Envia um item para a ecf com desconto de R$  0.5
        ECF.add_item(
            item_code='00000000000123',
            item_description='KIT ITEM',
            item_price=Decimal('20.00'),
            taxcode='F',
            items_quantity=Decimal('1.0'),
            unit=UnitType.CUSTOM,
            discount=Decimal('0.5'),
            surcharge=Decimal('0'),
            unit_desc='CX'
        )
        # totaliza o cupom com R$ 0.5 de acréscimo, e aguarda o pagamento
        ECF.totalize(surcharge=Decimal('0.6'), taxcode=TaxType.ICMS)
        # efetua o pagamento com indice 01 - Dinheiro na bematech
        ECF.add_payment('01', Decimal('37'), '')
        # fecha o cupom
        ECF.close()

    def conecta_ecf(self, fabricante):
        if fabricante == 'bematech':
            model = 'MP2100'
        elif fabricante == 'daruma':
            model = 'FS600MFD'
        elif fabricante ==  'dataregis':
            model = 'EP375'
        elif fabricante ==  'elgin':
            model = 'Pay2023'
        elif fabricante == 'sweda':
            model = 'IFS9000I'

        port = SerialPort('/dev/ttyS0')
        port.open()

        return FiscalPrinter(brand=fabricante, model=model, port=port)

    def executar_sql(self):
        sql_local = SQL(HOST_P, USER_P, PWD_P, DB_P).run_sql
        sql_super = SQL(HOST_C, USER_C, PWD_C, DB_C).run_sql
        print '--- SQL LOCAL ------------------------------------------------------------------------------------------'
        print sql_local('SELECT * FROM TblCad_Itens LIMIT 1') # Retorna Dicionario
        print sql_local('SELECT * FROM TblCad_Itens LIMIT 1', 'F') # Retorna Lista
        print '--- SQL SUPERVISOR -------------------------------------------------------------------------------------'
        print sql_super('SELECT * FROM TblCad_Itens LIMIT 1') # Retorna Dicionario
        print sql_super('SELECT * FROM TblCad_Itens LIMIT 1', 'F') # Retorna Lista

import types

def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
    '''
    Returns a bytestring version of 's', encoded as specified in 'encoding'.

    If strings_only is True, don't convert (some) non-string-like objects.
    '''
    if strings_only and isinstance(s, (types.NoneType, int)):
        return s
    if not isinstance(s, basestring):
        try:
            return str(s)
        except UnicodeEncodeError:
            if isinstance(s, Exception):
                # An Exception subclass containing non-ASCII data that doesn't
                # know how to print itself properly. We shouldn't raise a
                # further exception.
                return ' '.join([smart_str(arg, encoding, strings_only,
                    errors) for arg in s])
            return unicode(s).encode(encoding, errors)
    elif isinstance(s, unicode):
        return s.encode(encoding, errors)
    elif s and encoding != 'utf-8':
        return s.decode('utf-8', errors).encode(encoding, errors)
    else:
        return s
def force_unicode(s, encoding='utf-8', strings_only=False, errors='strict'):
    '''
    Similar to smart_unicode, except that lazy instances are resolved to
    strings, rather than kept as lazy objects.

    If strings_only is True, don't convert (some) non-string-like objects.
    '''
    # Handle the common case first, saves 30-40% in performance when s
    # is an instance of unicode. This function gets called often in that
    # setting.
    if isinstance(s, unicode):
        return s
    if strings_only:
        return s
    try:
        if not isinstance(s, basestring,):
            if hasattr(s, '__unicode__'):
                s = unicode(s)
            else:
                try:
                    s = unicode(str(s), encoding, errors)
                except UnicodeEncodeError:
                    if not isinstance(s, Exception):
                        raise
                        # If we get to here, the caller has passed in an Exception
                    # subclass populated with non-ASCII data without special
                    # handling to display as a string. We need to handle this
                    # without raising a further exception. We do an
                    # approximation to what the Exception's standard str()
                    # output should be.
                    s = ' '.join([force_unicode(arg, encoding, strings_only,
                        errors) for arg in s])
        elif not isinstance(s, unicode):
            # Note: We use .decode() here, instead of unicode(s, encoding,
            # errors), so that if s is a SafeString, it ends up being a
            # SafeUnicode at the end.
            s = s.decode(encoding, errors)
    except UnicodeDecodeError as e:
        if not isinstance(s, Exception):
            raise Exception(e)
        else:
            # If we get to here, the caller has passed in an Exception
            # subclass populated with non-ASCII bytestring data without a
            # working unicode method. Try to handle this without raising a
            # further exception by individually forcing the exception args
            # to unicode.
            s = ' '.join([force_unicode(arg, encoding, strings_only,
                errors) for arg in s])
    return s
if __name__ == '__main__':
    #c = sig()
    from models import tbl_mov
#    emitente = tbl_cfg.configuracao.get_by(Loja=1)


    #port = SerialPort('/home/SIG/christian.douglas/.wine/dosdevices/com4')
    port = SerialPort('/dev/ttyS0')
#
    #ecf = FiscalPrinter(brand='daruma', model='FS600MFD', port=port)
    #ecf = FiscalPrinter(brand='sweda', model='IFS9000I', port=port)
    ecf = FiscalPrinter(brand='bematech', model='MP2100', port=port)

    #sintegra = ecf.get_sintegra()
    #print sintegra

    print ecf.summarize()
    coo = ecf.get_coo()
    print 'COO', coo

    def gerar_reducao_xz(tipo):
        """
        Emiti redução zx gerando os dados na base de dados
        """
        # A variável abaixo tem a finalidade dar continuidade na aplicação caso um comando não seja executado na ECF.
        # Por padrão, ao mandar um comando (_cmd) e este não for executado, a aplicação entra em loop ate
        # conseguir executar o comando. Passando o parametro (stay_in_loop=False), permite a aplicação prosseguir.
        stay_in_loop_XZ = False
        #        try:
        try:
            coo = False
            try:
                log.info('Tentando comunicar com o ECF')
                coo = ecf.get_coo()
            except Exception as e:
                log.error('Erro ao Reconectar: %s' % e)
            if not coo:
                return False
#            open('pdvsig.sig')
#            if os.path.exists(os.path.join(ROOT_PATH, 'gt.sig')):
#                checar_gt(self._printer)
            data_ini = ecf.get_ecf_date()
            if not data_ini:
                return False
            sintegra = ecf.get_sintegra()
            if not sintegra:
                return False
            gt_atual = sintegra.total
            gt_anterior = gt_atual - sintegra.period_total
            if tipo == 'X':
                ecf.summarize()
            elif tipo == 'Z':
                try:
                    ecf.close_till()
                except CouponOpenError:
                    ecf.cancel()
                    cupom = tbl_mov.capa.get_by(
                        Loja=config.get('loja', config['loja']),
                        Caixa=config.get('caixa', config['caixa']),
                        NroCupom=sintegra.coo
                    )
                    if cupom:
                        print 'CANCELA CUPOM'

                    else:
                        print 'NAO ACHEI'
                    ##self.gera_capa_rg(tipo='C', tipo_c='NC')
                    ecf.close_till()
            else:
                pass
                # TOTALIZADORES
            tributacao = {}
            acrescimos_iss = 0
            acrescimos = 0
            cancelamentos_iss = 0
            cancelamentos = 0
            descontos_iss = 0
            descontos = 0
            for t in sintegra.taxes:
                if (t[0] == 'ACRESC') and (t[2] == 'ISS'):
                    acrescimos_iss = t[1]
                elif (t[0] == 'ACRESC') and (t[2] == 'ICMS'):
                    acrescimos = t[1]
                elif (t[0] == 'CANC') and (t[2] == 'ISS'):
                    cancelamentos_iss = t[1]
                elif (t[0] == 'CANC') and (t[2] == 'ICMS'):
                    cancelamentos = t[1]
                elif (t[0] == 'DESC') and (t[2] == 'ISS'):
                    descontos_iss = t[1]
                elif (t[0] == 'DESC') and (t[2] == 'ICMS'):
                    descontos = t[1]
                else:
                    tributacao[t[0]] = t[1]
                    # RECONECTAR
            timeout = 0
            while True:
                try:
                    timeout += 1
                    log.info('Tentando comunicar com o ECF* (%d)' % timeout)
                    sleep(1)
                    coo = ecf.get_coo()
                    int(coo)
                    if coo:
                        break
                except Exception as e:
                    log.error('Erro ao Reconectar*: %s' % e)
                    # GRAVAR NO BANCO
                if ecf.get_status().block_z:
                    log.info('Não foi possível emitir a Redução Z.')


            # Nao consegui obter o coo da ecf ou excedeu o numero de tentativas
            if not coo and timeout > 300:
                raise Exception('Foi excedido o numero de tentativas para obter o numero coo da Leitura Z')

            nro_cupom = coo  # self._cmd('get_coo()')
            coo_inicial_final = ecf.get_coo_inicial_final()
            sess.begin()
            tbl_mov.capa(
                DataCupom=datetime.now().date(),
                Loja=1,
                Caixa=5,
                NroCupom=nro_cupom,
                Operador=1,
                DataInicio=data_ini,
                DataFim=data_ini,
                Tipo=tipo,
                NroSerie=sintegra.serial,
                MarcaEcf=ecf.brand,
                ModeloEcf=ecf._driver.model_name,
                NroProprietario=1,
                CCF=ecf.get_ccf(),
                GNF=ecf.get_gnf(),
                CRZ=ecf.get_crz(),
                GRG=sintegra.grg,
                CDC=ecf.get_cdc(),
                TipoComprovante='NC'
            )
            sess.commit()
            tributos = tbl_cad.tributacao.query.all()
            sess.begin()
            for t in tributos:
                if t.TipoTributacao == 'T':
                    key = '%.2f' % t.Aliquota
                    key = key.replace('.', '').rjust(4, '0')
                else:
                    key = t.TipoTributacao

                try:
                    valor = tributacao[key]
                except:
                    valor = Decimal('0')

                tbl_mov.totalizadores_xz(
                    DataCupom=self._date_coupon,
                    Loja=self._loja.Loja,
                    Caixa=self._caixa_id,
                    NroCupom=nro_cupom,
                    CodigoTribEcf=t.CodigoTribEcf,
                    DescricaoTotalizador=t.Descricao,
                    Valor=valor,
                    Aliquota=t.Aliquota,
                    Base=t.Base,
                    TipoImposto=t.TipoImposto,
                    TipoTributacao=t.TipoTributacao,
                    CRZ=self.send_command('get_crz()', stay_in_loop=stay_in_loop_XZ),
                    NroSerie=self.send_command('get_serial()', stay_in_loop=stay_in_loop_XZ)
                )
            sess.commit()
            sess.begin()
            tbl_mov.corpo_xz(
                DataCupom=self._date_coupon,
                Loja=self._loja.Loja,
                Caixa=self._caixa_id,
                NroCupom=nro_cupom,
                DataMovimento=self._date_coupon,
                CCF=sintegra.ccf,
                GNF=sintegra.gnf,
                CRO=sintegra.cro,
                CupomInicial= str(coo_inicial_final)[:5],
                CupomFinal=nro_cupom,
                GTAtual=gt_atual,
                GTAnterior=gt_anterior,
                Cancelamentos=cancelamentos,
                Descontos=descontos,
                Acrescimos=acrescimos,
                DescontosISS=descontos_iss,
                AcrescimosISS=acrescimos_iss,
                CancelamentosISS=cancelamentos_iss,
                ModeloEcf=self._printer.brand
            )

            sess.commit()
            sess.begin()
            tbl_mov.cupom_servidor(
                DataCupom=self._date_coupon,
                NroCupom=nro_cupom,
                Loja=self._loja.Loja,
                Caixa=self._caixa_id,
                Enviado='N')
            sess.commit()
            if tipo == 'Z':
                self.gera_movimento_ecf(self._date_coupon, self._date_coupon)
                if self.z_pendente:
                    self.verificar_mesas_abertas(True)
                    self.z_pendente = False
                else:
                    if config['shutdownaposz']:
                        self._lock_keyboard(True)
                        enviou = 'N'
                        inc = 0
                        loop = 60
                        while enviou == 'N':
                            inc += 1
                            sql = 'SELECT Enviado FROM TblMov_CupomServidor WHERE Loja = %d AND Caixa = %d AND NroCupom = %d' % (config['loja'], config['caixa'], coo)
                            ret = self.exec_sql_c(sql)
                            if ret:
                                ret = ret[0]
                                enviou = ret['Enviado']
                            self.show_info_proccess(True, 'Desligando o Sistema: (%d-%d)' % (inc, loop))
                            QtGui.QApplication.processEvents()
                            print 'Desligando o Sistema: (%d-%d)' % (inc, loop)
                            sleep(1)
                            if inc == loop: break
                        os.system('exec /bin/shutdown -h now')
                        self.show_info_proccess(False)
            QtGui.QApplication.processEvents()
            return sintegra.cro
        except Exception:
            self.tratar_except()
            return False
        return True


