#!/usr/bin/python
# -*- encoding: utf-8 -*-

import os
from datetime import date, datetime, timedelta
from PySide import QtGui
from ctypes import CDLL
from subprocess import call
from sigpdv import root_path
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.control import utils
from sigpdv.tools.log import traced as Log
from sigpdv.tools.config import config
from sigpdv.ui.messagebox import message_dlg, MSG_QUESTION, YES_ROLE, MSG_INFO
from sigpdv.control.utils import remove_accents
import platform

ERRO_DARUMA = {0: 'Erro durante a execução.',
               1: 'Operação realizada com sucesso.',
              -1: 'Erro do Método.',
              -2: 'Parâmetro incorreto.',
              -3: 'Alíquota (Situação tributária) não programada.',
              -4: 'Chave do Registry não encontrada.',
              -5: 'Erro ao Abrir a porta de Comunicação.',
              -6: 'Impressora Desligada.',
              -7: 'Erro no Número do Banco.',
              -8: 'Erro ao Gravar as informações no arquivo de Status ou de Retorno de Info.',
              -9: 'Erro ao Fechar a porta de Comunicação.',
              -10: 'O ECF não tem a forma de pagamento e não permite cadastrar esta forma.',
              -12: 'A função executou o comando porém o ECF sinalizou Erro,\n chame a rStatusUltimoCmdInt_ECF_Daruma para identificar o Erro.',
              -13: 'A Emissao da RZ ira bloquear a ECF ate as 23:59:59h,\n Execute o comando novamente para confirmar a Emissao.',
              -24: 'Forma de Pagamento não Programada.',
              -27: 'Foi Detectado Erro ou Warning na Impressora.',
              -28: 'Time-Out.',
              -40: 'Tag XML Inválida.',
              -41: 'NCM não encontrado.',
              -42: 'Item não encontrado.',
              -50: 'Problemas ao Criar Chave no Registry.',
              -51: 'Erro ao Gravar LOG.',
              -52: 'Erro ao abrir arquivo.',
              -53: 'Fim de arquivo.',
              -60: 'Erro na tag de formatação DHTML.',
              -78: 'Impressora FS600 com SB 01.04, favor atualizar.',
              -90: 'Erro Configurar a Porta de Comunicação.',
              -99: 'Parâmetro inválido ou ponteiro nulo de parâmetro.',
              -101: 'Erro ao LER ou ESCREVER arquivo.',
              -102: 'Erro arquivo corrompido.',
              -103: 'Não foram encontradas as DLLs auxiliares (lebin.dll e LeituraMFDBin.dll).',
              -104: 'Data informada é inferior ao primeiro documento emitido.',
              -105: 'Data informada é maior que a ultima redução Z impressa.',
              -106: 'Não possui movimento no periodo solicitado.',
              -107: 'Porta de comunicação ocupada.',
              -110: 'Indica que o GT foi atualizado no arquivo de registro do PAF.',
              -112: 'O numero de série já existe no arquivo do PAF.',
              -113: 'ECF conectado não cadastrado no arquivo do PAF.',
              -114: 'MFD Danificada.',
              -115: 'Erro ao abrir arquivos .idx/.dat/.mf.',
              -116: 'Intervalo solicitado não é válido.',
              -117: 'Impressora não identificada durante download dos binários.',
              -118: 'Erro ao abrir porta serial.',
              -119: 'Leitura dos binários abortada.',
               }

class MFD(object):
    def __init__(self, parent):
        self.parent = parent

    @Log
    def emitir_mfd(self, loja, caixa, tipo_mfd=1, tipo_intevalo='D', intervalo_inicial=None,
                   intervalo_final=None, operador_id=None, imprimir=False, mfd_mensal=False):
        """
        :param loja: loja_id
        :param caixa: caixa_id
        :param tipo_mfd: tipo=1 (Espelho MFD), tipo=2 (Arquivo MFD), tipo=3(Arquivo TDM)
        :param tipo_intevalo: 'D' (Intervalo por Data), 'C' (Intervalo por COO)
        :return boolean
        """

        relatorio = self.parent.ui.lblFiscalInfo.text()
        file_download = os.path.join(root_path, 'down.mfd')

        op = 2
        start = intervalo_inicial  # intervalo por coo
        end = intervalo_final
        if tipo_intevalo == 'D':  # intervalo por data
            start = intervalo_inicial.strftime('%d%m%y')
            end = intervalo_final.strftime('%d%m%y')
            op = 1

        linuxmfd_args = '%s %s %d %s %s %.4d' % (config['ecf'], file_download, op,
                                                 str(start), str(end), loja)
        try:
            if self.parent.obj_fiscal_printer.printer.brand == 'bematech':
                self.parent.infoText('Processo demorado! Aguarde...')
                QtGui.QApplication.processEvents(10)
                err_cod = {1: 'RETORNO ERRO NA SERIAL',
                           2: 'RETORNO ERRO NO ARQUIVO',
                           3: 'RETORNO IMPRESSORA NAO RESPONDE',
                           4: 'RETORNO SINTAXE INCORRETA',
                           5: 'RETORNO PORTA COMUNICACAO INVALIDA',
                           6: 'RETORNO TIPO DOWNLOAD INVALIDO',
                           7: 'RETORNO DATA INVALIDA',
                           8: 'RETORNO DATA INICIAL POSTERIOR A FINAL',
                           9: 'RETORNO COO INVALIDO',
                           10: 'RETORNO USUARIO INVALIDO',
                           11: 'RETORNO COO INICIAL POSTERIOR AO FINAL',
                           12: 'RETORNO POUCA MEMORIA PARA EXECUCAO'}

                ret_cod = call('./libs/linuxmfd %s' % linuxmfd_args, shell=True)

                if ret_cod in err_cod.keys():
                    str_err = '%d - %s' % (ret_cod, err_cod[ret_cod])
                    self.parent.show_info_proccess(False)
                    raise Exception, str_err
        except:
            self.parent._set_screen_state(2)
            self.parent.show_info_proccess(False)
            return False

        dir_mfd = str(root_path + '/MFD/').replace('//', '/')
        dir_mfd_arquivo = str(root_path + '/MFD/arquivo/').replace('//', '/')

        if tipo_mfd == 1: # espelho
            if op == 1:
                if mfd_mensal:
                    mes_passado = date.today().month - 1
                    mes_por_extenso = utils.get_mes_por_extenso(mes_passado)
                    file_name = os.path.join(dir_mfd, 'Espelho_MFD_%s.txt' % str(mes_por_extenso))
                else:
                    file_name = os.path.join(dir_mfd, 'Espelho_MFD_Data_%s.txt' % str(date.today()))
            else:
                file_name = os.path.join(dir_mfd, 'Espelho_MFD_Reducao_%s.txt' % str(date.today()))
        elif tipo_mfd == 2: # arquivo MFD
            if tipo_intevalo.upper() == 'D':
                op = 7
                file_name = os.path.join(dir_mfd_arquivo, 'Arq_MFD_Data_%s_%s.txt' %
                                         (str(start), str(end)))
            elif tipo_intevalo.upper() == 'C':
                op = 8
                file_name = os.path.join(dir_mfd_arquivo, 'Arq_MFD_Coo_%s_%s.txt' %
                                         (str(start), str(end)))

        else: # tipo = 3 (TDM)
            if tipo_intevalo.upper() == 'D':
                op = 9
                file_name = os.path.join(dir_mfd_arquivo, 'Arq_TDM_Data_%s_%s.txt' %
                                         (str(start), str(end)))
            elif tipo_intevalo.upper() == 'C':
                op = 10
                file_name = os.path.join(dir_mfd_arquivo, 'Arq_TDM_Coo_%s_%s.txt' %
                                         (str(start), str(end)))

        if self.parent.obj_fiscal_printer.printer.brand == 'bematech':
            if not self.mfd_tdm_bematech(file_name, file_download, op, start, end):
                self.parent.show_info_proccess(False)
                return False

        elif self.parent.obj_fiscal_printer.printer.brand == 'elgin':
            if not self.mfd_eugin(tipo_mfd, imprimir, tipo_intevalo, file_name, intervalo_inicial,
                                  intervalo_final):
                self.parent.show_info_proccess(False)
                return False

        elif self.parent.obj_fiscal_printer.printer.brand == 'daruma':
            if not self.mfd_tdm_daruma(tipo_mfd, tipo_intevalo, file_name, start, end):
                self.parent.show_info_proccess(False)
                return False

        elif self.parent.obj_fiscal_printer.printer.brand == 'sweda':
            if not self.mfd_sweda(relatorio, tipo_mfd, tipo_intevalo, start, end, file_name):
                self.parent.show_info_proccess(False)
                return False

        if imprimir:
            f = open(file_name, 'r')
            raw = f.read()
            f.close()
            GerencialReport(report_id=config['rg_relatorio_gerencial'],
                            text=raw, loja_id=loja, caixa_id=caixa, operador_id=operador_id,
                            parent=self.parent)
        # usa a funcao get_printer para reconfigurar a porta da ecf que estava sendo alterada
        # pela lib da bematech ao tirar um espelho da MFD ou Arquivo MFD  e entao perdia cominica
        # cao com a ecf.
        self.parent.obj_fiscal_printer.get_printer()
        message_dlg(self.parent, MSG_INFO, text='Arquivo gerado:', file=file_name)

        return True

    @Log
    def mfd_tdm_bematech(self, file_name, file_download, op, start, end):
        """
        Emite o arquivo MFD(op=7,8) ou TDM(op=9,10)
        """
        err_cod_bem = {1: 'Arquivo de entrada inválido',
                       2: 'Arquivo de saída vazio',
                       30: 'Erro geral',
                       4: 'Erro ao abrir arquivo MFD',
                       5: 'Arquivo MFD sem número de série',
                       6: 'Data inválida',
                       7: 'Erro ao abrir arquivo TXT',
                       8: 'Erro ao abrir arquivo MDB',
                       9: 'Arquivo sem dados válidos',
                       10: 'Comando não executado',
                       11: 'Erro de comunicação',
                       12: 'Tipo invâlido',
                       13: 'Parâmetros invâlidos'}

        bemamfd2_args = '%s %s %d %s %s 001' % (file_name, file_download, op, str(start), str(end))

        Log.warning('RUN bemamfd2: [./libs/bemamfd2 %s]' % bemamfd2_args)
        ret_cod = call('./libs/bemamfd2 %s' % bemamfd2_args, shell=True)

        if ret_cod in err_cod_bem.keys():
            str_err = '%d - %s' % (ret_cod, err_cod_bem[ret_cod])
            self.parent.show_info_proccess(False)
            message_dlg(self.parent, MSG_INFO, str_err)
            raise False

        return True

    @Log
    def espelho_mfd_elgin(self, imprimir, tipo_intervalo, file_name, start, end):
        TipoDocumento = '11' # Leitura da Memoria Fiscal, de acordo com a tabela do manual de programacao da elgin
        destino = 'I' if imprimir else 'S' # 'I' = imprimir na ecf, 'S' = gerar arquivo arquivo
        if tipo_intervalo == 'D':   # intervalo por data
            res = self.parent.obj_fiscal_printer.send_command(
                """emite_leitura_fita_detalhe(COOFinal={COOFinal}, COOInicial={COOInicial},
                                              yf={yf}, mf={mf}, df={df}, yi={yi}, mi={mi}, di={di},
                                              Destino='{Destino}', Proprietario='{Proprietario}',
                                              TipoDocumento='{TipoDocumento}')""".format(
                                                         COOFinal=False,
                                                         COOInicial=False,
                                                         yf=end.year,   mf=end.month,   df=end.day,
                                                         yi=start.year, mi=start.month, di=start.day,
                                                         Destino=destino,
                                                         Proprietario='',
                                                         TipoDocumento=TipoDocumento))
        else:   # intevalo por coo
            res = self.parent.obj_fiscal_printer.send_command(
                """emite_leitura_fita_detalhe(COOFinal={COOFinal}, COOInicial={COOInicial},
                                              yf={yf}, mf={mf}, df={df}, yi={yi}, mi={mi}, di={di},
                                              Destino='{Destino}', Proprietario='{Proprietario}',
                                              TipoDocumento='{TipoDocumento}')""".format(
                                                        COOFinal= end,
                                                        COOInicial=start,
                                                        yf=False, mf=False, df=False,
                                                        yi=False, mi=False, di=False,
                                                        Destino=destino,
                                                        Proprietario='',
                                                        TipoDocumento=TipoDocumento))

        return res

    @Log
    def mfd_eugin(self, tipo_mfd, imprimir, tipo_intervalo, file_name, start, end):
        res = False
        try:
            if tipo_mfd == 1: # 1=Espelho, 2=Arquivo ato/cotepe(no linux não tem como gerar opção 2)
                res = self.espelho_mfd_elgin(imprimir, tipo_intervalo, file_name, start, end)
                res = remove_accents(res)

        except Exception as e:
            Log.error(str(e))

        if not res:
            return False
        try:
            f = open(file_name, 'w')
            f.write(res)
            f.close()
        except Exception as e:
            Log.error(str(e))

        return True

    @Log
    def mfd_tdm_daruma(self, tipo_mfd, tipo_intervalo, file_name, start, end):
        if platform.machine() == 'x86_64':
            root_path = '/usr/local/sigpdv/libs/daruma/x86_64'

        lib_path = os.path.join(root_path, 'libDarumaFramework.so')
        lib = CDLL(lib_path)
        if tipo_mfd == 1: # Espelho MFD        if tipo_mfd == 1: # Espelho MFD
            if tipo_intervalo.upper() == 'C': # por Coo
                ret_cod = lib.rGerarEspelhoMFD_ECF_Daruma('2', '%06d' % start, '%06d' % end)
            else: # por Data
                ret_cod = lib.rGerarEspelhoMFD_ECF_Daruma('1', start, end)

            os.system('mv /usr/local/sigpdv/MDF/Espelho_MFD.txt %s' % file_name)
            if ret_cod != 1:
                Log.error('Espelho MFD, Não foi possível obter a MFD: err %d' % ret_cod)
                raise Exception('Não foi possível obter a MFD')
        elif tipo_mfd in (2,3): # arquivo mfd(ato cotepe)(=2) ou arquivo TDM(=3)
            arq = 'MFD' if tipo_mfd == 2 else 'TDM'
            if tipo_intervalo.upper() == 'D': # intervalo por data
                ret_cod = lib.rGerarRelatorio_ECF_Daruma(arq, 'DATAM',
                                                         start,
                                                         end)
                os.system('mv /usr/local/sigpdv/MFD/ATO_%s_DATA.TXT %s' % (arq, file_name))

            elif tipo_intervalo.upper() == 'C': # intervalo por COO
                ret_cod = lib.rGerarRelatorio_ECF_Daruma(arq, 'COO', '%06d' % start,
                                                         '%06d' % end)
                os.system('mv /usr/local/sigpdv/MFD/ATO_%s_COO.TXT %s' % (arq, file_name))

            if ret_cod != 1:
                Log.error('Não foi possível obter o Arquivo %s: err %s' %
                          (arq, ERRO_DARUMA.get(ret_cod)))
                raise Exception('Não foi possível obter o Arquivo %s: err %s' %
                                (arq, ERRO_DARUMA.get(ret_cod)))

        return True

    @Log
    def mfd_sweda(self, relatorio, tipo_mfd, tipo_intervalo, start, end, file_name):
        try:
            if tipo_mfd == 1:
                if tipo_intervalo.upper() == 'D':
                    self.parent.obj_fiscal_printer.send_command("get_emitir_mfd('%s', '%s','%s', '%s', '%s')"
                                                    % (relatorio, start, end, file_name,
                                                       config['ecf']))
                elif tipo_intervalo.upper() == 'C':
                    self.parent.obj_fiscal_printer.send_command(
                        "get_emitir_mfd('%s', '%s', '%s', '%s', '%s', '%s')"
                        % (relatorio, start, end, file_name, config['ecf'], '2'))
            else:
                if tipo_intervalo.upper() == 'D':
                    self.parent.obj_fiscal_printer.send_command("get_emitir_mfd('%s', '%s','%s', '%s', '%s', '%s')"
                                    % (relatorio, start, end, file_name, config['ecf'], tipo_mfd))
                elif tipo_intervalo.upper() == 'C':
                    self.parent.obj_fiscal_printer.send_command(
                        "get_emitir_mfd('%s', '%s', '%s', '%s', '%s', '%s')"
                        % (relatorio, start, end, file_name, config['ecf'], tipo_mfd))
        except Exception as e:
            Log.error(str(e))

        return True

    @Log
    def mfd_mensal(self, loja, caixa):
        data_atual = date.today()

        # Se for dia 1 e já gerou mfd, retorna False
        if data_atual.day == 1 and config['gerou_mfd']:
            return False

        # Sempre que não for dia 1, irá setar o parâmetro Gerou Mfd para False
        elif not data_atual.day == 1:
            config.set('gerou_mfd', False)
            config.save()
            return False

        self.parent._set_screen_state(2)

        mes_passado = data_atual.month - 1
        mes_por_extenso = utils.get_mes_por_extenso(mes_passado)

        msg = 'É necessário gerar mensalmente um arquivo de backup do Espelho MFD da ECF!\n\n' \
              'Deseja gerar Espelho MFD do mês de %s ?' % mes_por_extenso

        res = False
        if message_dlg(self.parent, MSG_QUESTION, msg) == YES_ROLE:
            QtGui.QApplication.processEvents()
            self.parent.infoText('Processo demorado! Aguarde...')
            self.parent.show_info_proccess(True,
                                           msg='Gerando Espelho MFD do mês de %s!' % mes_por_extenso)

            intervalo_inicial = date(data_atual.year, mes_passado, data_atual.day)
            intervalo_final = data_atual - timedelta(days=1)

            try:
                res = self.emitir_mfd(loja, caixa, intervalo_inicial=intervalo_inicial,
                                      intervalo_final=intervalo_final, mfd_mensal=True)
                config.set('gerou_mfd', True)
                config.save()

            except Exception as e:
                Log.error(str(e))
                res = False

        self.parent.show_info_proccess(False)
        return res
