#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.tools.log import traced as Log
from sigpdv.models.mov_capa_data import CaixaMovCapa
from sigpdv.models.db_utils import sess, exec_sql
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from datetime import datetime, date
from sigpdv.ui.messagebox import NO_ROLE, MSG_QUESTION, message_dlg


class MovCapa():
    def __init__(self, parent=None):
        self.parent = parent

    @Log
    def search(self, vals, order_by=None, limit=0):
        res = False
        try:
            filters = ''
            for key in vals:
                value = vals[key]
                if type(value) in [unicode, str, date, datetime]:
                    filters += ".filter(CaixaMovCapa.%s == '%s')" % (key, value)
                else:
                    filters += ".filter(CaixaMovCapa.%s == %s)" % (key, value)

            query = 'CaixaMovCapa.query%s' % filters

            if order_by:
                query += '.order_by(%s)' % order_by

            if limit:
                query += '.limit(%s)' % limit

            res_query = eval(query)
            if res_query.count():
                if res_query.count() == 1:
                    res = res_query.one()
                else:
                    res = res_query.all()
        except Exception as err:
            res = False
            Log.error(str(err))

        return res

    @Log
    def create(self, vals):
        if self.parent:
            printer = self.parent.obj_fiscal_printer
            vals['supcap_nro_serie'] = printer.printer._serial
            vals['supcap_marca_ecf'] = printer.printer.brand
            vals['supcap_modelo_ecf'] = printer.printer.model
        else:
            printer = FiscalPrinterHelper()

        res = False
        try:
            # este while foi implementado por causa da reducaoZ, no caso da impressora ainda
            # nao ter teminado seus processos
            timeout = 0
            while True:
                Log.info('Tentando comunicar com o ECF* (%d)' % timeout)
                try:
                    if printer.send_command('get_status().block_z'):
                        Log.info('Impressora ocupada. Vou tentar novamente...')
                        if timeout < 3:
                            timeout += 1
                            continue

                    if not vals.get('supcap_ccf'):
                        vals['supcap_ccf'] = 0
                        vals['supcap_ccf'] = printer.send_command("get_ccf()")
                    if not vals.get('supcap_gnf'):
                        vals['supcap_gnf'] = 0
                        vals['supcap_gnf'] = printer.send_command("get_gnf()")
                    if not vals.get('supcap_crz'):
                        vals['supcap_crz'] = 0
                        vals['supcap_crz'] = printer.send_command("get_crz()")
                    if not vals.get('supcap_grg'):
                        vals['supcap_grg'] = 0
                        vals['supcap_grg'] = printer.send_command("get_grg()")
                    if not vals.get('supcap_cdc'):
                        vals['supcap_cdc'] = 0
                        vals['supcap_cdc'] = printer.send_command("get_cdc()")
                    if not vals.get('supcap_nro_serie'):
                        vals['supcap_nro_serie'] = ''
                        vals['supcap_nro_serie'] = printer.send_command("get_serial()")
                    if not vals.get('supcap_marca_ecf'):
                        vals['supcap_marca_ecf'] = ''
                        vals['supcap_marca_ecf'] = printer.send_command("brand")
                    if not vals.get('supcap_modelo_ecf'):
                        vals['supcap_modelo_ecf'] = ''
                        vals['supcap_modelo_ecf'] = printer.send_command("_driver.model_name")

                    break
                except Exception as e:
                    Log.error('Erro de comunicacao com a ECF: %s' % str(e))
                    # caso nao cosiga pegar algum elemento, grava com os elemtens zerados
                    # a partir do que deu erro
                    if message_dlg(self.parent, MSG_QUESTION, 'Erro ao conectar com a ecf .\n'
                                               'Verifique se a impressora está OFF-LINE, com o cabo solto,'
                                               ' com a tampa aberta, ou sem papel.'
                                               ' Deseja tentar novamente?') == NO_ROLE:
                        break

            if not vals.get('supcap_operador_id') and vals.get('supcap_tipo') not in ['V', 'C']:
                op_id  = exec_sql("""SELECT id
                                     FROM caixa_cad_operadores
                                     ORDER BY id
                                     LIMIT 1""", 'C', ret=True)
                if not op_id:
                    raise Exception('Nenhum registro encontrado na tabela de operadores!')

                vals['supcap_operador_id'] = op_id[0].get('id')

            values = ''
            for key in vals:
                value = vals[key]
                if type(value) in [unicode, str, date, datetime]:
                    values += "%s='%s', " % (key, value)
                else:
                    values += "%s=%s, " % (key, value)

            sess.begin()
            res = eval("CaixaMovCapa(%s)" % values)
            sess.commit()
            return res
        except Exception as err:
            sess.rollback()
            Log.error('Erro ao gravar Capa do Cupom! \n%s' % str(err))
            return False


    @Log
    def write(self, query_capa, vals):
        res = False
        try:
            sess.begin()
            query_capa.from_dict(vals)
            sess.commit()
            res = True
        except Exception as err:
            sess.rollback()
            Log.error(str(err))

        return res
