#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.tools.log import traced as Log
from sigpdv.tools.config import config
from sigpdv.models.mov_corpos_data import CaixaMovCorpos
from sigpdv.control.cad_tributacao import CadTributacao
from sigpdv.models.cad_tributacao_data import CaixaCadTributacao
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.db_utils import sess, exec_sql
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from datetime import datetime, date
from decimal import Decimal, ROUND_DOWN, ROUND_UP
from sigpdv.ui.messagebox import YES_ROLE, MSG_QUESTION, message_dlg

TWOPLACES = Decimal(10) ** -2
TREPLACES = Decimal(10) ** -3


class MovCorpos():
    def __init__(self, parent=None):
        self.parent = parent

    @Log
    def set_rateio_desc_acre_itens(self, totalizadores, indicador_desc_acre, mov_corpos):
        '''
        Esta fucao rateia o desconto/acrescimo de acordo com os 'totalizadores' que sao a diferenca dos totalizadores
        depois da venda do cupom com os de antes da venda do cupom, para garantir que o rateio sera feito corretamente
        para as tributacoes.
        :param totalizadores: diferenca dos tots. de depois e antes da venda do cupom. Eh uma lisca contendo tuplas,
         na tupla temos nome do totalizador e valor nesta ordem.
        :param indicador_desc_acre: indica se eh desconto('D') ou acrescimo('A')
        :param mov_corpos: itens do cupom
        :return:
        '''
        if not totalizadores:
            return {}

        tipo_imposto = {'C': 'ICMS', 'S': 'ISS'}
        dados_itens = {}
        total_por_tributacao = {}
        dict_totalizadores = {}
        for tot in totalizadores:
            chave = str(tot[0]) + '-' + tot[2]
            dict_totalizadores[chave] = tot[1]

        for item in mov_corpos:
            tributacao_id = CaixaCadItens.get_by(supite_codigo_item=item.supcor_item,
                                      supite_loja=self.parent.loja.supcfc_loja).supite_tributacao_id
            tributacao = CaixaCadTributacao.get_by(id=tributacao_id)
            if not total_por_tributacao.get(tributacao):
                total_por_tributacao[tributacao] = \
                    {'total_itens': item.supcor_preco_total- item.supcor_desconto_item + \
                                    item.supcor_acrescimo_item,
                     'maior_item': item,
                     'suptri_tipo_tributacao': tributacao.suptri_tipo_tributacao,
                     'suptri_aliquota': tributacao.suptri_aliquota,
                     'itens': [item],
                     }
            else:
                total_por_tributacao[tributacao]['total_itens'] += item.supcor_preco_total - \
                                                                   item.supcor_desconto_item + \
                                                                   item.supcor_acrescimo_item
                total_por_tributacao[tributacao]['itens'].append(item)

            # marca o item de maior valor por tributacao
            if item.supcor_preco_total > \
                                  total_por_tributacao[tributacao]['maior_item'].supcor_preco_total:
                total_por_tributacao[tributacao]['maior_item'] = item

        for trib in total_por_tributacao:
            if trib.suptri_tipo_tributacao in ('F', 'I', 'N'):
                codigo_trib_na_ecf = trib.suptri_tipo_tributacao
            else:
                codigo_trib_na_ecf = '%02d' % int(trib.suptri_codigo_trib_ecf)

            # este teste eh feito para as tributacoes de ICMS e ISS que possuem o mesmo codigo na
            # ECF como por exemplo na Sweda.
            chave = codigo_trib_na_ecf + '-' + tipo_imposto.get(trib.suptri_tipo_imposto)
            dif = (dict_totalizadores.get(chave) or Decimal(0))  - \
                  (total_por_tributacao.get(trib).get('total_itens') or Decimal(0))
            if not dif:
                continue

            if dif < 0:
                dif = dif * -1

            total_liquido = total_por_tributacao.get(trib).get('total_itens')
            percent_efetivo = Decimal((dif / total_liquido) * 100) .quantize(TWOPLACES,
                                                                             rounding=ROUND_DOWN)
            des_aplicado = 0
            acres_aplicado = 0
            for item in total_por_tributacao.get(trib).get('itens'):
                desc_anterior = item.supcor_desconto_item
                acres_anterior = item.supcor_acrescimo_item
                desconto = 0
                acrescimo = 0
                if indicador_desc_acre == 'D':
                    desconto = Decimal(((item.supcor_preco_total - desc_anterior) * percent_efetivo)
                                    / 100 + desc_anterior ).quantize(TWOPLACES, rounding=ROUND_DOWN)
                    des_aplicado += desconto - desc_anterior
                else:
                    acrescimo = Decimal(((item.supcor_preco_total + acres_anterior) *
                                     percent_efetivo) / 100 + acres_anterior).quantize(
                                                                     TWOPLACES, rounding=ROUND_DOWN)
                    acres_aplicado += acrescimo - acres_anterior

                dados_itens[item.id] = dict(supcor_desconto_item=desconto,
                                            supcor_acrescimo_item=acrescimo)
            if indicador_desc_acre == 'D':
                residuo = dif - des_aplicado
            else:
                residuo = dif - acres_aplicado

            if not residuo:
                continue

            maior_item = total_por_tributacao.get(trib).get('maior_item')
            if indicador_desc_acre == 'D':
                dados_itens[maior_item.id]['supcor_desconto_item'] += residuo
            else:
                dados_itens[maior_item.id]['supcor_acrescimo_item'] += residuo

        return dados_itens

    @staticmethod
    def search(vals, order_by=None, limit=0):
        res = []
        try:
            filters = ''
            for key in vals:
                value = vals[key]
                if type(value) in [unicode, str, date, datetime]:
                    filters += ".filter(CaixaMovCorpos.%s == '%s')" % (key, value)
                else:
                    filters += ".filter(CaixaMovCorpos.%s == %s)" % (key, value)

            query = 'CaixaMovCorpos.query%s' % filters

            if order_by:
                query += ".order_by('%s')" % order_by

            if limit:
                query += '.limit(%s)' % limit

            res_query = eval(query)
            if res_query.count():
                if res_query.count() == 1:
                    res = res_query.one()
                else:
                    res = res_query.all()
        except Exception as err:
            res = False
            Log.error(str(err))

        return res

    @Log
    def create(self, vals):
        if self.parent:
            printer = self.parent.obj_fiscal_printer
            vals['supcor_nro_serie'] = printer.printer._serial
            vals['supcor_marca_ecf'] = printer.printer.brand
            vals['supcor_modelo_ecf'] = printer.printer.model
        else:
            printer = FiscalPrinterHelper()

        timeout = 0
        try:
            dict_trib = CadTributacao().get_tributacao()
            cod_trib_ecf = vals['supcor_tributacao_ecf']

            vals['supcor_totalizador_parcial'] = CadTributacao().format_st(cod_trib_ecf)
            vals['supcor_tipo_tributacao'] = dict_trib[cod_trib_ecf].get('TipoTributacao')
            vals['supcor_tributacao_retag'] = dict_trib[cod_trib_ecf].get('TribRetag')
            vals['supcor_aliquota'] = dict_trib[cod_trib_ecf].get('Aliquota')
            vals['supcor_base'] = dict_trib[cod_trib_ecf].get('Base')

            campos = ['supcor_ccf', 'supcor_nro_serie']
            while True:
                timeout += 1
                try:
                    if vals.get('supcor_ccf') == None:
                        vals['supcor_ccf'] = printer.send_command("get_ccf()")

                    if vals.get('supcor_nro_serie') == None:
                        vals['supcor_nro_serie'] = printer.send_command("get_serial()")

                except Exception as e:
                    Log.error('Erro de comunicacao com a ECF: %s' % str(e))

                termina = True
                for campo in campos:
                    if campo not in vals:
                        termina = False

                if timeout > 60 or termina:
                    break

            for campo in campos:
                if campo not in vals:
                    if message_dlg(None, MSG_QUESTION, 'Erro ao conectar com a ecf para pegar os seguintes valores:\n'
                                               'CCF, No. Serial.\n'
                                               'Verifique se a impressora está OFF-LINE, com o cabo solto,'
                                               ' com a tampa aberta, ou sem papel.'
                                               ' Deseja tentar novamente?') == YES_ROLE:
                        return self.create(vals)

                    return False

            values = []
            for key in vals:
                value = vals[key]
                if type(value) in [unicode, str, date, datetime]:
                    values.append("%s='%s'" % (key, self.parent.remove_accents(value)))
                else:
                    values.append("%s=%s" % (key, self.parent.remove_accents(value)))

            sess.begin()
            res = eval("CaixaMovCorpos(%s)" % ', '.join(values))
            sess.commit()
        except Exception as err:
            res = False
            sess.rollback()
            Log.error('Erro ao gravar Corpo do Cupom! \n%s' % str(err))

        return res

    @Log
    def write(self, query_corpo, vals):
        res = False
        try:
            sess.begin()
            query_corpo.from_dict(vals)
            sess.commit()
            res = True
        except Exception as err:
            sess.rollback()
            Log.error(str(err))

        return res
