#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.tools.log import traced as Log
from sigpdv.models.mov_fechamentos_data import CaixaMovFechamentos
from sigpdv.models.db_utils import sess
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from datetime import datetime, date
from sigpdv.ui.messagebox import YES_ROLE, MSG_QUESTION, message_dlg


class MovFechamentos():
    def __init__(self, parent=False):
        self.parent = parent

    @staticmethod
    def search(vals, order_by=None, limit=0):
        res = False
        try:
            filters = ''
            for key in vals:
                value = vals[key]
                if type(value) in [unicode, str, date, datetime]:
                    filters += ".filter(CaixaMovFechamentos.%s == '%s')" % (key, value)
                else:
                    filters += ".filter(CaixaMovFechamentos.%s == %s)" % (key, value)

            query = 'CaixaMovFechamentos.query%s' % filters

            if order_by:
                query += '.order_by(%s)' % order_by

            if limit:
                query += '.limit(%s)' % limit

            res_query = eval(query)
            if res_query.count():
                if res_query.count() == 1:
                    res = res_query.one()
                else:
                    res = res_query.all()
        except Exception as err:
            res = False
            Log.error(str(err))

        return res

    @Log
    def create(self, vals):
        if self.parent:
            printer = self.parent.obj_fiscal_printer
            vals['supfec_nro_serie'] = printer.printer._serial
            vals['supfec_marca_ecf'] = printer.printer.brand
            vals['supfec_modelo_ecf'] = printer.printer.model
        else:
            printer = FiscalPrinterHelper()

        res = False
        timeout = 0
        try:
            campos = ['supfec_ccf', 'supfec_gnf', 'supfec_cdc', 'supfec_nro_serie']
            while True:
                timeout += 1
                try:
                    if vals.get('supfec_ccf') == None:
                        vals['supfec_ccf'] = printer.send_command("get_ccf()")

                    if vals.get('supfec_gnf') == None:
                        vals['supfec_gnf'] = printer.send_command("get_gnf()")

                    if vals.get('supfec_cdc') == None:
                        vals['supfec_cdc'] = printer.send_command("get_cdc()")

                    if vals.get('supfec_nro_serie') == None:
                        vals['supfec_nro_serie'] = printer.send_command("get_serial()")

                except Exception as e:
                    Log.error('Erro de comunicacao com a ECF: %s' % str(e))

                termina = True
                for campo in campos:
                    if campo not in vals:
                        termina = False

                if timeout > 60 or termina:
                    break

            for campo in campos:
                if campo not in vals:
                    if message_dlg(self.parent, MSG_QUESTION, 'Erro ao conectar com a ecf para pegar os seguintes valores:\n'
                                               'CCF, GNF, CDC, No. Serial.\n'
                                               'Verifique se a impressora está OFF-LINE, com o cabo solto,'
                                               ' com a tampa aberta, ou sem papel.'
                                               ' Deseja tentar novamente?') == YES_ROLE:
                        return self.create(vals)

                    return False

            values = ''
            for key in vals:
                value = vals[key]
                if type(value) in [unicode, str, date, datetime]:
                    values += "%s='%s', " % (key, value)
                else:
                    values += "%s=%s, " % (key, value)

            sess.begin()
            res = eval("CaixaMovFechamentos(%s)" % values)
            sess.commit()
        except Exception as err:
            sess.rollback()
            Log.error('Erro ao gravar Fechamento do Cupom! \n%s' % str(err))

        return res

    @Log
    def write(self, query_fechamentos, vals):
        res = False
        try:
            sess.begin()
            query_fechamentos.from_dict(vals)
            sess.commit()
            res = True
        except Exception as err:
            sess.rollback()
            Log.error(str(err))

        return res
