#!/usr/bin/python
# -*- encoding: utf-8 -*-

import os
from datetime import date, datetime, timedelta
from time import sleep
from decimal import Decimal
from PySide import QtCore, QtGui
from sigpdv.control.mov_capa import MovCapa
from sigpdv.control.mov_cupom_servidor import MovCupomServidor
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from sigpdv.control.importacao_automatica import ImportacaoAutomatica
from sigpdv.control.movimento_ecf import MovimentoEcf
from sigpdv.models.mov_totalizadores_xz_data import CaixaMovTotalizadoresXZ
from sigpdv.models.mov_corpo_xz_data import CaixaMovCorpoXZ
from sigpdv.models.cad_operadores_data import CaixaCadOperadores
from sigpdv.models.cad_tributacao_data import CaixaCadTributacao
from sigpdv.models.db_utils import exec_sql, sess, pemite_testar_conexao, testa_conexao_pg
from sigpdv.sigrestaurante.controle_mesa import ControleMesa
from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from sigpdv.ui.dlpPwd import password_dlg
from stoqdrivers.exceptions import CouponOpenError
from sigpdv.ui.messagebox import MSG_ERROR, YES_ROLE, MSG_INFO, MSG_QUESTION, message_dlg
from sigpdv.control.mov_caixa import MovCaixa

# controle de conexao, nao permitir testar conexao quando estiver fazendo reducao Z
permite_local = pemite_testar_conexao


class ReducaoXZ(object):
    def __init__(self, parent):
        self.parent = parent

    @Log
    def gerar_reducao_xz(self, loja, caixa, tipo, codigo_operador):
        """
        Emiti redução x/z gerando os dados na base de dados.
        """
        permite_local['valor'] = False
        permite_local['z'] = True
        testa_conexao_pg('','','','')
        caixa_turno = MovCaixa().search({'supcax_loja': config['loja'], 'supcax_fechado': 'A'},
                                  order_by='id DESC', limit=1)

        cad_operador = False
        obj_fiscal_printer = self.parent.obj_fiscal_printer
        try:
            status = obj_fiscal_printer.send_command('get_status()')
            if caixa_turno and tipo == 'Z':
                try:
                    status.check_error()
                    if not status.block_z:
                        # fecha o turno caso a reducao Z nao seja pendente e tenha um turno aberto
                        self.parent.obj_abertura_fechamento_turno.fechar_turno(caixa_turno,
                                                           config['loja'], caixa_turno.supcax_caixa,
                                         (self.parent.user_logged.id, self.parent.user_logged.name))
                except:
                    pass

        except Exception as e:
            Log.error('Erro na Reducao X/Z: %s' % e)
            permite_local['valor'] = True
            permite_local['z'] = False
            return message_dlg(self.parent, MSG_ERROR, 'Atenção!\n'
                                                'Verifique se o papel acabou ou se a tampa da ECF '
                                                'está aberta')

        if codigo_operador:
            cad_operador = CaixaCadOperadores.get_by(supope_codigo=codigo_operador)

        try:
            texto = ''
            if tipo == 'X':  # Leitura X
                texto = 'Redução X'
            elif tipo == 'Z':
                texto = 'Redução Z'

            if self.parent.teclado:
                self.parent.teclado.envia_display('Imprimindo %s...' % texto)

            self.parent.show_info_proccess(True, texto)
            data_ini = obj_fiscal_printer.get_data_ecf()
            dif_time_ecf = data_ini - datetime.strptime(datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                                                        "%Y-%m-%d %H:%M:%S")
            data_cupom = datetime.date(data_ini)
            if not data_cupom or str(data_cupom) == '2000-01-01':
                data_cupom = date.today()

            sintegra = self.get_sintegra_ecf()
            self.tributacao_ecf = obj_fiscal_printer.send_command('get_totalizadores(total=True)')
            cro = sintegra.cro if sintegra else None

            # Nro cupom, este eh pego antes da reducaoX/Z e somado 1, pois quando ha Z pendente do mes anterior,
            # automaticamente eh emitida uma MFD na impressora entao o coo pego depois do Z sera da MFD
            coo = self.get_coo_ecf(obj_fiscal_printer) # Atencao ^
            if coo:
                coo += 1

            if tipo == 'X':  # Leitura X
                obj_fiscal_printer.send_command('summarize()')

            elif tipo == 'Z':  # Redução Z
                try:
                    obj_fiscal_printer.printer._port.setTimeout(20) # evita que pegue dados errados ou não pegue dados apos a reducao Z
                    obj_fiscal_printer.send_command('close_till()')
                except CouponOpenError:
                    obj_fiscal_printer.send_command('cancel()')
                    GerencialReport(loja_id=loja,
                                    caixa_id=caixa,
                                    operador_id=cad_operador.id,
                                    nro_cupom_vinculado=self.get_coo_ecf(obj_fiscal_printer),
                                    tipo='C', tipo_c='NC',
                                    parent=self.parent)
                    try:
                        obj_fiscal_printer.send_command('close_till()')
                    except Exception as e:
                        Log.error('Erro Redução Z: %s' % str(e))

                # pode haver erro na elgin ao ler a resposta do comando da reducao Z,
                # o comando foi enviado, mas nao rescebeu resposta
                except Exception as e:
                    Log.error('Erro Redução Z: %s' % str(e))
                    # trata o erro na reducao Z caso acabe o papel no meio da impressao
                    if '7003' in str(e) or '7004' in str(e): # Elgin
                        status = obj_fiscal_printer.send_command('get_status()')
                        while status.open:
                            message_dlg(self.parent, MSG_ERROR, str(e))
                            try:
                                status = obj_fiscal_printer.send_command('get_status()')
                            except Exception as e2:
                                Log.error(str(e2))
                        obj_fiscal_printer.send_command('get_corta_papel()')
                    # trata fila de comandos cheia ao tentar emitir red. Z
                    elif '11016' in str(e): # Elgin
                        while True:
                            try:
                                sleep(5)
                                obj_fiscal_printer.send_command('get_status()')
                                break
                            except Exception as e:
                                Log.error('Fila cheia: %s' % str(e))

                executei_sem_erro = False
                tentativas = 0
                while True:
                    try:
                        sleep(1)
                        tentativas += 1
                        if tentativas > 30:
                            data_cupom = datetime.today().date()
                            break

                        data_cupom = obj_fiscal_printer.send_command('get_date_last_mov_rz()')
                        # para a bematec quando tira um Z que nao eh pendente em um dia que nao tem movimentacao,
                        # a data do movimento do Z nao existe entao deve-se pegar a data do dia que emitiu o Z
                        if data_cupom == 0 and tentativas <= 30:
                            continue

                        elif data_cupom == 0:
                            data_cupom = datetime.today().date()

                        executei_sem_erro = True
                    except Exception as e:
                        Log.error('Erro ao pegar data da Reducao Z: %s' % str(e))

                    if executei_sem_erro:
                        break

            if not coo:
                obj_fiscal_printer.printer._port.setTimeout(1)
                permite_local['valor'] = True
                permite_local['z'] = False
                # Todo: Tratar este tipo de erro no pos.py e nao aqui. EX: retorna um erro e no pos.py decide o que fazer.
                return message_dlg(self.parent, MSG_ERROR, 'Não foi possível pegar o número do COO após a Redução X/Z',
                                                           'O cupom não foi salvo no Banco de dados')

            try:
                # Grava a capa do cupom
                dados_capa = dict(supcap_data_cupom=data_cupom,
                                  supcap_loja=loja,
                                  supcap_caixa=caixa,
                                  supcap_nro_cupom=coo,
                                  supcap_data_inicio=data_ini,
                                  supcap_data_fim=data_ini,
                                  supcap_tipo=tipo,
                                  supcap_nro_proprietario=loja,
                                  supcap_tipo_comprovante='NC')
                if cad_operador:
                    dados_capa['supcap_operador_id']=cad_operador.id,

                self.parent.mov_capa = MovCapa().create(dados_capa)
                obj_fiscal_printer.printer._port.setTimeout(1) # apos gravar o cupom o timeout volta para 1

                if not self.parent.mov_capa:
                    permite_local['valor'] = True
                    permite_local['z'] = False
                    # Todo: Tratar este tipo de erro no pos.py e nao aqui. EX: retorna um erro e no pos.py decide o que fazer.
                    return message_dlg(self.parent, MSG_ERROR, 'Não foi possível gravar o cupom no banco!')

                # Grava totalizadores e corpo xz
                self.totalizar_xz(data_cupom, coo, sintegra, obj_fiscal_printer)

                # Grava cupom servidor
                MovCupomServidor().create({'supcse_capa_id': self.parent.mov_capa.id})

                # Atualiza data fim
                data_fim = obj_fiscal_printer.get_data_ecf(mostra_msg=False)
                if not data_fim:
                    # data do computador aplicada a diferenca entre a hora do computador e a ecf
                    data_fim = datetime.strptime(datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                                                        "%Y-%m-%d %H:%M:%S") + dif_time_ecf

                MovCapa().write(self.parent.mov_capa, {'supcap_data_fim': data_fim,
                                                       'supcap_nro_cupom': coo,})
                self.parent.verifica_z_emitido()
            except Exception as e:
                obj_fiscal_printer.printer._port.setTimeout(1)
                Log.error('Erro ao gravar redução_xz \n %s' % str(e))

            if tipo == 'Z':
                if self.parent.acao_pendente_z:
                    ImportacaoAutomatica(self.parent).del_acao_pendente('reducao_z', loja_id=loja, caixa_id=caixa)

                MovimentoEcf(self.parent).gera_movimento_ecf(data_cupom, data_cupom)

                if config['ativar_restaurante']:
                    ControleMesa().verificar_mesas_abertas(True)

                if self.parent.z_pendente:
                    self.parent.z_pendente = False

                elif config['shutdownaposz']:
                    permite_local['valor'] = True
                    permite_local['z'] = False
                    self.parent._lock_keyboard(True)

                    inc = 0
                    loop = 60
                    supcse_enviado = False

                    while not supcse_enviado:
                        inc += 1
                        sql = """
                            SELECT mcs.supcse_enviado
                            FROM caixa_mov_cupom_servidor AS mcs
                            INNER JOIN caixa_mov_capa     AS cmc ON mcs.supcse_capa_id = cmc.id
                            WHERE cmc.supcap_loja = %d
                              AND cmc.supcap_caixa = %d
                              AND cmc.supcap_nro_cupom = %d
                        """ % (config['loja'], config['caixa'], coo)
                        try:
                            mov_cupom_serv = exec_sql(sql, 'C', ret=True)
                        except Exception as e:
                            Log.error('Erro de consulta: %s' % e)
                            mov_cupom_serv = False

                        if mov_cupom_serv and isinstance(mov_cupom_serv, list):
                            supcse_enviado = mov_cupom_serv[0].get('supcse_enviado')

                        self.parent.show_info_proccess(True, 'Desligando o sistema: (%d-%d)'
                                                       % (inc, loop))

                        QtGui.QApplication.processEvents()
                        Log.info('Desligando o Sistema: (%d-%d)' % (inc, loop))
                        sleep(1)

                        if inc == loop:
                            break

                    sleep(5)
                    os.system('echo %s | sudo -S %s' % ('sig', 'shutdown -h now'))
                    self.parent.show_info_proccess(False)

            QtGui.QApplication.processEvents()

        except Exception as e:
            Log.error(str(e))
            self.parent.show_info_proccess(False)
            permite_local['valor'] = True
            permite_local['z'] = False
            return False

        self.parent.show_info_proccess(False)

        permite_local['valor'] = True
        permite_local['z'] = False
        return cro

    @Log
    def totalizar_xz(self, data_movimento, coo, sintegra=None, obj_fiscal_printer=None):
        try:
            ccf = self.parent.mov_capa.supcap_ccf
            gnf = self.parent.mov_capa.supcap_gnf
            crz = self.parent.mov_capa.supcap_crz
            nro_serie = self.parent.mov_capa.supcap_nro_serie
            cro = sintegra.cro if sintegra else None

            cupom_ini = int(sintegra.coupon_start) if sintegra else None
            cupom_fin = coo

            gt_atual = sintegra.total if sintegra else None
            # gt anterior = gt autal - venda bruta
            gt_anterior = gt_atual - sintegra.period_total if sintegra else None

            # Totalizadores
            tributacao = {}
            acrescimos_iss = 0
            acrescimos = 0
            cancelamentos_iss = 0
            cancelamentos = 0
            descontos_iss = 0
            descontos = 0
            for trib in self.tributacao_ecf:
                if (trib[0] == 'ACRESC') and (trib[2] == 'ISS'):
                    acrescimos_iss = trib[1]
                elif (trib[0] == 'ACRESC') and (trib[2] == 'ICMS'):
                    acrescimos = trib[1]
                elif (trib[0] == 'CANC') and (trib[2] == 'ISS'):
                    cancelamentos_iss = trib[1]
                elif (trib[0] == 'CANC') and (trib[2] == 'ICMS'):
                    cancelamentos = trib[1]
                elif (trib[0] == 'DESC') and (trib[2] == 'ISS'):
                    descontos_iss = trib[1]
                elif (trib[0] == 'DESC') and (trib[2] == 'ICMS'):
                    descontos = trib[1]
                else:
                    key = str(trib[0].zfill(2)) + '-' + trib[2]
                    tributacao[key] = trib[1]

            for cad_tributacao in CaixaCadTributacao.query.all():
                tipo_trib = ({'C': 'ICMS', 'S':'ISS'}.get(cad_tributacao.suptri_tipo_imposto)) or \
                            'Tributacao Sem Tipo'
                key = str(cad_tributacao.suptri_codigo_trib_ecf).zfill(2) + '-' + tipo_trib
                try:
                    valor = tributacao[key]
                except:
                    Log.error("Erro ao pegar valor da tributacao(cod. na ecf): %s" % key)
                    valor = Decimal('0')

                sess.begin()
                CaixaMovTotalizadoresXZ(
                    suptxz_codigo_trib_ecf=cad_tributacao.suptri_codigo_trib_ecf,
                    suptxz_descricao_totalizador=cad_tributacao.suptri_name,
                    suptxz_valor=valor,
                    suptxz_aliquota=cad_tributacao.suptri_aliquota,
                    suptxz_base=cad_tributacao.suptri_base,
                    suptxz_tipo_imposto=cad_tributacao.suptri_tipo_imposto,
                    suptxz_tipo_tributacao=cad_tributacao.suptri_tipo_tributacao,
                    suptxz_crz=crz,
                    suptxz_nro_serie=nro_serie,
                    suptxz_capa_id=self.parent.mov_capa.id
                )
                sess.commit()

            sess.begin()
            CaixaMovCorpoXZ(
                supcxz_data_movimento=data_movimento,
                supcxz_ccf=ccf, supcxz_gnf=gnf, supcxz_cro=cro,
                supcxz_cupom_inicial=cupom_ini, supcxz_cupom_final=cupom_fin,
                supcxz_gt_atual=gt_atual, supcxz_gt_anterior=gt_anterior,
                supcxz_cancelamentos=cancelamentos, supcxz_descontos=descontos,
                supcxz_acrescimos=acrescimos, supcxz_descontos_iss=descontos_iss,
                supcxz_acrescimos_iss=acrescimos_iss, supcxz_cancelamentos_iss=cancelamentos_iss,
                supcxz_modelo_ecf=obj_fiscal_printer.printer.brand,
                supcxz_capa_id=self.parent.mov_capa.id)
            sess.commit()
        except Exception as err:
            Log.error(str(err))
            sess.rollback()
            return False

        return coo

    @Log
    def get_sintegra_ecf(self):
        try:
            sintegra = self.parent.obj_fiscal_printer.send_command('get_sintegra()')
        except Exception:
            sintegra = None
        if not sintegra:
            msg = "Não foi possível enviar o comando 'get_sintegra()' para o ECF\n" \
                  "\nTENTAR NOVAMENTE?"
            if message_dlg(self.parent, MSG_QUESTION, msg) == YES_ROLE:
                sintegra = self.get_sintegra_ecf()
        return sintegra

    @Log
    def get_coo_ecf(self, obj_fiscal_printer, tentativas=60):
        timeout = coo = 0
        while True:
            timeout += 1
            Log.info('Tentando comunicar com o ECF* (%d)' % timeout)
            try:
                coo = int(obj_fiscal_printer.send_command("get_coo()"))
                if coo or timeout > tentativas:
                    break

            except Exception as e:
                Log.error('Erro de comunicacao com a ECF: %s' % e)

        if not coo:
            if message_dlg(self.parent, MSG_QUESTION, 'Erro ao reconectar com ECF, foi excedido o numero de tentativas para '
                                               'obter o numero coo da Leitura Z.\n'
                                               'Verifique se a impressora está OFF-LINE, com o cabo solto,'
                                               ' com a tampa aberta, ou sem papel.'
                                               ' Deseja tentar novamente?') == YES_ROLE:
                return self.get_coo_ecf(obj_fiscal_printer)



        return coo


