#!/usr/bin/python
# -*- encoding: utf-8 -*-

from decimal import Decimal
from PySide import QtGui
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from sigpdv.models.db_utils import sess, exec_sql
from sigpdv.models.mov_capa_data import CaixaMovCapa
from sigpdv.models.mov_fechamentos_data import CaixaMovFechamentos
from sigpdv.models.mov_cupom_servidor_data import CaixaMovCupomServidor
from sigpdv.control.mov_meios_pagamentos import MovMeiosPagamentos
from sigpdv.ui.messagebox import MSG_INFO, message_dlg


class Sangria():
    def __init__(self, parent):
        self.parent = parent
        self.printer = self.parent.obj_fiscal_printer

    @Log
    def sangrar(self, loja_id, caixa_id, operador_id):
        if not self.parent:
            return False

        data_cupom = self.parent._date_coupon
        if not data_cupom:
            self.parent.show_info_proccess(False)
            return message_dlg(self, MSG_INFO, "Ainda não houve venda para este turno!")

        # Bloqueia enter, pois ha casos que emite mais de uma sangria por apertar a tecla
        # indiscrimidadamente, a liberacao eh feita ao terminar a sangria ou clicar em cancelar
        self.bloqueia_tecla = True
        Log.info('bloqueia_tecla: True')

        self.parent.show_info_proccess(True, 'Efetuando Sangria')
        QtGui.QApplication.processEvents()

        data_ini = self.printer.get_data_ecf()

        inc = 0
        tipo_finalizadora = ('D', 'H', 'C', 'T', 'O')

        desc_sangria = (False,
                        config['sangriadiscriminarcheques'],
                        config['sangriadiscriminarcartoes'],
                        config['sangriadiscriminartickets'],
                        config['sangriadiscriminaroutros'])

        nome_finalizadora = {0: 'Dinheiro',
                             1: 'Cheque',
                             2: 'Cartao',
                             3: 'Ticket',
                             4: 'Outros'}

        texto_cupom_vinc = ''
        for control in (self.parent.ui.spinSSDinheiro,
                        self.parent.ui.spinSSCheque,
                        self.parent.ui.spinSSCartao,
                        self.parent.ui.spinSSTickets,
                        self.parent.ui.spinSSOutros):

            value = control.valueFromText(control.text())
            if value > 0:
                try:
                    value = Decimal(str(value))
                    if not self.printer.add_value_to_counter('SA', value, tipo_finalizadora[inc],
                                                             self.parent.user_logged.name):
                        inc += 1
                        continue

                    coo = self.printer.send_command("get_coo()")

                    fechamentos = self.get_fechamentos(loja_id, caixa_id, data_cupom,
                                                       tipo_finalizadora[inc])

                    self.gravar_sangria(loja_id, caixa_id, coo, data_cupom, data_ini, value,
                                        operador_id, inc + 1, tipo_finalizadora[inc])
                    if fechamentos and (desc_sangria[inc]):
                        total = 0
                        texto_cupom_vinc += """* Fechamentos em %s\n\nCOO    Finalizadora
                            Valor\n------ ---------------- ------------\n""" % nome_finalizadora[inc]

                        for fechamento in fechamentos:
                            if fechamento['supcap_tipo'] == 'C':
                                total -= fechamento['supfec_valor']
                                texto_cupom_vinc += '%06d %s  %12.2f Canc.\n' \
                                                    % (fechamento['supcap_nro_cupom'],
                                                       (fechamento['supfec_nome_finalizadora']
                                                        or 'Sem finalizadora').ljust(16, ' '),
                                                       fechamento['supfec_valor'] * -1)
                            else:
                                total += fechamento['supfec_valor']
                                texto_cupom_vinc += '%06d %s  %12.2f\n' \
                                                    % (fechamento['supcap_nro_cupom'],
                                                       (fechamento['supfec_nome_finalizadora']
                                                        or 'Sem finalizadora').ljust(16, ' '),
                                                       fechamento['supfec_valor'])

                        if total:
                            texto_cupom_vinc += '------ ---------------- ------------\n'
                            texto_cupom_vinc += '                  TOTAL %12.2f\n\n' % total

                except Exception as err:
                    if self.parent:
                        self.parent.tratar_except()
                    Log.error(str(err))
                    return False

            inc += 1

        if texto_cupom_vinc:
            self.parent.show_info_proccess(True, 'Imprimindo Relatório')
            GerencialReport(report_id=config['rg_relatorio_gerencial'],
                            text=str(texto_cupom_vinc),
                            loja_id=loja_id,
                            caixa_id=caixa_id,
                            operador_id=operador_id,
                            parent=self.parent)

        self.parent.show_info_proccess(False)

        return True

    @staticmethod
    def get_ultima_sangria(loja_id, caixa_id, data_cupom, tipo_finalizadora):
        ultima_sangria = None
        sql = """
         SELECT mov_capa.supcap_data_cupom,
                mov_capa.supcap_nro_cupom,
                mov_capa.id
         FROM   caixa_mov_capa mov_capa
                INNER JOIN caixa_mov_fechamentos mov_fechamentos
                        ON mov_fechamentos.supfec_capa_id = mov_capa.id
         WHERE  mov_capa.supcap_tipo = 'A'
                AND mov_capa.supcap_loja = %d
                AND mov_capa.supcap_caixa = %d
                AND mov_capa.supcap_data_cupom = '%s'
                AND mov_fechamentos.supfec_tipo_finalizadora = '%s'
         ORDER
            BY  id DESC
         LIMIT  1
        """ % (loja_id, caixa_id, data_cupom, tipo_finalizadora)

        try:
            ultima_sangria = exec_sql(sql, 'C', ret=True)
        except Exception as err:
            Log.error('Erro ao obter dados da última sangria! \n %s' % str(err))

        Log.info(sql)
        Log.info(ultima_sangria)

        return ultima_sangria

    @Log
    def get_fechamentos(self, loja_id, caixa_id, data_cupom, tipo_finalizadora):
        ultima_sangria = self.get_ultima_sangria(loja_id, caixa_id, data_cupom, tipo_finalizadora)
        if ultima_sangria:
            sql = """SELECT    c.supcap_nro_cupom,
                               f.supfec_valor,
                               f.supfec_nome_finalizadora,
                               c.supcap_tipo
                        FROM   caixa_mov_capa AS c
                               INNER JOIN caixa_mov_fechamentos AS f
                                       ON ( c.id = f.supfec_capa_id )
                        WHERE  f.supfec_tipo_finalizadora = '%s'
                               AND f.supfec_finalizadora <> 0
                               AND c.supcap_nro_cupom > %d
                               AND c.supcap_data_cupom = '%s'
                               AND c.supcap_loja = %d
                               AND c.supcap_caixa = %d;
            """ % (tipo_finalizadora, ultima_sangria[0]['supcap_nro_cupom'],
                   ultima_sangria[0]['supcap_data_cupom'], loja_id, caixa_id)
        else:
            sql = """SELECT c.supcap_nro_cupom,
                            f.supfec_valor,
                            f.supfec_nome_finalizadora,
                            c.supcap_tipo
                     FROM   caixa_mov_capa AS c
                            INNER JOIN caixa_mov_fechamentos AS f
                                    ON ( c.id = f.supfec_capa_id )
                     WHERE  f.supfec_tipo_finalizadora = '%s'
                            AND c.supcap_data_cupom = '%s'
                            AND c.supcap_loja = %d
                            AND c.supcap_caixa = %d;
            """ % (tipo_finalizadora, data_cupom, loja_id, caixa_id)

        fechamentos = exec_sql(sql, 'C', ret=True)
        return fechamentos

    @Log
    def gravar_sangria(self, loja_id, caixa_id, nro_cupom, data_cupom, data_inicio, total_cupom,
                       operador_id, sequencia, tipo_finalizadora):

        self.parent._coupon_id = nro_cupom
        ccf = self.printer.send_command("get_ccf()")
        gnf = self.printer.send_command("get_gnf()")
        crz = self.printer.send_command("get_crz()")
        cdc = self.printer.send_command("get_cdc()")
        nro_serie = self.printer.send_command("get_serial()")
        marca_ecf = self.printer.send_command("brand")
        modelo_ecf = self.printer.send_command("_driver.model_name")

        sess.begin()
        capa = CaixaMovCapa(
            supcap_data_cupom=data_cupom,
            supcap_loja=loja_id,
            supcap_caixa=caixa_id,
            supcap_nro_cupom=nro_cupom,
            supcap_operador_id=operador_id,
            supcap_cliente=None,
            supcap_data_inicio=data_inicio,
            supcap_data_fim=data_inicio,
            supcap_tipo='A',
            supcap_alteracao=None,
            supcap_valor_desc_acres=0,
            supcap_total_cupom=total_cupom,
            supcap_nro_serie=nro_serie,
            supcap_mf_adicional=False,
            supcap_marca_ecf=marca_ecf,
            supcap_modelo_ecf=modelo_ecf,
            supcap_nro_proprietario=1,
            supcap_ccf=ccf,
            supcap_gnf=gnf,
            supcap_crz=crz,
            supcap_grc=gnf,
            supcap_cdc=cdc,
            supcap_tipo_comprovante='CN',
            supcap_sub_total=self.parent._coupon_subtotal,
            supcap_indicador_desconto=None,
            supcap__indicador_acrescimo=None,
            supcap_indicador_cancelamento='N',
            supcap_cancelamento_acrescimo=0,
            supcap_ordem_desconto_acrescimo=None,
            supcap_cancelado_posteriormente=False,
            supcap_nome_cliente=None,
            supcap_cod_vendedor=None
        )
        sess.commit()

        cod_finalizadora = exec_sql("""SELECT  fne.supfec_codigo_na_ecf
                                       FROM caixa_cad_finalizadora_na_ecf AS fne 
                                       INNER JOIN caixa_cad_finalizadora AS ccf ON ccf.supfin_codigo_na_ecf = fne.id
                                       WHERE ccf.supfin_tipo_finalizadora = '%s' 
                                       GROUP BY fne.supfec_codigo_na_ecf""" % tipo_finalizadora)
        if cod_finalizadora:
            cod_finalizadora = cod_finalizadora[0].get('supfec_codigo_na_ecf')
        else:
            cod_finalizadora = 0

        sess.begin()
        CaixaMovFechamentos(
            supfec_capa_id=capa.id,
            supfec_sequencia=sequencia,
            supfec_finalizadora=cod_finalizadora,
            supfec_tipo_finalizadora=tipo_finalizadora,
            supfec_valor=total_cupom,
            supfec_ccf=ccf,
            supfec_gbf=gnf,
            supfec_cdc=cdc,
            supfec_nro_serie=nro_serie)
        sess.commit()

        sess.begin()
        CaixaMovCupomServidor(supcse_capa_id=capa.id, supcse_enviado=False)
        sess.commit()

        MovMeiosPagamentos.gravar_meios_pagamento(loja_id, caixa_id, data_cupom, capa.supcap_tipo,
                                                  capa.supcap_tipo_comprovante, total_cupom)

        data_fim = self.printer.get_data_ecf()
        sess.begin()
        capa.supcap_data_fim = data_fim
        sess.commit()

        return True