#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.models.cad_finalizadora_data import CaixaCadFinalizadora
from sigpdv.control.mov_capa import MovCapa
from sigpdv.control.mov_fechamentos import MovFechamentos
from sigpdv.control.mov_cupom_servidor import MovCupomServidor
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from sigpdv.control.mov_meios_pagamentos import MovMeiosPagamentos
from sigpdv.ui.messagebox import MSG_INFO, MSG_ERROR, message_dlg


class Suprimento():
    def __init__(self, parent):
        self.parent = parent
        self.printer = FiscalPrinterHelper(self.parent)

    @Log
    def add_suprimento(self, loja_id, caixa_id, operador_id, value):
        self.parent.infoText('Suprimento')
        data_ini = self.printer.get_data_ecf()

        if value <= 0:
            message_dlg(self, MSG_INFO, 'Valor não pode ser menor ou igual a zero')
            return False
        try:
            try:
                totalizador = int(config['suprimento'][:2])
                meio = int(config['suprimento'][3:])
            except:
                totalizador = False
                meio = False

            if not totalizador and not isinstance(totalizador, int):
                message_dlg(self.parent, MSG_ERROR, 'Não foi possível efetuar o Suprimento',
                                   'Totalizador de suprimento não configurado no '
                                   'PDV-Config')
                return False

            self.printer.send_command("till_add_cash(%r, totalizador=%r, meio=%r)" % (value,
                                                                                      totalizador,
                                                                                      meio))
        except Exception as e:
            Log.error(str(e))
            return False

        self.gravar_suprimento(loja_id, caixa_id, operador_id, value, data_ini)

        if config['usagaveta']:
            self.printer.send_command("open_cash_drawer(50)")
            self.parent._timer_end_sale.start(500)

        return True

    @Log
    def gravar_suprimento(self, loja_id, caixa_id, operador_id, value, data_ini):
        try:
            coo = self.printer.send_command("get_coo()")
            self.parent._coupon_id = coo

            self.parent.mov_capa = MovCapa().create(dict(
                supcap_data_cupom=self.parent._date_coupon, supcap_loja=loja_id,
                supcap_caixa=caixa_id, supcap_nro_cupom=coo,
                supcap_operador_id=operador_id, supcap_data_inicio=data_ini,
                supcap_data_fim=data_ini,
                supcap_tipo='U', supcap_total_cupom=value,
                supcap_tipo_comprovante='CN', supcap_sub_total=value))

            query_cad_finalizadora = CaixaCadFinalizadora.get_by(supfin_tipo_finalizadora='D')

            MovFechamentos().create(dict(
                supfec_capa_id=self.parent.mov_capa.id, supfec_sequencia=1, supfec_valor=value,
                supfec_finalizadora=query_cad_finalizadora.supfin_codigo,
                supfec_tipo_finalizadora=query_cad_finalizadora.supfin_tipo_finalizadora,
                supfec_nome_finalizadora=query_cad_finalizadora.name))

            MovMeiosPagamentos().gravar_meios_pagamento(loja_id, caixa_id,
                                                        self.parent._date_coupon,
                                                        self.parent.mov_capa.supcap_tipo,
                                                        self.parent.mov_capa.supcap_tipo_comprovante,
                                                        value,
                                                        query_cad_finalizadora.supfin_codigo,
                                                        query_cad_finalizadora.name)

            MovCapa().write(self.parent.mov_capa, {'supcap_data_fim': self.printer.get_data_ecf()})

            MovCupomServidor().create({'supcse_capa_id': self.parent.mov_capa.id})
        except Exception as e:
            Log.error(str(e))
            return False

        return True