# -*- encoding: utf-8 -*-
__author__ = 'Dener Rodrigues'

import re
from datetime import datetime
from sigpdv.tools.log import traced
from sigpdv.models.supervisor import SupervisorDavCapa, SupervisorDavCorpo, SupervisorDavFechamento,\
    SupervisorCadVendedor, SupervisorDavVencimentos
from sigpdv.models.cad_clientes_data import CaixaCadClientes
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.cad_finalizadora_data import CaixaCadFinalizadora
from sigpdv.tools.config import config
from sigpdv.tools.rpc_timeout import ServerProxy
from sigpdv.models.db_utils import testa_conexao_rpc, b_sess, testa_conexao_pg, exec_sql
from sigpdv.control.utils import remove_accents
import time

server = config['db_ges_host']
username = config['db_ges_user']
pwd = config['db_ges_pass']
dbname = config['db_ges_name']
port = config['db_ges_port']
dbemp = config['db_ges_emp']

server_sup = config['db_sup_host']
dbname_sup = config['db_sup_name']
pwd_sup = config['db_sup_pass']
username_sup = config['db_sup_user']
uid = None


class ErrConnRPC(Exception):
    """Base"""


@traced
def get_state_pedido_dav(pedido_id):
    """
    Retorna o state do pedido do erp
    """
    check = testa_conexao_rpc(server, username, pwd, dbname, port)
    if not check.get('res'):
        raise ErrConnRPC(check.get('msg'))

    sock_common = ServerProxy('http://%s:%s/xmlrpc/common' % (server, port))
    uid = sock_common.login(dbname, username, pwd)
    sock = ServerProxy('http://%s:%s/xmlrpc/object' % (server, port))
    args = [('id', '=', pedido_id)]
    if dbemp:
        emp = dbemp.replace(' ', '').replace('-', ',').split(',')
        emp_list = []
        for x in emp:
            emp_list.append(int(x))
        emp_list = list(set(emp_list))
        epc_company_id = sock.execute(dbname, uid, pwd, 'adm.empresa.objeto', 'getEmpresa',
                                      'estoquepedidos.pedido.capa', False, {}, emp_list[0])
        args.append(('company_id', '=', epc_company_id))

    pedido_ids = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.capa', 'search', args, 0, '',
                              '', {'sig_force_company': True})

    ret_val = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.capa', 'read', pedido_ids,
                           ['state', 'exportado_frente'])

    if not ret_val:
        return 'I', True

    if isinstance(ret_val, list):
        ret_val = ret_val[0]

    return ret_val.get('state'), ret_val.get('exportado_frente')

@traced
def cancelar_pedidos(dav_ids=[]):
    res = False
    check = testa_conexao_rpc(server, username, pwd, dbname, port)
    check_supervisor = testa_conexao_pg(dbname_sup, username_sup, server_sup, pwd_sup)
    if not check.get('res'):
        msg = 'Erro ao conectar com SigERP \n\n' + str(check.get('msg'))
        traced.error(msg)
        raise ErrConnRPC(msg)
    if not check_supervisor:
        msg = 'Erro ao conectar com Supervisor \nSem conexão'
        traced.error(msg)
        raise ErrConnRPC(msg)
    sock_common = ServerProxy('http://%s:%s/xmlrpc/common' % (server, port))
    uid = sock_common.login(dbname, username, pwd)
    sock = ServerProxy('http://%s:%s/xmlrpc/object' % (server, port))

    if not dav_ids:
        dav_ids = [dav.id for dav in SupervisorDavCapa.query.filter('supdva_restricao IN (0, 1)').all()]

    if dav_ids:
        pedidos = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.capa', 'read', dav_ids, ['state'])
        for pedido in pedidos:
            if pedido.get('state') == 'C':
                dav_capa = SupervisorDavCapa.get_by(id=pedido.get('id'))
                b_sess.begin()
                dav_capa.supdva_restricao = -1
                b_sess.commit()
                res = True

    return res


@traced
def fatura_pedido_dav(pedido_id, loja='', caixa='', coo=''):
    check = testa_conexao_rpc(server, username, pwd, dbname, port)
    if not check.get('res'):
        msg = 'Erro ao conectar com SigERP \n\n' + str(check.get('msg'))
        traced.error(msg)
        raise ErrConnRPC(msg)

    sock_common = ServerProxy('http://%s:%s/xmlrpc/common' % (server, port))
    uid = sock_common.login(dbname, username, pwd)
    sock = ServerProxy('http://%s:%s/xmlrpc/object' % (server, port))
    pedido_dados = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.capa', 'read',
                                pedido_id, ['state'])
    if pedido_dados and isinstance(pedido_dados, list):
        pedido_dados = pedido_dados[0]

    if pedido_dados.get('state') == 'F': # o pedido ja foi faturado no erp
        return True

    vals = {'state': 'F', 'loja': '%d' % loja, 'caixa': '%d' % caixa, 'coo': '%d' % coo, 'exportado_frente': True}
    ret_val = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.capa', 'write', pedido_id, vals)

    # o pedido pode ter mais de um titulo, deletar todos eles, pois estao em previsao.
    titulo_ids = sock.execute(dbname, uid, pwd, 'financeiro.titulo', 'search', [('fintit_pedido_id','=',pedido_id)])
    sock.execute(dbname, uid, pwd, 'financeiro.titulo', 'unlink', titulo_ids)
    return ret_val


@traced
def faturar_pedidos_pendentes(dav, loja='', caixa='', coo=''):
    res = []
    try:
        f = fatura_pedido_dav(dav.id, loja, caixa, coo)
        if not f:
            raise Exception('Pedido nao faturado')
    except Exception as e:
        traced.error('DAV{0:010d} (N)'.format(dav.supdva_nro_pedido) + '\n' + e)
        res.append('DAV{0:010d}'.format(dav.supdva_nro_pedido))
    return res


def import_dav(parent=None):
    try:
        check = testa_conexao_rpc(server, username, pwd, dbname, port)
        # Get the uid
        sock_common = ServerProxy('http://%s:%s/xmlrpc/common' % (server, port))
        uid = sock_common.login(dbname, username, pwd)
        sock = ServerProxy('http://%s:%s/xmlrpc/object' % (server, port))
    except Exception as e:
        msg = 'Erro ao conectar com SigERP \n\n' + str(e)
        raise ErrConnRPC(msg)

    def get_cpf_cnpj_cliente(id):
        """
        Obtem o documento do cliente informado (no gestão)
        retorna o documento com 15 caracteres e 0 a esquerda
        """
        doc = sock.execute(dbname, uid, pwd, 'adm.pessoa', 'read', id, ['cpf_cnpj'])
        if isinstance(doc, list):
            doc = doc[0]
        doc = doc.get('cpf_cnpj', '')
        if not doc:
            doc = '0'
        doc = re.sub('[^0-9]', '', doc)
        return doc.rjust(15, '0')

    def get_qtda_emb(p_embalagem_id):
        """
        Obtem a quantidade da embalem informada
        retorna int
        """
        res = {'qtda': 1,
               'sigla': 'UN'}

        if isinstance(p_embalagem_id, list):
            p_embalagem_id = p_embalagem_id[0]

        if not p_embalagem_id:
            return res

        embalagem_data = sock.execute(
            dbname, uid, pwd, 'estoque.produto.embalagem', 'read', p_embalagem_id, ['embalagem_id'])

        embalagem_id = embalagem_data.get('embalagem_id')

        if embalagem_id and isinstance(embalagem_id, list):
            embalagem_id = embalagem_id[0]

            emb = sock.execute(dbname, uid, pwd,
                               'estoque.embalagem', 'read', embalagem_id,
                               ['quantidade_na_embalagem', 'sigla', 'unidade_id'])
            unidade_id = emb.get('unidade_id')[0]
            unidade = sock.execute(dbname, uid, pwd,
                               'estoque.embalagem.unidade', 'read', unidade_id, ['unidade'])

            res.update({'qtda': emb.get('quantidade_na_embalagem') or 1, 'sigla': unidade.get('unidade')})

        return res

    def get_menor_qtd_emb(grade_id, emp_list):
        args = [('grade_id', '=', grade_id)]
        if emp_list:
            epc_company_id = sock.execute(dbname, uid, pwd, 'adm.empresa.objeto', 'getEmpresa',
                                          'estoquepedidos.pedido.capa', False, {}, emp_list[0])
            args.append(('company_id', '=', epc_company_id))

        e = sock.execute(dbname, uid, pwd, 'automacao.codigo.barra', 'search',
                         args, 0, '', '', {'sig_force_company': True})
        e_data = sock.execute(dbname, uid, pwd, 'automacao.codigo.barra', 'read', e,
                              ['codigo_barra',
                               'produto_embalagem_id'])
        m = False
        e_id = False
        cod_b = False
        for x in e_data:
            print '*', x
            qtda = get_qtda_emb(x['produto_embalagem_id'][0]).get('qtda')
            if not m:
                m, cod_b, e_id = qtda, x.get('codigo_barra'), x.get('id')

            if qtda < m:
                m, cod_b, e_id = qtda, x.get('codigo_barra'), x.get('id')

        if not cod_b:
            return False

        return e_id, cod_b

    def get_codigo_barra(item, emp_list):
        """
        Obtem o codigo de barra do item informado (no gestão)
        retorna o codigo de barra com 14 caracteres e 0 a esquerda
        """
        qtde_pedida = item.get('qtde_pedida')
        grade_id = item.get('grade_id')[0]
        emb_id = item.get('produto_embalagem_id')[0]
        print 'GET COD BARRA', grade_id, emb_id
        args = [('grade_id', '=', grade_id),
                ('active', '=', True)]
        if emp_list:
            acb_company_id = sock.execute(dbname, uid, pwd, 'adm.empresa.objeto', 'getEmpresa',
                                          'automacao.codigo.barra', False, {},
                                          emp_list[0])
            args.append(('company_id', '=', acb_company_id))

        barra_ids = sock.execute(dbname, uid, pwd, 'automacao.codigo.barra', 'search', args, 0, '',
                                 '', {'sig_force_company': True})
        print 'barra_ids', barra_ids
        codigo_barra = []
        menor_qtd = 0
        for i in sock.execute(dbname, uid, pwd, 'automacao.codigo.barra', 'read', barra_ids,
                                    ['codigo_barra', 'produto_embalagem_id'], {}, '_classic_write'):
            qtd = get_qtda_emb(i.get('produto_embalagem_id'))['qtda']
            menor_qtd = not menor_qtd and qtd or menor_qtd
            if i.get('produto_embalagem_id') == emb_id:
                codigo_barra = (qtd, i.get('codigo_barra'))
                break

            if qtd <= menor_qtd and qtde_pedida % qtd == 0:
                menor_qtd = qtd
                codigo_barra = (qtd, i.get('codigo_barra'))


        return codigo_barra

    # -------------------------------------------- // -------------------------------------------- #

    # BUSCAR OS PEDIDOS QUE NAO ESTAO EM ESBOCO E AINDA NAO FORAM ENVIADOS
    args = [('state', 'in', ['V', 'C', 'F', 'D', 'O']), ('tipo', '=', 'V'),
            ('exportado_frente', '=', False)]
    if dbemp:
        emp = dbemp.replace(' ', '').replace('-', ',').split(',')
        emp_list = []
        for x in emp:
            emp_list.append(int(x))
        emp_list = list(set(emp_list))
        epc_company_id = sock.execute(dbname, uid, pwd, 'adm.empresa.objeto', 'getEmpresa',
                                      'estoquepedidos.pedido.capa', False, {}, emp_list[0])
        args.append(('company_id', '=', epc_company_id))
    pedido_ids = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.capa', 'search', args, 0, '',
                              '', {'sig_force_company': True})
    err_dict = {}
    dav_super = SupervisorDavCapa.query.filter("supdva_restricao=1").all()
    if dav_super:
        nro_pedidos = map(lambda x: 'PVE-%010d' % x.supdva_nro_pedido, dav_super)
        args = [('state', '=', 'R'), ('tipo', '=', 'V'), ('numero_pedido','in', nro_pedidos)]
        pedido_ids.extend(sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.capa', 'search',
                                       args, 0, '', '', {'sig_force_company': True}))
    if not pedido_ids:
        return err_dict

    # OBTER OS DADOS DOS PEDIDOS ENCONTRADOS
    # READ PEDIDOS
    fields = ['create_date', 'numero_pedido', 'pessoa_id', 'vendedor_id', 'vr_desconto',
              'valor_total', 'company_id', 'state']
    pedido_data = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.capa', 'read',
                               pedido_ids, fields)

    parent.show_info_proccess(True, 'Aguarde! Processando %s pedido(s)...' % (len(pedido_ids)) )
    parent.QtGui.QApplication.processEvents()
    parent.QtGui.QApplication.processEvents() # quando chama este metodo apenas uma vez, a mensagem fica com a tela preta
    time.sleep(0.5)
    pedidos_enviados_ids = [] # pedidos que nao sao para aparecer mais no frente(faturados e cancelados no ERP)
    list_pedidos_ids = [] # pedidos enviados com sucesso para o Supervisor
    for pedido in pedido_data:
        pedido_id = pedido.get('id')
        nro_pedido = int(re.sub('[^0-9]', '', pedido.get('numero_pedido')))
        loja = pedido.get('company_id') or None
        if loja and isinstance(loja, list):
            loja = loja[0]
        caixa = config['caixa']
        data_hora = pedido.get('create_date')
        juros = 0.0
        descontos = pedido.get('vr_desconto')
        data_cupom = None
        restricao = 1
        total_dav = pedido.get('valor_total')
        err_dict.update({nro_pedido: []})

        # *** CHECAR CLIENTE DO PEDIDO *** #
        dav_super = SupervisorDavCapa.query.filter("id=%s" % pedido_id).all()

        # pedidos que foram faturados ou cancelados no erp e nao estao no supervisor
        if not dav_super:
            if pedido.get('state') in ['F', 'C']:
                pedidos_enviados_ids.append(pedido.get('id'))
                continue

        else: # pedido que foi modificado e já está no supervisor
            if isinstance(dav_super, list):
                dav_super = dav_super[0]

            if pedido.get('state') == 'F': # pedidos que foram faturados no erp
                dav_super.supdva_restricao = 2
                pedidos_enviados_ids.append(pedido.get('id'))
                continue
            elif pedido.get('state') == 'C': # pedidos que foram cancelados no erp
                dav_super.supdva_restricao = -1
                pedidos_enviados_ids.append(pedido.get('id'))
                continue

            if dav_super.supdva_nro_cupom:  # Verifica se já foi emitido o cupom p/ o pedido
                err_dict[nro_pedido] += ['Pedido já foi emitido']
                pedidos_enviados_ids.append(pedido.get('id'))
                continue

            b_sess.begin()
            dav_itens = SupervisorDavCorpo.query.filter(SupervisorDavCorpo.supdvo_capa_id == pedido_id).all()
            dav_fecha = SupervisorDavFechamento.query.filter(
                SupervisorDavFechamento.supdvf_capa_id == pedido_id).all()
            for f in dav_fecha:
                dav_venci = SupervisorDavVencimentos.query.filter(
                                            SupervisorDavVencimentos.supdvv_fechamento_id == f.id).all()
                for venc in dav_venci:
                    b_sess.delete(venc)  # APAGA OS VENCIMENTOS DOS FECHAMENTOS

                # sem esta consulta dava erro ao fazer o commit
                SupervisorDavVencimentos.query.filter(
                                SupervisorDavVencimentos.supdvv_fechamento_id == f.id).all()

            for f in dav_fecha:
                b_sess.delete(f)  # APAGA O FECHAMENTO

            for d in dav_itens:
                b_sess.delete(d)  # APAGA O ITEM DO DAV

            # sem estas consultas dava erro ao fazer o commit
            SupervisorDavCorpo.query.filter(SupervisorDavCorpo.supdvo_capa_id == pedido_id).all()
            SupervisorDavFechamento.query.filter(SupervisorDavFechamento.supdvf_capa_id == pedido_id).all()

            b_sess.delete(dav_super)  # APAGA O DAV

            b_sess.commit()

            del (dav_super)

        cliente = pedido.get('pessoa_id') and pedido.get('pessoa_id')[0]
        nome_cliente = remove_accents(pedido.get('pessoa_id') and pedido.get('pessoa_id')[1] or '')
        cnpj_cpf_cliente = cliente and get_cpf_cnpj_cliente(cliente) or ''
        cliente = False
        if cnpj_cpf_cliente:
            cliente = CaixaCadClientes.get_by(supcli_cnpj_cpf=cnpj_cpf_cliente)
        if not cliente and cnpj_cpf_cliente:
            err_dict[nro_pedido] += [
                'Cliente {0:>s} nao esta cadastrado no Frente'.format(nome_cliente)]
            continue

        if pedido.get('state') == 'R': # pedidos em esboço sao deletados e nao sao criados no sup.
            continue

        nome_cliente = cliente and cliente.name or ''
        cpf_cnpj_cliente = cliente and cliente.supcli_cnpj_cpf or ''

        # CHECAR VENDEDOR DO PEDIDO SE É OPERADOR DO FRENTE
        vendedor = vendedor_id = pedido.get('vendedor_id') or None
        if vendedor and isinstance(vendedor, list):
            vendedor_id = vendedor[0]
            nome_vendedor = remove_accents(vendedor[1])
            cad_vendedor = SupervisorCadVendedor.get_by(supven_codigo=vendedor_id)
            if cad_vendedor:
                vendedor_id = cad_vendedor.id
            else:
                err_dict[nro_pedido] += [
                    'Vendedor %s-%s nao esta cadastrado no Frente' % (vendedor_id,
                                                                      nome_vendedor)]
                continue

        # *** OBTER OS DADOS DOS ITENS *** #
        item_ids = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.item', 'search',
                                [('pedido_id', '=', pedido_id)])
        if not item_ids:
            err_dict[nro_pedido] += ['Sem itens lancados']
            continue

        items_data = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.item', 'read', item_ids,
                                  ['produto_id', 'grade_id', 'produto_embalagem_id', 'vr_unitario',
                                   'valor_total', 'vr_desconto', 'qtde_pedida'], {})
        sequence_item = 1
        items_list = []
        for item in items_data:
            traced.info('dav id ' + str(pedido_id))
            traced.info('Nro. Pedido: ' + str(nro_pedido))
            item_descricao = remove_accents(item.get('produto_id')[1])
            codigo_barra = get_codigo_barra(item, emp_list)
            traced.info('EAN ' + str(codigo_barra))
            if not codigo_barra:
                err_dict[nro_pedido] += [
                    'Produto ({0:>s}) sem codigo de barras no SigERP '
                    'para \n\tGrade: "{1:>s}" e \n\tEmbalagem: "{2:>s}"'.format(
                        item_descricao,
                        item.get('grade_id')[1],
                        item.get('produto_embalagem_id')[1])]
                b_sess.rollback()
                break

            traced.info('cbarra ' + str(codigo_barra[1]))
            codigo_barra_e = CaixaCadItens.get_by(supite_codigo_item=codigo_barra[1],
                                                  supite_loja=parent.loja.supcfc_loja)
            if not codigo_barra_e:
                err_dict[nro_pedido] += [
                    'Produto ({0:>s}) sem codigo de barras no Frente'.format(
                         item_descricao)]
                b_sess.rollback()
                break

            m = get_menor_qtd_emb(item.get('grade_id')[0], emp_list)
            qtd_emb_orig = get_qtda_emb(item.get('produto_embalagem_id'))
            traced.info(get_qtda_emb(item.get('produto_embalagem_id')))

            traced.info('emb ' + str(qtd_emb_orig) + str(codigo_barra) + str(m))
            vorig = item.get('vr_unitario') / qtd_emb_orig['qtda'] * qtd_emb_orig['qtda']
            vajus = item.get('vr_unitario') / qtd_emb_orig['qtda'] * codigo_barra[0]
            qorig = (item.get('qtde_pedida') * qtd_emb_orig['qtda']) / qtd_emb_orig['qtda']
            qajus = (item.get('qtde_pedida') * qtd_emb_orig['qtda']) / codigo_barra[0]

            traced.info(str(vorig * qorig) + '|' + str(vorig) + '-' + str(qorig))
            traced.info(str(vajus * qajus) + '|' + str(vajus) + '-'  + str(qajus))

            items_list.append(dict(
                create_date=datetime.strftime(datetime.today(), '%Y-%m-%d %H:%M:%S'),
                supdvo_capa_id=pedido_id,
                supdvo_sequencia=sequence_item,
                supdvo_numero_do_dav=nro_pedido,
                supdvo_data_hora=data_hora,
                supdvo_item=codigo_barra[1],
                supdvo_quantidade=qajus,
                supdvo_quantidade_conferida=item.get('qtde_atendida') or 0,
                supdvo_vr_total=item.get('valor_total'),
                supdvo_desconto_item=item.get('vr_desconto') or 0,
                supdvo_acrescimo_item=item.get('vr_acrescimo') or 0,
                supdvo_venda_cancelamento='V',
                supdvo_descricao=item_descricao.split(" - ")[1],
                supdvo_unidade=qtd_emb_orig.get('sigla'),
                supdvo_vr_unit=vajus,
                supdvo_totalizador_parcial='',
            ))
            sequence_item += 1

        if err_dict[nro_pedido]:
            continue

        # *** OBTER OS DADOS DOS FECHAMENTOS *** #
        fechamento_ids = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.condicao',
                                      'search',
                                      [('capa_id', '=', pedido_id)], 0, 0, 'id')
        if not fechamento_ids:
            err_dict[nro_pedido] += ['Sem fechamentos lançado']
            continue

        fechamentos_data = sock.execute(dbname, uid, pwd,
                                        'estoquepedidos.pedido.condicao', 'read',
                                        fechamento_ids, [])
        sequence_fechamento = 1
        fechamento_list = []
        sem_finalizadora_frente = False
        for fechamento in fechamentos_data:
            cod_ecf = sock.execute(dbname, uid, pwd, 'adm.condicao.forma', 'read',
                                   fechamento['condicao_forma_id'][0], [])
            cod_ecf = cod_ecf.get('finalizadora_na_ecf')
            finalizadora = CaixaCadFinalizadora.get_by(supfin_codigo=cod_ecf)
            if not finalizadora:
                err_dict[nro_pedido] += [
                    'Condição de pagamento: %d, não cadastrado no Frente' % cod_ecf]
                sem_finalizadora_frente = True
                break

            fechamento_list.append(dict(
                id=fechamento.get('id'),
                supdvf_cheque_agencia=fechamento.get('pedcdc_chq_agencia') or 0,
                supdvf_cheque_banco=fechamento.get('pedcdc_chq_banco') or 0,
                supdvf_cheque_cnpj=fechamento.get('pedcdc_chq_emitente') or '',
                supdvf_cheque_conta=int(re.sub('[^0-9]', '',
                                               (str(fechamento.get('pedcdc_chq_conta') or '0')))),
                supdvf_cheque_numero=int(re.sub('[^0-9]', '',
                                               (str(fechamento.get('pedcdc_chq_numero') or '0')))),
                supdvf_capa_id=pedido_id,
                supdvf_sequencia=sequence_fechamento,
                supdvf_finalizadora=cod_ecf,
                supdvf_valor=fechamento['valor']
            ))
            sequence_fechamento += 1

        if sem_finalizadora_frente:
            continue

        # SE TUDO DEU CERTO VAMOS LANÇAR O PEDIDO COMO DAV
        b_sess.begin()  # INICIA TRANSAÇÃO NO BANCO
        SupervisorDavCapa(id=pedido_id,
                          create_date=datetime.strftime(datetime.today(), '%Y-%m-%d %H:%M:%S'),
                          supdva_nro_pedido=nro_pedido,
                          supdva_data_hora=datetime.now(),
                          supdva_cliente=cpf_cnpj_cliente,
                          supdva_nome_cliente=nome_cliente,
                          supdva_vendedor_id=vendedor_id,
                          supdva_juros=juros,
                          supdva_descontos=descontos,
                          supdva_data_cupom=data_cupom,
                          supdva_loja=loja,
                          supdva_caixa=caixa,
                          supdva_restricao=restricao,
                          supdva_total_dav=total_dav)
        b_sess.commit()

        for item in items_list:
            print '>>>>>', item
            b_sess.begin()
            SupervisorDavCorpo().from_dict(item)
            b_sess.commit()

        for fechamento in fechamento_list:
            id_erp = fechamento.get('id')
            fechamento.pop('id')
            b_sess.begin()
            SupervisorDavFechamento().from_dict(fechamento)
            b_sess.commit()
            id_dav = SupervisorDavFechamento.query.filter(
                    SupervisorDavFechamento.supdvf_capa_id == pedido_id).order_by('id desc').limit(1).one().id
            # vencimentos do fechamento
            vencimentos_ids = sock.execute(dbname, uid, pwd, 'estoquepedidos.pedido.parcela', 'search',
                                          [('pedido_condicao_id', '=', id_erp)], 0, 0, 'data_vencimento')
            if not vencimentos_ids:
                err_dict[nro_pedido] += ['Sem vencimento lançado']
                continue

            vencimentos_data = sock.execute(dbname, uid, pwd,
                                            'estoquepedidos.pedido.parcela', 'read',
                                            vencimentos_ids, [])
            sequence_vencimento = 1
            for vencimento in vencimentos_data:
                b_sess.begin()
                SupervisorDavVencimentos().from_dict(dict(supdvv_seq_vencimento=sequence_vencimento,
                                                          supdvv_fechamento_id=id_dav,
                                                          supdvv_valor_parcela=vencimento.get('vr_parcela'),
                                                          supdvv_vencimento=vencimento.get('data_vencimento')
                                                         )
                                                     )
                b_sess.commit()
                sequence_vencimento += 1

        list_pedidos_ids.append(pedido_id)

    if list_pedidos_ids:
        sql = ''' UPDATE estoquepedidos_pedido_capa
                  SET exportado_frente = True,
                      loja = %s,
                      caixa = %s
                  WHERE id IN (%s) ''' % (config['loja'],
                                          caixa,
                                          str(list_pedidos_ids).replace('[','').replace(']', ''))
        exec_sql(sql, 'E')


    # grava como enviado os pedidos que nao sao mais para aparecer na tela de dav do frente
    if pedidos_enviados_ids:
        sql = ''' UPDATE estoquepedidos_pedido_capa
                  SET exportado_frente = True
                  WHERE id IN (%s) ''' % str(pedidos_enviados_ids).replace('[','').replace(']', '')
        exec_sql(sql, 'E')

    parent.show_info_proccess(False)

    return err_dict

