#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8 -*-

__author__ = 'Dener Rodrigues'

from socket import socket, AF_INET, SOCK_STREAM
from sigpdv.tools.config import config
from sigpdv.tools.log import traced
from decimal import Decimal
from datetime import datetime
from sigpdv.models.db_utils import b_sess
from sigpdv.models.mov_caixa_data import CaixaMovCaixa
from sigpdv.models.cad_operadores_data import CaixaCadOperadores
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.supervisor import SupervisorDavCapa, SupervisorDavCorpo, SupervisorDavFechamento,\
    SupervisorDavVencimentos, SupervisorCadVendedor, SupervisorCadOperadores


class DavGestao(object):
    def __init__(self, server_host, server_port):
        self._server_host = server_host
        self.server_port = server_port
        self._sock = self._connect(server_host, server_port)
        self._cmd_cobol = config.get('cmd_cobol')

    @traced
    def _connect(self, server_host, server_port):
        sock = socket(AF_INET, SOCK_STREAM)
        try:
            sock.connect((server_host, server_port))
        except Exception as e:
            traced.error(str(e))
            raise Exception(e)
        return sock

    @traced
    def send_data(self, data):
        return self._sock.send(data)

    @traced
    def receive_data(self, rsize=1024):
        return self._sock.recv(rsize)

    @traced
    def get_lista(self):
        cmds = ['ESTA AI?',
                'DIR:/mm/arquivos',
                'ARQ NOME:/usr/local/sigpdv/dav.cmd',
                'ARQ DADOS:LISTAR',
                'ARQ FECHAR:',
                'EXECUTAR:%s' % self._cmd_cobol,
                'LER:/usr/local/sigpdv/dav.ret',
                'REMOVER:/usr/local/sigpdv/dav.ret'
                ]

        davs = []
        for cmd in cmds:
            self.send_data(cmd)
            print cmd
            ret = self.receive_data()
            print ret
            if ret.startswith('OK!'):
                continue
            if ret.startswith('ERRO!'):
                raise Exception('Erro ao enviar comando: %s' % cmd)
            if cmd.startswith('LER') and not ret.startswith('FIM'):
                davs.append(dict(
                    [
                        ('data', datetime.strptime(ret[:8], '%d%m%Y').date()),
                        ('num_dav', int(ret[8:18])),
                        ('cliente', ret[18:58].rstrip()),
                        ('total', Decimal(ret[58:69]) / Decimal('1e2'))
                    ]
                ))

                while True:
                    # ret = self.receive_data(203)
                    if ret.startswith('OK!') or ret.startswith('FIM') or ret == '':
                        break
                    else:
                        davs.append(dict(
                            [
                                ('data', datetime.strptime(ret[:8], '%d%m%Y').date()),
                                ('num_dav', int(ret[8:18])),
                                ('cliente', ret[18:58].rstrip()),
                                ('total', Decimal(ret[58:69]) / Decimal('1e2'))
                            ]
                        ))
                        break

        print 'davs >>>', davs
        return davs

    @traced
    def fatura_dav(self, num_dav):
        if isinstance(num_dav, (int, long)):
            num_dav = [num_dav]

        for n_dav in num_dav:
            cmds = ['DIR:/mm/arquivos',
                    'ARQ NOME:/usr/local/sigpdv/dav.cmd',
                    'ARQ DADOS:DAV IMPOR NRO: %010d' % n_dav,
                    'ARQ FECHAR:',
                    'EXECUTAR:%s' % self._cmd_cobol,
                    'LER:/usr/local/sigpdv/dav.ret',
                    'REMOVER:/usr/local/sigpdv/dav.ret']

            for cmd in cmds:

                self.send_data(cmd)
                # if not cmd.startswith('LER'):
                ret = self.receive_data(203)

                if ret.startswith('OK!') and not cmd.startswith('LER'):
                    continue
                if ret.startswith('ERRO!'):
                    raise Exception('Erro ao enviar comando: %s' % cmd)
        return True

    @traced
    def export_dav(self, num_dav):
        if isinstance(num_dav, int):
            num_dav = [num_dav]

        davs_data = ()

        for n_dav in num_dav:
            cmds = ['DIR:/mm/arquivos',
                    'ARQ NOME:/usr/local/sigpdv/dav.cmd',
                    'ARQ DADOS:DAV EXPOR NRO: %010d' % n_dav,
                    'ARQ FECHAR:',
                    'EXECUTAR:%s' % self._cmd_cobol,
                    'LER:/usr/local/sigpdv/dav.ret',
                    # 'REMOVER:/usr/local/sigpdv/dav.ret'
                    ]

            dict_capa = dict(num_dav=n_dav, itens=(), finalizadoras=())

            for cmd in cmds:
                self.send_data(cmd)
                # if not cmd.startswith('LER'):
                ret = self.receive_data(203)
                if ret.startswith('OK!') and not cmd.startswith('LER'):
                    continue
                if ret.startswith('ERRO!'):
                    raise Exception('Erro ao enviar comando: %s' % cmd)
                if cmd.startswith('LER') and not ret.startswith('FIM'):
                    # ret = self.receive_data(203)
                    if ret.startswith('1'):
                        dict_capa.update(dict(
                            cpf=ret[1:16],
                            desc=Decimal(ret[16:25]) / Decimal('1e2'),
                            vendedor=int(ret[16:29]),
                            cliente=ret[29:69].rstrip(),
                            condicoes=ret[69],
                            situacao=ret[70],
                            restricao=3 if ret[72] == 'N' else 1
                            # -1=Cancelamento, 1=Emitir, 2=Emitido, 3=Avisos
                        ))
                    while True:
                        ret = self.receive_data(203)
                        if ret.startswith('OK!') or ret.startswith('FIM') or ret == '':
                            break
                        elif ret.startswith('2'):
                            dict_capa.update(dict(
                                itens=dict_capa.get('itens') + (dict(
                                    codigo=ret[1:15],
                                    quantidade=Decimal(ret[15:24]) / Decimal('1e3'),
                                    valor=Decimal(ret[24:35]) / Decimal('1e2')
                                ),)
                            ))
                        elif ret.startswith('3'):
                            parcelas = ()
                            qtd_parcelas = int(ret[15:17])
                            if qtd_parcelas:
                                i = 17
                                for x in range(qtd_parcelas):
                                    line = ret[i:i + 19]
                                    try:
                                        dt = datetime.strptime(dt, '%d%m%Y').date()
                                    except:
                                        traced.error('Erro ao obter a parcela do dav')
                                        dt = '01-01-1900'
                                    parcelas += (dict(
                                        parcela=x + 1,
                                        vencimento=dt,
                                        valor=Decimal(line[8:]) / Decimal('1e2')
                                    ),)
                                    i += 19
                            dict_capa.update(dict(
                                finalizadoras=dict_capa.get('finalizadoras') + (dict(
                                    codigo=int(ret[1:4]),
                                    valor=Decimal(ret[4:15]) / Decimal('1e2'),
                                    quantidade=qtd_parcelas,
                                    vencimentos=parcelas,
                                ),)
                            ))
                        # else:
                        #     break
                    if not dict_capa.get('itens'):
                        dict_capa = {}
            davs_data += (dict_capa,) if dict_capa else ()
        return davs_data


@traced
def truncate_dav_super(id_dav):
    dav_super = SupervisorDavCapa.get_by(id=id_dav)
    ret = False
    if dav_super:
        if dav_super.supdva_nro_cupom:  # Verifica se já foi emitido o cupom p/ o pedido
            traced.info('Não posso atualizar o DAV # %010d! ***CUPOM %06d EMITIDO***' % (
                id_dav, dav_super.supdva_nro_cupom))
        elif dav_super:
            dav_itens = SupervisorDavCorpo.query.filter(SupervisorDavCorpo.id == id_dav).all()
            dav_fecha = SupervisorDavFechamento.query.filter(SupervisorDavFechamento.id == id_dav).all()
            dav_venci = SupervisorDavVencimentos.query.filter(SupervisorDavFechamento.id == id_dav).all()
            b_sess.begin()
            b_sess.delete(dav_super)  # APAGA O DAV
            for d in dav_itens:
                b_sess.delete(d)  # APAGA O ITEM DO DAV
            for f in dav_fecha:
                b_sess.delete(f)  # APAGA O FECHAMENTO
            for v in dav_venci:
                b_sess.delete(v)  # APAGA O VENCIMENTO

            b_sess.commit()
            traced.info('DAV # %010d! ***EXCLUIDO***' % (id_dav,))
        ret = dav_super.id
    del (dav_super)
    return ret


@traced
def importar_davs_gestao(lista=None):
    dav_restricao = {
        'C': -1,
        'A': 1,
        'F': 2,
        'W': 3
    }
    o_dav = DavGestao(config.get('ip_gestao'), 5019)
    if isinstance(lista, int):
        lista = [lista]

    if not lista:
        lista_dict = o_dav.get_lista()
        lista = []
        for y in lista_dict:
            lista.append(int(y.get('num_dav')))

    for l in lista:
        id_dav = l
        dav = o_dav.export_dav(num_dav=id_dav)
        for d in dav:
            msg_aviso = []
            dav_exist = SupervisorDavCapa.get_by(id=id_dav)
            if dav_exist:
                if dav_exist.supdva_nro_cupom:
                    traced.info('Não posso atualizar o DAV # %010d! ***CUPOM %06d EMITIDO***' % (
                        id_dav, dav_exist.supdva_nro_cupom))
                    continue
                else:
                    truncate_dav_super(id_dav)

            #/ ITENS DO DAV /#
            seq = 1
            total_dav = []
            items_list = []
            for item in d.get('itens'):
                r_item = CaixaCadItens.get_by(supite_codigo_item=item.get('codigo'))
                desc_item = ''
                if not r_item:
                    traced.warning(
                        'Item: %s não encontrado no Banco de Dados do SigFrente!' % item.get(
                            'codigo'))
                    msg_aviso.append('COD.BARRA #%s NAO ENCONTRADO' % item.get('codigo'))
                    d.update(dict(restricao=3))
                else:
                    desc_item = str(r_item.name)
                items_list.append(dict(
                    create_date=datetime.strftime(datetime.today(), '%Y-%m-%d %H:%M:%S'),
                    supdvo_sequencia=seq,
                    supdvo_numero_do_dav=id_dav,
                    supdvo_capa_id=id_dav,
                    supdvo_data_hora=datetime.now(),
                    supdvo_item=item.get('codigo'),
                    supdvo_quantidade=item.get('quantidade'),
                    supdvo_vr_total=item.get('valor'),
                    supdvo_desconto_item=item.get('vr_desconto') or 0,
                    supdvo_acrescimo_item=item.get('vr_acrescimo') or 0,
                    supdvo_venda_cancelamento='V',
                    supdvo_descricao=desc_item,
                    supdvo_unidade='UN',
                    supdvo_vr_unit=item.get('valor') / item.get('quantidade'),
                    supdvo_totalizador_parcial=''))
                total_dav.append(item.get('valor'))
                seq += 1

            #/ FINALIZADORAS DO DAV /#
            seq = 1
            fechamento_list = []
            finalizadora = []
            for finalizadora in d.get('finalizadoras'):
                fechamento_list.append(dict(
                    supdvf_capa_id=id_dav,
                    supdvf_sequencia=seq,
                    supdvf_finalizadora=finalizadora.get('codigo'),
                    supdvf_valor=finalizadora.get('valor')))
                seq += 1

            #/ VENCIMENTOS DO DAV /#
            vencimento_list = []
            for venc in finalizadora.get('vencimentos'):
                vencimento_list.append(dict(
                    supdvv_fechamento_id=SupervisorDavFechamento.get_by(supdvf_capa_id=id_dav).id,
                    supdvv_seq_vencimento=venc.get('parcela'),
                    supdvv_valor_parcela=venc.get('valor'),
                    supdvv_vencimento=venc.get('vencimento')))

            #/ CAPA DO DAV /#
            if d.get('situacao') == 'A' and d.get('restricao') == 3:
                d.update(dict(situacao='W'))
            restricao = dav_restricao.get(d.get('situacao'))
            cod_cliente = d.get('cpf')
            nome_cliente = d.get('cliente')
            descontos = d.get('desc')

            # CHECAR VENDEDOR DO PEDIDO SE É OPERADOR DO FRENTE
            vendedor_id = None
            operador_id = None
            vendedor = SupervisorCadVendedor.get_by(supven_codigo=d.get('vendedor'))
            operador_caixa = CaixaMovCaixa.query.filter(CaixaMovCaixa.supcax_fechado == 'A').filter(
                CaixaMovCaixa.supcax_data_hora_fechamento == None).order_by(
                CaixaMovCaixa.supcax_data_hora_abertura).limit(1)
            if vendedor:
                vendedor_id = vendedor.id
            if operador_caixa:
                for x in operador_caixa:
                    supcax_operador_id = x.supcax_operador_id
                    operador_cax = CaixaCadOperadores.get_by(id=supcax_operador_id).supope_codigo
                    operador_id = SupervisorCadOperadores.get_by(id=operador_cax).supope_codigo

            # SE TUDO DEU CERTO VAMOS LANÇAR O PEDIDO COMO DAV
            b_sess.begin()  # INICIA TRANSAÇÃO NO BANCO
            SupervisorDavCapa(
                id=id_dav,
                create_date=datetime.strftime(datetime.today(), '%Y-%m-%d %H:%M:%S'),
                supdva_data_hora=datetime.now(),
                supdva_cliente=cod_cliente,
                supdva_nome_cliente=nome_cliente,
                supdva_vendedor_id=vendedor_id,
                supdva_operador_id=operador_id,
                supdva_descontos=descontos,
                supdva_restricao=restricao,
                supdva_total_dav=sum(total_dav) - (d.get('desc') or 0)
            )
            b_sess.commit()

            for item in items_list:
                b_sess.begin()
                SupervisorDavCorpo().from_dict(item)
                b_sess.commit()

            for fechamento in fechamento_list:
                b_sess.begin()
                SupervisorDavFechamento().from_dict(fechamento)
                b_sess.commit()

            for vencimento in vencimento_list:
                b_sess.begin()
                SupervisorDavVencimentos().from_dict(vencimento)
                b_sess.commit()

    return lista


if __name__ == '__main__':
    importar_davs_gestao(8)
