#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8 -*-

import sys
from sigpdv.ui.messagebox import YES_ROLE, MSG_ERROR, MSG_QUESTION, MSG_INFO, MSG_NONE, message_dlg
from PySide import QtCore, QtGui
from sigpdv.ui.busca_produto import BuscaProduto
from sigpdv.ui.busca_cliente import BuscaCliente
from sigpdv.ui.login import LoginForm
from sigpdv.ui.dlg_acresc_desc import DlgAcrescDesc
from sigpdv.gui.ui.ui_pve import Ui_MainWindow
from sigpdv.models.db_utils import exec_sql, b_session as sess
from sigpdv.models.supervisor import SupervisorPveCapa, SupervisorPveCorpo,\
    SupervisorCfgConfiguracao, SupervisorCfgParametros, SupervisorCadItens

from decimal import Decimal
from sigpdv import root_path
import os
import traceback
from sigpdv.control.utils import checar_conexao
from SigUtils.decorators import TraceContext
from datetime import datetime
from datetime import timedelta
from sigpdv.tools.config import config
from stoqdrivers.readers.barcode.reader import BarcodeReader
from sigpdv.__init__ import get_version

from elixir import create_all, setup_all
try:
    setup_all()
    create_all()
except Exception as e:
    print "Erro elixir: %s" % e

TWOPLACES = Decimal(10) ** -2
TREEPLACES = Decimal(10) ** -3

LOG_FILE = os.path.join(root_path, 'logs/sigpve.%s.log' % datetime.now().strftime('%d-%m-%Y'))

log = TraceContext(logfile=LOG_FILE)


@log
def exception_message_box(text, btn_quit=False):
    """
    Extract exception information and display a message box with exception details in
    a details extension of the message.
    """
    msg = QtGui.QMessageBox()
    msg.setWindowTitle(QtGui.QApplication.applicationName())
    msg.setText(text)
    #        msg.setInformativeText(e)
    exc_type, exc_value, exc_traceback = sys.exc_info()
    traceback.print_exc()
    msg.setDetailedText(''.join(traceback.format_exception(exc_type, exc_value, exc_traceback)))
    msg.setStandardButtons(QtGui.QMessageBox.Ok)
    msg.setIcon(QtGui.QMessageBox.Critical)
    if btn_quit:
        msg.addButton(QtGui.QMessageBox.Close)
    ret = msg.exec_()
    if ret == QtGui.QMessageBox.Close:
        sys.exit()

@log
def get_loja():
    res = SupervisorCfgConfiguracao.get_by()
    if not res:
        res = SupervisorCfgConfiguracao.get_by()
        if not res or not res.supcfc_loja:
            msg = 'Empresa não cadastrada na tabela caixa_cfg_configuracao\n\n' + \
                  'A Aplicação não pode continuar'
            if not res.supcfc_loja:
                msg = "Atenção!\n\nLoja não está cadastrada no parâmetro do Supervisor!" + \
                      '\nA Aplicação não pode continuar'

            message_dlg(None, MSG_ERROR, msg)
            sys.exit()

    return res

class PveForm(QtGui.QMainWindow):
    def __init__(self, operador, parent=None):
        super(PveForm, self).__init__()

        self.ui = Ui_MainWindow()

        self.ui.setupUi(self)
        self.estacionamento = False
        self.teclado = False
        self.btn_cancel_info_dialog = False
        self.cancel_pinpad = False

        self._timer_principal()
        timer_principal = QtCore.QTimer(self)
        timer_principal.timeout.connect(self._timer_principal)

        sapce1 = QtGui.QLabel()
        sapce2 = QtGui.QLabel()
        self.ui.horizontalLayout_13.removeWidget(self.ui.lbl_operador)
        self.ui.horizontalLayout_13.removeWidget(self.ui.label_img_calendar)
        self.ui.horizontalLayout_13.removeWidget(self.ui.lbl_time)
        self.ui.horizontalLayout_13.removeWidget(self.ui.lbl_img_versao)
        self.ui.horizontalLayout_13.removeWidget(self.ui.lbl_versao)
        self.ui.frmbuttons.setVisible(True)
        self.ui.lbl_operador.setText(' Operador: %s' % operador.name)
        self.ui.lbl_versao.setText(u'Pré-Venda: 3.1 SVN-%s' % get_version())
        self.ui.statusbar.addWidget(self.ui.lbl_operador)
        self.ui.statusbar.addWidget(sapce1)
        self.ui.statusbar.addWidget(self.ui.label_img_calendar)
        self.ui.statusbar.addWidget(self.ui.lbl_time)
        self.ui.statusbar.addWidget(sapce2)
        self.ui.statusbar.addWidget(self.ui.lbl_img_versao)
        self.ui.statusbar.addWidget(self.ui.lbl_versao)

        self.ui.stackedWidget.setCurrentIndex(1)
        #Atalhos
        self.ui.buttonNovaPve.setShortcut("Ctrl+N")
        self.ui.buttonEditarPve.setShortcut("Ctrl+A")
        self.ui.buttonCancelarPVE.setShortcut("Ctrl+C")
        self.ui.buttonSair.setShortcut("Ctrl+R")
        self.ui.bt_desc.setShortcut("Ctrl+D")
        self.ui.buttonAlternar.setShortcut("Ctrl+O")

        # SINAIS
        #self.ui.buttonAddItem.pressed.connect(self._button_add_item_click)
        #self.ui.buttonBuscarProduto.pressed.connect(self._button_busca_item_click)
        self.ui.spinAcrescItem.valueChanged.connect(self._calcula_total)
        self.ui.spinDescItem.valueChanged.connect(self._calcula_total)
        self.ui.spinQtda.valueChanged.connect(self._calcula_total)
        self.ui.spinDescItem.editingFinished.connect(self.spinVlItem_editingFinished)
        self.ui.buttonCancelaItem.pressed.connect(self._cancela_item)
        self.ui.buttonNovaPve.pressed.connect(self._button_nova_pve_click)
        self.ui.buttonSair.pressed.connect(self.close)
        self.ui.buttonEditarPve.pressed.connect(self._editar_pve)
        self.ui.buttonBuscaPve.pressed.connect(self.filter_pve)
        self.ui.buttonAlternar.pressed.connect(self._alternar)
        self.ui.stackedWidget.currentChanged.connect(self._update_buttons)
        self.ui.buttonCancelarPVE.pressed.connect(self._cancelar_pve)
        self.ui.buttonBuscarCliente.pressed.connect(self._button_busca_cliente_click)
        self.ui.bt_desc.pressed.connect(self._add_desc)
        self.ui.edtFiltroCodPve.textChanged.connect(self.edtfiltrocodpve)

        self.loja = get_loja()

        # OUTROS
        self._operador = operador
        self._pve = None
        self._capaPve = None
        self._cli = ''
        # IMAGENS
        icon = QtGui.QIcon()
        icon.addPixmap(
            QtGui.QPixmap(os.path.join(root_path, "gui/ui/images/art/scalable/apps/xfce-edit.svg")),
            QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.setWindowIcon(icon)

        icon1 = QtGui.QIcon()
        icon1.addPixmap(
            QtGui.QPixmap(os.path.join(root_path, "gui/ui/images/art/16x16/actions/gtk-open.png")),
            QtGui.QIcon.Disabled, QtGui.QIcon.Off)
        self.ui.comboStatusPve.addItem(icon1, "")

        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(os.path.join(root_path, "resources/16x16/hourglass.png")),
                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon2.addPixmap(QtGui.QPixmap(os.path.join(root_path, "resources/16x16/hourglass.png")),
                        QtGui.QIcon.Disabled, QtGui.QIcon.Off)
        self.ui.comboStatusPve.addItem(icon2, "")

        icon3 = QtGui.QIcon()
        icon3.addPixmap(
            QtGui.QPixmap(os.path.join(root_path, "gui/ui/images/art/16x16/actions/fileprint.png")),
            QtGui.QIcon.Disabled, QtGui.QIcon.Off)
        self.ui.comboStatusPve.addItem(icon3, "")

        icon4 = QtGui.QIcon()
        icon4.addPixmap(
            QtGui.QPixmap(os.path.join(root_path, "gui/ui/images/art/16x16/actions/kfind.png")),
            QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.ui.buttonBuscarCliente.setIcon(icon4)

        params = SupervisorCfgParametros
        pve_desconto = pve_acrescimo = False
        self.ui.spinQtda.setDecimals(3)
        self.ui.spinQtda.setMinimum(0)
        self.ui.spinQtda.setMaximum(100000)
        if params:
            if params.supcfp_pre_permite_desconto:
                pve_desconto = True
            if params.supcfp_pre_permite_acrescimo:
                pve_acrescimo = True
        self.ui.spinDescItem.setEnabled(pve_desconto)
        self.ui.spinAcrescItem.setEnabled(pve_acrescimo)

        self.status_conexao_servidor = False

        checar_conexao(parent=self, var=True, path='../conexao.py')
        scanner_timer = QtCore.QTimer(self)
        scanner_timer.timeout.connect(self._timer)
        scanner_timer.start(3000)

        self.barcode_active = config['itens_scan_active']
        scanner_timer = QtCore.QTimer(self)
        scanner_timer.timeout.connect(self._active_scanner_timer)

        if self.barcode_active:
            try:
                self.barcode = BarcodeReader(brand='metrologic', model='MC630',
                                             device=config['scanner'])
                self.barcode._port.timeout = 0.001
            except Exception as err:
                exception_message_box(str(err), True)
                self.barcode_active = False
            scanner_timer.start(100)

        self._filtro_padrao()
        self.clear_widget_table(self.ui.tableBuscaPve)
        self.filter_pve(show_msg=False)

    def _timer_principal(self):
        self.ui.lbl_time.setText(datetime.now().strftime('%d/%m/%Y %H:%M'))

    def edtfiltrocodpve(self, value):
        self.ui.editBuscaPve.setEnabled(True)
        self.ui.frmPeriodo.setEnabled(True)
        self.ui.comboFiltroStatus.setEnabled(True)

        if value:
            if int(value):
                self.ui.editBuscaPve.setEnabled(False)
                self.ui.frmPeriodo.setEnabled(False)
                self.ui.comboFiltroStatus.setEnabled(False)

    def populate_tablepve(self, pves):
        table_pve = self.ui.tableBuscaPve
        self.clear_widget_table(table_pve)

        row = 0
        for pve in pves:
            table_pve.insertRow(row)
            # Status
            icon3 = QtGui.QIcon()
            if pve.suppva_restricao == -1:
                rest = 'Cancelado'
                icon3.addPixmap(QtGui.QPixmap(
                    os.path.join(root_path, "gui/ui/images/art/16x16/actions/gtk-cancel.png")),
                                QtGui.QIcon.Disabled, QtGui.QIcon.Off)
            elif pve.suppva_restricao == 0:
                rest = 'Aberto'
                icon3.addPixmap(QtGui.QPixmap(
                    os.path.join(root_path, "gui/ui/images/art/16x16/actions/gtk-open.png")),
                                QtGui.QIcon.Disabled, QtGui.QIcon.Off)
            elif pve.suppva_restricao == 1:
                rest = u'Aguardando Emissão'
                icon3.addPixmap(
                    QtGui.QPixmap(os.path.join(root_path, "resources/16x16/hourglass.png")),
                    QtGui.QIcon.Disabled, QtGui.QIcon.Off)
            elif pve.suppva_restricao == 4:
                rest = u'Anulada'
                icon3.addPixmap(QtGui.QPixmap(
                    os.path.join(root_path, "gui/ui/images/art/16x16/actions/gtk-cancel.png")),
                                QtGui.QIcon.Normal, QtGui.Disabled.Off)
            elif pve.suppva_restricao == 5:
                rest = u'Faturando'
                icon3.addPixmap(
                    QtGui.QPixmap(os.path.join(root_path, "resources/16x16/hourglass.png")),
                    QtGui.QIcon.Disabled, QtGui.QIcon.Off)
            else:
                rest = 'Fechado'
                icon3.addPixmap(QtGui.QPixmap(
                    os.path.join(root_path, "gui/ui/images/art/16x16/actions/fileprint.png")),
                                QtGui.QIcon.Disabled, QtGui.QIcon.Off)

            item_check = QtGui.QTableWidgetItem(rest)
            item_check.setIcon(icon3)
            # Codigo
            item_cod = QtGui.QTableWidgetItem('%010d' % pve.id)
            # Nro Cupom
            ncupom = 0 if not pve.suppva_nro_cupom else pve.suppva_nro_cupom
            nro_cupom = QtGui.QTableWidgetItem('%06d' % ncupom)
            # data
            item_data = QtGui.QTableWidgetItem(
                '%s' % pve.suppva_data_hora.strftime('%d/%m/%Y %H:%M'))
            # CPF
            item_cpf = QtGui.QTableWidgetItem(pve.suppva_cliente)
            # Nome Cliente
            item_nome = QtGui.QTableWidgetItem(pve.suppva_nome_cliente)
            # Valor
            if pve.suppva_descontos:
                totalpve = Decimal(pve.suppva_total_pve - pve.suppva_descontos)
            else:
                totalpve = pve.suppva_total_pve if pve.suppva_total_pve else 0
            item_vr = QtGui.QTableWidgetItem('%1.2f' % totalpve)
            item_vr.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

            # WIDGET OS NEW ITENS NA TABLE ITENS
            table_pve.setItem(row, 0, item_check)
            table_pve.setItem(row, 1, nro_cupom)
            table_pve.setItem(row, 2, item_cod)
            table_pve.setItem(row, 3, item_data)
            table_pve.setItem(row, 4, item_cpf)
            table_pve.setItem(row, 5, item_nome)
            table_pve.setItem(row, 6, item_vr)
            row += 1
            table_pve.resizeColumnsToContents()
            table_pve.horizontalHeader().setStretchLastSection(True)

    def filter_pve(self, show_msg=True):
        """
        Filtra a lista de pre vendas:
        """

        res_query = []
        str_filtro = 'O Filtro: '
        cod_pve = 0
        if self.ui.edtFiltroCodPve.text():
            cod_pve = int(self.ui.edtFiltroCodPve.text())

        if cod_pve:  # BUSCAR POR ID
            res_query = SupervisorPveCapa.query.filter(SupervisorPveCapa.id == cod_pve).all()
            str_filtro = u"Por Codigo da Pré-Venda = '%s'" % self.ui.edtFiltroCodPve.text()
        else:  # BUSCAR POR PERÍODO
            status_pve = 0
            curr = self.ui.comboFiltroStatus.currentIndex()
            if curr == 1:
                status_pve = -1  # Cancelada
            elif curr == 2:
                status_pve = 1  # Aguardando emissao
            elif curr == 3:
                status_pve = 2  # Fechado
            elif curr == 4:
                 status_pve = -2  # Anulada

            data_ini = self.ui.dateIni.dateTime().toPython()
            data_fim = self.ui.dateFim.dateTime().toPython()
            str_filtro = u"No Período de '%s' a '%s'" % (
            self.ui.dateIni.text(), self.ui.dateFim.text())
            data_atual = datetime.today().date()
            # Validar os valores dos controles DateTime
            msg = ''
            if data_ini.date() > data_atual:
                msg = "A 'Data Inicial: %s' não pode ser maior que a 'Data Atual: %s'" % (
                data_ini.date(), data_atual)
                self.ui.dateIni.selectAll()
                self.ui.dateIni.setFocus()
            elif data_ini.date() > data_fim.date():
                msg = "A 'Data Inicial: %s' não pode ser maior que a 'Data Final: %s'" % (
                data_ini.date(), data_fim)
                self.ui.dateIni.selectAll()
                self.ui.dateIni.setFocus()
            if data_fim.date() > data_atual:
                msg = "A 'Data Final: %s' não pode ser maior que a 'Data Atual: %s" % (
                data_fim.date(), data_atual)
                self.ui.dateFim.selectAll()
                self.ui.dateFim.setFocus()

            if msg:
                return message_dlg(self, MSG_INFO, msg)
            else:
                nome_cpf_cliente = self.ui.editBuscaPve.text()
                if nome_cpf_cliente:
                    if nome_cpf_cliente.isdigit():
                        str_filtro += u" e CPF = %r" % nome_cpf_cliente
                        res_query = SupervisorPveCapa.query.filter(
                            SupervisorPveCapa.suppva_data_hora.between(data_ini, data_fim)).filter(
                            SupervisorPveCapa.suppva_nome_cliente == nome_cpf_cliente
                        ).filter(SupervisorPveCapa.suppva_restricao == status_pve).all()
                    else:
                        str_filtro += u" e Nome do Cliente contêm '%s'" % nome_cpf_cliente
                        p_like = SupervisorPveCapa.suppva_nome_cliente.like('%s%s%s' % ('%', nome_cpf_cliente, '%'))
                        res_query = SupervisorPveCapa.query.filter(
                            SupervisorPveCapa.suppva_data_hora.between(data_ini, data_fim)).filter(
                            p_like
                        ).filter(SupervisorPveCapa.suppva_restricao == status_pve).all()
                else:
                    res_query = SupervisorPveCapa.query.filter(
                        SupervisorPveCapa.suppva_data_hora.between(data_ini, data_fim)).filter(
                        SupervisorPveCapa.suppva_restricao == status_pve).all()
        str_filtro += u" Restrição = '%s' " % self.ui.comboFiltroStatus.currentText()
        str_filtro += u". Retornou %d Registro(s)." % len(res_query)
        self.ui.lblResFiltro.setText(str_filtro)

        if not res_query and show_msg:
            self.clear_widget_table(self.ui.tableBuscaPve)
            #self._filtro_padrao()
            return message_dlg(self, MSG_INFO, 'A busca não retornou resultados')
        self.populate_tablepve(res_query)
        self.ui.tableBuscaPve.setFocus()
        self.ui.tableBuscaPve.selectRow(0)
        return res_query

    def _filtro_padrao(self):
        self.ui.edtFiltroCodPve.setText('0')
        self.ui.edtFiltroCodPve.selectAll()
        self.ui.edtFiltroCodPve.setFocus()

        date_ini = datetime.today() - timedelta(days=2)
        y, m, d = date_ini.year, date_ini.month, date_ini.day
        self.ui.dateIni.setDate(QtCore.QDate(y, m, d))
        date_fim = datetime.today()
        y, m, d = date_fim.year, date_fim.month, date_fim.day
        self.ui.dateFim.setDate(QtCore.QDate(y, m, d))

        self.ui.editBuscaPve.clear()
        self.ui.comboFiltroStatus.setCurrentIndex(2)

    def _timer(self):
        st = self.status_conexao_servidor
        self.ui.radioButton.setChecked(st)
        try:
            if not st:
                raise Exception('Sem conexao com o Banco de Dados Supervisor.')
        except Exception as err:
            exception_message_box(str(err), True)
        return st

    def _active_scanner_timer(self):
        if self.barcode_active:
            try:
                cod = self.barcode.get_code()
                if cod and self.ui.editCodItem.hasFocus():
                    self.ui.editCodItem.setText(cod)
                    ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Return,
                                         QtCore.Qt.KeyboardModifiers())
                    self.keyPressEvent(ev)

            except Exception as e:
                pass

    def exception_message_box(self, text):
        return exception_message_box(text)

    @log
    def _add_acresc_desc(self, tipo, valor):
        dlg = DlgAcrescDesc(tipo='D', valor=valor)
        dlg.exec_()
        valor = dlg.retorno
        return valor

    @log
    def _add_desc(self, *args):
        if not self._checar_libera_desconto():
            return message_dlg(self, MSG_INFO, 'PVE', 'Desconto não permitido!')

        if not self._pve:
            return message_dlg(self, MSG_INFO, 'Nenhum item inserido no PVE!')

        # VERIFICAR SE EXISTE ACRESCIMOS NOS ITENS AO DAR DESCONTO
        isacresc = SupervisorPveCorpo.query.filter(SupervisorPveCorpo.suppvo_acrescimo_item > 0) \
            .filter(SupervisorPveCorpo.suppvo_venda_cancelamento == 'V').filter(
            SupervisorPveCorpo.suppvo_capa_id == self._pve.id) \
            .limit(1).all()
        if isacresc:
            self.ui.spinDescItem.setValue(0)
            return message_dlg(self, MSG_INFO, 'DESCONTO PVE',
                               'Já há item(s) com Acréscimo. Não é possivel efetuar Desconto.')

        desc = self._add_acresc_desc(tipo='D', valor=self._pve.suppva_total_pve)

        if desc > self._pve.suppva_total_pve:  # Desconto Maior que o Valor do Item
            return message_dlg(self, MSG_INFO, 'DESCONTO PVE',
                               'Desconto não pode ser maior que o valor total.')
        elif desc:
            self.ui.spinDesc.setValue(desc)
            if message_dlg(self, MSG_QUESTION, 'DESCONTO PVE',
                           'Deseja adicionar o desconto e concluir a Pré-Venda?') == \
                    YES_ROLE:
                self.ui.spinDesc.setValue(desc)
                sess.begin()
                self._pve.suppva_descontos = desc
                sess.commit()
                self._alternar()
            else:
                self.ui.spinDesc.setValue(0.00)
        return desc

    @log
    def _update_buttons(self, *args):
        icon = QtGui.QIcon()
        idx = self.ui.stackedWidget.currentIndex()
        if idx == 0:
            icon.addPixmap(QtGui.QPixmap(
                os.path.join(root_path, "gui/ui/images/art/24x24/actions/edit-redo.png")),
                           QtGui.QIcon.Normal, QtGui.QIcon.Off)
            caption = '&Voltar'
            self.ui.buttonNovaPve.setVisible(False)
            self.ui.buttonSair.setEnabled(False)
        else:
            icon.addPixmap(QtGui.QPixmap(os.path.join(root_path,
                                                      "gui/ui/images/art/24x24/apps/accessories-text-editor.png")),
                           QtGui.QIcon.Normal, QtGui.QIcon.Off)
            caption = '&Alterar'
            self.ui.buttonNovaPve.setVisible(True)
            self.ui.buttonSair.setEnabled(True)
        self.ui.buttonEditarPve.setIcon(icon)
        self.ui.buttonEditarPve.setText(caption)
        self.ui.buttonCancelarPVE.setVisible(bool(idx))
        self._capaPve = None

    @log
    def _alternar(self, *args):
        if Decimal(str(self.ui.spinTotalPve.text().replace(',', '.'))) > 0:
            rest = 1
            if not SupervisorPveCorpo.get_by(suppvo_capa_id=self._pve.id, suppvo_venda_cancelamento='V'):
                rest = 0
            sess.begin()
            if self._cli and not self._pve.suppva_cliente:
                self._pve.suppva_cliente = self._cli.supcli_cnpj_cpf
                self._pve.suppva_nome_cliente = self._cli.name

            self._pve.suppva_restricao = rest
            sess.commit()
            self.ui.stackedWidget.setCurrentIndex(1)
            self._filtro_padrao()
            self.ui.buttonNovaPve.setVisible(True)
            self.ui.labelItem.clear()
            self._cli = ''
        else:
            if message_dlg(self, MSG_QUESTION, 'PVE',
                           'Não existe nenhum item adicionado na pre-venda.\nDeseje anular '
                           'essa pre-venda') == YES_ROLE:
                sess.begin()
                self._pve.suppva_restricao = -2
                sess.commit()

        self.filter_pve()

    @log
    def _clear_widgets(self):
        self.clear_widget_table(self.ui.tableItens)
        self.ui.editNumPve.clear()
        self.ui.dateEmissaoPve.clear()
        self.ui.comboStatusPve.setCurrentIndex(0)
        self.ui.editDocCliente.clear()
        self.ui.spinTotalPve.setValue(0)
        self.ui.editCodItem.clear()
        self.ui.spinDescItem.setValue(0)
        self.ui.spinQtda.setValue(1)
        self.ui.spinTotalPve.setValue(0)
        self.ui.spinVlItem.setValue(0)
        self.ui.spinVlTotal.setValue(0)

    @log
    def _cancelar_pve(self, *args):
        if self.ui.stackedWidget.currentIndex() == 1:
            if self.ui.tableBuscaPve.selectedItems():
                id_pve = self.ui.tableBuscaPve.item(self.ui.tableBuscaPve.currentRow(), 2).text()
                status = self.ui.tableBuscaPve.item(self.ui.tableBuscaPve.currentRow(), 0).text()
                id_pve = int(id_pve)
                pve = exec_sql(
                    'SELECT suppva_restricao, suppva_nro_cupom FROM supervisor_pve_capa '
                    'WHERE id = %d;' % id_pve, 'S')
                if not pve:
                    return message_dlg(self, MSG_INFO, 'PVE', 'Pré Venda não encontrada')

                pve = pve[0]

                st = pve.get('suppva_restricao')
                idx = self.ui.tableBuscaPve.currentIndex()

                if st in [0, 1] and not pve.get('suppva_nro_cupom'):
                    if message_dlg(self, MSG_QUESTION, 'PVE',
                                   'Deseja Cancelar a Pré-Venda: %010d' % id_pve) \
                            == YES_ROLE:
                        pve = SupervisorPveCapa.get_by(id=id_pve)
                        sess.begin()
                        pve.suppva_restricao = -1
                        sess.commit()
                        self.filter_pve()
                        self.ui.tableBuscaPve.setCurrentIndex(idx)
                elif st == 4:
                    message_dlg(self, MSG_INFO, 'PVE', 'PVE No. %010d Foi anulada.' % id_pve)
                elif st == 5:
                    message_dlg(self, MSG_INFO, 'PVE',
                                'PVE No. %010d está sendo processada pelo frente.' % id_pve)
                elif st == -1:
                    message_dlg(self, MSG_INFO, 'PVE', 'PVE No. %010d Já está cancelada.' % id_pve)
            else:
                message_dlg(self, MSG_INFO, 'PVE', 'Selecione uma Pré-Venda.')

    @log
    def _editar_pve(self, *args):
        if self.ui.stackedWidget.currentIndex() == 1:
            if self.ui.tableBuscaPve.selectedItems():
                id_pve = self.ui.tableBuscaPve.item(self.ui.tableBuscaPve.currentRow(), 2).text()
                status = self.ui.tableBuscaPve.item(self.ui.tableBuscaPve.currentRow(), 0).text()
                id_pve = int(id_pve)

                self.ui.editCodItem.setFocus()
                pve = exec_sql('SELECT * FROM supervisor_pve_capa WHERE id = %d;' % id_pve, 'S')
                # SupervisorPveCapa.get_by(IdPreVenda=id_pve)
                if not pve:
                    return message_dlg(self, MSG_INFO, 'PVE', 'Pré Venda não encontrada')

                pve = pve[0]
                rest = pve.get('suppva_restricao')
                if pve.get('NroCupom'):
                    if message_dlg(self, MSG_QUESTION, 'PVE',
                                   'Não é possivel alterar a Pré-Venda selecionada.\n '
                                   'Cupom: %06d já foi emitido.\n\nDeseja visualizar?'
                                           % pve.get('NroCupom')) == YES_ROLE:
                        self.editar_pve(id_pve, ro=False)
                        #self.buscar_pve()
                elif rest == 5:
                    if message_dlg(self, MSG_QUESTION, 'PVE',
                                   'Não é possivel alterar a Pré-Venda selecionada.\n\n'
                                   'Deseja visualizar?') == YES_ROLE:
                        self.editar_pve(id_pve, ro=False)
                        #self.buscar_pve()
                elif rest in [0, 1]:
                    self.editar_pve(id_pve)
                elif rest == -1:
                    self.editar_pve(id_pve, ro=False)
                else:
                    msg ='Não posso alterar PVE: %s' % str(status)
                    message_dlg(self, MSG_INFO, 'PVE', msg)
            else:
                message_dlg(self, MSG_INFO, 'PVE', 'Selecione uma Pré-Venda.')
        else:
            if self._pve:
                itens = SupervisorPveCorpo.get_by(suppvo_capa_id=self._pve.id, suppvo_venda_cancelamento='V')
                if self._pve.suppva_restricao == 0 and self.ui.frame_esquerdo.isEnabled() and itens:
                    return message_dlg(self, MSG_INFO, 'PVE', 'Conclua a Pré-Venda.')
            self.ui.stackedWidget.setCurrentIndex(1)
            self.ui.edtFiltroCodPve.setFocus()

    @log
    def editar_pve(self, id_pve, ro=True):
        self._clear_widgets()
        self._pve = pve = SupervisorPveCapa.get_by(id=id_pve)
        self.ui.stackedWidget.setCurrentIndex(0)
        self.ui.editNumPve.setText('%010d' % pve.id)
        d = pve.suppva_data_hora.timetuple()
        d = QtCore.QDateTime(d[0], d[1], d[2], d[3], d[4], d[5])
        self.ui.dateEmissaoPve.setDateTime(d)
        self.ui.comboStatusPve.setCurrentIndex(0)
        self.ui.editDocCliente.setText('[%s] %s' % (pve.suppva_cliente, pve.suppva_nome_cliente))
        self.ui.spinTotalPve.setValue(pve.suppva_total_pve or 0)
        desc = pve.suppva_descontos or 0
        # if pve.suppva_descontos:
        #     desc = pve.suppva_descontos
        #     ro = False
        #     self.ui.labelItem.setText(u'Pré-Venda não pode ser editada porque possui desconto no '
        #                               u'subtotal.')
        self.ui.spinDesc.setValue(desc)
        self.ui.comboStatusPve.setCurrentIndex(pve.suppva_restricao)
        corpos = SupervisorPveCorpo.query.filter(SupervisorPveCorpo.suppvo_capa_id == id_pve) \
            .order_by('suppvo_sequencia')
        for c in corpos:
            item = SupervisorCadItens.get_by(supite_codigo_item=c.suppvo_item)
            icon3 = QtGui.QIcon()
            if c.suppvo_venda_cancelamento == 'V':
                icon3.addPixmap(
                    QtGui.QPixmap(os.path.join(root_path, "resources/16x16/check2.png")),
                    QtGui.QIcon.Normal, QtGui.QIcon.Off)
            else:
                icon3.addPixmap(
                    QtGui.QPixmap(os.path.join(root_path, "resources/16x16/delete.png")),
                    QtGui.QIcon.Normal, QtGui.QIcon.Off)
            table_item = self.ui.tableItens
            row = table_item.rowCount()
            table_item.insertRow(row)
            # Codigo Item
            item_check = QtGui.QTableWidgetItem('')
            item_check.setIcon(icon3)
            # Codigo Item
            item_cod = QtGui.QTableWidgetItem(c.suppvo_item)
            # Descrição
            item_descr = QtGui.QTableWidgetItem(item.name)
            # Quantidade
            item_qtda = QtGui.QTableWidgetItem('%1.3f' % c.suppvo_quantidade)
            # Valor Unitário
            item_vuni = QtGui.QTableWidgetItem('R$ %1.2f' % item.supite_preco)
            item_vuni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            # Acrescimo
            item_acresc = QtGui.QTableWidgetItem('%1.2f' % c.suppvo_acrescimo_item)
            # Desconto
            item_desc = QtGui.QTableWidgetItem('%1.2f' % c.suppvo_desconto_item)
            # Total
            item_total = QtGui.QTableWidgetItem('R$ %1.2f' % c.suppvo_preco_total)

            # WIDGET OS NEW ITENS NA TABLE ITENS
            table_item.setItem(row, 0, item_check)
            table_item.setItem(row, 1, item_cod)
            table_item.setItem(row, 2, item_descr)
            table_item.setItem(row, 3, item_qtda)
            table_item.setItem(row, 4, item_vuni)
            table_item.setItem(row, 5, item_acresc)
            table_item.setItem(row, 6, item_desc)
            table_item.setItem(row, 7, item_total)
            table_item.resizeColumnsToContents()
            table_item.scrollToBottom()

        self.ui.frame_esquerdo.setEnabled(ro)
        self.ui.buttonBuscarCliente.setEnabled(ro)

    @log
    def _cancela_item(self, *args):
        if not self.ui.tableItens.rowCount():
            return message_dlg(self, MSG_INFO, 'PVE', 'Não há itens lançados.')
        if not self.ui.tableItens.selectedItems():
            return message_dlg(self, MSG_INFO, 'PVE', 'Selecione um item.')
        row = self.ui.tableItens.currentRow()
        tb_item = self.ui.tableItens.item(row, 0)
        item_des = self.ui.tableItens.item(row, 2).text()
        item_vr = self.ui.tableItens.item(row, 7).text()
        item = SupervisorPveCorpo.get_by(suppvo_capa_id=self._pve.id, suppvo_sequencia=row + 1)

        if item.suppvo_venda_cancelamento == 'C':
            return message_dlg(self, MSG_ERROR,
                               u'Item: <h2><font color="red"> %d-%s</font></h2> já foi cancelado.' %
                               ((row + 1), item_des))

        if message_dlg(self, MSG_QUESTION,
                       'Deseja cancelar o item:\n<h2><font color="red">%d-%s</font></h2>' % (
                               row + 1, str(item_des))) != YES_ROLE:
            return
        else:
            try:
                total = self.ui.spinTotalPve.value() - float(item_vr.split(' ')[-1])
                sess.begin()
                self._pve.suppva_total_pve = total
                item.suppvo_venda_cancelamento = 'C'
                sess.commit()
                # Limpar a o widget table
                self.ui.spinTotalPve.setValue(total)
                if not SupervisorPveCorpo.get_by(suppvo_capa_id=self._pve.id, suppvo_venda_cancelamento='V'):
                    self._pve.suppva_restricao = 0
            except Exception as err:
                sess.rollback()
                return self.exception_message_box(str(err))

        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(os.path.join(root_path, "resources/16x16/delete.png")),
                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        tb_item.setIcon(icon3)

    @log
    def spinVlItem_editingFinished(self, _return=False):
        self._calcula_total()
        return True

    @log
    def _calcula_total(self, *args):
        if args and args[0] == 0:
            return

        preco_item = self.ui.spinVlItem.value() * self.ui.spinQtda.value()
        if preco_item and self.ui.spinDescItem.value() and self.ui.spinDescItem.value() >= preco_item:
            self.ui.spinDescItem.setValue(0)
            total = preco_item + self.ui.spinAcrescItem.value() - self.ui.spinDescItem.value()
            self.ui.spinVlTotal.setValue(total)
            return message_dlg(self, MSG_ERROR,
                               'O valor do desconto deve ser menor que o preço do item')


        total = preco_item + self.ui.spinAcrescItem.value() - self.ui.spinDescItem.value()
        self.ui.spinVlTotal.setValue(total)
        return total

    @log
    def _button_busca_cliente_click(self, *args):
        busca_cli_form = BuscaCliente(self, 'S')

        if self._pve:
            if self._pve.suppva_cliente:
                return message_dlg(self, MSG_ERROR,
                                   'Já existe um cliente selecionado para a Pré-Venda.')

        if busca_cli_form.exec_():
            self._cli = busca_cli_form.retorno
            # if self._pve:
            #     try:
            #         self._pve.supcli_cliente = self._cli.supcli_cnpj_cpf
            #         self._pve.NomeCliente = self._cli.name
            #     except Exception as e:
            #         self.exception_message_box(str(e))
            #         return
            self.ui.editDocCliente.setText(
                '[%s] %s' % (self._cli.supcli_cnpj_cpf, self._cli.name))

    @log
    def _button_nova_pve_click(self, *args):
        self.ui.buttonNovaPve.setVisible(False)
        self._cli = ''
        self._pve = None
        self._clear_widgets()
        self.ui.spinDesc.setValue(0.00)
        self.ui.frame_esquerdo.setEnabled(True)
        self.ui.buttonBuscarCliente.setEnabled(True)
        self.ui.stackedWidget.setCurrentIndex(0)
        self.ui.dateEmissaoPve.setDateTime(QtCore.QDateTime.currentDateTime())
        self.ui.editCodItem.setFocus()

    @log
    def _button_busca_item_click(self):
        cod_item = self.ui.editCodItem.text()
        item = None
        if cod_item:
            item = SupervisorCadItens.get_by(supite_codigo_item=cod_item.rjust(14, '0'))
            if item:
                self.ui.editCodItem.setText(item.supite_codigo_item)
                self.ui.spinVlItem.setValue(item.supite_preco)
                self._calcula_total()
                self.ui.spinQtda.setFocus()
                self.ui.spinQtda.selectAll()
                self.ui.labelItem.setText(item.name)
            else:
                busca_produto_form = BuscaProduto(self)
                if busca_produto_form.exec_():
                    res = busca_produto_form.retorno
                    if res:
                        self.ui.editCodItem.setText(res.supite_codigo_item)
                        self.ui.spinVlItem.setValue(res.supite_preco)
                        self._calcula_total()
                        self.ui.labelItem.setText(res.name)
                        self.ui.spinQtda.setFocus()
                        self.ui.spinQtda.selectAll()
                else:
                    self.ui.editCodItem.selectAll()
        return item

    @log
    def _button_add_item_click(self):
        item = self._button_busca_item_click()
        if item:
            try:
                self._adicionar_item(item)
            except Exception as err:
                self.exception_message_box(str(err))

    @log
    def _checar_libera_desconto(self):
        libera = self._operador.supope_operador_supervisor or self._operador.supope_desconto_item
        if not libera:
            login_form = LoginForm(db='S')
            if login_form.exec_():
                libera = login_form.retorno.Operador_supervisor == 'S'
        return libera

    @log
    def _adicionar_item(self, item):
        # GET
        if not self._pve:
            self.gerar_capa_pve()
        cod = item.supite_codigo_item
        descr = item.name
        qtda = Decimal(str(self.ui.spinQtda.value())).quantize(TREEPLACES)
        vuni = item.supite_preco
        acresc = self.ui.spinAcrescItem.value()
        desc = Decimal(str(self.ui.spinDescItem.value()))
        print 'qtda', qtda
        if not qtda:
            self.ui.spinQtda.setFocus()
            self.ui.spinQtda.setValue(1)
            return message_dlg(self, MSG_INFO, 'PVE', 'Favor informe a quantidade desejada.')
            # VERIFICAR QUANTIDADE MIN/MAX
        params = SupervisorCfgParametros.get_by()
        if params:
            qtd_min = params.supcfp_qtd_minima if params.supcfp_qtd_minima else 0
            qtd_max = params.supcfp_qtd_maxima if params.supcfp_qtd_maxima else 0
            msg_qtda = ''
            qtd_min = Decimal(str(qtd_min)).quantize(TREEPLACES)
            qtd_max = Decimal(str(qtd_max)).quantize(TREEPLACES)
            if qtda < qtd_min:
                msg_qtda = 'Qtda. digitada menor que a permitida!'
            elif qtda > qtd_max:
                msg_qtda = 'Qtda. digitada maior que a permitida!'
            if msg_qtda:
                self.ui.spinQtda.setFocus()
                self.ui.spinQtda.setValue(1)
                return message_dlg(self, MSG_INFO, 'PVE', msg_qtda)

        if item.supite_fracionado:
            self.ui.spinQtda.setDecimals(3)
        else:
            if not Decimal(self.ui.spinQtda.text().replace(',', '.'))._isinteger():
                msg = '<center>A quantidade <h2><font color="red">{0!s}</font></h2> para o item  ' \
                      '<h3><font color="red">{1!s}</font></h3> não pode ser ' \
                      'fracionada.</center>'.format(self.ui.spinQtda.text(), item.name)
                self.ui.spinQtda.setValue(1)
                message_dlg(self, MSG_INFO, msg)
                return False

        total = Decimal(str(self.ui.spinVlTotal.value()))

        vr_tot = Decimal(vuni * qtda)

        # ADD ON TABLE
        if self.ui.spinAcrescItem.value() and self.ui.spinDescItem.value():
            self.ui.spinAcrescItem.setValue(0)
            self.ui.spinAcrescItem.setFocus()
            self.ui.spinDescItem.setValue(0)
            return MSG_INFO(self, MSG_INFO, 'PVE',
                            'Não é possivel dar Acréscimo e Desconto simultâneos.')

        # VERIFICAR SE EXISTE DESCONTO NOS ITENS AO DAR ACRESCIMO
        isdesc = SupervisorPveCorpo.query.filter(SupervisorPveCorpo.suppvo_desconto_item > 0).filter(
            SupervisorPveCorpo.suppvo_venda_cancelamento == 'V').filter(
            SupervisorPveCorpo.suppvo_capa_id == self._pve.id).all()
        if acresc and isdesc:
            self.ui.spinAcrescItem.setValue(0)
            return message_dlg(self, MSG_INFO, 'PVE',
                               'Já há item(s) com Desconto. Não é possivel efetuar Acréscimo.')
            # VERIFICAR SE EXISTE ACRESCIMOS NOS ITENS AO DAR DESCONTO
        isacresc = SupervisorPveCorpo.query.filter(SupervisorPveCorpo.suppvo_acrescimo_item > 0).filter(
            SupervisorPveCorpo.suppvo_venda_cancelamento == 'V').filter(
            SupervisorPveCorpo.suppvo_capa_id == self._pve.id).limit(1).all()
        if desc and isacresc:
            self.ui.spinDescItem.setValue(0)
            return message_dlg(self, MSG_INFO, 'PVE',
                               'Já há item(s) com Acréscimo. Não é possivel efetuar Desconto.')
        elif desc > vr_tot:  # Desconto Maior que o Valor do Item
            self.ui.spinAcrescItem.setValue(0)
            self.ui.spinDescItem.setFocus()
            self.ui.spinDescItem.setValue(0)
            return message_dlg(self, MSG_INFO, 'PVE',
                               'Desconto não pode ser maior que o valor total.')
        elif desc:
            if not self._checar_libera_desconto():
                self.ui.spinDescItem.setValue(0)
                return MSG_INFO(self, MSG_INFO, 'PVE', 'Desconto não permitido!')
        try:
            total_pve = self._pve.suppva_total_pve or 0
            sess.begin()
            self._pve.suppva_total_pve = total_pve + total
            sess.commit()
            self.ui.spinVlTotal.setValue(total_pve + total)
            sess.begin()
            SupervisorPveCorpo(suppvo_capa_id=self._pve.id,
                      suppvo_sequencia=self.ui.tableItens.rowCount() + 1,
                      suppvo_item=cod,
                      suppvo_quantidade=qtda,
                      suppvo_preco_total=total,
                      suppvo_desconto_item=desc,
                      suppvo_acrescimo_item=acresc,
                      suppvo_venda_cancelamento='V')
            sess.commit()
        except Exception as err:
            self.exception_message_box(str(err))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(os.path.join(root_path, "resources/16x16/check2.png")),
                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        table_item = self.ui.tableItens
        row = table_item.rowCount()
        table_item.insertRow(row)
        # Codigo Item
        item_check = QtGui.QTableWidgetItem('')
        item_check.setIcon(icon3)
        # Codigo Item
        item_cod = QtGui.QTableWidgetItem(cod)
        # Descrição
        item_descr = QtGui.QTableWidgetItem(descr)
        # Quantidade
        item_qtda = QtGui.QTableWidgetItem('%1.3f' % qtda)
        # Valor Unitário
        item_vuni = QtGui.QTableWidgetItem('R$ %1.2f' % vuni)
        item_vuni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        # Acrescimo
        item_acresc = QtGui.QTableWidgetItem('%1.2f' % acresc)
        # Desconto
        item_desc = QtGui.QTableWidgetItem('%1.2f' % desc)
        # Total
        item_total = QtGui.QTableWidgetItem('R$ %1.2f' % total)

        # WIDGET OS NEW ITENS NA TABLE ITENS
        table_item.setItem(row, 0, item_check)
        table_item.setItem(row, 1, item_cod)
        table_item.setItem(row, 2, item_descr)
        table_item.setItem(row, 3, item_qtda)
        table_item.setItem(row, 4, item_vuni)
        table_item.setItem(row, 5, item_acresc)
        table_item.setItem(row, 6, item_desc)
        table_item.setItem(row, 7, item_total)
        table_item.resizeColumnsToContents()
        table_item.scrollToBottom()
        self.ui.editCodItem.clear()
        self.ui.editCodItem.setFocus()
        self.ui.spinVlItem.setValue(0)
        self.ui.spinQtda.setValue(1)
        self.ui.spinDescItem.setValue(0)
        self.ui.spinAcrescItem.setValue(0)
        self.ui.spinVlTotal.setValue(0)
        self.ui.spinTotalPve.setValue(self.ui.spinTotalPve.value() + float(total))
        self.ui.labelItem.setText(u'Informe o Código do Produto')

    @log
    def clear_widget_table(self, table):
        # Apaga lista de itens
        rows_count = table.rowCount()
        # Verifica se hà dados
        if rows_count:
            rows = range(0, rows_count + 1)
            rows.reverse()
            for r in rows:
                table.removeRow(r)
        return rows_count

    @log
    def gerar_capa_pve(self):
        self.ui.editCodItem.setFocus()
        try:
            cli_doc = cli_nome = ''
            if self._cli:
                cli_doc = self._cli.supcli_cnpj_cpf
                cli_nome = self._cli.name

            sess.begin()
            self._pve = SupervisorPveCapa(suppva_cliente=cli_doc, suppva_nome_cliente=cli_nome,
                                 suppva_restricao=0, suppva_operador_id=self._operador.id)
            sess.commit()
            self.ui.editNumPve.setText('%010d' % self._pve.id)
            self.ui.comboStatusPve.setCurrentIndex(self._pve.suppva_restricao)
            self.ui.frame_esquerdo.setEnabled(True)
        except Exception as e:
            self.exception_message_box(str(e))
            sess.rollback()
            return
        return self._pve

    def keyPressEvent(self, ev):
        if ev.key() in (QtCore.Qt.Key_Return.__int__(), QtCore.Qt.Key_Enter.__int__()):
            if self.ui.spinQtda.hasFocus() or \
                    self.ui.spinAcrescItem.hasFocus() or self.ui.spinDescItem.hasFocus():
                self._button_add_item_click()
            elif self.ui.editCodItem.hasFocus():
                self._button_busca_item_click()
            elif self.ui.edtFiltroCodPve.hasFocus() or \
                    self.ui.dateIni.hasFocus() or self.ui.dateFim.hasFocus() or \
                    self.ui.editBuscaPve.hasFocus() or self.ui.comboFiltroStatus.hasFocus():
                self.filter_pve()

@log
def start():
    app = QtGui.QApplication(sys.argv)
    login_form = LoginForm(db='S')
    exec_form = login_form.exec_()
    if exec_form:
        form = PveForm(operador=login_form.retorno)
        form.setWindowTitle(u'SigPDV - Pré-Venda')
        form.showFullScreen()
        app.exec_()
    else:
        sys.exit(-1)

if __name__ == "__main__":
    try:
        start()
        pass
    except Exception as e:
        exception_message_box(str(e))
        sys.exit(-1)
