/*

EXEMPLO.C
 
    ÍNDICE:
  
     1. Tratamento dos erros gerados pela biblioteca
     2. Deteco automtica do ECF e conexo 
     3. Transferncia de dispositivos de memria do ECF
         3.1. Rotina de callback da funo SWEDA_CapturarDispositivo
         3.2. Funo SWEDA_CapturarDispositivo
     4. Gerao do arquivo SINTEGRA  
         4.1. Informaes do estabelecimento
         4.2. Rotina de callback da funo SWEDA_CapturarDispositivo
     5. Reproduo da fita-detalhe para arquivo em codificao ASCII
     6. Impresso da fita-detalhe
     7. Gerao do arquivo COTEPE
     8. Gerao da Tabela de Mercadorias e Servios
     9. Gerao do arquivo SINTEGRA
    10. Reproduo da Leitura da Memria Fiscal para arquivo em codificao ASCII
    11. Impresso da Leitura da Memria Fiscal
    12. Emisso do documento "Leitura X"
    13. Emisso do documento "Leitura de Parmetros do ECF"
    14. Configurao de parmetros da biblioteca	  
        14.1. Leitura da configurao atual
        14.2. Alterao da configurao atual
    15. Rotina auxiliar de exibio de arquivos no console
    16. Modulo de ligao dinmica da biblioteca
         16.1. Especificao do nome do arquivo de ligao dinmica
         16.2. Rotina de carregamento dinmico da biblioteca
         16.3. Rotina de descarregamento dinmico da biblioteca
    17. Menus da aplicao
    18. Funo main()
      18.1. Carregamento dinmico da biblioteca
      18.2. Execuo do menu principal	
      18.3. Descarregamento da biblioteca

*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "swmfd.h"      // Incluso do arquivo header da biblioteca
#include "menus.cpp"    // Rotinas de gerenciamento de menus (console)


// 1. Tratamento de erros

short int Erro (short int Codigo)
{  
// Lista de mensagens referntes ao cdigo de erro retornado por funo da biblioteca
   char *pszMensagemErro[] =
   {  "",
      "Falha de abertura do arquivo log!",
      "Data invlida!",
      "Faixa invlida!",
      "O nmero de arquivos excede a capacidade da fila! (100 arquivos)",
 	  "Nenhum arquivo encontrado! (opo: opArquivosPasta)",
	  "Nenhum arquivo selecionado! (opo: opListaEmArquivo)",
	  "Arquivo no encontrado! (Origem)",
	  "Memria insuficiente!",
	  "Falha em arquivo temporrio!",
	  "No foi possvel abrir o arquivo de sada!",
	  "Arquivo invlido!",
	  "Falha na criao da instncia!",
	  "O ms selecionado no est encerrado!",
	  "Buffer insuficiente",
      "Falha de gravao em arquivo!",
      "Falha de comunicao! (Timeout)",
      "Impressora fiscal est desligada ou no foi detectada!",
      "Tampa da impressora fiscal aberta!",
      "Impressora fiscal sem papel!",
      "Troque a bobina da impressora fiscal!",
      "Erro de sintaxe! Verifique os parmetros da funo.",
      "Comando invlido!",
      "Falha na execuo do comando fiscal!",
      "O aplicativo  incompatvel com o arquivo deste modelo de ECF ou com a verso de seu software bsico!",
   };

   if (Codigo && Codigo < (short int) (sizeof (pszMensagemErro) / sizeof (*pszMensagemErro)))
       printf ("\r\n%s\n\n", pszMensagemErro [Codigo]);

   return Codigo;
}


// 2. Deteco automtica do ECF e conexo 

short int DetectarECF (struct MENU::ITEM *Item)
   
// Detecta automticamente as configuraes da porta de comunicao serial 
// e obtem informaes sobre o ECF conectado.

{  short int Porta;
   long Velocidade;
   struct TPropriedadesDispositivo PropriedadesECF;

   if (SWEDA_DetectarECF(0, &Porta, &Velocidade) == 0 ||
       SWEDA_ConectarECF (0, &PropriedadesECF, sizeof (PropriedadesECF)) == 0)
      printf ("\n\nERRO: ECF no detectado!\n");
   else
      printf ("\n\n"
          "ECF detectado!\r\n\n"
          "   Modelo:     %s\r\n"
          "   FAB:        %s\r\n"
          "   MFD:        %s\r\n"
          "   Verso:     %s\r\n"
          "   Porta:      COM%d\r\n"
          "   Velocidade: %ld\r\n"
          , PropriedadesECF.ECF.Modelo
          , PropriedadesECF.ECF.FAB
          , PropriedadesECF.MFD.NS
          , PropriedadesECF.SB.Versao
          , Porta
          , Velocidade);

   return CONTINUAR;
}


// 3. Transferncia de dispositivos de memria do ECF


// 3.1. Rotina de callback da funo SWEDA_CapturarDispositivo

void CallBackCD (struct TInfoCallBackCD *Info, void *Parametro)

// Atravs da rotina de callback, a funo SWEDA_CapturarDispositivo comunica 
// aplicao eventos ocorridos durante a execuo.

{  switch (Info->Evento)
   {  case 200: printf ("Detectando o ECF...");
                break;
      case 110: printf ("\rVerificando dispositivos instalados...");
                break;
      case 130: printf ("\rArquivo: %-50s\r\n", Info->Arquivo);
                break;
      case 140: printf ("\rTransferindo... %ld de %ld bytes (%ld%%)", Info->Contador.Valor, Info->Contador.Maximo, (Info->Contador.Valor * 100) / Info->Contador.Maximo);
                break;
      case 198: printf ("\r\nProcesso interrompido pela aplicao!\n");
                break;
      case 199: printf ("\r%-60s\n", "Transferncia concluda!");
                break;
   }
}


// 3.2. Funo SWEDA_CapturarDispositivo

short int TransferirDispositivo (struct MENU::ITEM *Item)
{  printf ("\r\nDispositivo: %s\r\n", Item->Titulo);
   Erro (SWEDA_CapturarDispositivo ((short int) Item->Parametro, opAutoDetectarECF, 0, 0, 15000, CallBackCD, Item));
   return ENCERRAR_MENU;
}

/*

4. Gerao do arquivo SINTEGRA

4.1. Informaes do estabelecimento

Uma estrutura do tipo TDadosEstabelecimento  utilizada pela aplicao para 
passar argumentos  funo SWEDA_ArquivoSINTEGRA, que as utiliza para gerao 
dos registros tipo 10 e tipo 11 do arquivo SINTEGRA.

Opcionalmente, estas informaes podem ser configuradas na seo [SINTEGRA] 
do arquivo de inicilizao da biblioteca, denominado SWMFD.INI:

[SINTEGRA]
   CNPJ: ""
   IE: ""
   IM: ""
   RAZO SOCIAL: "SWEDA Informtica Ltda"
   LOGRADOURO: "Rua Dona Br­gida"
   NUMERO: "713"
   COMPLEMENTO: ""
   BAIRRO: "Vila Mariana"
   CIDADE: "So Paulo"
   UF: "SP"
   CEP: "04111-081"
   TELEFONE: "11 5574-5644
   FAX: "5572-4799"
   CONTATO: "SAC"



*/
struct TDadosEstabelecimento Estabelecimento =
{  "",                           // CNPJ
   "",                           // Inscrio Estadual
   "",                           // Inscrio Municipal
   "Sweda Informtica Ltda",     // Nome comercial (Razo Social) do contribuinte
   "Rua Dona Brgida",           // Logradouro
   "713",                        // Nmero
   "",                           // Complemento
   "Vila Mariana",               // Bairro
   "So Paulo",                  // Municpio onde est domiciliado o estabelecimento informante
   "SP",                         // Unidade da Federao referente ao Municpio
   "04111-081",                  // Cdigo de Endereamento Postal
   "11 5574-5644",               // Nmero de telefone para contato
   "5572-4799",                  // Fax
   "SAC"                         // Pessoa responsvel para contato
};


// 4.2. Rotina de callback da funo SWEDA_CapturarDispositivo 

void CallBackSintegra (struct TInfoCallBackSintegra *Info, void *Parametro)

// Atravs da rotina de callback, a funo SWEDA_CapturarDispositivo comunica 
// aplicao eventos ocorridos durante a execuo.

{  switch (Info->Evento)
   {  case 110: printf ("Aguarde...");
             	break;
      case 200: printf ("\rDetectando o ECF...");
                break;
      case 130: printf ("\rTransferindo a memria de fita-detalhe...\r\n");
                break;
      case 140: printf ("\r%ld/%ld bytes", Info->Contador.Valor, Info->Contador.Maximo);
                break;
      case 40:  printf ("\r\n\nCNPJ: %s\r\n", Info->Estabelecimento.CNPJ);

                if (*Info->Estabelecimento.IE)
                   printf ("IE: %s\r\n", Info->Estabelecimento.IE);

                if (*Info->Estabelecimento.IM)
                   printf ("IM: %s\r\n", Info->Estabelecimento.IM);

                if (*Info->Arquivo.FAB)
                   printf ("FAB: %s\r\n", Info->Arquivo.FAB);

                if (*Info->Arquivo.Loja)
                   printf ("Loja: %s\r\n", Info->Arquivo.Loja);

                if (*Info->Arquivo.ECF)
                   printf ("ECF: %s\r\n", Info->Arquivo.ECF);

                printf ("\r\n\n""Processando informaes da memria de fita-detalhe...\r\n");
                break;

      case 41:  printf ("\rCOO: %06ld/%06ld", Info->Contador.Valor, Info->Contador.Maximo);
                break;

      case 80:  printf ("\rConsolidando regitros...\r\n");
                break;

      case 81:  printf ("\rRegistros: %ld/%ld", Info->Contador.Valor, Info->Contador.Maximo);
                break;
      case 98:  printf ("\rProcesso interrompido pela aplicao!\n\n");
                break;
      case 99:  printf ("\rArquivo gerado com sucesso!\n\n");
                break;
   }
}


// Funo auxiliar para exibio de arquivos no console
void ExibirArquivo (char*);


short int SelecionarFaixa (struct MENU::ITEM *Item)
{  char Selecao[2][200];
   short int Opcao;
   const char *pszCriterio;

   Opcao = opTDM | opCodePage850;

   pszCriterio = (Item->Parametro & 32) ? "" : (Item->Parametro & 16 ? "o CRZ ou " : "o COO ou ");

   printf ("%s\r\n\n", Item->Titulo);
   printf ("Digite %sa data inicial: ", pszCriterio);
   scanf ("%20s", Selecao[0]);
   printf ("Digite %sa data final (opcional): ", pszCriterio);
   scanf ("%20s", Selecao[1]);
   
   switch (Item->Parametro)
   {
// 5. Reproduo da fita-detalhe para arquivo em codificao ASCII
      case 1:  if (Erro (SWEDA_ReproduzirFitaDetalhe ("", "temp.txt", Selecao[0], Selecao[1], opUTF_8, NULL, 0)))
                  return CONTINUAR;
               break;

// 6. Impresso da fita-detalhe
      case 2:  Erro (SWEDA_ImprimirFitaDetalhe (Selecao[0], Selecao[1]));
               return CONTINUAR;

// 7. Gerao do arquivo COTEPE
      case 20:  Opcao |= opCSV;
      case 19:  if (Erro (SWEDA_ArquivoEletronicoCOTEPE ("", "", "temp.txt", Selecao[0], Selecao[1], Opcao)))
                  return CONTINUAR;
               break;

// 8. Gerao da Tabela de Mercadorias e Servios
      case 33: if (Erro (SWEDA_TabelaMercadoriasServicos ("temp.txt", Selecao[0], Selecao[1], opCodePage850 | opOcultarJanela, "", "")))
                  return CONTINUAR;
               break;

// 9. Gerao do arquivo SINTEGRA
      case 34: if (Erro (SWEDA_ArquivoSINTEGRA ("temp.txt", Selecao[0], Selecao[1], &Estabelecimento,
		                  opSintegraTipo10 | opSintegraTipo11 | opSintegraTipo60M | opSintegraTipo60A | opSintegraTipo60D |
                  opSintegraTipo60I | opSintegraTipo60R | opSintegraTipo75 | opSintegraTipo90, opCodePage850 | opOcultarJanela, "", "LOG.TXT", CallBackSintegra, NULL, 0)))
               {  ExibirArquivo ("LOG.TXT");
				  return CONTINUAR;
			   }
               break;

// 10. Reproduo da Leitura da Memria Fiscal para arquivo em codificao ASCII
      case 17: if (Erro (SWEDA_ReproduzirLeituraMemoriaFiscal ("", "temp.txt", Selecao[0], Selecao[1], opLeituraCompleta, opUTF_8, NULL, 0)))
                  return CONTINUAR;
               break;

// 11. Impresso da Leitura da Memria Fiscal
      case 18: Erro (SWEDA_EmitirLeituraMemoriaFiscal (Selecao[0], Selecao[1], opLeituraCompleta));
               return ENCERRAR_MENU;
   }

   ExibirArquivo ("temp.txt");
   return CONTINUAR;
}

// 12. Emisso do documento "Leitura X"

short int EmitirLX (struct MENU::ITEM *Item)
{  Erro (SWEDA_EmitirLeituraX());
   return CONTINUAR;
}


// 13. Emisso do documento "Leitura de Parmetros do ECF"

short int EmitirLP (struct MENU::ITEM *Item)
{  Erro (SWEDA_EmitirLeituraParametros());
   return CONTINUAR;
}



// 14. Configurao de parmetros da biblioteca	  

// Parmetros de configurao da biblioteca
struct SWMFD_INI Parametros;


extern struct MENU::ITEM MenuConfiguracao[];

short int OpcaoConfigurar (struct MENU::ITEM *Item)
{  
// 14.1. Leitura da configurao atual
   SWMFD_ObterParametrosConfiguracao(&Parametros, sizeof (Parametros));

   MENU::Executar (MenuConfiguracao, Item->Titulo);
   Parametros.Estabelecimento = Estabelecimento;
	
// 14.2. Alterao da configurao atual
   SWMFD_AlterarParametrosConfiguracao(&Parametros, sizeof (Parametros));
	
   return CONTINUAR;
}


short int ConfigurarDiretorio (struct MENU::ITEM *Item)
{  return ENCERRAR_MENU;
}


short int ConfigurarPorta (struct MENU::ITEM *Item)
{  Parametros.Porta = (short int) Item->Parametro;
   printf ("\r\n\nOpo configurada: %s\n", Item->Titulo);
   return ENCERRAR_MENU;
}


short int ConfigurarVelocidadeSerial (struct MENU::ITEM *Item)
{  Parametros.Velocidade = Item->Parametro;
   printf ("\r\n\nOpo configurada: %s\n", Item->Titulo);
   return ENCERRAR_MENU;
}


// 15. Rotina auxiliar de exibio de arquivos no console

void ExibirArquivo (char *Arquivo)
{  FILE *handle;
   short int Character, Linhas, Colunas;
   char szLinha[200];

   Colunas = Linhas = 0;
   handle = fopen (Arquivo, "rb");
   
   if (handle == 0)
	  return;
   
   printf ("\n");
            
   while (1)
   {  Character = fgetc (handle);
      if (Character == EOF)
      {  printf ("\n%-36s* Fim *\r\n", Arquivo);
         getchar();
         break;
      }

      if ((unsigned int) Colunas < (sizeof (szLinha) - 1))
         szLinha[Colunas++] = Character;

      if (Character == '\n')
      {  szLinha[Colunas] = '\0';
         printf ("%s", szLinha);
         Colunas = 0;
         if ((++Linhas % 23) == 0)
         {  printf ("%-30s> >  P A U S A  < <", Arquivo);
            if (getchar() == 0x1B)
               break;
            printf ("\r%60s\r", "");
         }
      }
   }
   fclose (handle);
}


// 16. Modulo de ligao dinmica da biblioteca

#include "swmfd_dl.c"


// 17. Menus da aplicao


struct MENU::ITEM MenuSelecaoPorta [] =
{  {  "COM1", ConfigurarPorta, 1  },
   {  "COM2", ConfigurarPorta, 2  },
   {  "COM3", ConfigurarPorta, 3  },
   {  "COM4", ConfigurarPorta, 4  },
   {  "COM5", ConfigurarPorta, 5  },
   {  "COM6", ConfigurarPorta, 6  },
   {  "Detectar", ConfigurarPorta, 0  },
   {  "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuSelecaoVelocidade [] =
{  {  "9.600 bps", ConfigurarVelocidadeSerial, 9600  },
   {  "19.200 bps", ConfigurarVelocidadeSerial, 19200  },
   {  "38.400 bps", ConfigurarVelocidadeSerial, 38400  },
   {  "57.600 bps", ConfigurarVelocidadeSerial, 57600  },
   {  "115.200 bps", ConfigurarVelocidadeSerial, 115200  },
   {  "Detectar", ConfigurarVelocidadeSerial, 0  },
   {  "Retornar"  },
   {  0  },
};

struct MENU::ITEM MenuComunicacao [] =
{  {  "Porta", MENU::Submenu, (PARAMETRO) MenuSelecaoPorta  },
   {  "Velocidade", MENU::Submenu, (PARAMETRO) MenuSelecaoVelocidade  },
   {  "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuConfiguracao [] =
{  {  "Comunicao", MENU::Submenu, (PARAMETRO) MenuComunicacao   },
   {  "Diretorio de gravao", ConfigurarDiretorio  },
   {  "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuTransferencia [] =
{  {  "Memria Fiscal", TransferirDispositivo, opCapturarMF  },
   {  "Memria de Fita-Detalhe", TransferirDispositivo, opCapturarMFD },
   {  "Software Bsico", TransferirDispositivo, opCapturarSoftwareBasico  },
   {  "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuArquivosEletronicos [] =
{  {  "Reproduo da Fita-Detalhe", SelecionarFaixa, 1  },
   {  "Reproduo da Leitura de Memria Fiscal", SelecionarFaixa, 17  },
   {  "Arquivo COTEPE", SelecionarFaixa, 19  },
   {  "Planlia Eletrnica (.CSV)", SelecionarFaixa, 20  },
   {  "Arquivo SINTEGRA", SelecionarFaixa, 34  },
   {  "Tabela de Mercadorias e Servicos", SelecionarFaixa, 33  },
   {  "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuEmissao [] =
{  {  "Leitura X", EmitirLX  },
   {  "Leitura da Memria Fiscal", SelecionarFaixa, 18  },
   {  "Leitura de Parmetros do ECF", EmitirLP  },
   {  "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuPrincipal [] =
{  {  "Transferncia de dispositivos", MENU::Submenu, (PARAMETRO) MenuTransferencia  },
   {  "Gerao de arquivos eletrnicos", MENU::Submenu, (PARAMETRO) MenuArquivosEletronicos  },
   {  "Emisso de documentos", MENU::Submenu, (PARAMETRO) MenuEmissao  },
   {  "Impresso da fita-detalhe", SelecionarFaixa, 2  },
   {  "Deteco automtica do ECF", DetectarECF  },
   {  "Configurao", OpcaoConfigurar  },
   {  "Sair"  },
   {  0  },
};


// 18. Funo main()

int main()
{  
// 18.1. Carregamento dinmico da biblioteca...
   SWMFD_LoadLibrary();
	
   printf ("\n\nSWMFD Verso: %d.%02d", SWEDA_SWMFDVersao() / 100, SWEDA_SWMFDVersao() % 100);

// 18.2. Execuo do menu principal	
   MENU::Executar (MenuPrincipal, "MENU PRINCIPAL");
	
// 18.3. Descarregamento da biblioteca...
   SWMFD_FreeLibrary();
	
   return 0;
}
