#!/usr/bin/python
# -*- encoding: utf-8 -*-

__description__ = 'O exportalog.py é responsável por verificar o log da Exportação de cupom e ' \
                  'exportar os cupons que não foram exportados para o banco de dados do Supervisor'

__author__ = 'Dener Rodrigues'

import os
import getpass
import psycopg2
import psycopg2.extras


class Connection(object):
    def __init__(self):
        self.database = raw_input('Nome da base: ')
        self.host = raw_input('Host: ') or 'localhost'
        self.user = raw_input('Usuario: ') or 'openerp'
        self.password = getpass.getpass('Senha: ')
        self.port = raw_input('Porta: ') or 5432

    def get_connection(self):
        """
        Realiza uma nova conexão no banco de dados.
        :rtype: psycopg2._psycopg.connection
        """

        try:
            connection = psycopg2.connect(database=self.database, host=self.host, user=self.user,
                                          password=self.password, port=self.port)
            return connection
        except Exception as err:
            raise("Erro ao conectar no banco de dados ... \n\n%s" % str(err))


class DButils(object):

    @staticmethod
    def dict2sql(table, data):
        """
        Converte um dicionario para uma sql.

        :param table: nome da tabela
        :param data: dados à serem manipulados

        :type table: str
        :type data: dict

        :return: retorna uma string com a sql pronta à ser executada

        :rtype: str
        """

        try:
            data_keys = [tuple('%s' % k for k in data.keys())]
            lista = []

            for data_value in data.values():
                lista.append('%s' % data_value)

            data_value = tuple(lista)

            chave = str(data_keys).replace("u'", "'").replace("'", "").replace(')]', ')')
            chave = chave.replace('[(', '(')

            valor = str(data_value).replace("u'", "'").replace(')]', ')').replace('[(', '(')

            valor = valor.replace("Decimal('", "'").replace("'),", "',").replace("'None'", 'null')

            valor = valor.replace("False", "F").replace("True", "T").replace('"(', '(')

            valor = valor.replace(')"', ')').replace('"', '').replace("'null'", 'null')
        except Exception as e:
            print str(e)

        try:
            sql = "INSERT INTO %s %s VALUES %s RETURNING id;" % (table, chave, valor)
        except Exception as err:
            print sql
            print str(err)

        return sql

    @staticmethod
    def sql2dict(sql):
        """
        Converte uma sql para um dicionário.

        :param sql: Comando sql.

        :type sql: str

        :return: Retorna um dicionário com as colunas seus repectivos valores.

        :rtype: dict
        """

        sql = sql.replace("('", "'").replace("')", "'") # evita que quebre errado a string pois o campo datetime
                                                        # vem com valor: ex: ('2016-02-10 17:20:00' at time zone 'UTC')
        sql = sql.replace('(', '|').replace(')', '|')

        fields = tuple(sql.split('|')[1].replace(" ", "").split(','))

        values = sql.split('|')[2].replace('SELECT', '').replace('WHERE NOT EXISTS', '')

        values = values.replace(" '", "'").replace("',", "'|").replace(',', '.').replace("'|", "',")

        values = values.replace(' null', 'null').replace('null.', 'null,').strip().split(',')

        values = tuple(filter(lambda i: i not in [' ', ',', "'"], values))

        dict_sql = dict(zip(fields, values))

        for x in dict_sql:
            dict_sql[x] = "'%s'" % dict_sql[x].replace("'", "")

        return dict_sql


class ExportacaoLog(object):
    """
    Classe responsável por verificar o log da Exportação de cupom e exportar os cupons que não foram
    exportados para o banco de dados do Supervisor.
    """

    def __init__(self, cursor, log='exportar'):
        """
        Método __init__ da classe ExportacaoLog.

        :param cursor: cursor do banco de dados.
        :param log: nome do log que será verificado, podendo ser informado o nome completo do log
        (exportar.21-08-2015.log) ou somente as iniciais (exportar).

        :type cursor: psycopg2._psycopg.connection.cursor
        :type log: str
        """
        self.cursor = cursor
        self.log = log

        self.qtd_cupons = 0

        self.capa_id = None
        self.fechamento_id = None

        self.diretorio_raiz = os.path.dirname(os.path.realpath(__file__))

        self.verifica_logs(self.diretorio_raiz)

        print '%s cupons exportados!' % self.qtd_cupons

    def verifica_logs(self, arq_dir):
        """
        Método recursivo que verifica todos os arquivos/diretóros que estão dentro do diretório
        raiz. Para cada arquivo, é verificado se há falha no 'insert' dos cupons no banco de dados
        do Supervisor, então os mesmos são exportados.

        :param arq_dir: arquivo/diretório à ser verificado
        """

        if os.path.isfile(arq_dir) and self.log in arq_dir:
            arquivo_log = open(arq_dir, 'r')

            line_log = arquivo_log.readlines()

            for line in line_log:
                if 'id: ' in line and 'supervisor_cad_pdvs' not in line:
                    index = line_log.index(line)
                    sql = 'INSERT INTO %s' % str(line_log[:index]).split('INSERT INTO')[-1][:-5]
                    sql = sql.replace('"', '').replace("\\n", "").replace(" ' ", "")
                    sql = sql.replace(')),', '))').replace("',  ", "").replace('     ', ' ')
                    sql = sql.replace('SELECT 1,', 'SELECT 1')
                    sql = sql.replace("RETURNING id;',","RETURNING id;")

                    self.exportar(sql)

            arquivo_log.close()

            self.cursor.connection.commit()  # Grava todas as interações no banco de dados

        elif os.path.isdir(arq_dir):
            for arq in os.listdir(arq_dir):
                self.verifica_logs(os.path.join(arq_dir, arq))

    def exportar(self, sql):
        """
        Exporta os registros para o banco de dados do Supervisor.

        :param sql: Comando sql.

        :type sql: str

        :return: Retorna o id do registro inserido no banco de dados.
        """

        res_id = None

        dict_sql = DButils.sql2dict(sql)

        if 'supervisor_mov_capa' in sql and 'supcap_nro_cupom' in dict_sql:
            print 'Exportando cupom %s' % dict_sql['supcap_nro_cupom']

        if self.capa_id:
            if 'supervisor_mov_corpos' in sql:
                dict_sql['supcor_capa_id'] = self.capa_id
                sql = DButils.dict2sql('supervisor_mov_corpos', dict_sql)

            elif 'supervisor_mov_corpo_xz' in sql:
                dict_sql['supcxz_capa_id'] = self.capa_id
                sql = DButils.dict2sql('supervisor_mov_corpo_xz', dict_sql)

            elif 'supervisor_mov_totalizadores_xz' in sql:
                dict_sql['suptxz_capa_id'] = self.capa_id
                sql = DButils.dict2sql('supervisor_mov_totalizadores_xz', dict_sql)

            elif 'supervisor_mov_fechamentos' in sql:
                dict_sql['supfec_capa_id'] = self.capa_id
                sql = DButils.dict2sql('supervisor_mov_fechamentos', dict_sql)

            elif 'supervisor_mov_vencimentos' in sql and self.fechamento_id:
                dict_sql['supfve_fechamentos_id'] = self.fechamento_id
                sql = DButils.dict2sql('supervisor_mov_vencimentos', dict_sql)

            elif 'supervisor_mov_cupom_servidor' in sql:
                dict_sql['supcse_capa_id'] = self.capa_id
                sql = DButils.dict2sql('supervisor_mov_cupom_servidor', dict_sql)

                # Exporta a tabela cupom_servidor e seta capa_id e fechamento_id para None
                self.exec_sql(sql)
                self.capa_id = None
                self.fechamento_id = None

        if 'supervisor_mov_capa' in sql or 'supervisor_mov_capa' not in sql and self.capa_id:
            res_id = False
            if 'supervisor_mov_capa' in sql:
                select_sql = '''SELECT id
                                FROM supervisor_mov_capa
                                WHERE supcap_nro_cupom = {coo}
                                  AND supcap_caixa = {caixa}
                                  AND supcap_loja = {loja}
                                  AND supcap_data_cupom = {data}
                             '''.format(coo=dict_sql.get('supcap_nro_cupom').replace("'",""),
                                        caixa='%s' % dict_sql.get('supcap_caixa').replace("'",""),
                                        loja='%s' % dict_sql.get('supcap_loja').replace("'",""),
                                        data=dict_sql.get('supcap_data_cupom'))
                res_id = self.exec_sql(select_sql)

            if not res_id: # somete vai inserir o cupom se o mesmo nao estiver no banco de dados
                res_id = self.exec_sql(sql)

        if 'supervisor_mov_capa' in sql:
            self.capa_id = res_id
            if self.capa_id:
                self.qtd_cupons += 1

        elif 'supervisor_mov_fechamentos' in sql:
            self.fechamento_id = res_id

        return res_id

    def exec_sql(self, sql):
        """
        Executa uma sql.

        :param sql: Comando sql.

        :return: Retorna o id do registro inserido no banco de dados.
        """

        try:
            self.cursor.execute(sql)
            res = self.cursor.fetchall()[0][0]
        except Exception, e:
            # print sql
            print str(e)
            res = False
        # finally:
            self.cursor.connection.commit()

        return res


if __name__ == '__main__':
    print
    print __description__
    print __author__
    print

    obj_exporta_log = ExportacaoLog(Connection().get_connection().cursor())

