#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.models.db_utils import a_metadata, a_session
from datetime import datetime
from sqlalchemy.sql.expression import text
from sqlalchemy import UniqueConstraint
from elixir.options import using_options, using_table_options
from elixir import VARCHAR, CHAR, SmallInteger, DateTime, Enum, Integer, String, DECIMAL, \
    BigInteger, Entity, Field


class CaixaCadClientes(Entity):
    """
    Tabela de Cadastro de Clientes
    Prefixo: supcli
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_cad_clientes')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supcli_cnpj_cpf = Field(String(15), primary_key=True, autoincrement=False, index=True)
    supcli_tipo_pessoa = Field(Enum('F', 'J', name="tipo_pessoa", default='F'))
    name = Field(String(40))
    supcli_endereco = Field(String(40))
    supcli_status = Field(CHAR(1))
    supcli_limite_compras = Field(DECIMAL) # o erp cria no banco sem precisao de digitos,
    supcli_total_em_aberto = Field(DECIMAL(10, 2), server_default=text('0.00'))
    supcli_codigo_interno = Field(BigInteger(15))
    supcli_cod_municipio = Field(Integer(7))
    supcli_insc_estadual = Field(String(15))
    supcli_bairro = Field(String(20))
    supcli_cidade = Field(String(30))
    supcli_estado = Field(CHAR(2))
    supcli_endereco_numero = Field(VARCHAR(15))
    supcli_cep = Field(String(8))
    supcli_telefone = Field(String(14))
    supcli_rota = Field(SmallInteger(5))
    supcli_email = Field(String(100))
    last_update = Field(DateTime, default=datetime.now())

    # _sql_constraints
    using_table_options(UniqueConstraint('supcli_cnpj_cpf'))

    def __repr__(self):
        _repr = "<caixa_cad_clientes: %s, 'Nome=%s', 'Status=%s'>"
        vals = self.supcli_cnpj_cpf, self.name, self.supcli_status
        return _repr % vals