#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.models.db_utils import a_metadata, a_session
from datetime import datetime
from sqlalchemy.sql.expression import text
from sqlalchemy.schema import ForeignKey
from elixir.options import using_options
from elixir import Boolean, CHAR, SmallInteger, DateTime, Enum, Integer, String, DECIMAL, \
    Entity, Field


class CaixaCadFinalizadora(Entity):
    """
    Tabela de cadastro de finalizadoras
    Prefixo: supfin
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_cad_finalizadora')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supfin_codigo = Field(SmallInteger(6), nullable=True, unique=True, index=True)
    name = Field(String(100))
    supfin_permite_troco = Field(Boolean, server_default=text('False'))
    supfin_troco_maximo = Field(DECIMAL(10, 2), server_default=text('0'))
    supfin_tipo_finalizadora = Field(CHAR(1))
    supfin_codigo_na_ecf = Field(Integer(10), ForeignKey('caixa_cad_finalizadora_na_ecf.id',
                                                         onupdate="cascade",
                                                         ondelete="cascade"))
    supfin_avista_aprazo = Field(Enum('V', 'P', name='avista_aprazo'), default='P')
    supfin_usa_tef = Field(Boolean, server_default=text('False'))
    supfin_cartao_tipo = Field(CHAR(1), server_default=text("'C'"))
    supfin_cartao_parcelamento = Field(CHAR(1))
    supfin_tipo_midia = Field(CHAR(1))
    supfin_outros_qtd_parcelas = Field(SmallInteger(6))
    supfin_outros_taxa_juros = Field(DECIMAL(5, 2))
    supfin_outros_prazo_primeira = Field(SmallInteger(6), server_default=text('30'))
    supfin_outros_intervalo_parcelas = Field(SmallInteger(6), server_default=text('31'))
    supfin_outros_periodo = Field(CHAR(1))
    supfin_outros_semanal_dia = Field(String(3))
    supfin_outros_quinzenal_vencimento = Field(SmallInteger(6))
    supfin_outros_quinzenal_tipo = Field(CHAR(1))
    supfin_outros_mensal_vencimento = Field(SmallInteger(6))
    supfin_outros_mensal_tipo = Field(CHAR(1))
    supfin_outros_mensal_fecha_mes = Field(SmallInteger(6))
    supfin_conta_caixa_credito = Field(String(15))
    supfin_conta_caixa_debito = Field(String(15))
    supfin_conta_a_prazo = Field(String(15))
    supfin_conta_taxa_administracao = Field(String(15))
    supfin_cliente = Field(Integer(11))
    supfin_permite_desconto_a_vista = Field(CHAR(1))
    supfin_aceita_contra_vale = Field(Boolean)
    supfin_contra_vale = Field(Boolean)
    supfin_imp_promissoria = Field(Boolean)
    supfin_conta_descontos = Field(String(15))
    supfin_conta_acrescimos = Field(String(15))
    supfin_conta_troco = Field(String(15))
    supfin_convenio = Field(CHAR(1))
    supfin_somente_dav = Field(CHAR(1))
    supfin_mod_pgto_grupo = Field(String(2))
    supfin_mod_pgto_sub_grupo = Field(String(2))
    supfin_bandeira_cre = Field(Integer(11))
    supfin_bandeira_deb = Field(Integer(11))
    last_update = Field(DateTime, default=datetime.now())

    def __repr__(self):
        _repr = "<caixa_cad_finalizadora: %d, 'Descricao=[%s] %s', 'Codigo_na_ecf=%s'>"
        vals = self.supfin_codigo, self.supfin_tipo_finalizadora, self.name, self.supfin_codigo_na_ecf
        return _repr % vals