#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import datetime
from sigpdv.models.db_utils import a_metadata, a_session
from elixir import Entity, Field
from elixir.options import using_options, using_table_options
from sqlalchemy.sql.expression import text
from sqlalchemy import Boolean, SmallInteger, DateTime, Enum, Integer, String, Float, Date, \
    UniqueConstraint, ForeignKey, DECIMAL, CHAR


class CaixaCadItens(Entity):
    """
    Tabela de Cadastro de itens
    Prefixo: supite
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_cad_itens')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supite_loja = Field(SmallInteger(6))
    supite_codigo_item = Field(String(14), index=True)
    active = Field(Boolean, server_default=text('True'))
    name = Field(String(40))
    supite_tributacao_id = Field(Integer(11),
                                 ForeignKey('caixa_cad_tributacao.id', onupdate="cascade"))
    supite_aliquota = Field(DECIMAL(5, 2))
    supite_pesavel = Field(Boolean)
    supite_fracionado = Field(Boolean)
    supite_data_atualizacao = Field(Date)
    supite_preco = Field(DECIMAL(10, 2))
    supite_unidade = Field(String(3))
    supite_peso_da_embalagem = Field(Float)
    supite_codigo_retaguarda = Field(CHAR(20))
    supite_fator_multiplicador = Field(Float)
    supite_producao = Field(Enum('P', 'T', name='producao'))
    supite_iat = Field(Enum('A', 'T', name='iat'))
    supite_ncm = Field(CHAR(8))
    supite_cest = Field(CHAR(7))
    supite_aliquota_ibpt_fed = Field(DECIMAL(4, 2), server_default=text('0.00'))     # aqliquota federal
    supite_aliquota_ibpt_est = Field(DECIMAL(4, 2), server_default=text('0.00'))     # aqliquota estadual
    supite_aliquota_ibpt_mun = Field(DECIMAL(4, 2), server_default=text('0.00'))     # aqliquota municipal
    supite_ibpt_chave = Field(CHAR(10))                                              # chave da tablera ibpt
    supite_descricao_completa = Field(String(60))
    supite_peso_tara = Field(Float)
    scanntech = Field(Integer(11))
    last_update = Field(DateTime, default=datetime.now())

    # _sql_constraints
    using_table_options(UniqueConstraint('supite_codigo_item', 'supite_loja'))

    def __repr__(self):
        _repr = "<caixa_cad_itens: %s, 'Loja=%d', 'Descricao=%s', 'Valor=R$ %.2f'>"
        vals = self.supite_codigo_item, self.supite_loja, self.name, self.supite_preco
        return _repr % vals