#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import datetime
from elixir import Entity, Field
from elixir.options import using_options, using_table_options
from elixir.ext.perform_ddl import preload_data
from sqlalchemy.sql.expression import text
from sqlalchemy import Boolean, VARCHAR, DateTime, Enum, Integer, String, UniqueConstraint, Text
from sigpdv.models.db_utils import a_metadata, a_session


class CaixaCadOperadores(Entity):
    """
    Tabela de Cadastro de Operadores
    Prefixo: supope
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_cad_operadores')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supope_codigo = Field(Integer(11), nullable=True, index=True)
    name = Field(VARCHAR(30))
    active = Field(Boolean, server_default=text('True'))
    supope_senha = Field(String(15))
    supope_operador_supervisor = Field(Enum('S', 'O', name='operador_supervisor'))
    supope_cancela_item = Field(Boolean)
    supope_desconto_item = Field(Boolean)
    supope_libera_restricao = Field(Boolean)
    supope_biometria = Field(Text)
    last_update = Field(DateTime, default=datetime.now())

    # _sql_constraints
    using_table_options(UniqueConstraint('supope_codigo'))

    # ------------------------- defaults ------------------------- #
    fields = ('supope_codigo', 'name', 'supope_senha',
              'supope_operador_supervisor', 'supope_cancela_item',
              'supope_desconto_item', 'supope_libera_restricao')

    vals = [('1', u'SUPERVISOR', '123', 'S', 'T', 'T', 'T')]

    preload_data(fields, vals)
    # ------------------------------------------------------------ #

    def __repr__(self):
        _repr = "<caixa_cad_operadores: %d, 'Nome=%s'>"
        vals = self.supope_codigo, self.name
        return _repr % vals