#!/usr/bin/python
# -*- encoding: utf-8 -*-


from sigpdv.models.db_utils import a_metadata, a_session
from elixir import Entity, Field, using_table_options
from elixir.options import using_options
from sqlalchemy.sql.expression import text
from sqlalchemy import CHAR, SmallInteger, DateTime, Enum, Integer, String, \
    Date, DECIMAL, BOOLEAN, ForeignKey, UniqueConstraint


class CaixaMovCapa(Entity):
    """
    Tabela de Capa de cupons
    Prefixo: supcap
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_mov_capa')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supcap_data_cupom = Field(Date(), index=True)
    supcap_loja = Field(SmallInteger(6))
    supcap_caixa = Field(SmallInteger(6))
    supcap_nro_cupom = Field(Integer(11), index=True)
    supcap_numero_pedido = Field(String(20))
    supcap_operador_id = Field(Integer(11), ForeignKey('caixa_cad_operadores.id',
                                                       ondelete="restrict"), index=True)
    supcap_cliente = Field(String(15), server_default=text("''")) # cpf/cnpj
    supcap_data_inicio = Field(DateTime())
    supcap_data_fim = Field(DateTime(), index=True)
    supcap_tipo = Field(CHAR(1))
    supcap_alteracao = Field(Enum('A', 'D', '', name='alteracao', ))
    supcap_valor_desc_acres = Field(DECIMAL(10, 2), server_default=text('0')) # desconto ou acrescimo dado na capa
    supcap_total_cupom = Field(DECIMAL(10, 2), server_default=text('0'))
    supcap_combustivel = Field(String(1), server_default=text("''"))
    supcap_nro_serie = Field(String(20))
    supcap_mf_adicional = Field(BOOLEAN)
    supcap_marca_ecf = Field(String(20))
    supcap_modelo_ecf = Field(String(20))
    supcap_nro_proprietario = Field(Integer(11))
    supcap_ccf = Field(Integer(11))
    supcap_gnf = Field(Integer(11))
    supcap_crz = Field(Integer(11))
    supcap_grg = Field(Integer(11))
    supcap_cdc = Field(Integer(11))
    supcap_tipo_comprovante = Field(Enum('CM', 'RV', 'CC', 'CN', 'NC', 'RG',
                                         name='tipo_comprovante'))
    supcap_subtotal = Field(DECIMAL(10, 2))
    supcap_vr_imposto_fed = Field(DECIMAL(10, 2), server_default=text('0.00'))
    supcap_vr_imposto_est = Field(DECIMAL(10, 2), server_default=text('0.00'))
    supcap_vr_imposto_mun = Field(DECIMAL(10, 2), server_default=text('0.00'))
    supcap_desconto_subtotal = Field(DECIMAL(10, 2))
    supcap_indicador_desconto = Field(Enum('', 'V', 'P', name='ind_desc'))
    supcap_acrescimo_subtotal = Field(DECIMAL(10, 2))
    supcap_indicador_acrescimo = Field(Enum('', 'V', 'P', name='ind_acres'))
    supcap_indicador_cancelamento = Field(Enum('N', 'S', name='ind_cancel'))
    supcap_cancelamento_acrescimo = Field(DECIMAL(10, 2))
    supcap_cancelado_posteriormente = Field(BOOLEAN)
    supcap_nome_cliente = Field(String(40))
    supcap_nro_cupom_vinculado = Field(Integer(11))
    supcap_cod_vendedor = Field(Integer(11))
    supcap_cod_tot_nao_fiscal = Field(Integer(2))
    scanntech = Field(Integer(11))

    # _sql_constraints
    using_table_options(UniqueConstraint('supcap_data_cupom', 'supcap_loja',
                                         'supcap_caixa', 'supcap_nro_cupom'))

    def __repr__(self):
        _repr = "<caixa_mov_capa: 'NroCupom=%d', 'DataCupom=%s', 'Loja=%d', 'Caixa=%d>"
        vals = self.supcap_nro_cupom, self.supcap_data_cupom, self.supcap_loja, self.supcap_caixa
        return _repr % vals