#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.models.db_utils import a_metadata, a_session
from elixir import Entity, Field
from elixir.options import using_options
from sqlalchemy import Enum, SmallInteger, Integer, CHAR, VARCHAR, DECIMAL, ForeignKey
from sigpdv.models.db_utils import modulo_instalado_fichas


class CaixaMovCorpos(Entity):
    """
    Tabela de itens dos cupons
    Prefixo: supcor
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_mov_corpos')

    id = Field(Integer(11), primary_key=True, autoincrement=True, index=True)
    supcor_capa_id = Field(Integer(11),
                           ForeignKey('caixa_mov_capa.id', onupdate="cascade", ondelete="cascade"),
                           index=True)
    supcor_kit_vendido_id = Field(Integer(11), ForeignKey('caixa_mov_kits_vendidos.id',
                                                          ondelete="cascade"), index=True)
    supcor_sequencia = Field(Integer(11), index=True)
    supcor_nro_serie = Field(VARCHAR(20))
    supcor_item = Field(VARCHAR(14), index=True)
    supcor_nro_serie_item = Field(VARCHAR(30))
    supcor_quantidade = Field(DECIMAL(10, 3))
    supcor_preco_total = Field(DECIMAL(10, 3)) # preco bruto do item sem desconto ou acrescimo
    supcor_vr_imposto_fed = Field(DECIMAL(10, 2))
    supcor_vr_imposto_est = Field(DECIMAL(10, 2))
    supcor_vr_imposto_mun = Field(DECIMAL(10, 2))
    supcor_ibpt_chave = Field(CHAR(10))
    supcor_desconto_item = Field(DECIMAL(10, 3), index=True) # desconto do item mais desconto rateado da capa
    supcor_venda_cancelamento = Field(Enum('V', 'C', name='venda_cancel'), index=True)
    supcor_acrescimo_item = Field(DECIMAL(10, 3), index=True)
    supcor_desconto_item_sem_rat = Field(DECIMAL(10, 3)) # sem desconto da capa reateado(apenas sacnntech)
    supcor_acrescimo_item_sem_rat = Field(DECIMAL(10, 3)) # sem acrescimo da capa reateado(apenas sacnntech)
    supcor_tributacao_ecf = Field(VARCHAR(3))
    supcor_tributacao_retag = Field(SmallInteger(6))
    supcor_aliquota = Field(DECIMAL(5, 2))
    supcor_base = Field(DECIMAL(5, 2))
    supcor_nome_item = Field(VARCHAR(60))
    supcor_unidade = Field(VARCHAR(3))
    supcor_preco_unitario = Field(DECIMAL(10, 3))
    supcor_totalizador_parcial = Field(VARCHAR(7))
    supcor_tipo_tributacao = Field(Enum('T', 'F', 'I', 'N', 'O', name='tipo_tributacao'))
    supcor_indicador_arr_trunc = Field(Enum('T', 'A', name='arr_trunc'))
    supcor_indicador_ppt = Field(Enum('P', 'T', name='ppt'))
    supcor_ncm = Field(CHAR(8))
    supcor_cest = Field(CHAR(7))
    supcor_ccf = Field(Integer(11))
    supcor_cod_vendedor = Field(Integer(11))
    supcor_nome_vendedor = Field(VARCHAR(40))
    supcor_quantidade_promo = Field(DECIMAL(10, 3))
    supcor_id_promo = Field(Integer(11))
    supcor_tipo_promo = Field(VARCHAR(1), index=True) # '1' = Scanntech, '2' = Sig, '3' = Funcionario
    scanntech = Field(Integer(11))
    if modulo_instalado_fichas:
        supcor_codigo_ficha = Field(DECIMAL(18, 3), index=True)

    def __repr__(self):
        _repr = "<caixa_mov_corpos: %.3d, 'IdCapa=%d>"
        vals = self.supcor_sequencia, self.supcor_capa_id
        return _repr % vals