#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.models.db_utils import a_metadata, a_session
from elixir import VARCHAR, BIGINT, Entity, Field
from elixir.options import using_options
from sqlalchemy import Enum, Integer, DECIMAL, ForeignKey


class CaixaMovFechamentos(Entity):
    """
    Tabela de Fechamento do cupom
    Prefixo: supfec
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_mov_fechamentos')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supfec_capa_id = Field(Integer(11),
                           ForeignKey('caixa_mov_capa.id', onupdate='cascade', ondelete="cascade"),
                           index=True)
    supfec_sequencia = Field(Integer(11))
    supfec_ccf = Field(Integer(11))
    supfec_gnf = Field(Integer(11))
    supfec_cdc = Field(Integer(11))
    supfec_nro_serie = Field(VARCHAR(20))
    supfec_finalizadora = Field(Integer(11))
    supfec_tipo_finalizadora = Field(Enum('D', 'H', 'C', 'T', 'O', name='tipo_finalizadora'))
    supfec_valor = Field(DECIMAL(10, 2))
    supfec_troco = Field(DECIMAL(10, 2))
    supfec_nro_banco = Field(Integer(11))
    supfec_agencia = Field(Integer(11))
    supfec_nro_cheque = Field(BIGINT)
    supfec_conta = Field(Integer(11))
    supfec_juros = Field(DECIMAL(8, 2))
    supfec_cnpj = Field(VARCHAR(15))
    supfec_desconto_proporcional = Field(DECIMAL(8, 2))
    supfec_acrescimo_proporcional = Field(DECIMAL(8, 2))
    supfec_cnpj_convenio = Field(VARCHAR(15))
    supfec_nro_autorizacao = Field(VARCHAR(15))
    supfec_nome_finalizadora = Field(VARCHAR(100))
    scanntech = Field(Integer(11))

    def __repr__(self):
        _repr = "<caixa_mov_fechamentos: %.3d, 'IdCapa=%d', 'Tipo=%s'>"
        vals = self.supfec_sequencia, self.supfec_capa_id, self.supfec_tipo_finalizadora
        return _repr % vals