#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import datetime
from sigpdv.models.db_utils import a_metadata, a_session
from elixir.options import using_options
from elixir import Entity, Field
from sqlalchemy import BigInteger, Boolean, Integer, text, Date, Time, DateTime, VARCHAR, DECIMAL


class CaixaNfeCapa(Entity):
    """
    Tabela de capa da nfe
    Prefixo: supnfa
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_nfe_capa')

    id = Field(Integer(11), primary_key=True)
    supnfa_serie_nf = Field(VARCHAR(3), server_default=text('000'))
    supnfa_modelo_nf = Field(VARCHAR(2), server_default=text('55'))
    supnfa_destinatario_doc = Field(VARCHAR(15))
    supnfa_destinatario_fone = Field(VARCHAR(15))
    supfna_forma_pgto = Field(Integer(1), server_default=text('0'))
    supnfa_destinatario_nome = Field(VARCHAR(60))
    supnfa_destinatario_uf = Field(VARCHAR(2))
    supnfa_destinatario_municipio = Field(VARCHAR(255))
    supnfa_destinatario_cmunicipio = Field(Integer)
    supnfa_destinatario_cep = Field(VARCHAR(8))
    supnfa_destinatario_endereco = Field(VARCHAR(255))
    supnfa_destinatario_bairro = Field(VARCHAR(60))
    supnfa_destinatario_numero = Field(VARCHAR(20))
    supnfa_destinatario_cpl = Field(VARCHAR(255))
    supnfa_transportador_id = Field(BigInteger)
    supnfa_data_emissao = Field(Date, default=datetime.today().date())
    supnfa_data_saida_entrada = Field(DateTime, default=datetime.today())
    supnfa_hora_emissao = Field(Time, default=datetime.today().time())
    supnfa_natureza = Field(VARCHAR(255))
    supnfa_peso_liquido = Field(DECIMAL(10, 2))
    supnfa_numero_nf = Field(BigInteger)
    supnfa_tipo = Field(VARCHAR(16))
    supnfa_numero_volume = Field(VARCHAR(15))
    supnfa_peso_bruto = Field(DECIMAL(10, 2))
    supnfa_vr_base_icms_frete = Field(DECIMAL(10, 2))
    supnfa_quantidade_volume = Field(BigInteger)
    supnfa_nfe_lote = Field(VARCHAR(15))
    supnfa_nfe_protocolo = Field(VARCHAR(15))
    supnfa_vr_total_nf = Field(DECIMAL(10, 0), server_default=text('0.0'))
    supnfa_vr_desconto_ = Field(DECIMAL(10, 0), server_default=text('0.0'))
    supnfa_state = Field(Integer, server_default=text('-1'))
    supnfa_condicao_pgto_id = Field(Integer(1))
    supnfa_nfe_processo_emissao = Field(VARCHAR(16))
    supnfa_nfe_protocolo_cancelamento = Field(VARCHAR(15))
    supnfa_especie = Field(VARCHAR(15))
    supnfa_vr_despesas_acessorias = Field(DECIMAL(10, 0))
    supnfa_vr_frete = Field(DECIMAL(10, 0), server_default=text('0.0'))
    supnfa_nfe_chave_acesso = Field(VARCHAR(44), server_default=text("''"))
    supnfa_nfe_xml_cancelamento = Field(Boolean)
    supnfa_nfe_xml_gerado = Field(Boolean)
    supnfa_nfe_status = Field(VARCHAR(255), server_default=text(u"'Esboço'"))
    supnfa_nfe_xml_proc = Field(Boolean)
    supnfa_observacao = Field(VARCHAR(255), server_default=text("''"))
    supnfa_vr_total_deducao = Field(DECIMAL(10, 2), server_default=text('0.0'))

    def __repr__(self):
        _repr = "<nfe_capa: %s, 'ser=%s'>"
        vals = self.id, self.supnfa_serie_nf
        return _repr % vals