#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sqlalchemy.sql.expression import text
from elixir import Entity, Field, using_table_options
from elixir.options import using_options
from sqlalchemy import UniqueConstraint, CHAR, SmallInteger, DateTime, Enum, Integer, String, \
    Date, DECIMAL, BOOLEAN, ForeignKey, VARCHAR, Boolean, Float, BigInteger, BIGINT, SMALLINT, \
    Text
from datetime import datetime
from sigpdv.tools.config import config
from sigpdv.models.db_utils import b_metadata, b_session, exec_sql, remove_accents, \
    modulo_instalado_producao_comanda


# ================================================================================================ #
#                                      TABELAS DO SUPERVISOR                                       #
# ================================================================================================ #
class SupervisorCfgParametros(Entity):
    """
    Tabela de parametros do supervisor
    Prefixo: supcfp
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cfg_parametros')
    using_table_options(useexisting=True)

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supcfp_moeda_singular = Field(VARCHAR(20))
    supcfp_moeda_plural = Field(VARCHAR(20))
    supcfp_municipio = Field(VARCHAR(30))
    supcfp_favorecido = Field(VARCHAR(40))
    supcfp_qtd_minima = Field(SmallInteger(6))
    supcfp_qtd_maxima = Field(SmallInteger(6))
    supcfp_valor_bonus = Field(DECIMAL(10, 2))
    supcfp_pz_sem_juros = Field(Integer(11))
    supcfp_pz_prazo1 = Field(SmallInteger(6))
    supcfp_pz_prazo2 = Field(SmallInteger(6))
    supcfp_pz_prazo3 = Field(SmallInteger(6))
    supcfp_pz_prazo4 = Field(SmallInteger(6))
    supcfp_pz_prazo5 = Field(SmallInteger(6))
    supcfp_pz_valor1 = Field(DECIMAL(10, 2))
    supcfp_pz_valor2 = Field(DECIMAL(10, 2))
    supcfp_pz_valor3 = Field(DECIMAL(10, 2))
    supcfp_pz_valor4 = Field(DECIMAL(10, 2))
    supcfp_pz_valor5 = Field(DECIMAL(10, 2))
    supcfp_pz_juros1 = Field(DECIMAL(10, 2))
    supcfp_pz_juros2 = Field(DECIMAL(10, 2))
    supcfp_pz_juros3 = Field(DECIMAL(10, 2))
    supcfp_pz_juros4 = Field(DECIMAL(10, 2))
    supcfp_pz_juros5 = Field(DECIMAL(10, 2))
    supcfp_limite_sangria = Field(DECIMAL(10, 2))
    supcfp_cliente_contra_vale = Field(Integer(11))
    supcfp_conta_contra_vale = Field(VARCHAR(15))
    supcfp_conta_transfer_contra_vale = Field(VARCHAR(15))
    supcfp_pre_permite_desconto = Field(VARCHAR(1))
    supcfp_pre_permite_acrescimo = Field(VARCHAR(1))
    supcfp_limite_cheque_a_vista = Field(DECIMAL(10, 2))
    supcfp_arq_promocoes = Field(String(100)) # nao eh usado no frente, apenas no supervisor
    supcfp_restringe_desconto = Field(Boolean, server_default=text('False'))
    supcfp_mensagem_doacao = Field(Text)
    try:
        res = exec_sql("""SELECT supcfp_comissao 
                          FROM supervisor_cfg_parametros;""", 'S')
        if res:
            supcfp_comissao = Field(DECIMAL(2, 2))
    except:
        pass



class SupervisorCadItens(Entity):
    """
    Tabela de Cadastro de itens
    Prefixo: supite
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cad_itens')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supite_loja = Field(SmallInteger(6), primary_key=True, autoincrement=False)
    supite_codigo_item = Field(String(14), primary_key=True, autoincrement=False)
    active = Field(Boolean, server_default=text('True'))
    name = Field(String(30))
    supite_tributacao_id = Field(Integer(11))
    supite_pesavel = Field(Boolean)
    supite_fracionado = Field(Boolean)
    supite_preco = Field(DECIMAL(10, 2))
    supite_unidade = Field(String(3))
    supite_peso_da_embalagem = Field(Float)
    supite_codigo_retaguarda = Field(CHAR(20))
    supite_fator_multiplicador = Field(Float)
    supite_producao = Field(Enum('P', 'T', name='producao'))
    supite_iat = Field(Enum('A', 'T', name='iat'))
    supite_ncm = Field(CHAR(8))
    supite_cest = Field(CHAR(7))
    supite_aliquota_ibpt_fed = Field(DECIMAL(4, 2), server_default=text('0.00'))     # aqliquota federal
    supite_aliquota_ibpt_est = Field(DECIMAL(4, 2), server_default=text('0.00'))     # aqliquota estadual
    supite_aliquota_ibpt_mun = Field(DECIMAL(4, 2), server_default=text('0.00'))     # aqliquota municipal
    supite_ibpt_chave = Field(CHAR(10))                                              # chave da tablera ibpt
    supite_descricao_completa = Field(String(60))
    supite_peso_tara = Field(Float)
    scanntech = Field(Integer(11))
    if modulo_instalado_producao_comanda:
        supite_codigo_local = Field(Integer(10))


    # _sql_constraints
    using_table_options(UniqueConstraint('supite_codigo_item'))

    def __repr__(self):
        _repr = "<caixa_cad_itens: %s, 'Loja=%d', 'Descricao=%s', 'Valor=R$ %.2f'>"
        vals = self.supite_codigo_item, self.supite_loja, self.name, self.supite_preco
        return _repr % vals


class SupervisorCadSerie(Entity):
    """
    SupervisorCadSerie
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cad_serie')

    supser_codigo_item = Field(String(14), primary_key=True)
    supser_serie = Field(String(30), primary_key=True)
    supser_usado = Field(BOOLEAN, default=False)

    def __repr__(self):
        _repr = "<supervisor_cad_serie: 'Item=%s', 'Serie=%s'"
        vals = self.supser_codigo_item, self.supser_serie
        return _repr % vals


class SupervisorCadClientes(Entity):
    """
    Tabela de Cadastro de Clientes
    Prefixo: supcli
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cad_clientes')
    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supcli_cnpj_cpf = Field(String(15), primary_key=True, autoincrement=False)
    supcli_tipo_pessoa = Field(Enum('F', 'J', name="tipo_pessoa", default='F'))
    name = Field(String(40))
    supcli_endereco = Field(String(40))
    supcli_status = Field(CHAR(1))
    supcli_limite_compras = Field(DECIMAL(14, 2))
    supcli_total_em_aberto = Field(DECIMAL(10, 2), server_default=text('0.00'))
    supcli_codigo_interno = Field(BigInteger(15))
    supcli_cod_municipio = Field(Integer(7))
    supcli_insc_estadual = Field(String(15))
    supcli_bairro = Field(String(20))
    supcli_cidade = Field(String(30))
    supcli_estado = Field(CHAR(2))
    supcli_endereco_numero = Field(VARCHAR(15))
    supcli_cep = Field(String(8))
    supcli_telefone = Field(String(14))
    supcli_rota = Field(SmallInteger(5))
    supcli_email = Field(String(100))

    def __repr__(self):
        _repr = "<caixa_cad_clientes: %s, 'Nome=%s', 'Status=%s'>"
        vals = self.supcli_cnpj_cpf, self.name, self.supcli_status
        return _repr % vals


class SupervisorCadFinalizadora(Entity):
    """
    Tabela de cadastro de finalizadoras
    Prefixo: supfin
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cad_finalizadora')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supfin_codigo = Field(SmallInteger(6), nullable=True, unique=True)
    name = Field(String(100))
    supfin_permite_troco = Field(Boolean, server_default=text('False'))
    supfin_troco_maximo = Field(DECIMAL(10, 2), server_default=text('0'))
    supfin_tipo_finalizadora = Field(CHAR(1))
    supfin_codigo_na_ecf = Field(Integer(10),
                                 ForeignKey('supervisor_cad_finalizadora_na_ecf.supfec_codigo_na_ecf',
                                            onupdate="cascade", ondelete="cascade"))
    supfin_avista_aprazo = Field(Enum('V', 'P', name='avista_aprazo'), server_default=text('P'))
    supfin_usa_tef = Field(Boolean, server_default=text('False'))
    supfin_cartao_tipo = Field(CHAR(1), server_default=text('C'))
    supfin_cartao_parcelamento = Field(CHAR(1))
    supfin_tipo_midia = Field(CHAR(1))
    supfin_outros_qtd_parcelas = Field(SmallInteger(6))
    supfin_outros_taxa_juros = Field(DECIMAL(5, 2))
    supfin_outros_prazo_primeira = Field(SmallInteger(6), default=30)
    supfin_outros_intervalo_parcelas = Field(SmallInteger(6), default=31)
    supfin_outros_periodo = Field(CHAR(1))
    supfin_outros_semanal_dia = Field(String(3))
    supfin_outros_quinzenal_vencimento = Field(SmallInteger(6))
    supfin_outros_quinzenal_tipo = Field(CHAR(1))
    supfin_outros_mensal_vencimento = Field(SmallInteger(6))
    supfin_outros_mensal_tipo = Field(CHAR(1))
    supfin_outros_mensal_fecha_mes = Field(SmallInteger(6))
    supfin_conta_caixa_credito = Field(String(15))
    supfin_conta_caixa_debito = Field(String(15))
    supfin_conta_a_prazo = Field(String(15))
    supfin_cliente = Field(Integer(11))
    supfin_permite_desconto_a_vista = Field(CHAR(1))
    supfin_aceita_contra_vale = Field(Boolean)
    supfin_contra_vale = Field(Boolean)
    supfin_imp_promissoria = Field(Boolean)
    supfin_conta_descontos = Field(String(15))
    supfin_conta_acrescimos = Field(String(15))
    supfin_conta_troco = Field(String(15))
    supfin_convenio = Field(CHAR(1))
    supfin_somente_dav = Field(CHAR(1))
    supfin_mod_pgto_grupo = Field(String(2))
    supfin_mod_pgto_sub_grupo = Field(String(2))

    def __repr__(self):
        _repr = "<caixa_cad_finalizadora: %d, 'Descricao=[%s] %s', 'Codigo_na_ecf=%s'>"
        vals = self.supfin_codigo, self.supfin_tipo_finalizadora, self.name, self.supfin_codigo_na_ecf
        return _repr % vals


class SupervisorCadFinalizadoraNaEcf(Entity):
    """
    Cadastro de nomes de finalizadoras na ecf
    Prefixo: supfec
    """
    using_options(metadata=b_metadata, session=b_session,
                  tablename='supervisor_cad_finalizadora_na_ecf')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supfec_codigo_na_ecf = Field(SmallInteger(10))
    name = Field(String(30))
    last_update = Field(DateTime, default=datetime.now())

    # _sql_constraints
    using_table_options(UniqueConstraint('supfec_codigo_na_ecf'))

    def __repr__(self):
        _repr = "<supervisor_cad_finalizadora_na_ecf: %d - %s>"
        vals = self.supfec_codigo_na_ecf, self.name
        return _repr % vals

class SupervisorCadBandeiras(Entity):
    """
    Tabela de cadastro de bandeiras
    Prefixo: supban
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cad_bandeiras')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supban_codigo = Field(Integer(6), nullable=True)
    name = Field(String(30))
    supban_descricao_completa = Field(String(40))
    supban_tipo = Field(VARCHAR(1))

    # _sql_constraints
    using_table_options(UniqueConstraint('supban_codigo', 'name', 'supban_tipo'))

    def __repr__(self):
        _repr = "<b_session_cad_bandeiras: %d, 'Nome=%s'>"
        vals = self.supban_codigo, self.name
        return _repr % vals

class SupervisorCfgConfiguracao(Entity):
    """
    Tabela de configuracao
    Prefixo: supcfc
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cfg_configuracao')

    id = Field(Integer(6), primary_key=True, autoincrement=True)
    supcfc_loja = Field(SmallInteger(6))
    supcfc_cnpj = Field(String(15))
    supcfc_cliente = Field(String(40))
    supcfc_endereco = Field(String(40))
    supcfc_cidade = Field(String(30))
    supcfc_estado = Field(String(2))
    supcfc_cep = Field(String(8))
    supcfc_nro_serie = Field(String(5))
    supcfc_data_validade = Field(String(8))
    supcfc_inscricao_estadual = Field(String(20))
    supcfc_fax = Field(String(14))
    supcfc_telefone = Field(String(14))
    supcfc_numero = Field(SmallInteger(5))
    supcfc_complemento = Field(String(20))
    supcfc_bairro = Field(String(15))
    supcfc_responsavel = Field(String(25))
    supcfc_ultimo_acesso = Field(String(8))
    supcfc_inscricao_municipal = Field(String(20))
    supcfc_cidade_ibge = Field(String(7))
    supcfc_fantasia = Field(String(40))
    supcfc_contador_nome = Field(String(40))
    supcfc_contador_cpf = Field(String(11))
    supcfc_contador_crc = Field(String(15))
    supcfc_contador_cnpj = Field(String(14))
    supcfc_contador_cep = Field(Integer(11))
    supcfc_contador_end = Field(String(40))
    supcfc_contador_end_nro = Field(Integer(11))
    supcfc_contador_end_compl = Field(String(20))
    supcfc_contador_end_bairro = Field(String(30))
    supcfc_contador_end_mun_ibge = Field(Integer(11))
    supcfc_contador_fone = Field(String(10))
    supcfc_contador_fax = Field(String(10))
    supcfc_contador_email = Field(String(40))
    supcfc_pis = Field(Float(5, 4))
    supcfc_cofins = Field(Float(5, 4))

    def __repr__(self):
        _repr = "<supervisor_cfg_configuracao: %s, 'Loja=%d', 'Cliente=%s'"
        vals = self.supcfc_cnpj, self.supcfc_loja, remove_accents(self.supcfc_cliente)
        return _repr % vals


class SupervisorCadOperadores(Entity):
    """
    Tabela de Cadastro de Operadores
    Prefixo: supope
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cad_operadores')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supope_codigo = Field(Integer(11), nullable=True)
    name = Field(VARCHAR(30))
    active = Field(Boolean, server_default=text('True'))
    supope_senha = Field(String(15))
    supope_operador_supervisor = Field(Enum('S', 'O', name='operador_supervisor'))
    supope_cancela_item = Field(Boolean)
    supope_desconto_item = Field(Boolean)
    supope_libera_restricao = Field(Boolean)
    # supope_biometria = Field(Text) # Quando nao tem biomeria no supervisor, erro na comanda

    def __repr__(self):
        _repr = "<supervisor_cad_operadores: %d, 'Nome=%s'>"
        vals = self.supope_codigo, remove_accents(self.name)
        return _repr % vals


class SupervisorMovCapa(Entity):
    """
    Tabela de Capa de cupons
    Prefixo: supcap
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_mov_capa')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supcap_data_cupom = Field(Date())
    supcap_loja = Field(SmallInteger(6))
    supcap_caixa = Field(SmallInteger(6))
    supcap_nro_cupom = Field(Integer(11))
    supcap_numero_pedido = Field(String(20))
    supcap_operador_id = Field(Integer(11), ForeignKey('supervisor_cad_operadores.id',
                                                       onupdate="restrict",))
    supcap_cliente = Field(String(15), server_default=text("''"))
    supcap_data_inicio = Field(DateTime())
    supcap_data_fim = Field(DateTime())
    supcap_tipo = Field(CHAR(1))
    supcap_alteracao = Field(Enum('A', 'D', '', name='alteracao', ))
    supcap_valor_desc_acres = Field(DECIMAL(10, 2), server_default=text('0'))
    supcap_total_cupom = Field(DECIMAL(10, 2), server_default=text('0'))
    supcap_combustivel = Field(String(1), server_default=text("''"))
    supcap_nro_serie = Field(String(20))
    supcap_mf_adicional = Field(BOOLEAN)
    supcap_marca_ecf = Field(String(20))
    supcap_modelo_ecf = Field(String(20))
    supcap_nro_proprietario = Field(Integer(11))
    supcap_ccf = Field(Integer(11))
    supcap_gnf = Field(Integer(11))
    supcap_crz = Field(Integer(11))
    supcap_grg = Field(Integer(11))
    supcap_cdc = Field(Integer(11))
    supcap_tipo_comprovante = Field(Enum('CM', 'RV', 'CC', 'CN', 'NC', 'RG',
                                         name='tipo_comprovante'))
    supcap_subtotal = Field(DECIMAL(10, 2))
    supcap_vr_imposto_fed = Field(DECIMAL(10, 2), server_default=text('0.00'))
    supcap_vr_imposto_est = Field(DECIMAL(10, 2), server_default=text('0.00'))
    supcap_vr_imposto_mun = Field(DECIMAL(10, 2), server_default=text('0.00'))
    supcap_desconto_subtotal = Field(DECIMAL(10, 2))
    supcap_indicador_desconto = Field(Enum('', 'V', 'P', name='ind_desc'))
    supcap_acrescimo_subtotal = Field(DECIMAL(10, 2))
    supcap_indicador_acrescimo = Field(Enum('', 'V', 'P', name='ind_acres'))
    supcap_indicador_cancelamento = Field(Enum('N', 'S', name='ind_cancel'))
    supcap_cancelamento_acrescimo = Field(DECIMAL(10, 2))
    supcap_cancelado_posteriormente = Field(BOOLEAN)
    supcap_nome_cliente = Field(String(40))
    supcap_nro_cupom_vinculado = Field(Integer(11))
    supcap_cod_vendedor = Field(Integer(11))
    scanntech = Field(Integer(11))

    def __repr__(self):
        _repr = "<supervisor_mov_capa: COO:%d>"
        vals = self.supcap_nro_cupom
        return _repr % vals


class SupervisorMovCorpos(Entity):
    """
    Tabela de itens dos cupons
    Prefixo: supcor
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_mov_corpos')
    id = Field(Integer(11), primary_key=True, autoincrement=True)

    supcor_capa_id = Field(Integer(11),
                           ForeignKey('supervisor_mov_capa.id', onupdate="cascade", ondelete="cascade"))
    supcor_sequencia = Field(Integer(11))
    supcor_nro_serie = Field(VARCHAR(20))
    supcor_item = Field(VARCHAR(14))
    supcor_nro_serie_item = Field(VARCHAR(30))
    supcor_quantidade = Field(DECIMAL(10, 3))
    supcor_quantidade_promo = Field(DECIMAL(10, 3))
    supcor_preco_total = Field(DECIMAL(10, 3))
    supcor_vr_imposto_fed = Field(DECIMAL(10, 2))
    supcor_vr_imposto_est = Field(DECIMAL(10, 2))
    supcor_vr_imposto_mun = Field(DECIMAL(10, 2))
    supcor_ibpt_chave = Field(CHAR(10))
    supcor_desconto_item = Field(DECIMAL(10, 3))
    supcor_venda_cancelamento = Field(Enum('V', 'C', name='venda_cancel'))
    supcor_acrescimo_item = Field(DECIMAL(10, 3))
    supcor_desconto_item_sem_rat = Field(DECIMAL(10, 3)) # sem desconto da capa reateado(apenas sacnntech)
    supcor_acrescimo_item_sem_rat = Field(DECIMAL(10, 3)) # sem acrescimo da capa reateado(apenas sacnntech)
    supcor_tributacao_ecf = Field(VARCHAR(3))
    supcor_tributacao_retag = Field(SmallInteger(6))
    supcor_aliquota = Field(DECIMAL(5, 2))
    supcor_base = Field(DECIMAL(5, 2))
    supcor_nome_item = Field(VARCHAR(60))
    supcor_unidade = Field(VARCHAR(3))
    supcor_preco_unitario = Field(DECIMAL(10, 3))
    supcor_totalizador_parcial = Field(VARCHAR(7))
    supcor_tipo_tributacao = Field(Enum('T', 'F', 'I', 'N', 'O', name='tipo_tributacao'))
    supcor_indicador_arr_trunc = Field(Enum('T', 'A', name='arr_trunc'))
    supcor_indicador_ppt = Field(Enum('P', 'T', name='ppt'))
    supcor_ncm = Field(CHAR(8))
    supcor_cest = Field(CHAR(7))
    supcor_ccf = Field(Integer(11))
    supcor_cod_vendedor = Field(Integer(11))
    supcor_nome_vendedor = Field(VARCHAR(40))
    supcor_id_promo = Field(Integer(11))
    supcor_tipo_promo = Field(VARCHAR(1)) # '1' = Scanntech, '2' = Sig
    scanntech = Field(Integer(11))

    def __repr__(self):
        _repr = "<supervisor_mov_corpos: %.3d, 'IdCapa=%d>"
        vals = self.supcor_sequencia, self.supcor_capa_id
        return _repr % vals


class SupervisorMovFechamentos(Entity):
    """
    Tabela de Fechamento do cupom
    Prefixo: supfec
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_mov_fechamentos')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supfec_capa_id = Field(Integer(11),
                           ForeignKey('supervisor_mov_capa.id', onupdate='cascade', ondelete="cascade"))
    supfec_sequencia = Field(Integer(11))
    supfec_ccf = Field(Integer(11))
    supfec_gnf = Field(Integer(11))
    supfec_cdc = Field(Integer(11))
    supfec_nro_serie = Field(VARCHAR(20))
    supfec_finalizadora = Field(Integer(11))
    supfec_tipo_finalizadora = Field(Enum('D', 'H', 'C', 'T', 'O', name='tipo_finalizadora'))
    supfec_valor = Field(DECIMAL(10, 2))
    supfec_troco = Field(DECIMAL(10, 2))
    supfec_nro_banco = Field(Integer(11))
    supfec_agencia = Field(Integer(11))
    supfec_nro_cheque = Field(BIGINT)
    supfec_conta = Field(Integer(11))
    supfec_juros = Field(DECIMAL(8, 2))
    supfec_cnpj = Field(VARCHAR(15))
    supfec_desconto_proporcional = Field(DECIMAL(8, 2))
    supfec_acrescimo_proporcional = Field(DECIMAL(8, 2))
    supfec_cnpj_convenio = Field(VARCHAR(15))
    supfec_nro_autorizacao = Field(VARCHAR(15))
    supfec_nome_finalizadora = Field(VARCHAR(100))
    scanntech = Field(Integer(11))

    def __repr__(self):
        _repr = "<supervisor_mov_fechamentos: %.3d, 'IdCapa=%d', 'Tipo=%s'>"
        vals = self.supfec_sequencia, self.supfec_capa_id, self.supfec_tipo_finalizadora
        return _repr % vals


class SupervisorMovCupomServidor(Entity):
    """
    Tabela de cupons exportados para o servidor
    Prefixo: supcse
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_mov_cupom_servidor')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    create_date = Field(DateTime())
    supcse_enviado = Field(BOOLEAN)
    supcse_capa_id = Field(Integer(11))

    def __repr__(self):
        _repr = "<TblMov_CupomServidor: %s, 'IDCapa=%d'>"
        vals = self.supcse_enviado, self.supcse_capa_id
        return _repr % vals


class SupervisorMovSaldo(Entity):
    """
    Tabela de Saldo
    Prefixo: supsal
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_mov_saldo')

    supsal_loja = Field(SmallInteger(6), primary_key=True, autoincrement=False)
    supsal_item = Field(String(20), primary_key=True, autoincrement=False)
    supsal_saldo_inicial = Field(DECIMAL(16, 2))
    supsal_saldo = Field(DECIMAL(16, 2))
    supsal_data = Field(Date)

    def __repr__(self):
        _repr = "<supervisor_mov_saldo: 'Loja=%d', 'Item=%s', 'Saldo=R$ %.3f'>"
        vals = self.supsal_loja, self.supsal_item, self.supsal_saldo
        return _repr % vals


class SupervisorCadVendedor(Entity):
    """
    Tabela de Vendedores
    Prefixo: supven
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cad_vendedor')
    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supven_codigo = Field(Integer, primary_key=True, autoincrement=False)
    name = Field(VARCHAR(40), required=True)

    def __repr__(self):
        _repr = "<supervisor_cad_Vendedor: %s-%s'>"
        vals = self.supven_codigo, self.name
        return _repr % vals


# ================================================================================================ #
#                                          TABELAS DE DAV                                          #
# ================================================================================================ #
class SupervisorDavCapa(Entity):
    """
    Tabela de capa do DAV
    Prefixo: supdva

    Restrição: -1=Cancelamento, 1=Aguardando Emissão, 2=Emitido, 3=Aviso
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_dav_capa')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supdva_nro_pedido = Field(Integer(11))
    create_date = Field(DateTime(), default=datetime.now())
    supdva_nro_cupom = Field(SmallInteger(6))
    supdva_loja = Field(SmallInteger(6))
    supdva_caixa = Field(SmallInteger(6))
    supdva_data_cupom = Field(DateTime())
    supdva_data_hora = Field(DateTime(), default=datetime.now())
    supdva_cliente = Field(CHAR(15))
    supdva_nome_cliente = Field(VARCHAR(60))
    supdva_restricao = Field(SmallInteger(1), server_default=text('0'))
    supdva_operador_id = Field(Integer(10))
    supdva_vendedor_id = Field(Integer(10))
    supdva_juros = Field(Float(10, 2))
    supdva_descontos = Field(Float(10, 2))
    supdva_total_dav = Field(DECIMAL(10, 2), server_default=text('0.00'))
    supdva_avisos = Field(String(255))

    def __repr__(self):
        _repr = "<supervisor_dav_capa: %d - %s, 'CNPJ/CPF=%s'>"
        vals = self.id, self.supdva_restricao, self.supdva_cliente
        return _repr % vals


class SupervisorDavCorpo(Entity):
    """
    Tabela de itens do DAV
    Prefixo: supdvo
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_dav_corpo')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    create_date = Field(DateTime())
    supdvo_capa_id = Field(Integer(11))
    supdvo_numero_do_dav = Field(Integer(10))
    supdvo_sequencia = Field(SmallInteger(5), primary_key=True, autoincrement=False)
    supdvo_data_hora = Field(DateTime(), default=datetime.now())
    supdvo_item = Field(CHAR(14))
    supdvo_quantidade = Field(DECIMAL(10, 2))
    supdvo_quantidade_conferida = Field(DECIMAL(10, 2))
    supdvo_vr_total = Field(DECIMAL(10, 3))
    supdvo_desconto_item = Field(DECIMAL(10, 3))
    supdvo_acrescimo_item = Field(DECIMAL(10, 3))
    supdvo_venda_cancelamento = Field(VARCHAR(1), server_default=text('V'))
    supdvo_descricao = Field(String(60))
    supdvo_unidade = Field(String(3))
    supdvo_vr_unit = Field(DECIMAL(10, 2))
    supdvo_totalizador_parcial = Field(String(7), server_default=text(''))

    def __repr__(self):
        _repr = "<supervisor_dav_corpo: %d/%d, 'Item=%s'>"
        vals = self.id, self.supdvo_sequencia, self.supdvo_item
        return _repr % vals


class SupervisorDavFechamento(Entity):
    """
    Tabela de fechamento do DAV
    Prefixo: supdvf
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_dav_fechamento')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    create_date = Field(DateTime())
    supdvf_capa_id = Field(Integer(10))
    supdvf_sequencia = Field(SmallInteger(5), primary_key=True, autoincrement=False)
    supdvf_finalizadora = Field(SmallInteger(5))
    supdvf_valor = Field(Float(8, 2))
    supdvf_cheque_banco = Field(SmallInteger(5))
    supdvf_cheque_agencia = Field(SmallInteger(5))
    supdvf_cheque_numero = Field(CHAR(9))
    supdvf_cheque_conta = Field(BigInteger(10))
    supdvf_cheque_juros = Field(Float(8, 2))
    supdvf_cheque_cnpj = Field(CHAR(15))

    def __repr__(self):
        _repr = "<supervisor_dav_fechamento: %d/%d, 'Finalizadora=%d'>"
        vals = self.id, self.supdvf_sequencia, self.supdvf_finalizadora
        return _repr % vals


class SupervisorDavVencimentos(Entity):
    """
    Tabela de vencimentos do dav
    Prefixo: supdvv
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_dav_vencimentos')

    id = Field(Integer(10), primary_key=True, autoincrement=False)
    supdvv_fechamento_id = Field(Integer(10))
    supdvv_seq_vencimento = Field(SmallInteger(5), primary_key=True, autoincrement=False)
    supdvv_valor_parcela = Field(Float(8, 2))
    supdvv_vencimento = Field(Date())

    def __repr__(self):
        _repr = "<supervisor_dav_vencimentos: %d-%d/%d, 'Valor=R$ %.2f'>"
        vals = self.id, self.supdvv_seq_vencimento, self.supdvv_valor_parcela, self.supdvv_valor_parcela
        return _repr % vals


# ================================================================================================ #
#                                       TABELAS DE PRE VENDA                                       #
# ================================================================================================ #
class SupervisorPveCapa(Entity):
    """
    Tabela de Capa da Prevenda
    Prefixo: suppva

    Restrição: -1=Cancelamento, 1=Aguardando Emissão, 2=Emitido, 3=Aviso
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_pve_capa')

    id = Field(Integer(10), primary_key=True)
    suppva_data_hora = Field(DateTime(), default=datetime.now())
    suppva_cliente = Field(CHAR(15))
    suppva_operador_id = Field(Integer(10))
    suppva_loja = Field(SmallInteger(5))
    suppva_caixa = Field(SmallInteger(6))
    suppva_nro_cupom = Field(SmallInteger(6))
    suppva_juros = Field(Float(10, 2))
    suppva_descontos = Field(Float(10, 2))
    suppva_data_cupom = Field(DateTime())
    suppva_restricao = Field(Integer(1), server_default=text('0'))
    suppva_nome_cliente = Field(VARCHAR(15))
    suppva_total_pve = Field(DECIMAL(10, 2), server_default=text('0.0'))

    def __repr__(self):
        _repr = "<supervisor_pve_capa: %d - %s, 'CNPJ/CPF=%s'>"
        vals = self.id, self.suppva_restricao, self.suppva_cliente
        return _repr % vals


class SupervisorPveCorpo(Entity):
    """
    Tabela de itens da prevenda
    Prefixo: suppvo
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_pve_corpo')

    id = Field(Integer(10), primary_key=True, autoincrement=True)
    suppvo_capa_id = Field(Integer(10))  # Chave e_reprangeira na tabela supervisor_pve_capa
    suppvo_sequencia = Field(SMALLINT(5))
    suppvo_item = Field(CHAR(14))
    suppvo_quantidade = Field(Float(10, 2))
    suppvo_preco_total = Field(Float(10, 3))
    suppvo_desconto_item = Field(Float(10, 3))
    suppvo_acrescimo_item = Field(Float(10, 3))
    suppvo_venda_cancelamento = Field(VARCHAR(1), server_default=text("'V'"))

    def __repr__(self):
        _repr = "<supervisor_mov_corpo: %d/%d, 'Item=%s'>"
        vals = self.id, self.suppvo_sequencia, self.suppvo_item
        return _repr % vals


class SupervisorPveFechamento(Entity):
    """
    Tabela de Fechamento da prevenda
    Prefixo: suppvf
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_pve_fechamento')

    id = Field(Integer(10), primary_key=True, autoincrement=True)
    suppvf_capa_id = Field(Integer(10))
    suppvf_sequencia = Field(SmallInteger(5))
    suppvf_finalizadora = Field(SmallInteger(5))
    suppvf_valor = Field(Float(8, 2))
    suppvf_cheque_banco = Field(SmallInteger(5))
    suppvf_cheque_agencia = Field(SmallInteger(5))
    suppvf_cheque_numero = Field(Integer(10))
    suppvf_cheque_conta = Field(Integer(10))
    suppvf_cheque_juros = Field(Float(8, 2))
    suppvf_cheque_cnpj = Field(CHAR(15))

    def __repr__(self):
        _repr = "<supervisor_pve_fechamento: %d/%d, 'Finalizadora=%d'>"
        vals = self.id, self.suppvf_sequencia, self.suppvf_finalizadora
        return _repr % vals


class SupervisorPveVencimentos(Entity):
    """
    Tabela de vencimentos da prevenda
    Prefixo: suppvv
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_pve_vencimentos')

    id = Field(Integer(10), primary_key=True, autoincrement=True)
    suppvv_fechamento_id = Field(Integer(10))
    suppvv_seq_vencimento = Field(SmallInteger(5))
    suppvv_valor_parcela = Field(DECIMAL(10, 2))
    suppvv_vencimento = Field(Date())

    def __repr__(self):
        _repr = "<supervisor_pve_vencimento: %d/%d/, 'Valor=R$ %.2d'>"
        vals = self.id, self.suppvv_seq_vencimento, self.suppvv_valor_parcela
        return _repr % vals


# ================================================================================================ #
#                                        TABELAS DA COMANDA                                        #
# ================================================================================================ #
class SupervisorCdaCapa(Entity):
    """
    Tabela da capa da comanda
    Prefixo: supcda

    Restrição: -1=Cancelamento, 0=Aberta, 1=Aguardando Emissão, 2=Faturada, 4=Anulada, 5=Faturando
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cda_capa')

    id = Field(Integer, primary_key=True, autoincrement=True)
    supcda_codigo_comanda = Field(BIGINT)
    supcda_data_hora = Field(DateTime(), default=datetime.utcnow())  # data no horario de greenwich
    supcda_cliente = Field(CHAR(15))
    supcda_operador = Field(Integer)
    supcda_nome_cliente = Field(VARCHAR(100))
    supcda_juros = Field(Float(10, 2))
    supcda_descontos = Field(Float(10, 2))
    supcda_data_cupom = Field(DateTime())
    supcda_loja = Field(SmallInteger)
    supcda_caixa = Field(SmallInteger)
    if modulo_instalado_producao_comanda:
        supcda_mesa = Field(SmallInteger)

    supcda_nro_cupom = Field(Integer)
    supcda_restricao = Field(Integer, default=0)
    supcda_total_comanda = Field(DECIMAL(10, 2), default=0.0)

    def __repr__(self):
        _repr = "<supervisor_cda_capa: %s - %s, 'CNPJ/CPF=%s'>"
        vals = self.supcda_codigo_comanda, self.supcda_restricao, self.supcda_cliente
        return _repr % vals


class SupervisorCdaCorpo(Entity):
    """
    Tabela do corpo da comanda
    Prefixo: supcdo
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cda_corpo')

    id = Field(Integer, primary_key=True, autoincrement=True)
    create_date = Field(DateTime())
    supcdo_capa_id = Field(Integer, ForeignKey('supervisor_cda_capa.id', onupdate="cascade",
                                               ondelete="cascade"))
    supcdo_sequencia_id = Field(SmallInteger, primary_key=True, autoincrement=False)
    supcdo_item = Field(CHAR(14))
    supcdo_quantidade = Field(Float(10, 2))
    supcdo_preco_total = Field(Float(10, 3))
    supcdo_venda_cancelamento = Field(VARCHAR(1), default='V')
    supcdo_cod_vendedor = Field(Integer(11))
    supcdo_nome_vendedor = Field(VARCHAR(40))
    if modulo_instalado_producao_comanda:
        supcdo_observacao = Field(Text)
        supcdo_seq_impressao = Field(Integer(10))
        supcdo_data_hora_impressao = Field(DateTime)
        supcdo_imprimir = Field(Boolean)
        supcdo_entregue = Field(Boolean)

    def __repr__(self):
        _repr = "<supervisor_cda_corpo: %d/%d, 'Item=%s'>"
        vals = self.supcdo_capa_id, self.supcdo_sequencia_id, self.supcdo_item
        return _repr % vals

class SupervisorCdaItensFavoritos(Entity):
    """
    Tabela de itens favoritos da comanda
    Prefixo: supifa
    """
    using_options(metadata=b_metadata, session=b_session, tablename='supervisor_cda_itens_favoritos')

    id = Field(Integer, primary_key=True, autoincrement=True)
    supifa_item_id = Field(Integer, ForeignKey('supervisor_cad_itens.id', ondelete="cascade"))
    supifa_botao = Field(VARCHAR(15))
    supifa_botao_descricao = Field(VARCHAR(26))

    def __repr__(self):
        _repr = "<supervisor_cda_itens_favoritos: 'Item=%s', 'Descricao do Item=%s' 'Botao=%s'>"
        vals = self.supifa_item_id, self.supifa_botao_descricao, self.supifa_botao
        return _repr % vals


# ================================================================================================ #


print config.rcfile