# -*- coding: utf-8 -*-
from nfe.nf_e import nf_e
from nfe.pysped.nfe.manual_401 import NFe_200, Det_200
from nfe.pysped.nfe.webservices_flags import *
import re
import os
from sigpdv.models.tbl_nfe import NfeCapa
from sigpdv.models.tbl_cfg import configuracao
from decimal import Decimal
from sigpdv.crypto import md5_file
from sigpdv import root_path
from sigpdv.tools.config import config
from datetime import datetime
def processar_nfe(nf_id, motivo_scan=None):
    """
    Efetuar o Processamento da Nota Fiscal Eletrônica
    """
    #
    # CERTIFICADO
    #
    _TIPO_EMISSAO = '1'

    nf_ = nf_e()
    cert1 = os.path.join(root_path, 'nf', 'ass_digital_sig_2012.pfx')
    cert_str = open(cert1, 'rb').read()
    pwd = u'certf2805siginf'
    res = nf_.extrair_certificado_a1(cert_str, pwd)
    cert = res['cert']
    key = res['key']
    #
    nfe_capa = NfeCapa.get_by(id=nf_id)
    #
    # Instancia uma NF-e
    #
    nota = NFe_200()
    #
    # CONTIGENCIA SCAN
    #
    #

    if motivo_scan:
        _TIPO_EMISSAO = '3'
        nota.infNFe.ide.dhCont.valor = datetime.strftime(datetime.today(), u'%Y-%m-%dT%H:%M:%S')
        nota.infNFe.ide.xJust.valor = motivo_scan
        # 2011-05-11T10:59:07 data e hora da contigência SCAN

    # Identificação da NF-e
    #
    nota.infNFe.ide.cUF.valor = UF_CODIGO[u'MG']
    nota.infNFe.ide.natOp.valor = nfe_capa.natureza
    nota.infNFe.ide.indPag.valor = nfe_capa.forma_pgto
    nota.infNFe.ide.serie.valor = nfe_capa.serie_nf
    nota.infNFe.ide.cNF.valor = '%08d' % nfe_capa.numero_nf
    nota.infNFe.ide.nNF.valor = nfe_capa.numero_nf
    nota.infNFe.ide.dEmi.valor = nfe_capa.data_emissao
    nota.infNFe.ide.dSaiEnt.valor = nfe_capa.data_saida_entrada
    nota.infNFe.ide.cMunFG.valor = 3167202
    nota.infNFe.ide.tpImp.valor = 1
    nota.infNFe.ide.tpEmis.valor = _TIPO_EMISSAO               # Emissão Normal
    nota.infNFe.ide.finNFe.valor = 1               # Nfe Normal
    nota.infNFe.ide.tpAmb.valor = 2
    nota.infNFe.ide.procEmi.valor = 0
    nota.infNFe.ide.verProc.valor = u'SigErp 1.0'
    nota.infNFe.infAdic.infAdFisco.valor = 'MD-5:%s' % md5_file(os.path.join(root_path, 'lista_de_aplicativos.txt'))
    #
    emitente = configuracao.get_by(Loja=config['loja'])
    #
    nota.infNFe.emit.CNPJ.valor = emitente.CNPJ
    nota.infNFe.emit.xNome.valor = emitente.Cliente
    nota.infNFe.emit.xFant.valor = emitente.Cliente
    nota.infNFe.emit.enderEmit.xLgr.valor = emitente.Endereco
    nota.infNFe.emit.enderEmit.nro.valor = '%d' %(emitente.Numero or 0)
    nota.infNFe.emit.enderEmit.xCpl.valor = emitente.Complemento
    nota.infNFe.emit.enderEmit.xBairro.valor = emitente.Bairro
    nota.infNFe.emit.enderEmit.cMun.valor = emitente.CidadeIBGE
    nota.infNFe.emit.enderEmit.xMun.valor = emitente.Cidade
    nota.infNFe.emit.enderEmit.UF.valor = emitente.Estado
    nota.infNFe.emit.enderEmit.CEP.valor = emitente.Cep
    nota.infNFe.emit.enderEmit.fone.valor = re.sub('[^0-9]', '', emitente.Telefone)
    nota.infNFe.emit.IE.valor = emitente.InscricaoEstadual
    #
    # Regime tributário
    #
    nota.infNFe.emit.CRT.valor = 3
    #
    # Destinatário
    #
    nota.infNFe.dest.CPF.valor = nfe_capa.destinatario_doc
    nota.infNFe.dest.xNome.valor = nfe_capa.destinatario_nome
    nota.infNFe.dest.enderDest.xLgr.valor = nfe_capa.destinatario_endereco
    nota.infNFe.dest.enderDest.nro.valor = nfe_capa.destinatario_numero
    nota.infNFe.dest.enderDest.xCpl.valor = nfe_capa.destinatario_cpl
    nota.infNFe.dest.enderDest.xBairro.valor = nfe_capa.destinatario_bairro
    nota.infNFe.dest.enderDest.cMun.valor = nfe_capa.destinatario_cmunicipio
    nota.infNFe.dest.enderDest.xMun.valor = nfe_capa.destinatario_municipio
    nota.infNFe.dest.enderDest.UF.valor = nfe_capa.destinatario_uf
    nota.infNFe.dest.enderDest.CEP.valor = nfe_capa.destinatario_cep
    nota.infNFe.dest.enderDest.fone.valor = nfe_capa.destinatario_fone
    nota.infNFe.dest.IE.valor = u'ISENTO'
    nota.infNFe.dest.email.valor = u'emeio@servidor.com.br'
    #
    # Detalhes
    #
    produtos = nfe_capa.item_ids

    #EMITIDA
    v_prod = Decimal(0)
    v_icms = Decimal(0)
    v_bc = Decimal(0)
    v_icms = Decimal(0)

    for prod in produtos:
        d1 = Det_200()
        v_prod += prod.vr_total_item
        v_icms += prod.v_icms
        v_bc += prod.v_bc
        d1.nItem.valor = prod.sequencia
        d1.prod.cProd.valor = prod.codigo_item
        d1.prod.cEAN.valor = u''
        d1.prod.xProd.valor = prod.descricao
        d1.prod.NCM.valor = u'01'
        d1.prod.EXTIPI.valor = u''
        d1.prod.CFOP.valor = prod.cfop_id
        d1.prod.uCom.valor = 'UND' #prod.unidade.upper()
        d1.prod.qCom.valor = prod.quantidade
        d1.prod.vUnCom.valor = prod.vr_unitario
        d1.prod.vProd.valor = prod.vr_total_item
        d1.prod.cEANTrib.valor = u''
        d1.prod.uTrib.valor = 'und' #d1.prod.uCom.valor
        d1.prod.qTrib.valor = d1.prod.qCom.valor
        d1.prod.vUnTrib.valor = d1.prod.vUnCom.valor
        d1.prod.vFrete.valor = u'0.00'
        d1.prod.vSeg.valor = u'0.00'
        d1.prod.vDesc.valor = u'0.00'
        d1.prod.vOutro.valor = u'0.00'
        #
        # Produto entra no total da NF-e
        #
        d1.prod.indTot.valor = 1
        #
        # Imposto ICMS
        #
        d1.imposto.ICMS.regime_tributario = 3
        d1.imposto.ICMS.CSOSN.valor = u'00'
        d1.imposto.ICMS.CST.valor = '%02d' % prod.cst
        d1.imposto.ICMS.orig.valor = prod.orig
        d1.imposto.ICMS.modBC.valor = prod.mod_bc
        d1.imposto.ICMS.vBC.valor = prod.v_bc
        d1.imposto.ICMS.vICMS.valor = prod.v_icms
        d1.imposto.ICMS.pRedBC.valor = prod.p_red_bc
        d1.imposto.ICMS.pICMS.valor = prod.p_icms
        d1.imposto.PIS.CST.valor = u'07'
        d1.imposto.COFINS.CST.valor = u'07'
        nota.infNFe.det.append(d1)
        # Totais
    nota.infNFe.total.ICMSTot.vBC.valor = v_bc
    nota.infNFe.total.ICMSTot.vICMS.valor = v_icms
    nota.infNFe.total.ICMSTot.vProd.valor = v_prod
    #    nota.infNFe.total.ICMSTot.vFrete.valor = 0
    #    nota.infNFe.total.ICMSTot.vDesc.valor = 0
    #    nota.infNFe.total.ICMSTot.vII.valor = 0
    #    nota.infNFe.total.ICMSTot.vCOFINS.valor = 0
    #    nota.infNFe.total.ICMSTot.vOutro.valor = 0
    nota.infNFe.total.ICMSTot.vNF.valor = nfe_capa.vr_total_nf
    # Observação
    nota.infNFe.infAdic.infCpl.valor = nfe_capa.observacao
    #
    # XML Gerado
    #
    xml = nota.xml
    #
    open('nf.xml', 'w').write(xml)

    processo = nf_.processar_nfe(xml, cert, key, scan=bool(motivo_scan))
    return (xml, processo)

#processar_nfe(60)
#
#n = NfeCapa.get_by(id=58)
#print n.nfe_xml_gerado

r =    {'protocolo': u'131110027532184', 'status_motivo': u'Autorizado o uso da NF-e',
        'status_resposta': u'100',
        'envio': u'<?xml version="1.0" encoding="utf-8"?><consSitNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="2.00"><tpAmb>2</tpAmb><xServ>CONSULTAR</xServ><chNFe>31110901526558000178551010000003111160927619</chNFe></consSitNFe>',
        'resposta': u'<?xml version="1.0" encoding="utf-8"?><retConsSitNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="2.00"><tpAmb>2</tpAmb><verAplic>3_3_42</verAplic><cStat>100</cStat><xMotivo>Autorizado o uso da NF-e</xMotivo><cUF>31</cUF><chNFe>31110901526558000178551010000003111160927619</chNFe><protNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="2.00"><infProt Id="ID131110027532184"><tpAmb>2</tpAmb><verAplic>3_3_42</verAplic><chNFe>31110901526558000178551010000003111160927619</chNFe><dhRecbto>2011-09-02T11:34:16</dhRecbto><nProt>131110027532184</nProt><digVal>arpKuITEvPn0OlC8bzPWOYlIRZM=</digVal><cStat>100</cStat><xMotivo>Autorizado o uso da NF-e</xMotivo></infProt></protNFe></retConsSitNFe>',
        'chave_nfe': u'31110901526558000178551010000003111160927619',
        'reason': u'OK'
}
