#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PySide import QtCore, QtGui
from datetime import datetime
from sigpdv.models.db_utils import sess
from sigpdv.models.cad_municipios_data import CaixaCadMunicipios
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.cad_tributacao_data import CaixaCadTributacao
from sigpdv.models.cad_modelos_nfe_data import CaixaCadModelosNfe
from sigpdv.models.mov_cupom_servidor_data import CaixaMovCupomServidor
from sigpdv.models.nfe_capa_data import CaixaNfeCapa
from sigpdv.models.nfe_itens_data import CaixaNfeItens
from sigpdv.models.cad_cfop_data import CaixaCadNfeCfop
from sigpdv.ui.busca_cliente import BuscaCliente
from ui_nfe import Ui_MainWindow
from ui_nfe_detalhe_item import Ui_Dialog
import sys, os
from sigpdv import root_path
import traceback
from gerar import processar_nfe
from decimal import Decimal
UF_CODIGO = dict(AC=12, AL=27, AM=13, AP=16, BA=29, CE=23, DF=53, ES=32, GO=52, MA=21, MG=31, MS=50, MT=51, PA=15, PB=25
                 , PE=26, PI=22, PR=41, RJ=33, RN=24, RO=11, RR=14, RS=43, SC=42, SE=28, SP=35, TO=17)

class XMLHighlighter(QtGui.QSyntaxHighlighter):

    #INIT THE STUFF
    def __init__(self, parent=None):
        super(XMLHighlighter, self).__init__(parent)

        keywordFormat = QtGui.QTextCharFormat()
        keywordFormat.setForeground(QtCore.Qt.darkMagenta)
        keywordFormat.setFontWeight(QtGui.QFont.Bold)

        keywordPatterns = ["\\b?xml\\b", "/>", ">", "<"]

        self.highlightingRules = [(QtCore.QRegExp(pattern), keywordFormat)
                for pattern in keywordPatterns]

        xmlElementFormat = QtGui.QTextCharFormat()
        xmlElementFormat.setFontWeight(QtGui.QFont.Bold)
        xmlElementFormat.setForeground(QtCore.Qt.green)
        self.highlightingRules.append((QtCore.QRegExp("\\b[A-Za-z0-9_]+(?=[\s/>])"), xmlElementFormat))

        xmlAttributeFormat = QtGui.QTextCharFormat()
        xmlAttributeFormat.setFontItalic(True)
        xmlAttributeFormat.setForeground(QtCore.Qt.blue)
        self.highlightingRules.append((QtCore.QRegExp("\\b[A-Za-z0-9_]+(?=\\=)"), xmlAttributeFormat))

        self.valueFormat = QtGui.QTextCharFormat()
        self.valueFormat.setForeground(QtCore.Qt.red)

        self.valueStartExpression = QtCore.QRegExp("\"")
        self.valueEndExpression = QtCore.QRegExp("\"(?=[\s></])")

        singleLineCommentFormat = QtGui.QTextCharFormat()
        singleLineCommentFormat.setForeground(QtCore.Qt.gray)
        self.highlightingRules.append((QtCore.QRegExp("<!--[^\n]*-->"), singleLineCommentFormat))
        #VIRTUAL FUNCTION WE OVERRIDE THAT DOES ALL THE COLLORING
    def highlightBlock(self, text):

        #for every pattern
        for pattern, format in self.highlightingRules:

            #Create a regular expression from the retrieved pattern
            expression = QtCore.QRegExp(pattern)

            #Check what index that expression occurs at with the ENTIRE text
            index = expression.indexIn(text)

            #While the index is greater than 0
            while index >= 0:

                #Get the length of how long the expression is true, set the format from the start to the length with the text format
                length = expression.matchedLength()
                self.setFormat(index, length, format)

                #Set index to where the expression ends in the text
                index = expression.indexIn(text, index + length)

        #HANDLE QUOTATION MARKS NOW.. WE WANT TO START WITH " AND END WITH ".. A THIRD " SHOULD NOT CAUSE THE WORDS INBETWEEN SECOND AND THIRD TO BE COLORED
        self.setCurrentBlockState(0)

        startIndex = 0
        if self.previousBlockState() != 1:
            startIndex = self.valueStartExpression.indexIn(text)

        while startIndex >= 0:
            endIndex = self.valueEndExpression.indexIn(text, startIndex)

            if endIndex == -1:
                self.setCurrentBlockState(1)
                commentLength = len(text) - startIndex
            else:
                commentLength = endIndex - startIndex + self.valueEndExpression.matchedLength()

            self.setFormat(startIndex, commentLength, self.valueFormat)

            startIndex = self.valueStartExpression.indexIn(text, startIndex + commentLength);

def get_cod_municipio(nome_municipio):
    flike = CaixaCadMunicipios.nome.like('%s%s%s' % ('%', nome_municipio, '%'))
    cod_mun = CaixaCadMunicipios.query.filter(flike)
    if cod_mun.count() == 1:
        cod_mun = cod_mun.one().id
    else:
        cod_mun = 0
    return cod_mun


class ItensForm(QtGui.QDialog):
    def __init__(self, parent=None):
        super(ItensForm, self).__init__()

        self.ui = Ui_Dialog()

        self.ui.setupUi(self)

        self.parent = parent
        self._produto_id = None
        self._trib = None
        # SIGNALS
        self.ui.buttonBuscarProduto.pressed.connect(self.buttonBuscarProduto_click)
        self.ui.tableBuscaProduto.cellDoubleClicked.connect(self.tableBuscaProduto_click)
        self.ui.editCFOP.editingFinished.connect(self.editCFOP_finish)
        self.ui.spinDesc.valueChanged.connect(self._calcular_preco_total)
        self.ui.spinPreco.valueChanged.connect(self._calcular_preco_total)
        self.ui.spinQtda.valueChanged.connect(self._calcular_preco_total)
        self.ui.buttonCancelar.clicked.connect(self.buttonbox_click)
        self.ui.buttonOk.clicked.connect(self.buttonbox_click)
        # AJUSTES DE CONTROLES
        self.ui.spinNfeId.setValue(parent._nfe.id)
        self.ui.editBuscaProduto.setFocus()
        self.ui.stackedWidget.setCurrentIndex(1)
        self.ui.buttonOk.setText('Selecionar')
        # AJUSTES DE CONTROLES REQUERIDOS
        pal = QtGui.QPalette()
        pal.setColor(QtGui.QPalette.Base, QtCore.Qt.green)

        requeridos = [self.ui.editCFOP, self.ui.spinQtda, self.ui.spinPreco]
        self.requeridos = requeridos
        for w in requeridos:
            w.setPalette(pal)
        self.retorno = None

    def buttonbox_click(self):
        def check_erro():
            pal_err = QtGui.QPalette()
            pal_err.setColor(QtGui.QPalette.Base, QtCore.Qt.red)
            pal_ok = QtGui.QPalette()
            pal_ok.setColor(QtGui.QPalette.Base, QtCore.Qt.green)
            erro = False
            prime_control = False

            for w in self.requeridos:
                if type(w) == QtGui.QLineEdit:
                    if not w.text():
                        erro = True
                        if not prime_control:
                            prime_control = w
                        w.setPalette(pal_err)
                    else:
                        w.setPalette(pal_ok)
                elif type(w) == QtGui.QDoubleSpinBox:
                    if w.value() == 0:
                        erro = True
                        if not prime_control:
                            prime_control = w
                        w.setPalette(pal_err)
                    else:
                        w.setPalette(pal_ok)
            if erro:
                QtGui.QMessageBox.information(self, u'Sig NF-e', u'Campo(s) requerido(s) sem prenche ou com valor inválido.')
                prime_control.setFocus()
                prime_control.selectAll()
            return erro

        if self.ui.buttonOk.hasFocus():
            if self.ui.buttonOk.text() == 'Selecionar':
                if self.ui.tableBuscaProduto.selectedItems():
                    self.tableBuscaProduto_click(self.ui.tableBuscaProduto.currentRow())
                else:
                    QtGui.QMessageBox.information(self, u'Sig NF-e', 'Selecione um item p/ continuar ou clique em [Cancelar]')
            else:
                try:
                    if check_erro():
                        return
                    seq = CaixaNfeItens.query.filter(CaixaNfeItens.nf_id == self.ui.spinNfeId.value()).count() + 1
                    sess.begin()
                    res = CaixaNfeItens(
                        descricao=self.ui.editDescProduto.text(), nf_id=self.ui.spinNfeId.value(),
                        sequencia=seq, codigo_item=self.ui.editCodItem.text(), quantidade=self.ui.spinQtda.value(),
                        unidade=self.ui.editUnd.text(), vr_desconto=self.ui.spinDesc.value(),
                        vr_unitario=self.ui.spinPreco.value(), vr_preco_venda=self.ui.spinPreco.value(),
                        vr_total_item=self.ui.spinTotal.value(), cfop_id=int(self.ui.editCFOP.text()),
                        vr_acrescimos=0, orig=int(self.ui.combo_orig.currentText().split('-')[0]),
                        cst=int(self.ui.combo_cst.currentText().split('-')[0]),
                        mod_bc=int(self.ui.combo_mod_bc.currentText().split('-')[0]),
                        p_icms=self.ui.spin_p_icms.value(), p_red_bc=self.ui.spin_p_red_bc.value(),
                        v_bc=self.ui.spin_v_bc.value(), v_icms=self.ui.spin_v_icms.value()
                    )
                    sess.commit()
                    self.retorno = res
                    self.close()
                except Exception as e:
                    sess.rollback()
                    self.retorno = None
                    QtGui.QMessageBox.critical(self, u'Sig NF-e', str(e), QtGui.QMessageBox.Ok)
        else:
            self.close()


    def editCFOP_finish(self):
        cod_cfop = self.ui.editCFOP.text()
        cfop = CaixaCadNfeCfop.get_by(codigo=cod_cfop)
        if cod_cfop and self.isActiveWindow():
            if cfop:
                self.ui.editDescCfop.setText(cfop.descricao)
            else:
                QtGui.QMessageBox.information(None, u'Sig NF-e', u'CFOP: %s, não cadastrado. !' % cod_cfop,
                                              QtGui.QMessageBox.Ok)
                self.ui.editCFOP.clear()
                self.ui.editCFOP.setFocus()

    def tableBuscaProduto_click(self, row):
        self.ui.buttonOk.setText('Salvar')
        self.ui.stackedWidget.setCurrentIndex(0)
        self.ui.editCFOP.setFocus()
        cod_item = self.ui.tableBuscaProduto.item(row, 0).text()
        self._update_widgets(cod_item)


    def buttonBuscarProduto_click(self):
        texto_busca = self.ui.editBuscaProduto.text()
        if texto_busca:
            if self.ui.comboFiltroBusca.currentIndex() == 0: # Busca por Código do Produto
                cod = texto_busca.rjust(14, '0')
                res = CaixaCadItens.query.filter(CaixaCadItens.CodigoItem == cod)
            else:
                flike = CaixaCadItens.Descricao.like('%s%s%s' % ('%', texto_busca, '%'))
                res = CaixaCadItens.query.filter(flike).order_by(CaixaCadItens.Descricao).limit(20)
            if res and res.count() > 0:
                self._update_table_busca(res)
                self.ui.tableBuscaProduto.setFocus()
            else:
                QtGui.QMessageBox.information(None, u'Sig NF-e', u'Item não cadastrado!', QtGui.QMessageBox.Ok)
                self.ui.editBuscaProduto.clear()
                self.ui.editBuscaProduto.setFocus()
        else:
            QtGui.QMessageBox.information(None, u'Sig NF-e', u'Digite algum texto para efetuar a busca.',
                                          QtGui.QMessageBox.Ok)
            self.ui.editBuscaProduto.setFocus()
            #        self._button_buscar_nfe_click()

    def _calcular_preco_total(self):
        valor = self.ui.spinPreco.value() * self.ui.spinQtda.value()
        desc = (valor * self.ui.spinDesc.value()) / 100
        total = valor - desc
        self.ui.spinTotal.setValue(total)
        if self._trib:
            trib = self._trib

            v_aliq = float(trib.Aliquota)
            v_base = (float(trib.Base) * total) / 100
            v_base_r = 100 - trib.Base
            v_icms = (v_base * v_aliq) / 100
            self.ui.spin_p_icms.setValue(v_aliq)
            self.ui.spin_v_bc.setValue(v_base)
            self.ui.spin_p_red_bc.setValue(v_base_r)
            self.ui.spin_v_icms.setValue(v_icms)


    def _update_widgets(self, cod_item):
        item = CaixaCadItens.get_by(CodigoItem=cod_item, supite_loja=self.parent.loja.supcfc_loja)
        self.ui.editDescProduto.setText(item.Descricao)
        self.ui.editCodItem.setText(item.CodigoItem)
        self.ui.spinPreco.setValue(item.Preco)
        self.ui.editUnd.setText(item.Unidade)
        trib = CaixaCadTributacao.get_by(CodigoTribEcf=item.Tributacao)
        self._trib = trib

        if trib.TipoImposto == 'M':
            if trib.TipoTributacao == 'T':
                v_cst = 0 # 00 - Tributada integralmente
            elif trib.TipoTributacao == 'F':
                v_cst = 1 # 10 - Tributada e com cobrança do ICMS por substituição tributária
            elif trib.TipoTributacao == 'I':
                v_cst = 4 # 40 - Isenta
            elif trib.TipoTributacao == 'N':
                v_cst = 5 # 41 - Não Tributada
        else:
            v_cst = 5 # 41 - Não Tributada
        self.ui.combo_cst.setCurrentIndex(v_cst)
        self._calcular_preco_total()
        self.editCFOP_finish()

    def _update_table_busca(self, itens):
        tableBuscaProduto = self.ui.tableBuscaProduto
        # Apaga lista de itens
        rows_count = tableBuscaProduto.rowCount()
        # Verifica se hà dados
        if rows_count:
            rows = range(0, rows_count + 1)
            rows.reverse()
            for r in rows:
                tableBuscaProduto.removeRow(r)
        rows = 0
        for produto in itens:
            tableBuscaProduto.insertRow(rows)
            # Codigo Item
            newItem = QtGui.QTableWidgetItem(produto.CodigoItem)
            # Descrição
            newDesc = QtGui.QTableWidgetItem(produto.Descricao)
            #            newDesc.setFlags(QtCore.Qt.ItemIsEnabled)
            # Unidade
            newUnid = QtGui.QTableWidgetItem(produto.Unidade)
            newUnid.setTextAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)
            # Valor Unitário
            newVUni = QtGui.QTableWidgetItem('%1.2f' % produto.Preco)
            newVUni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            # WIDGET OS NEW ITENS NA TABLE ITENS
            tableBuscaProduto.setItem(rows, 0, newItem)
            tableBuscaProduto.setItem(rows, 1, newDesc)
            tableBuscaProduto.setItem(rows, 2, newUnid)
            tableBuscaProduto.setItem(rows, 3, newVUni)
            rows += 1
        tableBuscaProduto.resizeColumnsToContents()

    def keyPressEvent(self, ev):
        if ev.key() == QtCore.Qt.Key_Return:
            table_busca_produto = self.ui.tableBuscaProduto
            if table_busca_produto.hasFocus() and table_busca_produto.rowCount() > 0:
                if table_busca_produto.currentItem().isSelected():
                    self.tableBuscaProduto_click(self.ui.tableBuscaProduto.currentRow())

    def closeEvent(self, ev):
        return self.ui.editBuscaProduto.text()


class NfeForm(QtGui.QMainWindow):
    def __init__(self, parent=None):
        super(NfeForm, self).__init__()

        self.ui = Ui_MainWindow()

        self.ui.setupUi(self)

        self._nfe = None
        # SIGNALS
        self.ui.comboUf.currentIndexChanged.connect(self._update_municipios)
        self.ui.buttonAdicionarItem.pressed.connect(self._abrir_form_item)
        self.ui.buttonNovaNfe.pressed.connect(self._button_nova_nfe_click)
        self.ui.buttonBuscarNfe.pressed.connect(self._button_buscar_nfe_click)
        self.ui.comboModelosNfe.currentIndexChanged.connect(self._update_serie_numeracao)
        self.ui.buttonOk.clicked.connect(self._button_ok_click)
        self.ui.buttonEnviarNf.clicked.connect(self._button_enviar_click)
        self.ui.buttonCancelarNfe.clicked.connect(self._button_cancelar_click)

        # AJUSTAR CONTROLES
        for m in CaixaCadModelosNfe.query.filter(CaixaCadModelosNfe.supmnf_active==True):
            self.ui.comboModelosNfe.addItem('%s - %s' % (m.codigo, m.descricao))
            # CONTROLES REQUERIDOS
        
        idx_default = self.ui.comboModelosNfe.findText(u'55 - Nota Fiscal Eletronica')
        self.ui.comboUf.setCurrentIndex(self.ui.comboUf.findText('MG'))
        self._update_serie_numeracao()
        self._update_municipios()
        self._button_buscar_nfe_click()
        self.ui.stackedWidget.setCurrentIndex(1)
        self.ui.plainTextEdit.document().setModified(False)
        self.ui.plainTextEdit.setVisible(False)
        self.ui.frameInfo.setVisible(False)
        self.ui.buttonCancelarNfe.setVisible(True)
        self.ui.buttonEnviarNf.setVisible(True)
        self.ui.buttonNovaNfe.setVisible(True)
        self.ui.buttonOk.setVisible(False)
        if idx_default:
            self.ui.comboModelosNfe.setCurrentIndex(idx_default)
        pal = QtGui.QPalette()
        pal.setColor(QtGui.QPalette.Base, QtCore.Qt.green)

        requeridos = [self.ui.editDestinatario]
        for w in requeridos:
            w.setPalette(pal)
        #Create a QTextEdit widget
        xmlviewer = self.ui.textXmlView

        #Create our XMLHighlighter derived from QSyntaxHighlighter
        XMLHighlighter(xmlviewer.document())

        #Set some xml code by hand or with the use of the xml.dom.minidom.Document class
#        xmlviewer.setPlainText(open('teste.xml').read())
    def exception_message_box(self, text):
        """
        Extract exception information and display a message box with exception details in
        a details extension of the message.
        """
        msg = QtGui.QMessageBox()
        msg.setWindowTitle(QtGui.QApplication.applicationName())
        msg.setText( text )
        #        msg.setInformativeText(e)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        msg.setDetailedText( ''.join(traceback.format_exception(exc_type, exc_value, exc_traceback)) )
        msg.setStandardButtons( QtGui.QMessageBox.Ok )
        return msg.exec_()

    def enviar_nfe(self, nfe_id):
        self.ui.frameInfo.setVisible(True)
        QtGui.QApplication.processEvents()
        try:
            motivo_scan = self.ui.edtMotivoScan.text()
            if self.ui.groupScan.isChecked():
                if len(motivo_scan) < 15:
                    raise Exception('Motivo informado deve conter ao menos 15 caracteres.')
            else:
                motivo_scan = False
            nfe = CaixaNfeCapa.get_by(id=nfe_id)
            ret = processar_nfe(nfe_id, motivo_scan=motivo_scan)
            retxml = ret[0]
            retval = ret[1]

            if retxml and not nfe.nfe_xml_gerado:
                sess.begin()
                nfe.nfe_xml_gerado = retxml
                self.ui.textXmlView.setPlainText(retxml)
                sess.commit()

            if 'reason' in retval:
                if retval['reason'] != u'OK':
                    raise Exception(retval['reason'])

            if 'status_resposta' in retval:
                if retval['status_resposta'] == u'100':
                    sess.begin()
                    nfe.state = retval['status_resposta']
                    nfe.nfe_status = retval['status_motivo']
                    sess.commit()
                    self.grava_cupom_servidor(nfe.data_emissao, nfe.id)
                    if '</nfeProc>' in retval['resposta']:
                        nfe.nfe_xml_proc = retval['resposta']
                else:
                    raise Exception(retval['status_motivo'])

            if 'numero_lote' in retval and not nfe.nfe_lote:
                sess.begin()
                nfe.nfe_lote = retval['numero_lote']
                sess.commit()
                self.ui.editNfeLote.setText(retval['numero_lote'])

            if 'protocolo' in retval and not nfe.nfe_protocolo:
                sess.begin()
                nfe.nfe_protocolo = retval['protocolo']
                sess.commit()
                self.ui.editNfeProtocolo.setText(retval['protocolo'])

            if 'chave_nfe' in retval and not nfe.nfe_chave_acesso:
                sess.begin()
                nfe.nfe_chave_acesso = retval['chave_nfe']
                sess.commit()
                self.ui.editNfeChaveAcesso.setText(retval['chave_nfe'])
            self.ui.frameInfo.setVisible(False)
            msg = '%s - %s' % (retval['status_resposta'], retval['status_motivo'])
            QtGui.QMessageBox.information(self, u'Sig NF-e', msg)
            state = nfe.state
            self.ui.spinNfeStatus.setValue(int(state))
            self.ui.editNfeStatusStr.setText(retval['status_motivo'])
            self.ui.tabWidget.setCurrentIndex(1)
            self._button_buscar_nfe_click()
        except Exception as e:
            traceback.print_exc()
            self.exception_message_box(str(e))
            self.ui.frameInfo.setVisible(False)
            sess.rollback()

    def _button_enviar_click(self):
        if self.ui.stackedWidget.currentIndex() == 1:
            if self.ui.tableBuscaNfe.selectedItems():
                id_nf = self.ui.tableBuscaNfe.item(self.ui.tableBuscaNfe.currentRow(), 0).text()
                modelo_nf = self.ui.tableBuscaNfe.item(self.ui.tableBuscaNfe.currentRow(), 1).text()
                status_nf = self.ui.tableBuscaNfe.item(self.ui.tableBuscaNfe.currentRow(), 8).text()
                if modelo_nf == '55':
                    if 'Aguardando Envio' in status_nf:
                        self.enviar_nfe(id_nf)
                    else:
                        QtGui.QMessageBox.information(self, u'Sig NF-e', u'Não posso enviar NF com o estado em% s' % status_nf)
                else:
                    QtGui.QMessageBox.information(self, u'Sig NF-e', u'Só é permitido o envio de Nota Fiscal Eletrônica.')
            else:

                QtGui.QMessageBox.information(self, u'Sig NF-e', 'Selecione um item da lista.')

    def _button_cancelar_click(self):
        if self.ui.stackedWidget.currentIndex() == 1:
            if self.ui.tableBuscaNfe.selectedItems():
                current_idx = self.ui.tableBuscaNfe.currentIndex()
                id_nf = self.ui.tableBuscaNfe.item(self.ui.tableBuscaNfe.currentRow(), 0).text()
                modelo_nf = self.ui.tableBuscaNfe.item(self.ui.tableBuscaNfe.currentRow(), 1).text()

                nota = CaixaNfeCapa.get_by(id=int(id_nf))
                if nota.state == 0:
                    QtGui.QMessageBox.information(self, u'Sig NF-e', u'A nota: %010d, já foi cancelada.' % nota.numero_nf)
                else:
                    if QtGui.QMessageBox.question(self, u'Sig NF-e', 'Deseja cancelar a nota: %010d' % nota.numero_nf, QtGui.QMessageBox.Yes|QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
    #                        if modelo_nf == '55' and nota.state == 100:
    #                            print 'cancelar nfe'
    #                        else:
                        nota = CaixaNfeCapa.get_by(id=int(id_nf))
                        old_state = nota.state
                        sess.begin()
                        nota.state = 0
                        nota.nfe_status = 'Cancelada'
                        sess.commit()
                        self._button_buscar_nfe_click()
                        self.ui.tableBuscaNfe.setCurrentIndex(current_idx)
                        if old_state in [1, 100]:
                            self.grava_cupom_servidor(nota.data_emissao, nota.id)
            else:
                QtGui.QMessageBox.information(self, u'Sig NF-e', u'Selecione um item da lista.')

    def grava_cupom_servidor(self, data, nro_cupom, enviado='N'):
        try:
            sess.begin()
            c_existes = CaixaMovCupomServidor.get_by(DataCupom=data, Loja=1, Caixa=999, NroCupom=nro_cupom)
            if c_existes:
                c_existes.Enviado = 'N'
            else:
                CaixaMovCupomServidor(DataCupom=data, Loja=1, Caixa=999, NroCupom=nro_cupom,Enviado=enviado)
            sess.commit()
        except Exception as e:
            sess.rollback()
            QtGui.QMessageBox.critical(self, u'Sig NF-e', str(e), QtGui.QMessageBox.Ok)

        print CaixaMovCupomServidor.get_by(DataCupom=data, Caixa=999, NroCupom=nro_cupom)
                
    def _button_ok_click(self, volta=True):

        if QtGui.QMessageBox.question(self, u'Sig NF-e', 'Deseja salvar a nota: %010d' % self._nfe.numero_nf, QtGui.QMessageBox.Yes|QtGui.QMessageBox.No) == QtGui.QMessageBox.No:
            return
        try:
            cod_modelo = self.ui.comboModelosNfe.currentText().split('-')[0]
            modelo_nfe = CaixaCadModelosNfe.get_by(codigo=cod_modelo)
            state = 1
            status = 'Emitida'
            if cod_modelo == '55 ':
                state = 2
                status = 'Aguardando Envio'
            sess.begin()
            self._nfe.state = state
            self._nfe.nfe_status = status
            self._nfe.destinatario_doc = self.ui.editDestDoc.text()[4:]
            self._nfe.destinatario_nome = self.ui.editDestinatario.text()
            self._nfe.destinatario_uf = self.ui.comboUf.currentText()
            self._nfe.destinatario_municipio = self.ui.comboMunicipio.currentText()
            self._nfe.destinatario_cep = self.ui.editCep.text()
            self._nfe.destinatario_endereco = self.ui.editEndereco.text()
            self._nfe.destinatario_bairro = self.ui.editBairro.text()
            self._nfe.destinatario_numero = self.ui.editNumero.text()
            self._nfe.destinatario_cmunicipio = self.ui.spinCMunicipio.value()
            self._nfe.destinatario_cpl = self.ui.editComplemento.text()
            self._nfe.modelo_nf = self.ui.comboModelosNfe.currentText()[:2]
            self._nfe.serie_nf = self.ui.comboSerieNf.currentText()
            self._nfe.numero_nf = self.ui.spinNumero.value()
            modelo_nfe.numeracao = self.ui.spinNumero.value()
            self._nfe.data_saida_entrada = self.ui.dateSaidaEntrada.dateTime().toPython()
            self._nfe.condicao_pgto_id = int(self.ui.comboFormaPgto.currentText()[:1])
            self._nfe.observacao = self.ui.editObs.text()
            sess.commit()
            if state == 1:
                self.grava_cupom_servidor(self._nfe.data_emissao, self._nfe.id)
            if volta:
                self.ui.stackedWidget.setCurrentIndex(1)
                self._button_buscar_nfe_click()
                self.ui.buttonCancelarNfe.setVisible(True)
                self.ui.buttonEnviarNf.setVisible(True)
                self.ui.buttonNovaNfe.setVisible(True)
                self.ui.buttonOk.setVisible(False)
        except Exception as e:
            QtGui.QMessageBox.critical(self, u'Sig NF-e', str(e), QtGui.QMessageBox.Ok)
            sess.rollback()


    def _update_serie_numeracao(self):
        cod_modelo = self.ui.comboModelosNfe.currentText().split('-')[0]
        if cod_modelo:
            modelo_nfe = CaixaCadModelosNfe.get_by(codigo=cod_modelo)
            self.ui.spinNumero.setValue(modelo_nfe.numeracao + 1)
            self.ui.comboSerieNf.clear()
            if self.ui.groupScan.isChecked():
                modelo_nfe.serie += 900
            self.ui.comboSerieNf.addItem('%03d' % (modelo_nfe.serie + 1))


    def _update_municipios(self):
        uf = self.ui.comboUf.currentText()
        if uf:
            cod_uf = UF_CODIGO[uf]
            self.ui.comboMunicipio.clear()
            for x in CaixaCadMunicipios.query.filter(CaixaCadMunicipios.supmun_iduf == cod_uf).order_by('name'):
                self.ui.comboMunicipio.addItem(x.name)
            self.ui.comboMunicipio.setFocus()

    def _button_buscar_nfe_click(self):
        table_nfe = self.ui.tableBuscaNfe
        # Apaga lista de itens
        rows_count = table_nfe.rowCount()
        # Verifica se hà dados
        if rows_count:
            rows = range(0, rows_count + 1)
            rows.reverse()
            for r in rows:
                table_nfe.removeRow(r)
        rows = 0
        nfs = CaixaNfeCapa.query.order_by('data_emissao,id').all()
        for nf in nfs:
            table_nfe.insertRow(rows)
            #BRUSH
            if nf.state == -1:
                brush = QtGui.QBrush(QtCore.Qt.gray)
            elif nf.state == 0:
                brush = QtGui.QBrush(QtCore.Qt.magenta)
            elif nf.state == 1:
                brush = QtGui.QBrush(QtCore.Qt.cyan)
            elif nf.state == 2:
                brush = QtGui.QBrush(QtCore.Qt.yellow)
            elif nf.state == 100:
                brush = QtGui.QBrush(QtCore.Qt.green)
            else:
                brush = QtGui.QBrush(QtCore.Qt.darkCyan)
            brush.setStyle(QtCore.Qt.SolidPattern)
            # ID
            item_id = QtGui.QTableWidgetItem('%d' % nf.id)
            item_id.setBackground(brush)
            # modelo
            item_mod = QtGui.QTableWidgetItem(nf.modelo_nf)
            item_mod.setBackground(brush)
            # Numero NFE
            num_nf = '*' * 10
            if nf.numero_nf:
                num_nf = '%010d' % nf.numero_nf
            item_num = QtGui.QTableWidgetItem(num_nf)
            item_num.setBackground(brush)
            # NF Serie
            nserie = '000'
            if nf.serie_nf:
                nserie = nf.serie_nf
            item_serie = QtGui.QTableWidgetItem(nserie )
            item_serie.setBackground(brush)
            # Data Emissao
            item_data_emis = QtGui.QTableWidgetItem(nf.data_emissao.strftime("%d/%m/%Y"))
            item_data_emis.setBackground(brush)
            # Total NFe
            item_total = QtGui.QTableWidgetItem('%1.2f' % nf.vr_total_nf)
            item_total.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            item_total.setBackground(brush)
            # CNPJ/CPFHEAD
            item_dest_doc = QtGui.QTableWidgetItem(nf.destinatario_doc)
            item_dest_doc.setBackground(brush)
            # NOME CLIENTE
            item_dest_nome = QtGui.QTableWidgetItem('  %s  ' % nf.destinatario_nome)
            item_dest_nome.setBackground(brush)
            # STATUS
            item_status = QtGui.QTableWidgetItem('  %s' % nf.nfe_status)
            item_status.setBackground(brush)
            # WIDGET OS NEW ITENS NA TABLE ITENS
            table_nfe.setItem(rows, 0, item_id)
            table_nfe.setItem(rows, 1, item_mod)
            table_nfe.setItem(rows, 2, item_num)
            table_nfe.setItem(rows, 3, item_serie)
            table_nfe.setItem(rows, 4, item_data_emis)
            table_nfe.setItem(rows, 5, item_total)
            table_nfe.setItem(rows, 6, item_dest_doc)
            table_nfe.setItem(rows, 7, item_dest_nome)
            table_nfe.setItem(rows, 8, item_status)
            rows += 1
            self.ui.tableBuscaNfe.resizeColumnsToContents()

    def _button_nova_nfe_click(self):
        self._update_serie_numeracao()
        busca_cliente_form = BuscaCliente(db='C')
        if busca_cliente_form.exec_():
            cliente = busca_cliente_form.retorno
            try:
                self.ui.tabWidget.setCurrentIndex(0)
                sess.begin()
                nfe_capa = CaixaNfeCapa(destinatario_doc=cliente.CNPJCPF, destinatario_nome=cliente.Nome,
                                   destinatario_uf=cliente.Estado, destinatario_municipio=cliente.Cidade,
                                   destinatario_cmunicipio=cliente.CodMunicipio, destinatario_cep=cliente.Cep,
                                   destinatario_endereco=cliente.Endereco, destinatario_bairro=cliente.Bairro,
                                   destinatario_numero=cliente.EnderecoNumero, natureza='Venda',
                                   numero_nf=self.ui.spinNumero.value(),
                                   serie=self.ui.comboSerieNf.currentText())
                sess.commit()
                self._nfe = nfe_capa
                # POPULAR CONTROLES
                self.ui.editDestDoc.setText(cliente.CNPJCPF)
                self.ui.editDestinatario.setText(cliente.Nome)
                self.ui.editEndereco.setText(cliente.Endereco)
                uf = ''
                if cliente.Estado:
                    uf = cliente.Estado.upper()
                self.ui.comboUf.setCurrentIndex(self.ui.comboUf.findText(uf))
                self.ui.comboMunicipio.setCurrentIndex(self.ui.comboMunicipio.findText(cliente.Cidade))
                self.ui.editCep.setText(cliente.Cep)
                self.ui.editBairro.setText(cliente.Bairro)
                self.ui.editNumero.setText(cliente.EnderecoNumero)
                self.ui.spinCMunicipio.setValue(cliente.CodMunicipio)
                self.ui.dateEmissao.setDate(datetime.today().date())
                self.ui.dateSaidaEntrada.setDateTime(datetime.now())
                self.ui.stackedWidget.setCurrentIndex(0)
                self.ui.buttonAdicionarItem.setFocus()
                self.ui.buttonCancelarNfe.setVisible(False)
                self.ui.buttonEnviarNf.setVisible(False)
                self.ui.buttonNovaNfe.setVisible(False)
                self.ui.buttonOk.setVisible(True)
                self._abrir_form_item()
            except Exception as e:
                QtGui.QMessageBox.critical(self, u'Sig NF-e', str(e), QtGui.QMessageBox.Ok)
                sess.rollback()

    def _abrir_form_item(self):
        frm_itens = ItensForm(self)
        frm_itens.exec_()
        #        frm_itens.retorno
        self._update_table_itens()

    def _update_table_itens(self):
        table_itens = self.ui.tableItens
        rows_count = table_itens.rowCount()
        # Verifica se hà dados
        if rows_count:
            rows = range(0, rows_count + 1)
            rows.reverse()
            for r in rows:
                table_itens.removeRow(r)
        rows = 0
        total_nfe = Decimal('0')
        for i in self._nfe.item_ids:
            table_itens.insertRow(rows)
            # sequencia
            item_seq = QtGui.QTableWidgetItem('%03d' % i.sequencia)
            # CODIGO ITEM
            item_codigo = QtGui.QTableWidgetItem(i.codigo_item)
            # Descrição
            item_desc = QtGui.QTableWidgetItem(i.descricao)
            # Unidade
            item_und = QtGui.QTableWidgetItem(i.unidade)
            # Qtda
            item_qtda = QtGui.QTableWidgetItem('%.2d' % i.quantidade)
            # valor unit
            item_vunit = QtGui.QTableWidgetItem('%1.2f' % i.vr_unitario)
            item_vunit.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            # cfop
            item_cfop = QtGui.QTableWidgetItem('%d' % i.cfop_id)
            # bc icms
            item_v_bc = QtGui.QTableWidgetItem('%1.2f' % i.v_bc)
            item_v_bc.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            # valor icms
            item_icms = QtGui.QTableWidgetItem('%1.2f' % i.v_icms)
            item_icms.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            # valor total
            item_total = QtGui.QTableWidgetItem('%1.2f' % i.vr_total_item)
            item_total.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            total_nfe += i.vr_total_item
            # POPULAR
            table_itens.setItem(rows, 0, item_seq)
            table_itens.setItem(rows, 1, item_codigo)
            table_itens.setItem(rows, 2, item_desc)
            table_itens.setItem(rows, 3, item_und)
            table_itens.setItem(rows, 4, item_qtda)
            table_itens.setItem(rows, 5, item_vunit)
            table_itens.setItem(rows, 6, item_cfop)
            table_itens.setItem(rows, 7, item_v_bc)
            table_itens.setItem(rows, 8, item_icms)
            table_itens.setItem(rows, 9, item_total)
            rows += 1
        table_itens.resizeColumnsToContents()
        self.ui.spinTotalNf.setValue(total_nfe)
        try:
            sess.begin()
            self._nfe.vr_total_nf = total_nfe
            print 'total:', total_nfe
            sess.commit()
        except Exception as e:
            QtGui.QMessageBox.critical(self, u'Sig NF-e', str(e), QtGui.QMessageBox.Ok)

def run():
    app = QtGui.QApplication(sys.argv)
    nfeform = NfeForm()
    #    nfeform = ItensForm()
    nfeform.setWindowTitle('SigNfe')
    nfeform.showMaximized()
    qtTranslator = QtCore.QTranslator()
    if qtTranslator.load(os.path.join(root_path, "qt_pt.qm")):
        app.installTranslator(qtTranslator)
        print('installTranslator pt_BR')
    else:
        print('Arquivo de tradução não encontrado')
    app.exec_()

if __name__ == "__main__":
    run()