#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PySide import QtCore, QtGui
import sys
import serial
import os
from unicodedata import normalize, category
from sigpdv.models.db_utils import testa_conexao_rpc, modulo_instalado_producao_comanda
from sigpdv import root_path
from sigpdv.models.cfg_configuracao_data import CaixaCfgConfiguracao
from sigpdv.control.utils import get_loja, set_serial, exec_sql
from stoqdrivers.printers.fiscal import FiscalPrinter
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from stoqdrivers.serialbase import SerialPort
from gui.ui.ui_pdvconfig import Ui_Dialog
from gui.keycode import key_dlg_show
from tools import config
from sigpdv.tools.socat import Socat
from sigpdv.sigtef.interativo import SigTefI
import glob

try:
    from elixir import create_all, setup_all
    setup_all()
    create_all()
except Exception as e:
    print e


ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
TIPOS_TECLADO = {'rb_gertec_44_ps2': 1, 'rb_gertec_44_usb': 2, 'rb_gertec_55_ps2': 3,
                 'rb_gertec_55_usb': 4, 'rb_gertec_65_ps2': 5, 'rb_gertec_65_usb': 6,
                 'rb_gertec_65_unisys': 7, 'rb_smak_12': 8, 'rb_smak_44': 9, 'rb_smak_44_eth': 10,
                 'rb_smak_291_sc': 11, 'rb_smak_291_lcm': 12}

class PdvConfigForm(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self._inifile = config.config
        self.caption = 'PDV Config'
        self.setWindowTitle(u" %s - %s" % (self.caption, self._inifile.rcfile))

        self.ui.cbAtivarRes.setChecked(False)
        self.ui.cbAtivarRes.setDisabled(True)
        self.ui.checkUsarcodificado.stateChanged.connect(self.state_change_checkUsarcodificado)
        self.ui.checkUsarDisplay.stateChanged.connect(self.state_change_checkUsarDisplay)
        self.checa_teclado_reduzido()
        #-----------------------------------------------------------------------------------------------
        # DEFINIR SLOTS P/ OS CONTROLES
        #===============================================================================================
        #        timer = QtCore.QTimer(self)
        #        timer.timeout.connect(self.showTime)
        #        timer.start(1000)
        #        print self.hasFocus()

        self.ui.listWidget.currentItemChanged.connect(self.changePage)
        self.ui.listWidget.setCurrentRow(0)
        self.ui.stackedWidget.setCurrentIndex(0)
        self.ui.btnDirImages.clicked.connect(self.btn_dir_images_click)
        self.ui.btn_checkdb.clicked.connect(self.check_db_connection)
        self.ui.spin_caixa.valueChanged.connect(self._get_ecf_autorizada)
        self.ui.spin_loja.valueChanged.connect(self._get_ecf_autorizada)
        self.ui.btn_conectarEcf.clicked.connect(self.checkEcf)
        self.ui.btn_conectarPinPad.clicked.connect(self.check_pinpad)
        self.ui.btn_conectarSocat.clicked.connect(self.iniciar_socat)
        if config.config['usarcomanda'] or not modulo_instalado_producao_comanda:
            self.ui.edt_printer_conferencia_cmd.setVisible(False)
            self.ui.lbl_impressora_conferencia.setVisible(False)

        # BOTOES
        self.ui.btnKEY_0.clicked.connect(self.btnKEY_0_click)
        self.ui.btnKEY_1.clicked.connect(self.btnKEY_1_click)
        self.ui.btnKEY_2.clicked.connect(self.btnKEY_2_click)
        self.ui.btnKEY_3.clicked.connect(self.btnKEY_3_click)
        self.ui.btnKEY_4.clicked.connect(self.btnKEY_4_click)
        self.ui.btnKEY_5.clicked.connect(self.btnKEY_5_click)
        self.ui.btnKEY_6.clicked.connect(self.btnKEY_6_click)
        self.ui.btnKEY_7.clicked.connect(self.btnKEY_7_click)
        self.ui.btnKEY_8.clicked.connect(self.btnKEY_8_click)
        self.ui.btnKEY_9.clicked.connect(self.btnKEY_9_click)
        self.ui.btnKEY_COMMA.clicked.connect(self.btnKEY_COMMA_click)
        self.ui.btnKEY_QTDA.clicked.connect(self.btnKEY_QTDA_click)
        self.ui.btnKEY_CANCEL_ITENS.clicked.connect(self.btnKEY_CANCEL_ITENS_click)
        self.ui.btnKEY_BALANCA.clicked.connect(self.btnKEY_BALANCA_click)
        self.ui.btnKEY_CONSULTA.clicked.connect(self.btnKEY_CONSULTA_click)
        self.ui.btnKEY_UP.clicked.connect(self.btnKEY_UP_click)
        self.ui.btnKEY_DOWN.clicked.connect(self.btnKEY_DOWN_click)
        self.ui.btnKEY_LEFT.clicked.connect(self.btnKEY_LEFT_click)
        self.ui.btnKEY_RIGHT.clicked.connect(self.btnKEY_RIGHT_click)
        self.ui.btnKEY_DAV_PVE.clicked.connect(self.btnKEY_DAV_PVE_click)
        if config.config['usardav']:
            self.ui.btnKEY_DAV_PVE.setText('DAV')
        elif config.config['usarprevenda']:
            self.ui.btnKEY_DAV_PVE.setText('PVE')
        else: # desabilita o botao btnKEY_DAV_PVE
            self.desabilita_DAV_PVE()

        self.ui.btnKEY_BACKSPACE.clicked.connect(self.btnKEY_BACKSPACE_click)
        self.ui.btnKEY_ESCAPE.clicked.connect(self.btnKEY_ESCAPE_click)
        self.ui.btnKEY_MENU.clicked.connect(self.btnKEY_MENU_click)
        self.ui.btnKEY_MENU_FISCAL.clicked.connect(self.btnKEY_MENU_FISCAL_click)
        self.ui.btnKEY_RETURN.clicked.connect(self.btnKEY_RETURN_click)
        self.ui.btnKEY_SOCIO_TORCEDOR.clicked.connect(self.btnKEY_SOCIO_TORCEDOR_click)
        self.ui.btnKEY_DINHEIRO.clicked.connect(self.btnKEY_DINHEIRO_click)
        self.ui.btnKEY_CHEQUE.clicked.connect(self.btnKEY_CHEQUE_click)
        self.ui.btnKEY_CARTAO.clicked.connect(self.btnKEY_CARTAO_click)
        self.ui.btnKEY_TICKET.clicked.connect(self.btnKEY_TICKET_click)
        self.ui.btnKEY_ESTACIONAMENTO.clicked.connect(self.btnKEY_ESTACIONAMENTO_click)
        self.ui.btnKEY_DOACAO.clicked.connect(self.btnKEY_DOACAO_click)
        self.ui.btnKEY_OUTROS.clicked.connect(self.btnKEY_OUTROS_click)
        self.ui.btnKEY_MENSAGEM.clicked.connect(self.btnKEY_MENSAGEM_click)
        self.ui.btnKEY_ACRESCIMO.clicked.connect(self.btnKEY_ACRESCIMO_click)
        self.ui.btnKEY_CONSULTA_CH.clicked.connect(self.btnKEY_CONSULTA_CH_click)
        self.ui.btnKEY_DESCONTO.clicked.connect(self.btnKEY_DESCONTO_click)
        self.ui.btnKEY_CLIENTE.clicked.connect(self.btnKEY_CLIENTE_click)
        self.ui.btnKEY_VENDEDOR.clicked.connect(self.btnKEY_VENDEDOR_click)
        self.ui.btnKEY_COMANDA.clicked.connect(self.btnKEY_COMANDA_click)
        self.ui.btnKEY_SUPORTE.clicked.connect(self.btnKEY_SUPORTE_click)
        self.ui.btnKEY_RECARGA_CEL.clicked.connect(self.btnKEY_RECARGA_click)
        self.ui.btnCheckKeys.clicked.connect(self.check_keys)
        self.ui.cbAtivarRes.clicked.connect(self.load_parm_restaurante)
        self.ui.edtDirImagens.textChanged.connect(self.on_change_edtDirImagens)
        self.ui.edtDirImagens.editingFinished.connect(self.on_focus_out_edtDirImagens)
        self.ui.chk_usa_dav.stateChanged.connect(self.on_change_chk_dav)
        self.ui.chk_pre_venda.stateChanged.connect(self.on_change_chk_pre_venda)
        self.ui.chk_ficha.stateChanged.connect(self.on_change_chk_ficha)
        self.ui.chk_usa_comanda.stateChanged.connect(self.on_change_chk_usa_comanda)

        self.permitir_salvar = True
        self.msg_aviso = ''

        self.ui.chk_gerou_mfd.setVisible(False)

        self.load_vals()

    def desabilita_DAV_PVE(self):
        self.ui.btnKEY_DAV_PVE.setText('DAV/PVE')
        self.ui.btnKEY_DAV_PVE.setEnabled(False)

    def on_change_chk_ficha(self, value):
        if value:
            self.ui.spin_tot_ficha.setEnabled(True)
            self.ui.spin_tot_devolucao_ficha.setEnabled(True)
        else:
            self.ui.spin_tot_ficha.setEnabled(False)
            self.ui.spin_tot_devolucao_ficha.setEnabled(False)

    def on_change_chk_pre_venda(self, value):
        if value:
            self.ui.btnKEY_DAV_PVE.setText('PVE')
            self.ui.chk_usa_dav.setChecked(False)
            self.ui.btnKEY_DAV_PVE.setEnabled(True)
            
        elif not self.ui.chk_usa_dav.isChecked():
            self.desabilita_DAV_PVE()

    def on_change_chk_dav(self, value):
        if value:
            self.ui.btnKEY_DAV_PVE.setText('DAV')
            self.ui.chk_pre_venda.setChecked(False)
            self.ui.btnKEY_DAV_PVE.setEnabled(True)
            
        elif not self.ui.chk_pre_venda.isChecked():
            self.desabilita_DAV_PVE()

    def on_change_chk_usa_comanda(self, value):
        if value and modulo_instalado_producao_comanda:
            self.ui.edt_printer_conferencia_cmd.setVisible(True)
            self.ui.lbl_impressora_conferencia.setVisible(True)

        else:
            self.ui.edt_printer_conferencia_cmd.setVisible(False)
            self.ui.lbl_impressora_conferencia.setVisible(False)

    def checa_teclado_reduzido(self):
        if not self.ui.checkUsarcodificado.isChecked():
            self.state_change_checkUsarDisplay(False)

    def state_change_checkUsarcodificado(self, value):
        self.ui.checkUsarDisplay.setEnabled(value)
        self.ui.checkUsarDisplay.setChecked(False)

    def state_change_checkUsarDisplay(self, value):
        if value:
            self.ui.rb_gertec_44_usb.setEnabled(True)
            self.ui.rb_gertec_44_ps2.setEnabled(True)
            self.ui.rb_gertec_55_usb.setEnabled(True)
            self.ui.rb_gertec_55_ps2.setEnabled(True)
            self.ui.rb_gertec_65_usb.setEnabled(True)
            self.ui.rb_gertec_65_ps2.setEnabled(True)
            self.ui.rb_gertec_65_unisys.setEnabled(True)
            self.ui.rb_smak_12.setEnabled(True)
            self.ui.rb_smak_44.setEnabled(True)
            self.ui.rb_smak_44_eth.setEnabled(True)
            self.ui.rb_smak_291_sc.setEnabled(True)
            self.ui.rb_smak_291_lcm.setEnabled(True)
            self.ui.lbl_gertec.setEnabled(True)
            self.ui.lbl_smak.setEnabled(True)
            self.ui.lbl_gertec_usb.setEnabled(True)
            self.ui.lbl_gertec_ps2.setEnabled(True)
            self.ui.lbl_gertec_unisys.setEnabled(True)
        else:
            self.ui.rb_gertec_44_usb.setEnabled(False)
            self.ui.rb_gertec_44_ps2.setEnabled(False)
            self.ui.rb_gertec_55_usb.setEnabled(False)
            self.ui.rb_gertec_55_ps2.setEnabled(False)
            self.ui.rb_gertec_65_usb.setEnabled(False)
            self.ui.rb_gertec_65_ps2.setEnabled(False)
            self.ui.rb_gertec_65_unisys.setEnabled(False)
            self.ui.rb_smak_12.setEnabled(False)
            self.ui.rb_smak_44.setEnabled(False)
            self.ui.rb_smak_44_eth.setEnabled(False)
            self.ui.rb_smak_291_sc.setEnabled(False)
            self.ui.rb_smak_291_lcm.setEnabled(False)
            self.ui.lbl_gertec.setEnabled(False)
            self.ui.lbl_smak.setEnabled(False)
            self.ui.lbl_gertec_usb.setEnabled(False)
            self.ui.lbl_gertec_ps2.setEnabled(False)
            self.ui.lbl_gertec_unisys.setEnabled(False)

        return True

    def load_parm_restaurante(self):
        # Metodo responsavel po verificar se a opção restaurantes, bares e similares está ativa.
        # Caso esteja, o sistema então carrega os parametros referente a esse modulo. Caso nao
        # esteja ativado, os parametros sao alterardos para ivisiveis. Esse metodo eh acionado na
        # hora que a aplicacao pdvconfig eh iniciada e quando o botao cbAtivarRes tem alguma acao.

        if self.ui.cbAtivarRes.isChecked():
            pdv_ini = self._inifile
            opt = True
            self.ui.spin_transMesas.setVisible(opt)
            self.ui.spin_confeMesas.setVisible(opt)
            self.ui.spin_mesasAbertas.setVisible(opt)
            self.ui.labelTransMesas.setVisible(opt)
            self.ui.labelConfeMesas.setVisible(opt)
            self.ui.labelMesasAbertas.setVisible(opt)
            self.ui.spin_transMesas.setValue(pdv_ini.get('trans_mesas'))
            self.ui.spin_confeMesas.setValue(pdv_ini.get('confe_mesas'))
            self.ui.spin_mesasAbertas.setValue(pdv_ini.get('mesas_abertas'))
        else:
            opt = False
            self.ui.spin_transMesas.setVisible(opt)
            self.ui.spin_confeMesas.setVisible(opt)
            self.ui.spin_mesasAbertas.setVisible(opt)
            self.ui.labelTransMesas.setVisible(opt)
            self.ui.labelConfeMesas.setVisible(opt)
            self.ui.labelMesasAbertas.setVisible(opt)

    def load_vals(self):
        #-----------------------------------------------------------------------------------------------
        # TRATAR ARQUIVO .INI
        #===============================================================================================
        # VERIFICA EXISTENCIA DO .INI, SE NÃO EXISTIR CRIA UM ARQUIVO PADRÃO
        pdv_ini = self._inifile

        #-----------------------------------------------------------------------------------------------
        # POPULAR OS CONTROLES COM OS DADOS DO ARQUIVO .INI
        #===============================================================================================
        self.ui.spin_loja.setValue(pdv_ini.get('loja'))

        self.ui.spin_caixa.setValue(pdv_ini.get('caixa'))
        # CAIXA MYSQL
        self.ui.edt_sup_hostname.setText(pdv_ini.get('db_sup_host'))
        self.ui.edt_sup_database.setText(pdv_ini.get('db_sup_name'))
        self.ui.edtUserSuper.setText(pdv_ini.get('db_sup_user'))
        self.ui.edtSenhaSuper.setText(pdv_ini.get('db_sup_pass'))

        # supervisor master
        self.ui.edt_sup_hostname_master.setText(pdv_ini.get('db_sup_host_master'))
        self.ui.edt_sup_database_master.setText(pdv_ini.get('db_sup_name_master'))
        self.ui.edtUserSuper_master.setText(pdv_ini.get('db_sup_user_master'))
        self.ui.edtSenhaSuper_master.setText(pdv_ini.get('db_sup_pass_master'))

        self.ui.edt_cx_hostname.setText(pdv_ini.get('db_pdv_host'))
        self.ui.edt_cx_database.setText(pdv_ini.get('db_pdv_name'))
        self.ui.edt_username.setText(pdv_ini.get('db_pdv_user'))
        self.ui.edt_password.setText(pdv_ini.get('db_pdv_pass'))
        self.ui.spin_font_grid.setValue(pdv_ini.get('font_grid'))

        # SUPRIMENTO/SANGRIA
        self.ui.edt_suprimento.setText(str(pdv_ini.get('suprimento')))
        self.ui.edt_sangria_dinheiro.setText(str(pdv_ini.get('sangria_dinheiro')))
        self.ui.edt_sangria_cheque.setText(str(pdv_ini.get('sangria_cheque')))
        self.ui.edt_sangria_cartao.setText(str(pdv_ini.get('sangria_cartao')))
        self.ui.edt_sangria_ticket.setText(str(pdv_ini.get('sangria_ticket')))
        self.ui.edt_sangria_outros.setText(str(pdv_ini.get('sangria_outros')))

        self.check_serial()

        # ECF
        self.ui.spin_printer_speed_port.setValue(pdv_ini.get('printer_speed_port') or 115200)
        fiscal_printer_active = pdv_ini.get('fiscal_printer_active')
        self.ui.group_fiscal_printer.setChecked(fiscal_printer_active)
        if fiscal_printer_active:
            porta_ecf = pdv_ini.get('ecf')
            porta_idx = self.ui.cbx_fiscal_printer_port.findText(porta_ecf)
            if porta_idx < 0:
                self.ui.cbx_fiscal_printer_port.addItem(porta_ecf)
            self.ui.cbx_fiscal_printer_port.setCurrentIndex(
                self.ui.cbx_fiscal_printer_port.findText(porta_ecf))

        #ficha
        if not self.ui.chk_ficha.isChecked():
            self.ui.spin_tot_ficha.setEnabled(False)
            self.ui.spin_tot_devolucao_ficha.setEnabled(False)

        # ECF Virtualizada
        self.ui.group_fiscal_printer_virtual.setChecked(pdv_ini.get('virtual_printer_active'))
        self.ui.edt_ecf_virtual_srv.setText(pdv_ini.get('virtual_printer_srv'))
        self.ui.spin_ecf_port_tcp.setValue(pdv_ini.get('ecf_port_tcp'))

        # Config. Nepos
        self.ui.group_fiscal_printer_nepos.setChecked(pdv_ini.get('config_nepos_active'))
        self.ui.edt_nepos_port.setText(pdv_ini.get('nepos_ip'))
        self.ui.spin_nepos_ip.setValue(pdv_ini.get('nepos_port_tcp'))


        # Totalizador Doacao
        self.ui.spip_totalizador_doacao.setValue(pdv_ini.get('totalizador_doacao'))

        # SCANNER DE DOCUMENTOS
        doc_scan_active = pdv_ini.get('doc_scan_active')
        self.ui.group_doc_scanner.setChecked(doc_scan_active)
        if doc_scan_active:
            self.ui.cbx_doc_scan_port.setCurrentIndex(
                self.ui.cbx_doc_scan_port.findText(pdv_ini.get('scannerdocporta')))
            self.ui.cbx_doc_scan_speed.setCurrentIndex(
                self.ui.cbx_doc_scan_speed.findText(pdv_ini.get('scannerdocbaud')))
            self.ui.spin_doc_scan_solicitaleitura.setValue(pdv_ini.get('scannerdocsolicitaleitura'))
            self.ui.spin_doc_scan_charini.setValue(pdv_ini.get('scannerdoccharinicio'))
            self.ui.spin_doc_scan_charfim.setValue(pdv_ini.get('scannerdoccharfim'))
        # SCANNER DE ITENS
        itens_scan_active = pdv_ini.get('itens_scan_active')
        self.ui.group_itens_scanner.setChecked(itens_scan_active)
        if itens_scan_active:
            porta_scan = pdv_ini.get('scanner')
            porta_idx = self.ui.cbx_itens_scan_port.findText(porta_scan)
            if porta_idx < 0:
                self.ui.cbx_itens_scan_port.addItem(porta_scan)
            self.ui.cbx_itens_scan_port.setCurrentIndex(
                self.ui.cbx_itens_scan_port.findText(porta_scan))
            self.ui.cbx_itens_scan_speed.setCurrentIndex(
                self.ui.cbx_itens_scan_speed.findText(pdv_ini.get('scannerbaud')))
            #self.ui.cbx_itens_scan_stopbits.setCurrentIndex(self.ui.cbx_itens_scan_stopbits.findText(pdv_ini.get('scannerstopbit')))
            #self.ui.cbx_itens_scan_bits.setCurrentIndex(self.ui.cbx_itens_scan_bits.findText(pdv_ini.get('scannertambyte')))
            #self.ui.cbx_itens_scan_parity.setCurrentIndex(self.ui.cbx_itens_scan_parity.findText(pdv_ini.get('scannerparidade')))
        # BALANÇA
        balanca_active = pdv_ini.get('balanca_active')
        self.ui.group_balanca.setChecked(balanca_active)
        self.ui.spin_balanca_qtdadig.setValue(pdv_ini.get('digitos'))
        self.ui.ckb_balanca_dv.setChecked(pdv_ini.get('codigocomdigito'))
        self.ui.spin_balanca_pesomax.setValue(pdv_ini.get('balancapesomaximo'))
        self.ui.spin_balanca_solicitaleitura.setValue(pdv_ini.get('solicitaleitura'))
        self.ui.spin_balanca_charini.setValue(pdv_ini.get('charinicio'))
        self.ui.spin_balanca_charfim.setValue(pdv_ini.get('charfim'))

        if balanca_active:
            porta_bal = pdv_ini.get('balancaporta')
            porta_idx = self.ui.cbx_balanca_port.findText(porta_bal)
            if porta_idx < 0:
                self.ui.cbx_balanca_port.addItem(porta_bal)
            self.ui.cbx_balanca_port.setCurrentIndex(self.ui.cbx_balanca_port.findText(porta_bal))
            self.ui.cbx_balanca_speed.setCurrentIndex(
                self.ui.cbx_balanca_speed.findText(pdv_ini.get('balancabaud')))
            self.ui.cbx_modelo_balanca.setCurrentIndex(
                self.ui.cbx_modelo_balanca.findText(pdv_ini.get('modelo_balanca')))

        # TEF
        self.ui.edt_tef_servidor.setText(pdv_ini.get('servidor'))
        self.ui.edt_tef_empresa.setText(pdv_ini.get('empresa'))
        self.ui.spin_tef_qtdavias.setValue(pdv_ini.get('qtdvias'))
        self.ui.cbx_tef_digitacao.setCurrentIndex(self.ui.cbx_tef_digitacao.findText(pdv_ini.get('digitacaocheque')))
        self.ui.chk_usa_recarga_cel.setChecked(pdv_ini.get('usa_recarga_cel'))
        self.ui.chk_usa_socio_torcedor.setChecked(pdv_ini.get('usa_socio_torcedor'))
        self.ui.spin_totalize_recarga.setValue(pdv_ini.get('totalizador_recarga_cel'))
        self.ui.group_fiscal_printer_3.setChecked(pdv_ini.get('config_pinpad_active'))
        self.ui.cbx_pinpad_port.setCurrentIndex(self.ui.cbx_pinpad_port.findText(pdv_ini.get('porta_pinpad')))

        # PARAMETROS
        self.ui.chk_pre_venda.setChecked(pdv_ini.get('usarprevenda'))
        self.ui.chk_ficha.setChecked(pdv_ini.get('usar_ficha'))
        self.ui.check_print_doacao_2_via.setChecked(pdv_ini.get('imprimir_2via_doacao'))
        self.ui.spin_tot_ficha.setValue(int(pdv_ini.get('tot_ficha')))
        self.ui.spin_tot_devolucao_ficha.setValue(int(pdv_ini.get('tot_devolucao_ficha')))
        self.ui.chk_usa_comanda.setChecked(pdv_ini.get('usarcomanda'))
        self.ui.chk_pedir_autor.setChecked(pdv_ini.get('autorizacaosup'))
        self.ui.chk_mult_itens.setChecked(pdv_ini.get('autorizacaomultiplicaitens'))
        self.ui.chk_disc_outros.setChecked(pdv_ini.get('sangriadiscriminaroutros'))
        self.ui.chk_disc_ch_sangria.setChecked(pdv_ini.get('sangriadiscriminarcheques'))
        self.ui.chk_itens_pesaveis.setChecked(pdv_ini.get('itempesaveletiquetabalanca'))
        self.ui.chk_itens_pesaveis_cda.setChecked(pdv_ini.get('itempesavelcomanda'))
        self.ui.chk_disc_cc_sangria.setChecked(pdv_ini.get('sangriadiscriminarcartoes'))
        self.ui.chk_disc_tick_sangria.setChecked(pdv_ini.get('sangriadiscriminartickets'))
        self.ui.chk_desligar_pdv.setChecked(pdv_ini.get('shutdownaposz'))
        self.ui.chk_usa_dav.setChecked(pdv_ini.get('usardav'))
        self.ui.chk_corta_papel.setChecked(pdv_ini.get('cortapapel'))
        self.ui.chk_informa_limite.setChecked(pdv_ini.get('restricoesclientes'))
        self.ui.chk_permite_consulta.setChecked(pdv_ini.get('consultaprodutos'))
        self.ui.chk_usa_gaveta.setChecked(pdv_ini.get('usagaveta'))
        self.ui.chk_sangria_x_z.setChecked(pdv_ini.get('sangria_x_z_suprimento'))
        self.ui.chk_todos_vlr_sangria.setChecked(pdv_ini.get('sangriamanual'))
        self.ui.chk_confere_dav.setChecked(pdv_ini.get('conferenciadav'))
        self.ui.chk_inverter_sinal.setChecked(pdv_ini.get('invertesinalgaveta'))
        self.ui.cbx_cliente_cod.setCurrentIndex(pdv_ini.get('imprimircpfcodigointerno'))
        self.ui.chk_imprimi_end.setChecked(pdv_ini.get('imprimirendereco'))
        self.ui.chk_imprimi_ie.setChecked(pdv_ini.get('imprimirinscricaoestadual'))
        self.ui.chk_imprimi_cidade.setChecked(pdv_ini.get('imprimircidade'))
        self.ui.spin_excluir_log.setValue(pdv_ini.get('max_dias_log'))
        self.ui.edt_sangria.setText(pdv_ini.get('frase_sangria'))
        hora_z = pdv_ini.get('horaminemitirz').split(':')
        self.ui.time_hora_z.setTime(QtCore.QTime(int(hora_z[0]), int(hora_z[1])))
        self.ui.time_intervencao.setValue(pdv_ini.get('temposupintervencao'))
        self.ui.time_comanda.setValue(pdv_ini.get('time_comanda'))
        self.ui.timeImportacao.setValue(pdv_ini.get('time_importacao'))
        self.ui.edt_printer_conferencia_cmd.setText(pdv_ini.get('impressora_conferencia'))
        self.ui.edt_alterar_data.setText(pdv_ini.get('cmd_alterar_data'))
        self.ui.num_limite_cda.setValue(pdv_ini.get('limite_comanda'))
        self.ui.chkImportFull.setChecked(pdv_ini.get('importacao_full'))
        self.ui.timeExport.setValue(pdv_ini.get('time_export'))
        self.ui.chkTruncate.setChecked(pdv_ini.get('truncar_tabelas'))
        self.ui.chk_itens_descricao_resumida.setChecked(pdv_ini.get('descricao_resumida'))
        self.ui.chk_mfd_mensal.setChecked(pdv_ini.get('mfd_mensal'))
        self.ui.chk_gerou_mfd.setChecked(pdv_ini.get('gerou_mfd'))
        self.ui.chk_usar_biometria.setChecked(pdv_ini.get('usar_biometria'))
        #TECLADO
        self.ui.checkUsarcodificado.setChecked(pdv_ini.get('usarcodificado'))
        self.ui.checkUsarDisplay.setChecked(pdv_ini.get('usar_display'))
        if pdv_ini.get('teclado_selecionado'):
            eval("self.ui.{campo}.setChecked(True)".format(
                                                          campo=pdv_ini.get('teclado_selecionado')))

        # CORESPONDENTE BANCARIO
        self.ui.edt_bb_agencia.setText(pdv_ini.get('bbagencia'))
        self.ui.spin_bb_cheque.setValue(pdv_ini.get('bbfinalizcheque'))
        self.ui.edt_bb_convenio.setText(pdv_ini.get('bbconvenio'))
        self.ui.spin_bb_dinheiro.setValue(pdv_ini.get('bbfinalizdinheiro'))
        self.ui.spin_bb_estorno.setValue(pdv_ini.get('bbtotalizadornfiscale'))
        self.ui.spin_bb_loja.setValue(pdv_ini.get('bbloja'))
        self.ui.spin_bb_recebimento.setValue(pdv_ini.get('bbtotalizadornfiscalr'))
        self.ui.edt_bb_servidor.setText(pdv_ini.get('bbservidor'))
        self.ui.spin_bb_porta.setValue(pdv_ini.get('bbporta'))
        # IMAGENScmd_teamviewer
        self.ui.check_dir_logo_itens.setChecked(pdv_ini.get('check_dir_logo_itens'))
        self.ui.edtDirImagens.setText(pdv_ini.get('local'))
        #RELATORIO GERENCIAL
        self.ui.spin_param_conf.setValue(pdv_ini.get('rg_parametro_conf'))
        self.ui.spin_meios_pgto.setValue(pdv_ini.get('rg_meios_de_pgto'))
        self.ui.spin_identPaf.setValue(pdv_ini.get('rg_identificacao_pafecf'))
        self.ui.spin_rel_geren.setValue(pdv_ini.get('rg_relatorio_gerencial'))
        self.ui.spin_dav_emit.setValue(pdv_ini.get('rg_davs_emitidos'))
        # CONEXAO ERP
        self.ui.edt_dbg_ip.setText(pdv_ini.get('db_ges_host'))
        self.ui.edt_dbg_user.setText(pdv_ini.get('db_ges_user'))
        self.ui.edt_dbg_emp.setText(pdv_ini.get('db_ges_emp'))
        self.ui.edt_dbg_pwd.setText(pdv_ini.get('db_ges_pass'))
        self.ui.edt_dbg_dbname.setText(pdv_ini.get('db_ges_name'))
        self.ui.spin_dbg_port.setValue(pdv_ini.get('db_ges_port'))
        # SUPORTE
        self.ui.edt_cmd_tv.setText(pdv_ini.get('cmd_teamviewer'))
        self.ui.chk_atualiza.setChecked(pdv_ini.get('check_atualiza'))
        self.ui.edt_link_sig.setText(pdv_ini.get('link_sig'))
        self.ui.spin_tries_wget.setValue(pdv_ini.get('tentativas_atualiza'))
        # APP DAV
        self.ui.cbAppDav.setCurrentIndex(self.ui.cbAppDav.findText(pdv_ini.get('aplicativo_dav')))
        # Conexão postgres
        self.ui.edt_pg_pass.setText(pdv_ini.get('pg_pass'))
        self.ui.edt_pg_user.setText(pdv_ini.get('pg_user'))
        self.ui.edt_pg_server.setText(pdv_ini.get('pg_server'))
        self.ui.edt_pg_db.setText(pdv_ini.get('pg_db'))
        self.ui.spin_pg_port.setValue(pdv_ini.get('pg_port'))

        # NOTA FISCAL LEGAL
        try:
            l = get_loja()
            if str(l.Estado).upper() == 'DF':
                self.ui.group_box_nflegal.setEnabled(True)
            else:
                self.ui.group_box_nflegal.setEnabled(False)
        except:
            pass
        self.ui.cb_nflegal.setChecked(pdv_ini.get('nf_legal'))
        # RUMCOBOL
        self.ui.edt_ip.setText(pdv_ini.get('ip_gestao'))
        self.ui.edt_cmd_dav.setText(pdv_ini.get('cmd_cobol'))
        # BARES RESTAURANTES
        self.ui.cbAtivarRes.setChecked(pdv_ini.get('ativar_restaurante'))
        self.load_parm_restaurante()  # Carrega os parametros referentes a restuarantes.

    def btnKEY_0_click(self):
        self.btk_key_press('bt_num_0')

    def btnKEY_1_click(self):
        self.btk_key_press('bt_num_1')

    def btnKEY_2_click(self):
        self.btk_key_press('bt_num_2')

    def btnKEY_3_click(self):
        self.btk_key_press('bt_num_3')

    def btnKEY_4_click(self):
        self.btk_key_press('bt_num_4')

    def btnKEY_5_click(self):
        self.btk_key_press('bt_num_5')

    def btnKEY_6_click(self):
        self.btk_key_press('bt_num_6')

    def btnKEY_7_click(self):
        self.btk_key_press('bt_num_7')

    def btnKEY_8_click(self):
        self.btk_key_press('bt_num_8')

    def btnKEY_9_click(self):
        self.btk_key_press('bt_num_9')

    def btnKEY_COMMA_click(self):
        self.btk_key_press('bt_num_.')

    def btnKEY_QTDA_click(self):
        self.btk_key_press('bt_quantidade')

    def btnKEY_CANCEL_ITENS_click(self):
        self.btk_key_press('bt_cancelar_item')

    def btnKEY_BALANCA_click(self):
        self.btk_key_press('bt_balanca')

    def btnKEY_CONSULTA_click(self):
        self.btk_key_press('bt_consulta')

    def btnKEY_UP_click(self):
        self.btk_key_press('bt_cima')

    def btnKEY_DOWN_click(self):
        self.btk_key_press('bt_baixo')

    def btnKEY_LEFT_click(self):
        self.btk_key_press('bt_esquerda')

    def btnKEY_RIGHT_click(self):
        self.btk_key_press('bt_direita')

    def btnKEY_DAV_PVE_click(self):
        self.btk_key_press('bt_dav_pve')

    def btnKEY_BACKSPACE_click(self):
        self.btk_key_press('bt_limpar')

    def btnKEY_ESCAPE_click(self):
        self.btk_key_press('bt_sair')

    def btnKEY_MENU_click(self):
        self.btk_key_press('bt_menu')

    def btnKEY_MENU_FISCAL_click(self):
        self.btk_key_press('bt_menufiscal')

    def btnKEY_RETURN_click(self):
        self.btk_key_press('bt_entrar')

    def btnKEY_SOCIO_TORCEDOR_click(self):
        self.btk_key_press('bt_socio_torcedor')

    def btnKEY_DINHEIRO_click(self):
        self.btk_key_press('bt_dinheiro')

    def btnKEY_CHEQUE_click(self):
        self.btk_key_press('bt_cheque')

    def btnKEY_CARTAO_click(self):
        self.btk_key_press('bt_cartao')

    def btnKEY_TICKET_click(self):
        self.btk_key_press('bt_ticket')

    def btnKEY_ESTACIONAMENTO_click(self):
        self.btk_key_press('bt_estacionamento')

    def btnKEY_DOACAO_click(self):
        self.btk_key_press('bt_doacao')

    def btnKEY_OUTROS_click(self):
        self.btk_key_press('bt_outros')

    def btnKEY_MENSAGEM_click(self):
        self.btk_key_press('bt_mensagem')

    def btnKEY_ACRESCIMO_click(self):
        self.btk_key_press('bt_acrescimo')

    def btnKEY_CONSULTA_CH_click(self):
        self.btk_key_press('bt_consulta_cheque')

    def btnKEY_DESCONTO_click(self):
        self.btk_key_press('bt_desconto')

    def btnKEY_CLIENTE_click(self):
        self.btk_key_press('bt_cliente')

    def btnKEY_VENDEDOR_click(self):
        self.btk_key_press('bt_vendedor')

    def btnKEY_COMANDA_click(self):
        self.btk_key_press('bt_comanda')

    def btnKEY_SUPORTE_click(self):
        self.btk_key_press('bt_suporte')

    def btnKEY_RECARGA_click(self):
        self.btk_key_press('bt_recarga_cel')

    def on_focus_out_edtDirImagens(self, _return=False):
        edt = self.ui.edtDirImagens.text()
        return self.verificar_arquivo(edt)

    def on_change_edtDirImagens(self, edt):
        return self.verificar_arquivo(edt)

    def verificar_arquivo(self, edt=''):
        if not self.ui.check_dir_logo_itens.isChecked():
            self.permitir_salvar = True
            self.ui.edtDirImagens.setText('')
            return True

        arquivos = glob.glob(str('%s/*') % edt)
        msg = ''
        for arq in arquivos:
            nome_arquivo = arq.split(str(edt + '/'))[1]
            if '.png' in nome_arquivo or '.jpg' in nome_arquivo or '.jpeg' in nome_arquivo or \
                '.gif' in nome_arquivo:
                tam_img = os.path.getsize(str(edt + '/') + nome_arquivo)
                # cria mensagem para os arquivos de imagem maiores que 100k bytes
                if tam_img / 1000 > 100:
                    msg += '\n' + nome_arquivo

        if not msg:
            self.permitir_salvar = True
            return True

        self.msg_aviso = msg
        if msg:
            self.permitir_salvar = False
            msg = u'Os arquivos a seguir no diretório\nde imagens possuem tamanho superior\nà 100k bytes:\n' + msg
            QtGui.QMessageBox.critical(None, u'Atenção!', msg)
            return False


        self.permitir_salvar = True
        return True

    def check_db_connection(self):
        pg_super, pg_pdv, pg_local, pg_gestao, pg_master = range(5)
        idx = self.ui.page_connection.currentIndex()

        def conexaoPostgreSQL(db, user, host, passwd):
            import psycopg2

            res = dict(res=True, msg="Conexão estabelecida: psycopg2://%s:%s@%s/%s>" % (
            user, '*' * len(passwd), host, db))
            # STRING DE CONEXÃO COM O BANCO DE DADOS
            _string_conexao = "dbname='%s' user='%s' host='%s' password='%s'" % (
            db, user, host, passwd)
            try:
                conn = psycopg2.connect(_string_conexao)
            except Exception as e:
                res = dict(res=False, msg=str(e))
                pass
            return res


        if idx == pg_super:
            host = self.ui.edt_sup_hostname.text().encode('utf-8')
            user = self.ui.edtUserSuper.text().encode('utf-8')
            passwd = self.ui.edtSenhaSuper.text().encode('utf-8')
            db = self.ui.edt_sup_database.text().encode('utf-8')
            res = conexaoPostgreSQL(db, user, host, passwd)

        elif idx == pg_pdv:
            host = self.ui.edt_cx_hostname.text().encode('utf-8')
            user = self.ui.edt_username.text().encode('utf-8')
            passwd = self.ui.edt_password.text().encode('utf-8')
            db = self.ui.edt_cx_database.text().encode('utf-8')
            res = conexaoPostgreSQL(db, user, host, passwd)

        elif idx == pg_local:
            host = self.ui.edt_dbg_ip.text().encode('utf-8')
            user = self.ui.edt_dbg_user.text().encode('utf-8')
            passwd = self.ui.edt_dbg_pwd.text().encode('utf-8')
            db = self.ui.edt_dbg_dbname.text().encode('utf-8')
            port = self.ui.spin_dbg_port.text().encode('utf-8')
            res = testa_conexao_rpc(host, user, passwd, db, int(port))

        elif idx == pg_gestao:
            host = self.ui.edt_dbg_ip.text().encode('utf-8')
            user = self.ui.edt_dbg_user.text().encode('utf-8')
            passwd = self.ui.edt_dbg_pwd.text().encode('utf-8')
            db = self.ui.edt_dbg_dbname.text().encode('utf-8')
            res = testa_conexao_rpc(host, user, passwd, db)
        elif idx == pg_master:
            host = self.ui.edt_sup_hostname_master.text().encode('utf-8')
            user = self.ui.edtUserSuper_master.text().encode('utf-8')
            passwd = self.ui.edtSenhaSuper_master.text().encode('utf-8')
            db = self.ui.edt_sup_database_master.text().encode('utf-8')
            res = conexaoPostgreSQL(db, user, host, passwd)

        QtGui.QMessageBox.question(None, self.caption, self.trUtf8(res['msg']),
                                   QtGui.QMessageBox.StandardButtons(QtGui.QMessageBox.Ok))

    def check_serial(self):
        set_serial()
        portas = []

        for ttyser in range(50):
            name = '/dev/ttyS%d' % ttyser
            try:
                serial.Serial(name)
                portas.append(name)
            except:
                pass

        for ttyser in range(50):
            name = '/dev/ttySNX%d' % ttyser
            try:
                serial.Serial(name)
                portas.append(name)
            except:
                pass

        for ttyusb in range(50):
            name = '/dev/ttyUSB%d' % ttyusb
            try:
                serial.Serial(name)
                portas.append(name)
            except:
                pass

        for ttyacm in range(50):
            name = '/dev/ttyACM%d' % ttyacm
            try:
                serial.Serial(name)
                portas.append(name)
            except:
                pass

        for ttyv in range(50):
            name = '/dev/ttyV%d' % ttyv
            try:
                serial.Serial(name)
                portas.append(name)
            except:
                pass

        self.ui.cbx_fiscal_printer_port.clear()
        self.ui.cbx_doc_scan_port.clear()
        self.ui.cbx_itens_scan_port.clear()
        self.ui.cbx_balanca_port.clear()
        self.ui.cbx_pinpad_port.clear()

        for p in portas:
            self.ui.cbx_fiscal_printer_port.addItem(p)
            self.ui.cbx_doc_scan_port.addItem(p)
            self.ui.cbx_itens_scan_port.addItem(p)
            self.ui.cbx_balanca_port.addItem(p)
            self.ui.cbx_pinpad_port.addItem(p)

        return True

    def checkEcf(self):
        loja = int(self.ui.spin_loja.text())
        caixa = int(self.ui.spin_caixa.text())
        try:
            obj_fiscal_printer_helper = FiscalPrinterHelper()
            ecf = obj_fiscal_printer_helper.get_ecf_autorizada(loja, caixa, os.path.join(ROOT_PATH, 'pdvsig.sig'), True)
            brand = None
            ecf_fabricante = (ecf.CFabricante or '').lower()
            if 'bematech' in ecf_fabricante:
                brand = 'bematech'
                if '4000' in ecf.CModelo:
                    model = 'MP4000'
                else:
                    model = 'MP2100'
            elif 'daruma' in ecf_fabricante:
                brand = 'daruma'
                model = 'FS600MFD'
            elif 'dataregis' in ecf_fabricante:
                brand = 'dataregis'
                model = 'EP375'
            elif 'elgin' in ecf_fabricante:
                brand = 'elgin'
                model = 'Pay2023'
            elif 'sweda' in ecf_fabricante:
                brand = 'sweda'
                model = 'IFS9000I'
            elif 'epson' in ecf_fabricante:
                raise Exception('ECF EPSON nao suportada')
            elif 'zpm' in ecf_fabricante:
                raise Exception('ECF ZOM nao suportada')
            elif 'urano' in ecf_fabricante:
                raise Exception('ECF URANO nao suportada')
            else:
                raise Exception('Fabricante da ECF não identificado no Arquivo de Configuração')

            port = SerialPort(str(self.ui.cbx_fiscal_printer_port.currentText()))
            printer = obj_fiscal_printer_helper.printer
            port.set_options(baudrate=self.ui.spin_printer_speed_port.text())

            printer._port.setTimeout(1)
            printer.get_coo()
            QtGui.QMessageBox.critical(None, 'PDV Config', 'Conectado!!!')
        except Exception as e:
            unicode_string = str(e)
            erro = unicode(filter(lambda c: category(c) != 'Mn',
                                  normalize('NFKD', unicode_string.decode('utf-8', 'replace')))
            )
            QtGui.QMessageBox.critical(None, 'PDV Config', 'Erro ao tentar conectar: %s' % erro)

    def check_pinpad(self):
        """
        Altera a porta configurada no arquivo CliSiTef.ini para a porta selecionada no PdvConfig,
        e verifica a presença do PinPad nesta porta.

        :return: Conectado/Desconectado
        :rtype bool

        :autor: Dener Luiz Rodrigues da Silva <d3ner.rodrigues[at]gmail.com>
        """
        res = False

        port = str(self.ui.cbx_pinpad_port.currentText())

        clisitef = open('%s/CliSiTef.ini' % ROOT_PATH, 'r+')
        file_lines = clisitef.readlines()

        for line in file_lines:
            if line[:6] == 'Porta=':
                index = file_lines.index(line)
                del file_lines[index]
                file_lines.insert(index, 'Porta=%s\n' % port)
                break

        clisitef.write('')
        clisitef = open('%s/CliSiTef.ini' % ROOT_PATH, 'r+')
        clisitef.writelines(file_lines)
        clisitef.close()

        obj_sigtef = SigTefI(None)

        if obj_sigtef.verifica_pinpad():
            res = True
            QtGui.QMessageBox.information(None, 'PDV Config', 'Conectado!!!')
        else:
            QtGui.QMessageBox.critical(None, 'PDV Config', 'Erro ao tentar conectar!')

        # Salva a porta no pdvconfig.cfg sem necessidade de clicar em Gravar
        pdv_ini = self._inifile
        pdv_ini.set('porta_pinpad', str(self.ui.cbx_pinpad_port.currentText()))
        pdv_ini.save()

        return res

    def iniciar_socat(self):
        server = self.ui.edt_ecf_virtual_srv.text()
        port = self.ui.spin_ecf_port_tcp.value()

        try:
            res = Socat().connect(server, port)
            QtGui.QMessageBox.information(None, 'PDV Config', 'Conectado!!!')
        except Exception as e:
            QtGui.QMessageBox.critical(None, 'PDV Config', 'Erro ao tentar conectar!')
            return False

        index = self.ui.cbx_fiscal_printer_port.findText('/dev/ttyV0')

        if index < 0:
            self.ui.cbx_fiscal_printer_port.addItem('/dev/ttyV0')

        self.ui.cbx_fiscal_printer_port.setCurrentIndex(self.ui.cbx_fiscal_printer_port.findText('/dev/ttyV0'))

        return res

    def btk_key_press(self, btn_name):
        key = self._inifile.get(btn_name)
        retval = key_dlg_show(parent=self, key=key)
        if retval:
            self._inifile.set(btn_name, retval)

    def fun_change(self, *t):
        self.showFullScreen()

    def keyReleaseEvent(self, event):

        obj_name = self.focusWidget().objectName()
        if obj_name[:10] == 'edt_tecla_' or obj_name == 'spinKeyCode':
            #if event.nativeScanCode() not in (23, 37, 50): 
            if obj_name == 'spinKeyCode':
                self.focusWidget().setValue(event.key())
            else:
                self.focusWidget().setText(str(event.key()))

            for name, cod in QtCore.Qt.Key.values.iteritems():
                if cod.__int__() == event.key():
                    print name.split('_')

        try:
            self.ui.lbl_key.setText(
                "<body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
                "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Key Press: [ </span><span style=\" font-weight:600; color:#0000ff;\">%s - %d</span><span style=\" font-weight:600;\"> ]</span></p></body>" % (
                chr(event.nativeVirtualKey()), event.nativeVirtualKey()))

        except:
            pass

    def changePage(self, current, previous):
        """ Função responsável por mudar a página quando selecionado um item na lista """
        if not current:
            current = previous
        self.ui.stackedWidget.setCurrentIndex(self.ui.listWidget.row(current))

    def closeEvent(self, ev):
        if not self.permitir_salvar:
            ev.ignore()
            if not self.verificar_arquivo(self.ui.edtDirImagens.text()):
                return False

        opt = QtGui.QMessageBox.question(self, self.caption,
                                         self.trUtf8("Deseja encerar a aplicação?"),
                                         QtGui.QMessageBox.StandardButtons(
                                             QtGui.QMessageBox.Save | QtGui.QMessageBox.Close | QtGui.QMessageBox.Cancel))

        if opt == QtGui.QMessageBox.StandardButton.Cancel:
            ev.ignore()
        elif opt == QtGui.QMessageBox.StandardButton.Save:
            opt = QtGui.QMessageBox.question(None, self.caption,
                                             self.trUtf8("Confirmar as Alterações?"),
                                             QtGui.QMessageBox.StandardButtons(
                                                 QtGui.QMessageBox.Ok | QtGui.QMessageBox.Cancel))
            if opt == QtGui.QMessageBox.StandardButton.Ok:
                self.gravar_ini()
                ev.accept()
            else:
                ev.ignore()
        else:
            ev.accept()

    def check_keys(self):
        """
        Verifica duplicidade de teclas
        :return:
        """
        check = self._inifile.check_codificado()
        msg = 'Teclas configuradas corretamente.'
        retval = False
        if check:
            retval = True
            msg = ''
            for key, value in check.iteritems():
                msg += 'Tecla: %r, configurada p: [%s]\n' % (
                QtCore.Qt.Key(key).name[4:], '|'.join(value))
        QtGui.QMessageBox.information(None, self.caption, self.trUtf8(msg),
                                      QtGui.QMessageBox.StandardButtons(
                                          QtGui.QMessageBox.Ok | QtGui.QMessageBox.Cancel))
        return retval

    def btn_dir_images_click(self):
        res = QtGui.QFileDialog.getExistingDirectory(None, self.trUtf8(
            "Diretório de Importação/Exportação"), root_path,
                                                     QtGui.QFileDialog.Options(
                                                         QtGui.QFileDialog.ShowDirsOnly))
        if res:
            self.ui.edtDirImagens.setText(res)

    def get_key_pressed(self, event):
        pass

    def nextPageFinalizadora(self):
        current = self.ui.pageFinalizadora.currentIndex()
        if not current:
            current = 1
        else:
            current = 0
        self.ui.pageFinalizadora.setCurrentIndex(current)

    def _get_ecf_autorizada(self):
        loja = self.ui.spin_loja.value()
        caixa = self.ui.spin_caixa.value()
        cfabricante = cmodelo = cversao = cdatasb = chorasb = cnserie = cnacional = '?'
        try:
            ecf = FiscalPrinterHelper().get_ecf_autorizada(loja, caixa, os.path.join(root_path, 'pdvsig.sig'), True)
            cfabricante = ecf.CFabricante
            cmodelo = ecf.CModelo
            cversao = ecf.CVersao
            cdatasb = ecf.CDataSB + ' ' + ecf.CHoraSB
            cnserie = ecf.CNSerie
            cnacional = ecf.CNacional
        except Exception, err:
            #QtGui.QMessageBox.critical(None, 'PDV Config', str(err))
            pass
        self.ui.lblEcfFabricante.setText(cfabricante)
        self.ui.lblEcfModelo.setText(cmodelo)
        self.ui.lblEcfVersao.setText(cversao)
        self.ui.lblEcfSB.setText(cdatasb)
        self.ui.lblEcfSerial.setText(cnserie)
        self.ui.lblEcfNacional.setText(cnacional)

    def get_telcado_selecionado(self):
        campos = ['rb_gertec_44_usb', 'rb_gertec_44_ps2', 'rb_gertec_55_usb', 'rb_gertec_55_ps2',
                  'rb_gertec_65_ps2', 'rb_gertec_65_usb', 'rb_gertec_65_unisys', 'rb_smak_12',
                  'rb_smak_44', 'rb_smak_44_eth', 'rb_smak_291_sc', 'rb_smak_291_lcm']
        for campo in campos:
            if eval('self.ui.%(campo)s.isChecked()' % locals()):
                return campo

        return False

    def gravar_ini(self):
        """ Grava os dados dos controles no arquivo pdvconfig.cfg """

        pdv_ini = self._inifile
        pdv_ini.set('usarcodificado', self.ui.checkUsarcodificado.isChecked())
        pdv_ini.set('usar_display', self.ui.checkUsarDisplay.isChecked())
        pdv_ini.set('teclado_selecionado', self.get_telcado_selecionado())
        if pdv_ini.get('usarcodificado') and pdv_ini.check_codificado():
            msg = 'Alguns códigos de teclas estão repetidos.\n\nO teclado codificado foi desativado.'
            ret = QtGui.QMessageBox.information(None, self.caption, self.trUtf8(msg),
                                                QtGui.QMessageBox.StandardButtons(
                                                    QtGui.QMessageBox.Ok | QtGui.QMessageBox.Cancel))
            pdv_ini.set('usarcodificado', False)

        pdv_ini.set('loja', self.ui.spin_loja.text())
        pdv_ini.set('caixa', self.ui.spin_caixa.text())
        # CAIXA MYSQL
        pdv_ini.set('db_pdv_name', self.ui.edt_cx_database.text())
        pdv_ini.set('db_pdv_host', self.ui.edt_cx_hostname.text())
        pdv_ini.set('db_pdv_user', self.ui.edt_username.text())
        pdv_ini.set('db_pdv_pass', self.ui.edt_password.text())

        # supervisor
        pdv_ini.set('db_sup_name', self.ui.edt_sup_database.text())
        pdv_ini.set('db_sup_host', self.ui.edt_sup_hostname.text())
        pdv_ini.set('db_sup_user', self.ui.edtUserSuper.text())
        pdv_ini.set('db_sup_pass', self.ui.edtSenhaSuper.text())

        # Supervisor Master
        pdv_ini.set('db_sup_name_master', self.ui.edt_sup_database_master.text())
        pdv_ini.set('db_sup_host_master', self.ui.edt_sup_hostname_master.text())
        pdv_ini.set('db_sup_user_master', self.ui.edtUserSuper_master.text())
        pdv_ini.set('db_sup_pass_master', self.ui.edtSenhaSuper_master.text())

        pdv_ini.set('font_grid', self.ui.spin_font_grid.value())
        # PERIFERICOS
        pdv_ini.set('printer_speed_port', self.ui.spin_printer_speed_port.value())
        pdv_ini.set('fiscal_printer_active', self.ui.group_fiscal_printer.isChecked())
        pdv_ini.set('itens_scan_active', self.ui.group_itens_scanner.isChecked())
        pdv_ini.set('balanca_active', self.ui.group_balanca.isChecked())
        pdv_ini.set('doc_scan_active', self.ui.group_doc_scanner.isChecked())
        pdv_ini.set('virtual_printer_active', self.ui.group_fiscal_printer_virtual.isChecked())
        pdv_ini.set('virtual_printer_srv', self.ui.edt_ecf_virtual_srv.text())
        pdv_ini.set('ecf_port_tcp', self.ui.spin_ecf_port_tcp.value())
        pdv_ini.set('config_nepos_active', self.ui.group_fiscal_printer_nepos.isChecked())
        pdv_ini.set('nepos_ip', self.ui.edt_nepos_port.text())
        pdv_ini.set('nepos_port_tcp', self.ui.spin_nepos_ip.value())

        pdv_ini.set('ecf', self.ui.cbx_fiscal_printer_port.currentText())
        pdv_ini.set('scannerdocporta', self.ui.cbx_doc_scan_port.currentText())
        pdv_ini.set('scannerdocbaud', self.ui.cbx_doc_scan_speed.currentText())
        pdv_ini.set('scannerdocsolicitaleitura', self.ui.spin_doc_scan_solicitaleitura.value())
        pdv_ini.set('scannerdoccharinicio', self.ui.spin_doc_scan_charini.value())
        pdv_ini.set('scannerdoccharfim', self.ui.spin_doc_scan_charfim.value())
        # SCANNER DE ITENS
        pdv_ini.set('scanner', self.ui.cbx_itens_scan_port.currentText())
        pdv_ini.set('scannerbaud', self.ui.cbx_itens_scan_speed.currentText())
        # BALANÇA
        pdv_ini.set('balancaporta', self.ui.cbx_balanca_port.currentText())
        pdv_ini.set('digitos', self.ui.spin_balanca_qtdadig.text())
        pdv_ini.set('codigocomdigito', str(self.ui.ckb_balanca_dv.isChecked()))
        pdv_ini.set('balancapesomaximo', self.ui.spin_balanca_pesomax.text())
        pdv_ini.set('solicitaleitura', self.ui.spin_balanca_solicitaleitura.value())
        pdv_ini.set('charinicio', self.ui.spin_balanca_charini.value())
        pdv_ini.set('charfim', self.ui.spin_balanca_charfim.value())
        pdv_ini.set('balancabaud', self.ui.cbx_balanca_speed.currentText())
        pdv_ini.set('modelo_balanca', self.ui.cbx_modelo_balanca.currentText())

        # TEF
        pdv_ini.set('servidor', self.ui.edt_tef_servidor.text())
        pdv_ini.set('empresa', self.ui.edt_tef_empresa.text())
        pdv_ini.set('qtdvias', self.ui.spin_tef_qtdavias.text())
        pdv_ini.set('digitacaocheque', str(self.ui.cbx_tef_digitacao.currentIndex()))
        pdv_ini.set('usa_recarga_cel', self.ui.chk_usa_recarga_cel.isChecked())
        pdv_ini.set('usa_socio_torcedor', self.ui.chk_usa_socio_torcedor.isChecked())
        pdv_ini.set('totalizador_recarga_cel', self.ui.spin_totalize_recarga.value())
        pdv_ini.set('totalizador_doacao', self.ui.spip_totalizador_doacao.value())
        pdv_ini.set('porta_pinpad', str(self.ui.cbx_pinpad_port.currentText()))
        pdv_ini.set('config_pinpad_active', self.ui.group_fiscal_printer_3.isChecked())

        # PARAMETROS
        pdv_ini.set('usarprevenda', str(self.ui.chk_pre_venda.isChecked()))
        pdv_ini.set('usar_ficha', str(self.ui.chk_ficha.isChecked()))
        pdv_ini.set('imprimir_2via_doacao', str(self.ui.check_print_doacao_2_via.isChecked()))
        pdv_ini.set('tot_ficha', str(self.ui.spin_tot_ficha.value()))
        pdv_ini.set('tot_devolucao_ficha', str(self.ui.spin_tot_devolucao_ficha.value()))
        pdv_ini.set('usarcomanda', str(self.ui.chk_usa_comanda.isChecked()))
        pdv_ini.set('autorizacaosup', str(self.ui.chk_pedir_autor.isChecked()))
        pdv_ini.set('autorizacaomultiplicaitens', str(self.ui.chk_mult_itens.isChecked()))
        pdv_ini.set('sangriadiscriminaroutros', str(self.ui.chk_disc_outros.isChecked()))
        pdv_ini.set('sangriadiscriminarcheques', str(self.ui.chk_disc_ch_sangria.isChecked()))
        pdv_ini.set('itempesaveletiquetabalanca', str(self.ui.chk_itens_pesaveis.isChecked()))
        pdv_ini.set('itempesavelcomanda', str(self.ui.chk_itens_pesaveis_cda.isChecked()))
        pdv_ini.set('sangriadiscriminarcartoes', str(self.ui.chk_disc_cc_sangria.isChecked()))
        pdv_ini.set('sangriadiscriminartickets', str(self.ui.chk_disc_tick_sangria.isChecked()))
        pdv_ini.set('shutdownaposz', str(self.ui.chk_desligar_pdv.isChecked()))
        pdv_ini.set('usardav', str(self.ui.chk_usa_dav.isChecked()))
        pdv_ini.set('cortapapel', str(self.ui.chk_corta_papel.isChecked()))
        pdv_ini.set('restricoesclientes', str(self.ui.chk_informa_limite.isChecked()))
        pdv_ini.set('consultaprodutos', str(self.ui.chk_permite_consulta.isChecked()))
        pdv_ini.set('usagaveta', str(self.ui.chk_usa_gaveta.isChecked()))
        pdv_ini.set('sangria_x_z_suprimento', str(self.ui.chk_sangria_x_z.isChecked()))
        pdv_ini.set('sangriamanual', str(self.ui.chk_todos_vlr_sangria.isChecked()))
        pdv_ini.set('conferenciadav', str(self.ui.chk_confere_dav.isChecked()))
        pdv_ini.set('invertesinalgaveta', str(self.ui.chk_inverter_sinal.isChecked()))
        pdv_ini.set('imprimircpfcodigointerno', str(self.ui.cbx_cliente_cod.currentIndex()))
        pdv_ini.set('imprimirendereco', str(self.ui.chk_imprimi_end.isChecked()))
        pdv_ini.set('imprimirinscricaoestadual', str(self.ui.chk_imprimi_ie.isChecked()))
        pdv_ini.set('imprimircidade', str(self.ui.chk_imprimi_cidade.isChecked()))
        pdv_ini.set('max_dias_log', self.ui.spin_excluir_log.text())
        pdv_ini.set('frase_sangria', self.ui.edt_sangria.text())
        pdv_ini.set('horaminemitirz', self.ui.time_hora_z.text())
        pdv_ini.set('temposupintervencao', self.ui.time_intervencao.value())
        pdv_ini.set('time_comanda', self.ui.time_comanda.value())
        pdv_ini.set('time_importacao', self.ui.timeImportacao.text())
        pdv_ini.set('impressora_conferencia', self.ui.edt_printer_conferencia_cmd.text())
        pdv_ini.set('cmd_alterar_data', self.ui.edt_alterar_data.text())
        pdv_ini.set('limite_comanda', self.ui.num_limite_cda.text())
        pdv_ini.set('importacao_full', self.ui.chkImportFull.isChecked())
        pdv_ini.set('time_export', self.ui.timeExport.text())
        pdv_ini.set('truncar_tabelas', str(self.ui.chkTruncate.isChecked()))
        pdv_ini.set('descricao_resumida', str(self.ui.chk_itens_descricao_resumida.isChecked()))
        pdv_ini.set('mfd_mensal', str(self.ui.chk_mfd_mensal.isChecked()))
        pdv_ini.set('gerou_mfd', str(self.ui.chk_gerou_mfd.isChecked()))
        pdv_ini.set('usar_biometria', str(self.ui.chk_usar_biometria.isChecked()))

        #TECLADO

        # SUPRIMENTO/SANGRIA
        pdv_ini.set('suprimento', self.ui.edt_suprimento.text())
        pdv_ini.set('sangria_dinheiro', self.ui.edt_sangria_dinheiro.text())
        pdv_ini.set('sangria_cheque', self.ui.edt_sangria_cheque.text())
        pdv_ini.set('sangria_cartao', self.ui.edt_sangria_cartao.text())
        pdv_ini.set('sangria_ticket', self.ui.edt_sangria_ticket.text())
        pdv_ini.set('sangria_outros', self.ui.edt_sangria_outros.text())
        # CORESPONDENTE BANCARIO
        pdv_ini.set('bbagencia', self.ui.edt_bb_agencia.text())
        pdv_ini.set('bbfinalizcheque', self.ui.spin_bb_cheque.value())
        pdv_ini.set('bbconvenio', self.ui.edt_bb_convenio.text())
        pdv_ini.set('bbfinalizdinheiro', self.ui.spin_bb_dinheiro.value())
        pdv_ini.set('bbtotalizadornfiscale', self.ui.spin_bb_estorno.value())
        pdv_ini.set('bbloja', self.ui.spin_bb_loja.value())
        pdv_ini.set('bbtotalizadornfiscalr', self.ui.spin_bb_recebimento.value())
        pdv_ini.set('bbservidor', self.ui.edt_bb_servidor.text())
        pdv_ini.set('bbporta', self.ui.spin_bb_porta.value())
        # Imagens
        pdv_ini.set('check_dir_logo_itens', self.ui.check_dir_logo_itens.isChecked())
        pdv_ini.set('local', self.ui.edtDirImagens.text())
        # Rlatório Gerencial
        pdv_ini.set('rg_relatorio_gerencial', self.ui.spin_rel_geren.value())
        pdv_ini.set('rg_meios_de_pgto', self.ui.spin_meios_pgto.value())
        pdv_ini.set('rg_davs_emitidos', self.ui.spin_dav_emit.value())
        pdv_ini.set('rg_identificacao_pafecf', self.ui.spin_identPaf.value())
        pdv_ini.set('rg_parametro_conf', self.ui.spin_param_conf.value())
        if self.ui.cbAtivarRes.isChecked():
            pdv_ini.set('trans_mesas', self.ui.spin_transMesas.value())
            pdv_ini.set('confe_mesas', self.ui.spin_confeMesas.value())
            pdv_ini.set('mesas_abertas', self.ui.spin_mesasAbertas.value())
        # CONEXAO ERP
        pdv_ini.set('db_ges_host', self.ui.edt_dbg_ip.text())
        pdv_ini.set('db_ges_user', self.ui.edt_dbg_user.text())
        pdv_ini.set('db_ges_emp', self.ui.edt_dbg_emp.text())
        pdv_ini.set('db_ges_pass', self.ui.edt_dbg_pwd.text())
        pdv_ini.set('db_ges_name', self.ui.edt_dbg_dbname.text())
        pdv_ini.set('db_ges_port', self.ui.spin_dbg_port.value())
        #SUPORTE
        pdv_ini.set('cmd_teamviewer', self.ui.edt_cmd_tv.text())
        pdv_ini.set('check_atualiza', self.ui.chk_atualiza.isChecked())
        pdv_ini.set('link_sig', self.ui.edt_link_sig.text())
        pdv_ini.set('tentativas_atualiza', self.ui.spin_tries_wget.value())
        # APP DAV
        pdv_ini.set('aplicativo_dav', self.ui.cbAppDav.currentText())
        # NOTA FISCAL LEGAL
        pdv_ini.set('nf_legal', self.ui.cb_nflegal.isChecked())
        # RUM COBOL
        pdv_ini.set('ip_gestao', self.ui.edt_ip.text())
        pdv_ini.set('cmd_cobol', self.ui.edt_cmd_dav.text())
        # Conexão Postgres
        pdv_ini.set('pg_user', self.ui.edt_pg_user.text())
        pdv_ini.set('pg_pass', self.ui.edt_pg_pass.text())
        pdv_ini.set('pg_server', self.ui.edt_pg_server.text())
        pdv_ini.set('pg_db', self.ui.edt_pg_db.text())
        pdv_ini.set('pg_port', self.ui.spin_pg_port.value())
        # BARES RESTAURANTES
        pdv_ini.set('ativar_restaurante', self.ui.cbAtivarRes.isChecked())

        pdv_ini.save()


def run():
    app = QtGui.QApplication(sys.argv)
    if os.name == 'posix':
        import pwd

        if pwd.getpwuid(os.getuid())[0] == 'root':
            msg = u"Tentativa de executar o PDV-Config como root. Isso não é bom, abortando.\n"
            QtGui.QMessageBox.critical(None, 'PDV Config', msg)
            sys.exit(1)
    dialog = PdvConfigForm()
    dialog.showMaximized()
    # Tradução
    qtTranslator = QtCore.QTranslator()
    if qtTranslator.load("qt_pt.qm"):
        app.installTranslator(qtTranslator)
    app.exec_()


if __name__ == "__main__":
    run()
