#!/usr/bin/python
# -*- coding: utf-8 -*-

from datetime import datetime
from sigpdv.tools.log_restaurante import traced as log
from sigpdv.control.utils import remove_accents, converte_tz
from sigpdv.models.db_utils import b_sess, exec_sql
from sigpdv.models.supervisor import SupervisorCadItens, SupervisorCdaCapa, SupervisorCdaCorpo, \
    SupervisorCfgConfiguracao
import subprocess
import time

class ImprimirComanda(object):

    def __init__(self, parent):
        self.parent = parent

    def _get_realacao_locais(self):
        locais = {}
        map(lambda x: locais.update({x.get('sucloc_codigo'): x.get('sucloc_impressora')}),
            exec_sql("""SELECT sucloc_impressora, sucloc_codigo 
                          FROM supervisor_producao_local
                          WHERE sucloc_impressora IS NOT NULL """, 'S', ret=True)
            )
        return locais

    def imprimir_pedidos(self, impressoras):
        path = '/tmp/pedido_cozinha.txt'
        for impressora, impressoes in impressoras.items():
            try:
                file = open(path, 'w')
                file.write('\n'.join(impressoes))
                file.close()
                subprocess.call(["lp", "-n1", '-d%s' % impressora, path])
            except Exception as e:
                log.error('Erro ao imprimir pedido de cozinha %s' % e)

            time.sleep(1)

    def _get_impressora_impressao(self, dados_corpo, dict_itens, dict_capa):
        """Retorna um  dict com a chave sendo a impressora e o valor as impressoes"""
        remove_acentuacao = remove_accents
        dict_locais = self._get_realacao_locais()
        impressoras = {}
        if not dict_locais:
            return impressoras

        ini_negrito = '%s%s' % (chr(27), chr(69))
        fim_negrito = '%s%s' % (chr(27), chr(70))
        fonte_i = '\x1B\x57\x01'
        fonte_f = '\x1B\x57\x00'
        
        nome_empresa = remove_accents(SupervisorCfgConfiguracao.get_by().supcfc_cliente or '')
        data_sem_fuso = datetime.now()
        data = datetime.strftime(data_sem_fuso, '%d/%m/%Y %H:%M:%S')[:19]
        ultima_capa_id = False
        cabecalho = ['%s%s%s' % (ini_negrito, (nome_empresa).center(48), fim_negrito),
                     ('DATA IMPRESSAO: %s' % data).rjust(48),
                     '',
                     '%s--------------* PEDIDO DE COZINHA *-------------%s' % (ini_negrito,
                                                                               fim_negrito),
                     '']
        rodape = ['DESENVOLVIDO POR:',
                  '%sSig Solucoes em Informatica Ltda. (31)2106-1700%s' % (ini_negrito,
                                                                           fim_negrito),
                  '%s%s' % (chr(27), chr(109)),  # corta papel
        ]
        impressao_comanda = []
        ultima_printer = False
        for corpo in dados_corpo:
            # itens cancelados nao imprime
            if corpo.supcdo_venda_cancelamento == 'C':
                b_sess.begin()
                corpo.supcdo_imprimir = '1'
                b_sess.commit()
                continue

            create_date = converte_tz(self, (datetime.strptime(('%s' % (corpo.create_date
                                            or ('%s' % data_sem_fuso)))[:19], '%Y-%m-%d %H:%M:%S')),
                     tz_origem='UTC', tz_destino='America/Sao_Paulo').strftime('%d/%m/%Y %H:%M:%S')
            item = dict_itens.get(corpo.supcdo_item)
            printer = False
            if item.supite_codigo_local and dict_locais.get(item.supite_codigo_local):
                printer = dict_locais.get(item.supite_codigo_local)

            nome_cliente = remove_acentuacao(dict_capa.get(corpo.supcdo_capa_id)[1] or '')
            comanda = dict_capa.get(corpo.supcdo_capa_id)[2]
            garcom = remove_acentuacao(corpo.supcdo_nome_vendedor or '')
            mesa = dict_capa.get(corpo.supcdo_capa_id)[3]
            seq_item = corpo.supcdo_sequencia_id
            descricao = remove_acentuacao(item.supite_descricao_completa)
            qtd = corpo.supcdo_quantidade
            if len(descricao) > 42:
                descricao = '\n      '.join([descricao[:42], descricao[42:]])

            obs = remove_acentuacao(corpo.supcdo_observacao or '')
            if len(obs) > 48:
                obs = '\n'.join([obs[:48], obs[48:]])

            seq_impressao = (corpo.supcdo_seq_impressao or 0) + 1
            if ultima_capa_id != corpo.supcdo_capa_id:
                if impressao_comanda:
                    impressao_comanda.extend(rodape)
                    if ultima_printer:
                        if not impressoras.get(ultima_printer):
                            impressoras[ultima_printer] = ['\n'.join(impressao_comanda)]
                        else:
                            impressoras[ultima_printer].append('\n'.join(impressao_comanda))
                    else:
                        log.error('Impressora nao cadastrada no local do item: %s' %
                                  item.supite_codigo_item)


                ultima_capa_id = corpo.supcdo_capa_id
                impressao_comanda = []
                impressao_comanda.extend(cabecalho)
                if seq_impressao > 1:
                    impressao_comanda.append('%s Reimpressao de pedido%s' % (fonte_i, fonte_f))

                if nome_cliente:
                    impressao_comanda.append('CLIENTE: %s' % nome_cliente)

                impressao_comanda.extend([
                             'GARCOM: %s%s%s' % (ini_negrito, garcom, fim_negrito),
                             'COMANDA: %s%06d%s' % (ini_negrito, comanda, fim_negrito),
                             'MESA: %s%03d%s' % (ini_negrito,mesa, fim_negrito),
                             'DATA: %s' % create_date,
                             '',
                             '%s------------------------------------------------%s' % (ini_negrito,
                                                                                       fim_negrito),
                             ])

            impressao_comanda.extend([
                             'Seq.Item: %s%03d%s' % (ini_negrito, seq_item, fim_negrito),
                             'SEQ.IMPRESSAO: %s' % seq_impressao,
                             'ITEM: %s%s%s' % (fonte_i, descricao, fonte_f),
                             'QTD: %s%s%s%s%s' % (fonte_i, ini_negrito, qtd, fim_negrito, fonte_f),
                             '',
                             'OBS:\n%s*%s\%s' % (fonte_i, obs, fonte_f),
                             '',])
            ultima_printer = printer
            if not printer:
                continue

            b_sess.begin()
            data = converte_tz(self, (datetime.strptime(data, '%d/%m/%Y %H:%M:%S')),
                     tz_origem='America/Sao_Paulo', tz_destino='UTC').strftime('%d/%m/%Y %H:%M:%S')
            corpo.supcdo_seq_impressao = seq_impressao
            corpo.supcdo_data_hora_impressao = data
            corpo.supcdo_imprimir = '4'
            b_sess.commit()
            
        if impressao_comanda:
            impressao_comanda.extend(rodape)
            if ultima_printer:
                if not impressoras.get(ultima_printer):
                    impressoras[ultima_printer] = ['\n'.join(impressao_comanda)]
                else:
                    impressoras[ultima_printer].append('\n'.join(impressao_comanda))
            else:
                log.error('Impressora nao cadastrada no local do item: %s' %
                          item.supite_codigo_item)

        return impressoras

    def imprimir_pedidos_cozinha(self):
        dados_corpo = SupervisorCdaCorpo.query.filter(
            SupervisorCdaCorpo.supcdo_capa_id == self.parent._pve.id). \
            filter(SupervisorCdaCorpo.supcdo_imprimir == '3').order_by('id')

        if not dados_corpo.all():
            return True

        codigos = []
        capa_ids = []
        for i in dados_corpo:
            codigos.append(i.supcdo_item)
            capa_ids.append(i.supcdo_capa_id)

        # ------------ pega a relacao dos itens ---------------
        dados_item = SupervisorCadItens.query.filter(
                                                SupervisorCadItens.supite_codigo_item.in_(codigos))

        dict_itens = {}
        map(lambda x: dict_itens.update( {x.supite_codigo_item: x}), dados_item)

        # ------------ pega a relacao das capas ---------------
        dict_capa = {}
        
        for i in SupervisorCdaCapa.query.filter(SupervisorCdaCapa.id.in_(capa_ids)):
            cliente = i.supcda_nome_cliente or ''
            comanda = i.supcda_codigo_comanda
            mesa = i.supcda_mesa or 0
            chave = '%s-%s' % (i.supcda_loja, i.supcda_caixa)
            dict_capa.update({i.id: (chave, cliente, comanda, mesa)})

        impressoras = self._get_impressora_impressao(dados_corpo, dict_itens, dict_capa)
        if not impressoras:
            log.error('Nenhuma impressora cadastrada nos locais')
            return True

        self.imprimir_pedidos(impressoras)
        return True