#!/usr/bin/python
# -*- encoding: utf-8 -*-

__autor__ = 'Dener Rodrigues'

from datetime import datetime
from sigpdv.models.mov_capa_data import CaixaMovCapa
from sigpdv.models.mov_fechamentos_data import CaixaMovFechamentos
from sigpdv.models.mov_cupom_servidor_data import CaixaMovCupomServidor
from sigpdv.models.db_utils import sess
from sigpdv.control.fiscal_printer import FiscalPrinterHelper


class RecargaCelular():
    def __init__(self, parent):
        self._parent = parent

    def totalizar_recarga(self, value, supfin_tipo_finalizadora):
        self._parent.obj_fiscal_printer.add_value_to_counter('RC', value, supfin_tipo_finalizadora)
        self.gerar_capa(value, supfin_tipo_finalizadora)

    def gerar_capa(self, total_cupom, tipo_finalizadora):
        obj_impressora_fiscal = FiscalPrinterHelper(self._parent)
        printer = obj_impressora_fiscal.printer
        data_inicio = obj_impressora_fiscal.get_data_ecf()
        nro_serie = printer.get_serial()
        contadores = printer.get_counters()
        nro_cupom = contadores['coo']
        data_cupom = datetime.date(obj_impressora_fiscal.get_data_ecf())
        loja_id = self._parent.loja.supcfc_loja
        caixa_id = self._parent.caixa_id
        operador_id = self._parent.user_logged.id
        cliente_id = self._parent.cliente.supcli_cnpj_cpf or None
        nome_cliente = self._parent.cliente.name
        vendedor_codigo = self._parent.vendedor_codigo

        sess.begin()
        capa = CaixaMovCapa(
            supcap_data_cupom=data_cupom,
            supcap_loja=loja_id,
            supcap_caixa=caixa_id,
            supcap_nro_cupom=nro_cupom,
            supcap_operador_id=operador_id,
            supcap_cliente=cliente_id,
            supcap_data_inicio=data_inicio,
            supcap_data_fim=obj_impressora_fiscal.get_data_ecf(),
            supcap_tipo='R',
            supcap_alteracao=None,
            supcap_valor_desc_acres=0,
            supcap_total_cupom=total_cupom,
            supcap_nro_serie=nro_serie,
            supcap_mf_adicional=False,
            supcap_marca_ecf=printer.brand,
            supcap_modelo_ecf=printer._driver.model_name,
            supcap_nro_proprietario=1,
            supcap_ccf=contadores['ccf'],
            supcap_gnf=contadores['gnf'],
            supcap_crz=contadores['crz'],
            supcap_grc=contadores['gnf'],
            supcap_cdc=contadores['cdc'],
            supcap_tipo_comprovante='CN',
            supcap_sub_total=self._parent._coupon_subtotal,
            supcap_indicador_desconto=None,
            supcap_indicador_acrescimo=None,
            supcap_indicador_cancelamento='N',
            supcap_cancelamento_acrescimo=0,
            supcap_ordem_desconto_acrescimo=None,
            supcap_cancelado_posteriormente=False,
            supcap_nome_cliente=nome_cliente,
            supcap_cod_vendedor=vendedor_codigo
        )
        sess.commit()

        self.gerar_fechamentos(capa.id, total_cupom, tipo_finalizadora)

        sess.begin()
        capa.supcap_data_fim = obj_impressora_fiscal.get_data_ecf()
        sess.commit()

        sess.begin()
        CaixaMovCupomServidor(supcse_capa_id=capa.id, supcse_enviado='F')
        sess.commit()

        return capa.id

    def gerar_fechamentos(self, capa_id, total_cupom, tipo_finalizadora):
        obj_impressora_fiscal = FiscalPrinterHelper(self._parent)
        printer = obj_impressora_fiscal.printer
        nro_serie = printer.get_serial()
        contadores = printer.get_counters()

        sess.begin()
        fechamentos = CaixaMovFechamentos(supfec_capa_id=capa_id,
                                          supfec_sequencia=1,
                                          supfec_finalizadora=0,
                                          supfec_tipo_finalizadora=tipo_finalizadora,
                                          supfec_valor=total_cupom,
                                          supfec_ccf=contadores['ccf'],
                                          supfec_gbf=contadores['gnf'],
                                          supfec_cdc=contadores['cdc'],
                                          supfec_nro_serie=nro_serie)
        sess.commit()
        return fechamentos.id