#!/usr/bin/python2
# -*- coding: utf-8 -*-

import threading
import time
import sys


class BackgroundTask(threading.Thread):
    """
    A subclass of threading.Thread whose run() method repeats.

    Use this class for most repeating tasks. It uses time.sleep() to wait
    for each interval, which isn't very responsive; that is, even if you call
    self.cancel(), you'll have to wait until the sleep() call finishes before
    the thread stops. To compensate, it defaults to being daemonic, which means
    it won't delay stopping the whole process.
    """

    def __init__(self, interval, function, args=[], kwargs={}, bus=None):
        threading.Thread.__init__(self)
        self.interval = interval
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.running = False
        self._start = False

    def cancel(self):
        self.running = False

    def run(self):
        self.running = True
        while self.running:
            if self._start:
                time.sleep(self.interval)
            else:
                self._start = True
            if not self.running:
                return False
            try:
                self.function(*self.args, **self.kwargs)
            except Exception as e:
                self.cancel()
                sys.exit(-1)

    def _set_daemon(self):
        return True
