# -*- coding: utf-8 -*-

import ConfigParser
import optparse
import os
import sys
from sigpdv import root_path
from datetime import datetime

LOG_FILE = os.path.join(root_path, 'logs/sigpdv.%s.log' % datetime.now().strftime('%d-%m-%Y'))
LOG_FILE_EXPORTAR = os.path.join(root_path,
                                 'logs/exportar.%s.log' % datetime.now().strftime('%d-%m-%Y'))
LOG_FILE_RESTAURANTE = os.path.join(root_path,
                                    'logs/restaurante.%s.log' % datetime.now().strftime('%d-%m-%Y'))
LOG_FILE_UPDATE = os.path.join(root_path,
                               'logs/update.%s.log' % datetime.now().strftime('%d-%m-%Y'))

_GVERSAO = 'Versão do Sistema'
_GDBPDV = 'Configurações de conexão com o BD do Frente'
_GDBSUP = 'Configurações de conexão com o BD do Supervisor'
_GDBGES = 'Configurações de conexão com o BD do Gestão'

KEYS_CODIFICADO = ['bt_num_0', 'bt_num_1', 'bt_num_2', 'bt_num_3', 'bt_num_4', 'bt_num_5',
                   'bt_num_6', 'bt_num_7',
                   'bt_num_8', 'bt_num_9', 'bt_num_.', 'bt_quantidade', 'bt_cancelar_item',
                   'bt_balanca', 'bt_consulta',
                   'bt_cima', 'bt_baixo', 'bt_esquerda', 'bt_direita', 'bt_dav_pve', 'bt_doacao',
                   'bt_limpar', 'bt_sair',
                   'bt_menu', 'bt_menufiscal', 'bt_entrar', 'bt_dinheiro', 'bt_cheque', 'bt_cartao',
                   'bt_ticket','bt_estacionamento', 'bt_socio_torcedor',
                   'bt_outros', 'bt_recebimento', 'bt_mensagem', 'bt_acrescimo', 'bt_comanda', 'bt_desconto',
                   'bt_cliente',
                   'bt_vendedor', 'bt_consulta_cheque', 'bt_suporte', 'bt_recarga_cel'
                   ]

_OPTIONS = (
    ('pidfile', '/tmp/sigfrente.pid'),
    ('root_path', None),
    # Section: Versão
    ('versao', '3.1'),
    ('data_instalacao', False),
    # Configuração do Banco do Frente
    ('db_pdv_name', 'sigpdv'),
    ('db_pdv_host', 'localhost'),
    ('db_pdv_user', ''),
    ('db_pdv_pass', ''),
    # Configuração do Banco do Supervisor
    ('db_sup_name', 'super_sigpdv'),
    ('db_sup_host', 'localhost'),
    ('db_sup_user', 'sig'),
    ('db_sup_pass', 'sig'),
    # Configuração do Banco do Supervisor Master
    ('db_sup_name_master', ''),
    ('db_sup_host_master', ''),
    ('db_sup_user_master', ''),
    ('db_sup_pass_master', ''),
    # Configuração do Banco do ERP
    ('db_ges_name', 'pdvsig'),
    ('db_ges_host', 'localhost'),
    ('db_ges_user', 'sig'),
    ('db_ges_pass', 'sig'),
    ('db_ges_port', 8069),
    ('db_ges_emp', '1'),
    # Configurações de Log
    ('max_dias_log', 0),
    ('max_size_log', 0),
    # Correspondente Bancário
    ('bbconvenio', ''),
    ('bbtotalizadornfiscale', 0),
    ('bbtotalizadornfiscalr', 0),
    ('bbporta', 0),
    ('bbloja', 0),
    ('bbagencia', ''),
    ('bbservidor', ''),
    ('bbfinalizdinheiro', 0),
    ('bbfinalizcheque', 0),
    # Section: PDV
    ('imprimirinscricaoestadual', False),
    ('conferenciadav', False),
    ('usarprevenda', False),
    ('usar_ficha', False),
    ('imprimir_2via_doacao', True),
    ('tot_ficha', 0),
    ('tot_devolucao_ficha', 0),
    ('usarcomanda', False),
    ('cortapapel', False),
    ('sangriadiscriminaroutros', False),
    ('invertesinalgaveta', False),
    ('usagaveta', False),
    ('consultaprodutos', True),
    ('dirimportaexporta', False),
    ('sangria_ticket', False),
    ('logimpressora', True),
    ('usardav', True),
    ('horaminemitirz', '18:00'),
    ('restricoesclientes', False),
    ('loja', 1),
    ('ecf', '/dev/ttyS0'),
    ('caixa', 1),
    ('qtdpdvs', 1),
    ('font_grid', 12),
    ('sangria_x_z_suprimento', False),
    ('imprimircpfcodigointerno', 0),
    ('sangriadiscriminarcartoes', False),
    ('sangriadiscriminartickets', False),
    ('sangriadiscriminarcheques', False),
    ('suprimento', ''),
    ('sangria_dinheiro', ''),
    ('sangria_cheque', ''),
    ('sangria_cartao', ''),
    ('sangria_ticket', ''),
    ('sangria_outros', ''),
    ('imprimircidade', False),
    ('imprimirendereco', False),
    ('itempesaveletiquetabalanca', False),
    ('itempesavelcomanda', False),
    ('temposupintervencao', 60),
    ('time_comanda', 60),
    ('shutdownaposz', False),
    ('autorizacaosup', False),
    ('autorizacaomultiplicaitens', False),
    ('sangriamanual', False),
    ('imprimir_ie', False),
    ('frase_sangria', 'Favor efetuar sangria!'),
    ('time_importacao', 0),
    ('impressora_conferencia', ''),
    ('cmd_alterar_data', 'date -s'),
    ('limite_comanda', 99),
    ('importacao_full', False),
    ('time_export', 10),
    ('truncar_tabelas', False),
    ('descricao_resumida', False),
    ('mfd_mensal', False),
    ('gerou_mfd', False),
    # Section: PORTAS
    ('printer_speed_port', 115200),
    ('fiscal_printer_active', True),
    ('doc_scan_active', False),
    ('itens_scan_active', False),
    ('balanca_active', False),
    ('scannerdoccharfim', 3),
    ('scanner', ''),
    ('scannerbaud', '9600'),
    ('scannerdoccharinicio', 2),
    ('scannerdocsolicitaleitura', 5),
    ('scannerdocporta', ''),
    ('scannerdocbaud', '9600'),
    ('balancaporta', ''),
    ('balancabaud', '9600'),
    ('balancapesomaximo', 20),
    ('modelo_balanca', 'Toledo'),
    # Section: TEF
    ('empresa', '00000000'),
    ('qtdvias', 2),
    ('servidor', ''),
    ('usa_recarga_cel', False),
    ('totalizador_recarga_cel', 0),
    ('usa_socio_torcedor', False),
    ('config_pinpad_active', True),
    ('porta_pinpad', ''),
    # Section: BALANCA
    ('solicitaleitura', 5),
    ('charinicio', 2),
    ('codigocomdigito', True),
    ('digitos', 4),
    ('charfim', 3),
    # Section: TECLADO
    ('usarcodificado', False),
    ('usar_display', False),
    ('teclado_selecionado', ''),
    ('bt_num_.', 44),
    ('bt_num_0', 45),
    ('bt_num_1', 49),
    ('bt_num_2', 50),
    ('bt_num_3', 51),
    ('bt_num_4', 52),
    ('bt_num_5', 53),
    ('bt_num_6', 54),
    ('bt_num_7', 55),
    ('bt_num_8', 56),
    ('bt_num_9', 57),
    ('bt_limpar', 16777219),
    ('bt_esquerda', 16777234),
    ('bt_cima', 16777235),
    ('bt_direita', 16777236),
    ('bt_baixo', 16777237),
    ('bt_entrar', 16777220),
    ('bt_acrescimo', 65),
    ('bt_balanca', 66),
    ('bt_cartao', 67),
    ('bt_socio_torcedor', 74),
    ('bt_dinheiro', 68),
    ('bt_menufiscal', 70),
    ('bt_cheque', 72),
    ('bt_cancelar_item', 73),
    ('bt_ticket', 75),
    ('bt_estacionamento', 94),
    ('bt_consulta_cheque', 76),
    ('bt_menu', 77),
    ('bt_vendedor', 78),
    ('bt_outros', 79),
    ('bt_consulta', 80),
    ('bt_quantidade', 81),
    ('bt_recarga_cel', 82),
    ('bt_comanda', 83),
    ('bt_cliente', 84),
    ('bt_desconto', 85),
    ('bt_cupom', 86),
    ('bt_dav_pve', 87),
    ('bt_doacao', 42),
    ('bt_recebimento', 93),
    ('bt_mensagem', 71),
    ('bt_sair', 16777216),
    ('bt_suporte', 89),
    # Relatorio Gerencial
    ('rg_parametro_conf', 0),
    ('rg_meios_de_pgto', 1),
    ('rg_identificacao_pafecf', 2),
    ('rg_relatorio_gerencial', 3),
    ('rg_davs_emitidos', 4),
    ('trans_mesas', 5),
    ('confe_mesas', 6),
    ('mesas_abertas', 7),

    # TEMP
    ('digitacaocheque', 'Manual'),
    ('ipretaguarda', ''),
    ('check_dir_logo_itens', False),
    ('local', ''),
    ('cmd_teamviewer', ''),
    ('check_atualiza', False),
    ('link_sig', 'download.sigsolucoes.net.br/download/versao/sigfrente/sigfrente3.1/'),
    ('tentativas_atualiza', 3),
    ('aplicativo_dav', 'Local'),
    # Nota Fiscal Legal
    ('nf_legal', False),
    # ServTCP runcobol
    ('ip_gestao', 'localhost'),
    ('cmd_cobol', 'runcobol /mm/objetos/venau001 -a="L 001" k'),
    # Conexão Postgres
    ('pg_user', ''),
    ('pg_pass', ''),
    ('pg_db', ''),
    ('pg_server', ''),
    ('pg_port', 5432),
    # Bares e Restaurantes
    ('ativar_restaurante', False),
    # Habilitar numero de série
    ('habilitar_nserie', False),

    # ECF Virtual
    ('virtual_printer_active', False),
    ('virtual_printer_srv', ''),
    ('ecf_port_tcp', 2000),

    # Config. Nepos Virtual
    ('config_nepos_active', False),
    ('nepos_ip', ''),
    ('nepos_port_tcp', 2000),

    # Config. Usar biometria
    ('usar_biometria', False),

    # Config totalizador nao fiscal para doacao
    ('totalizador_doacao', 0)
)


class configmanager(object):
    def __init__(self, fname=None):
        self.options = dict([(x[0], x[1]) for x in _OPTIONS])

        self.blacklist_for_save = set(["publisher_warranty_url", "load_language"])

        self.misc = {}
        self.config_file = fname
        # self.rcfile = fname
        version = '3.1'
        self.parser = parser = optparse.OptionParser(version=version)

        parser.add_option("-c", "--config", dest="config", help="specify alternate config file")
        parser.add_option("-s", "--save", action="store_true", dest="save", default=False,
                          help="save configuration to ~/.openerp_serverrc")
        parser.add_option("--pidfile", dest="pidfile",
                          help="file where the server pid will be stored")

        for p in _OPTIONS:
            try:
                parser.add_option("--" + p[0], dest=p[0], help='"' + p[0] + '"')
            except:
                pass

    def parse_config(self):
        opt = self.parser.parse_args()[0]

        def die(cond, msg):
            if cond:
                print msg
                sys.exit(1)


        # Check if the config file exists (-c used, but not -s)
        die(not opt.save and opt.config and not os.path.exists(opt.config),
            "The config file '%s' selected with -c/--config doesn't exist, " \
            "use -s/--save if you want to generate it" % (opt.config))

        # place/search the config file on Win32 near the server installation
        # (../etc from the server)
        # if the server is run by an unprivileged user, he has to specify location of a config file where he has the rights to write,
        # else he won't be able to save the configurations, or even to start the server...

        rcfilepath = os.path.join(root_path, 'pdvconfig.cfg')


        # rcfilepath = os.path.join(APP_CONFIG_DIR, 'config.cfg')
        self.rcfile = os.path.abspath(
            self.config_file or opt.config or rcfilepath)
        self.load()


        # the same for the pidfile
        if self.options['pidfile'] in ('None', 'False'):
            self.options['pidfile'] = False

        keys = self.options.keys()
        for arg in keys:
            try:
                if getattr(opt, arg):
                    self.options[arg] = getattr(opt, arg)
            except AttributeError as e:
                print e, 'Remova do arquivo'

        for arg in keys:
            try:
                if getattr(opt, arg) is not None:
                    self.options[arg] = getattr(opt, arg)
            except AttributeError as e:
                print e, 'Remova do arquivo'

        if not self.options['root_path'] or self.options['root_path'] == 'None':
            self.options['root_path'] = os.path.abspath(os.path.dirname(sys.argv[0]))

        if not self.options['db_pdv_user']:
            try:
                import getpass

                self.options['db_pdv_user'] = getpass.getuser()
            except Exception:
                self.options['db_pdv_user'] = None

        die(not self.options['db_pdv_user'],
            'ERROR: No user specified for the connection to the database')

        if opt.save:
            self.save()

    def _generate_pgpassfile(self):
        """
        Generate the pgpass file with the parameters from the command line (db_host, db_user,
        db_password)

        Used because pg_dump and pg_restore can not accept the password on the command line.
        """
        is_win32 = sys.platform == 'win32'
        if is_win32:
            filename = os.path.join(os.environ['APPDATA'], 'pgpass.conf')
        else:
            filename = os.path.join(os.environ['HOME'], '.pgpass')

        text_to_add = "%(db_host)s:*:*:%(db_user)s:%(db_password)s" % self.options

        if os.path.exists(filename):
            content = [x.strip() for x in file(filename, 'r').readlines()]
            if text_to_add in content:
                return

        fp = file(filename, 'a+')
        fp.write(text_to_add + "\n")
        fp.close()

        if is_win32:
            try:
                import _winreg
            except ImportError:
                _winreg = None
            x = _winreg.ConnectRegistry(None, _winreg.HKEY_LOCAL_MACHINE)
            y = _winreg.OpenKey(x, r"SYSTEM\CurrentControlSet\Control\Session Manager\Environment",
                                0, _winreg.KEY_ALL_ACCESS)
            _winreg.SetValueEx(y, "PGPASSFILE", 0, _winreg.REG_EXPAND_SZ, filename)
            _winreg.CloseKey(y)
            _winreg.CloseKey(x)
        else:
            import stat

            os.chmod(filename, stat.S_IRUSR + stat.S_IWUSR)

    def _check_addons_path(self, option, opt, value, parser):
        res = os.path.abspath(os.path.expanduser(value))
        if not os.path.exists(res):
            raise optparse.OptionValueError("option %s: no such directory: %r" % (opt, value))

        contains_addons = False
        for f in os.listdir(res):
            modpath = os.path.join(res, f)
            if os.path.isdir(modpath) and \
                    os.path.exists(os.path.join(modpath, '__init__.py')) and \
                    (os.path.exists(os.path.join(modpath, '__openerp__.py')) or \
                             os.path.exists(os.path.join(modpath, '__terp__.py'))):
                contains_addons = True
                break

        if not contains_addons:
            raise optparse.OptionValueError(
                "option %s: The addons-path %r does not seem to a be a valid Addons Directory!" % (
                opt, value))

        setattr(parser.values, option.dest, res)

    def load(self):
        self.deleta_parametros_antigos()
        session = [('Banco de Dados', 'db'), ('Balanca', 'balanca'), ('Convenio', 'bb'),
                   ('Ecf', 'nada'),
                   ('Parametros', 'nada'), ('Relatorio Gerencial', 'rg'), ('Sangria', 'sangria'),
                   ('Scanner', 'scanner'),
                   ('Tef', 'nada'), ('Teclado', 'bt')]
        p = ConfigParser.ConfigParser()
        try:
            p.read([self.rcfile])
            for keySession, valueKey in session:
                for (name, value) in p.items(keySession):
                    if name not in self.options:
                        p.remove_option(keySession, name)
                        continue

                    if value == 'True' or value == 'true':
                        value = True
                    elif value == 'False' or value == 'false':
                        value = False

                    elif isinstance(self.options[name], int):
                        try:
                            value = int(value)
                        except ValueError:
                            value = self.options[name]
                    self.options[name] = value
                    # parse the other sections, as well
                for sec in p.sections():
                    for keySession, valueKey in session:
                        if sec == keySession:
                            continue
                        if not self.misc.has_key(sec):
                            self.misc[sec] = {}
                        for (name, value) in p.items(sec):
                            if value == 'True' or value == 'true':
                                value = True
                            if value == 'False' or value == 'false':
                                value = False
                            self.misc[sec][name] = value
        except IOError:
            pass
        except ConfigParser.NoSectionError:
            pass

    def save(self):
        p = ConfigParser.ConfigParser()
        session = [('Banco de Dados', 'db'), ('Balanca', 'balanca'), ('Convenio', 'bb'),
                   ('Ecf', 'nada'),
                   ('Parametros', 'nada'), ('Relatorio Gerencial', 'rg'), ('Sangria', 'sangria'),
                   ('Scanner', 'scanner'),
                   ('Tef', 'nada'), ('Teclado', 'bt')]
        sessionOutros = [('Ecf', 'loja'), ('Ecf', 'caixa'), ('Ecf', 'ecf'),
                         ('Ecf', 'printer_speed_port'),
                         ('Balanca', 'char'), ('Balanca', 'codigo'), ('Sangria', 'suprimento'),
                         ('Sangria', 'frase_sangria'), ('Balanca', 'digitos'), ('Tef', 'empresa'),
                         ('Tef', 'servidor'),
                         ('Tef', 'qtdvias')]

        for sess, opt2 in session:
            p.add_section(sess)
        sess = 'Parametros'
        for opt in sorted(self.options.keys()):
            if opt in ('version', 'language', 'translate_out', 'translate_in',
                       'overwrite_existing_translations', 'init', 'update'):
                continue
            if opt in self.blacklist_for_save:
                continue
            for sessOpt, opt2 in session:
                if str(opt).startswith(opt2):
                    sess = sessOpt
                    p.set(sess, opt, self.options[opt])
            for sessOpt, opt2 in sessionOutros:
                if str(opt).startswith(opt2):
                    sess = sessOpt
                    p.set(sess, opt, self.options[opt])
            p.set(sess, opt, self.options[opt])
            sess = 'Parametros'
        # try to create the directories and write the file
        try:
            rc_exists = os.path.exists(self.rcfile)
            if not rc_exists and not os.path.exists(os.path.dirname(self.rcfile)):
                os.makedirs(os.path.dirname(self.rcfile))
            try:
                p.write(file(self.rcfile, 'w'))
                if not rc_exists:
                    os.chmod(self.rcfile, 0600)
            except IOError:
                sys.stderr.write("ERROR: couldn't write the config file\n")

        except OSError:
            # what to do if impossible?
            sys.stderr.write("ERROR: couldn't create the config directory\n")

    def check_codificado(self):
        keys_dict = {}
        for key in KEYS_CODIFICADO:
            v = self.get(key)
            if v not in keys_dict:
                keys_dict[v] = (key,)
            else:
                keys_dict[v] += (key,)
        retval = {}
        for key, value in keys_dict.iteritems():
            if len(value) > 1:
                retval[key] = value
        return retval

    @staticmethod
    def deleta_parametros_antigos():
        res = False
        parametros = ['diretorio', 'comandodav', 'utiliza', 'habilitar_nroserie']
        diretorio = os.path.join(root_path, 'pdvconfig.cfg')
        if not os.path.exists(diretorio):
            return False
        pdv_ini = open(diretorio, 'r')
        file_lines = pdv_ini.readlines()

        for line in file_lines:
            index = file_lines.index(line)
            line = line.replace(' = ', '').replace('False', '').replace('True', '').replace('\n', '')
            if line in parametros:
                del file_lines[index]
                res = True

        pdv_ini.close()

        if res:
            pdv_ini = open(os.path.join(root_path, 'pdvconfig.cfg'), 'w')
            pdv_ini.writelines(file_lines)
            pdv_ini.close()

        return res

    def get(self, key, default=None):
        return self.options.get(key, default)

    def get_misc(self, sect, key, default=None):
        return self.misc.get(sect, {}).get(key, default)

    def set(self, key, value):
        return self.__setitem__(key, value)

    def __setitem__(self, key, value):
        self.options[key] = value

    def __getitem__(self, key):
        return self.options[key]


config = configmanager()

# FIXME:following line should be called explicitly by the server
# when it starts, to allow doing 'import tools.config' from
# other python executables without parsing *their* args.
config.parse_config()