#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8 -*-

import sys
from PySide import QtCore, QtGui
from busca_cliente_ui import Ui_Dialog
from sigpdv.models.db_utils import exec_sql


class BuscaCliente(QtGui.QDialog):
    def __init__(self, parent=None, db='C'):
        super(BuscaCliente, self).__init__()

        self._clientes = None
        if db == 'C':
            from sigpdv.models.cad_clientes_data import CaixaCadClientes

            self._clientes = CaixaCadClientes
        elif db == 'S':
            from sigpdv.models.supervisor import SupervisorCadClientes

            self._clientes = SupervisorCadClientes

        self.ui = Ui_Dialog()

        self.ui.setupUi(self)

        self.parent = parent
        self.retorno = False
        # SIGNALS
        self.ui.buttonBuscarCliente.pressed.connect(self._button_buscar_cliente_click)
        self.ui.tableBuscaCliente.cellClicked.connect(self.table_busca_cliente_click)
        # AJUSTES
        self.ui.tableBuscaCliente.resizeColumnsToContents()
        self.ui.editBuscaCliente.setFocus()

    def accept(self):
        if self.ui.tableBuscaCliente.selectedItems():
            cod_cli = self.ui.tableBuscaCliente.item(self.ui.tableBuscaCliente.currentRow(),
                0).text()
            self.retorno = self._clientes.get_by(supcli_cnpj_cpf=cod_cli)
            super(BuscaCliente, self).accept()

    def _button_buscar_cliente_click(self):
        texto_busca = self.ui.editBuscaCliente.text()
        if texto_busca:
            if self.ui.comboFiltroBusca.currentIndex() == 0:  # Busca por CPF/CNPJ
                cod = texto_busca.rjust(15, '0')
                res = self._clientes.query.filter(self._clientes.supcli_cnpj_cpf == cod)
            else:
                sql = """ SELECT id
                          FROM supervisor_cad_clientes
                          WHERE name ilike '%%%s%%'
                """ % texto_busca
                result = exec_sql(sql, 'S')
                ids = map(lambda x: x.get('id'), result)
                res = []
                if ids:
                    res = self._clientes.query.filter(self._clientes.id.in_(ids)).order_by(
                        self._clientes.name)

            if res and res.count() > 0:
                self._update_table_busca(res)
                self.ui.tableBuscaCliente.setFocus()
            else:
                QtGui.QMessageBox.information(None, u'Pesquisa cliente', u'Cliente não cadastrado!',
                                              QtGui.QMessageBox.Ok)
                self.ui.editBuscaCliente.clear()
                self.ui.editBuscaCliente.setFocus()
        else:
            QtGui.QMessageBox.information(None, u'Pesquisa cliente',
                                          u'Digite algum texto para efetuar a busca.',
                                          QtGui.QMessageBox.Ok)
            self.ui.editBuscaCliente.setFocus()

    def _update_table_busca(self, cli):
        tableBuscaCliente = self.ui.tableBuscaCliente
        # Apaga lista de itens
        rows_count = tableBuscaCliente.rowCount()
        # Verifica se hà dados
        if rows_count:
            rows = range(0, rows_count + 1)
            rows.reverse()
            for r in rows:
                tableBuscaCliente.removeRow(r)
        rows = 0
        for c in cli:
            tableBuscaCliente.insertRow(rows)
            # CPF/CNPJ
            c_cpf = QtGui.QTableWidgetItem(c.supcli_cnpj_cpf)
            # Nome
            c_nome = QtGui.QTableWidgetItem(c.name)
            # newDesc.setFlags(QtCore.Qt.ItemIsEnabled)
            # Status
            c_status = QtGui.QTableWidgetItem(c.supcli_status)
            c_status.setTextAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)
            # LIMITE
            c_limite = QtGui.QTableWidgetItem('%1.2f' % (c.supcli_limite_compras or 0))
            c_limite.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            # FONE
            c_fone = QtGui.QTableWidgetItem(c.supcli_telefone)

            # WIDGET OS NEW ITENS NA TABLE ITENS
            tableBuscaCliente.setItem(rows, 0, c_cpf)
            tableBuscaCliente.setItem(rows, 1, c_nome)
            tableBuscaCliente.setItem(rows, 2, c_status)
            tableBuscaCliente.setItem(rows, 3, c_limite)
            tableBuscaCliente.setItem(rows, 4, c_fone)
            rows += 1
        tableBuscaCliente.resizeColumnsToContents()

    def table_busca_cliente_click(self, row):
        print row


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    form = BuscaCliente()
    form.show()
    sys.exit(app.exec_())
