# -.- coding: utf-8 -.-

from PySide import QtCore
from PySide import QtGui
from uiDlgPassword import Ui_Dialog
from sigpdv.models.cad_operadores_data import CaixaCadOperadores
from sigpdv.tools.config import config
from SigUtils.simple_crypto import simple_num_crypt, flag_decrypt
from sigpdv.tools.utils import key_generator
from sigpdv.tools.log import traced as Log
import sys
from stoqdrivers.readers.barcode.reader import BarcodeReader

style_btn_enabled = """
    QPushButton {
        border-image: url(:/btn/img/bt.png);
        min-width: 100;
        min-height: 40;
        color: rgb(216, 216, 216);
        font: bold;
}
"""


class dlg_pwd(QtGui.QDialog):
    def __init__(self, parent=None, btFocus='Ok', msg_info='', senha=[]):
        QtGui.QDialog.__init__(self, parent, QtCore.Qt.Dialog | QtCore.Qt.FramelessWindowHint)
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)

        # Botoes
        OkButton = QtGui.QPushButton(self.tr("Ok"))
        OkButton.setFocusPolicy(QtCore.Qt.NoFocus)
        CancelButton = QtGui.QPushButton(self.tr("Cancela"))
        CancelButton.setFocusPolicy(QtCore.Qt.NoFocus)

        # Focu inicial
        if btFocus == 'Ok':
            button_focus = OkButton
        elif btFocus == 'Cancel':
            button_focus = CancelButton
        self._button_focus = button_focus
        self.ui.lbl_info.setText(msg_info)

        # Variaveis
        self._OkButton = OkButton
        self._CancelButton = CancelButton
        self._parent = parent

        if parent:
            self._parent._close_window = False

        self.ui.buttonBox.addButton(OkButton, QtGui.QDialogButtonBox.YesRole)
        self.ui.buttonBox.addButton(CancelButton, QtGui.QDialogButtonBox.NoRole)

        # Sinais
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.accept)
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("rejected()"), self.reject)
        self.ui.label_biometria_pwd.setVisible(False)
        self.ui.frame_biometria_pwd.setVisible(False)
        if config['usar_biometria'] and parent:
            self.ui.label_biometria_pwd.setVisible(True)
            self.ui.frame_biometria_pwd.setVisible(True)
            self.ui.label.setVisible(False)
            self.ui.lbl_info.setVisible(False)
            self.ui.label_3.setVisible(False)
            self.ui.label_4.setVisible(False)
            self.ui.edtUser.setVisible(False)
            self.ui.edtPwd.setVisible(False)
            self.ui.btnKEY_0.setVisible(False)
            self.ui.btnKEY_1.setVisible(False)
            self.ui.btnKEY_2.setVisible(False)
            self.ui.btnKEY_3.setVisible(False)
            self.ui.btnKEY_4.setVisible(False)
            self.ui.btnKEY_5.setVisible(False)
            self.ui.btnKEY_6.setVisible(False)
            self.ui.btnKEY_7.setVisible(False)
            self.ui.btnKEY_8.setVisible(False)
            self.ui.btnKEY_9.setVisible(False)
            self.ui.btnKEY_UP.setVisible(False)
            self.ui.btnKEY_DOWN.setVisible(False)
            self.ui.btnKEY_RETURN.setVisible(False)
            self.ui.btnKEY_BACKSPACE.setVisible(False)
            self.ui.buttonBox.setVisible(False)

        # KeysNumPad
        self.ui.btnKEY_0.clicked.connect(self.KEY_0_Click)
        self.ui.btnKEY_1.clicked.connect(self.KEY_1_Click)
        self.ui.btnKEY_2.clicked.connect(self.KEY_2_Click)
        self.ui.btnKEY_3.clicked.connect(self.KEY_3_Click)
        self.ui.btnKEY_4.clicked.connect(self.KEY_4_Click)
        self.ui.btnKEY_5.clicked.connect(self.KEY_5_Click)
        self.ui.btnKEY_6.clicked.connect(self.KEY_6_Click)
        self.ui.btnKEY_7.clicked.connect(self.KEY_7_Click)
        self.ui.btnKEY_8.clicked.connect(self.KEY_8_Click)
        self.ui.btnKEY_9.clicked.connect(self.KEY_9_Click)
        self.ui.btnKEY_RETURN.clicked.connect(self.KEY_RETURN_Click)
        self.ui.btnKEY_BACKSPACE.clicked.connect(self.KEY_BACKSPACE_Click)
        self.ui.btnKEY_UP.clicked.connect(self.KEY_UP_Click)
        self.ui.btnKEY_DOWN.clicked.connect(self.KEY_DOWN_Click)

        def get_key_name(key):
            return QtCore.Qt.Key(key).name[4:]

        usar_codificado = config.get('usarcodificado')

        self.ui.btnKEY_RETURN.setShortcut(
            "Return" if not usar_codificado else get_key_name(config['bt_entrar']))
        self.ui.btnKEY_BACKSPACE.setShortcut(
            "Backspace" if not usar_codificado else get_key_name(config['bt_limpar']))
        self.ui.btnKEY_UP.setShortcut(
            "Up" if not usar_codificado else get_key_name(config['bt_cima']))
        self.ui.btnKEY_DOWN.setShortcut(
            "Down" if not usar_codificado else get_key_name(config['bt_baixo']))

        self.ui.edtPwd.returnPressed.connect(self._returnPressed_edtPwd)
        self.ui.edtUser.returnPressed.connect(self._returnPressed_edtUser)
        # Style
        self._control_focus = self.ui.edtUser
        self.retval = self.user = False
        self.ui.frm_contrasenha.setVisible(bool(senha))
        self._senha = senha
        self._key = key_generator(7)
        self._senha_key = self._new_pass(senha)
        self._num_tentativas = 3
        if senha:
            self._ajustar_form_contra_senha()
        self._control_focus.setStyleSheet("border-image: url(:/edit/img/edit_399x62_f.png);")

        self.barcode_active = config['itens_scan_active']
        scanner_timer = QtCore.QTimer(self)
        scanner_timer.timeout.connect(self._active_scanner_timer)
        self._timer = scanner_timer
        #        self.ui.frmInfoMsg.setVisible(False)
        if self.barcode_active:
            try:
                self.barcode = BarcodeReader(brand='metrologic', model='MC630',
                                             device=config['scanner'])
                self.barcode._port.timeout = 0.001
            except:
                msg_info.append(Exception('Verifique a conexão com o Leitor de Código de Barras'))
                self.barcode_active = False

    def _active_scanner_timer(self):
        if self._control_focus == self.ui.edtPwd:
            try:
                cod = self.barcode.get_code()
                if cod:
                    self._control_focus.setText(cod)
                    QtGui.QApplication.processEvents()
                    self.KEY_RETURN_Click()
            except Exception as e:
                pass
        else:
            self._timer.stop()

    def _ajustar_form_contra_senha(self):
        self.ui.lbl_info.setText(u'Ligue para: (31)2106-1700 para obter sua contra-senha')
        self.ui.frm_pwd.setVisible(False)
        self.ui.buttonBox.setVisible(False)
        soma = []
        nsenha = self._senha_key.split('-')
        for x in range(len(nsenha)):
            if x + 1 > 9:
                break
            if nsenha[x]:
                soma.append(int(nsenha[x]))
                eval('self.ui.lbl_pos%d.setText(%r)' % (x + 1, nsenha[x]))
        self._control_focus = self.ui.edt_contrasenha


    def _new_pass(self, senha):
        new_pass = []
        for x in range(len(self._senha)):
            if x < len(self._key):
                new_pass.append('%s%s' % (self._key[x], self._senha[x]))
            else:
                new_pass.append('%s' % (self._senha[x]))
        return simple_num_crypt('-'.join(new_pass))

    def emit_key(self, key, text=''):
        ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, key, QtCore.Qt.KeyboardModifiers(), text=text)
        self.keyPressEvent(ev)

    def KEY_0_Click(self):
        self.emit_key(QtCore.Qt.Key_0.__int__(), '0')

    def KEY_1_Click(self):
        self.emit_key(QtCore.Qt.Key_1.__int__(), '1')

    def KEY_2_Click(self):
        self.emit_key(QtCore.Qt.Key_2.__int__(), '2')

    def KEY_3_Click(self):
        self.emit_key(QtCore.Qt.Key_3.__int__(), '3')

    def KEY_4_Click(self):
        self.emit_key(QtCore.Qt.Key_4.__int__(), '4')

    def KEY_5_Click(self):
        self.emit_key(QtCore.Qt.Key_5.__int__(), '5')

    def KEY_6_Click(self):
        self.emit_key(QtCore.Qt.Key_6.__int__(), '6')

    def KEY_7_Click(self):
        self.emit_key(QtCore.Qt.Key_7.__int__(), '7')

    def KEY_8_Click(self):
        self.emit_key(QtCore.Qt.Key_8.__int__(), '8')

    def KEY_9_Click(self):
        self.emit_key(QtCore.Qt.Key_9.__int__(), '9')

    def KEY_RETURN_Click(self):
        self.emit_key(QtCore.Qt.Key_Return.__int__(), '')

    def KEY_BACKSPACE_Click(self):
        self.emit_key(QtCore.Qt.Key_Backspace.__int__(), '')

    def KEY_UP_Click(self):
        self.emit_key(QtCore.Qt.Key_Up.__int__(), '')

    def KEY_DOWN_Click(self):
        self.emit_key(QtCore.Qt.Key_Down.__int__(), '')


    def accept(self):
        if self.user:
            self.ui.edtUser.clear()
            self.ui.edtPwd.clear()
            if self._parent:
                self._parent._close_window = True
            # log.debug('<<< dlg_pwd, ok, user=%r' % self.user.name)
            res = super(dlg_pwd, self).accept()

            #        self._parent.activateWindow()
            self.retval = True
            return res

    def reject(self):
        self.ui.edtUser.clear()
        self.ui.edtPwd.clear()
        res = super(dlg_pwd, self).reject()
        if self._parent:
            self._parent._close_window = True
        # log.debug('<<< dlg_pwd, cancelou')
        #        self._parent.activateWindow()
        return res

    def keyPressEvent(self, ev):
        enter = False
        text = ev.text()
        if ev.key() == QtCore.Qt.Key_Backspace:
            if self._control_focus in [self.ui.edtPwd, self.ui.edtUser, self.ui.edt_contrasenha]:
                self._control_focus.clear()
            text = ''
            return
        elif ev.key() == config['bt_sair'] and config['usarcodificado']:
            self.close()
        elif ev.key() == QtCore.Qt.Key_Up:
            if self._control_focus == self.ui.buttonBox:
                self._control_focus = self.ui.edtPwd
                self.ui.edtPwd.setStyleSheet("border-image: url(:/edit/img/edit_399x62_f.png);")
                self._OkButton.setStyleSheet('')
                self._CancelButton.setStyleSheet('')
            elif self._control_focus == self.ui.edtPwd:
                self._control_focus = self.ui.edtUser
                self.ui.edtUser.setStyleSheet("border-image: url(:/edit/img/edit_399x62_f.png);")
                self.ui.edtPwd.setStyleSheet("")
        elif (ev.key() in [QtCore.Qt.Key_Left, config['bt_esquerda'], QtCore.Qt.Key_Right,
                           config['bt_direita']]) and (
                    self._control_focus == self.ui.buttonBox):
            if self._button_focus.text() == 'Cancela':
                self._button_focus = self._OkButton
                self._OkButton.setStyleSheet(style_btn_enabled)
                self._CancelButton.setStyleSheet('')
            else:
                self._button_focus = self._CancelButton
                self._OkButton.setStyleSheet('')
                self._CancelButton.setStyleSheet(style_btn_enabled)
        elif ev.key() in (QtCore.Qt.Key_Return, QtCore.Qt.Key_Enter):
            enter = True
            if self._control_focus == self.ui.buttonBox:
                if self._button_focus.text() == 'Cancela':
                    self.reject()
                elif self._button_focus.text() == 'Ok':
                    self.accept()
            elif self._control_focus == self.ui.edt_contrasenha:
                text = self.ui.edt_contrasenha.text()
                if not self.checar_contra_senha(text):
                    self._num_tentativas -= 1
                    self.ui.lbl_info.setText(
                        u'Você tem mais %d tentativas!\n*** Contra senha (%s) inválida! ***' %
                        (self._num_tentativas, text))
                    self.ui.edt_contrasenha.clear()
                    if not self._num_tentativas:
                        self._key = key_generator(7)
                        self._senha_key = self._new_pass(self._senha)
                        self._num_tentativas = 3
                        self._ajustar_form_contra_senha()
                        self.ui.lbl_info.setText(u'Número de tentativas excedido.\n'
                                                 u'*** Foi gerado uma nova sequencia ***')
                else:
                    self.user = True
                    self.accept()
        elif ev.key() == QtCore.Qt.Key_Escape:
            self.reject()

        if enter:
            n_ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Return,
                                   QtCore.Qt.KeyboardModifiers(),
                                   text=text)
        else:
            n_ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, ev.key(), QtCore.Qt.KeyboardModifiers(),
                                   text=text)
        self._control_focus.keyPressEvent(n_ev)

    def checar_contra_senha(self, psenha):
        soma = []
        for x in self._senha_key.split('-'):
            if x:
                soma.append(int(x))
        s = simple_num_crypt(str(sum(soma)))
        print '>>>>>', s, psenha, 's', sum(soma), '\n', self._senha_key
        return s == psenha

    @Log
    def biometria_dlpPwd(self):
        res = self._parent.biometria(autorizacao=True)
        if isinstance(res, dict) and res.get('autorizado'):
            self.user = res.get('operador')
            self.accept()
            n_ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Return,
                                   QtCore.Qt.KeyboardModifiers(),
                                   text='')
            self.ui.buttonBox.keyPressEvent(n_ev)


    def _returnPressed_edtUser(self):
        if config['usar_biometria'] and self._parent:
            return self.biometria_dlpPwd()

        user = CaixaCadOperadores.get_by(supope_codigo=self.ui.edtUser.text(), active=True)
        if user:
            if user.supope_operador_supervisor == 'S':
                self.ui.lbl_info.setText(u'Olá %s' % user.name)
                self._control_focus = self.ui.edtPwd
                self._timer.start(100)
                self.ui.edtUser.setStyleSheet('')
                self.ui.edtPwd.clear()
                self.ui.edtPwd.setStyleSheet("border-image: url(:/edit/img/edit_399x62_f.png);")
            else:
                self.ui.lbl_info.setText(
                    u'Este operador não é Supervisor! Tente Novamente.')
                self.ui.edtUser.clear()
        else:
            self.ui.lbl_info.setText(u'Usuário não cadastrado ou inativo!')
            self.ui.edtUser.clear()

    def _returnPressed_edtPwd(self):
        user = CaixaCadOperadores.get_by(supope_codigo=self.ui.edtUser.text(), active=True)
        if user:
            if user.supope_senha == self.ui.edtPwd.text():
                self.ui.lbl_info.setText(u'Confirme a operação %s' % user.name)
                self._control_focus = self.ui.buttonBox
                self.ui.edtPwd.setStyleSheet('')
                self._button_focus.setStyleSheet(style_btn_enabled)
                self.user = user
            else:
                self.ui.lbl_info.setText(u'Senha Incorreta! Tente novamente.')

        else:
            self.ui.lbl_info.setText(u'Usuário não cadastrado ou inativo!')
            self.ui.edtUser.clear()

@Log
def password_dlg(parent=None, bt_focus='Ok', msg_info='', senha=[]):
    try:
        app = QtGui.QApplication(sys.argv)
    except:
        pass

    if parent and 'teclado' in parent.__dict__ and parent.teclado:
        parent.teclado.envia_display("Operacao do Supervisor, infome-o")

    dlg = dlg_pwd(parent=parent, btFocus=bt_focus, msg_info=msg_info, senha=senha)
    dlg.exec_()
    return dlg


if __name__ == "__main__":
    dlg = password_dlg(msg_info='teste')
    if dlg.retval:
        print dlg.user


