# -*- coding: utf-8 -*-
__author__ = 'Dener Rodrigues'

from PySide import QtGui, QtCore
from sigpdv.ui.message_dialog import Ui_Dialog
import os
import sys
from sigpdv.tools.config import config
from sigpdv.tools.log import traced
ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
from sigpdv import root_path
MSG_INFO, MSG_ERROR, MSG_QUESTION, MSG_ALL, MSG_NONE = range(5)

MSG_NAMES = {
    MSG_INFO: u'Informação',
    MSG_ERROR: u'Erro',
    MSG_QUESTION: u'Questão',
    MSG_ALL: u'None',
    MSG_NONE: u'Informação: Pressione a Tecla SAIR/ESC'
}

NO_ROLE = QtGui.QDialogButtonBox.NoRole
YES_ROLE = QtGui.QDialogButtonBox.YesRole
ACCEPT_ROLE = QtGui.QDialogButtonBox.AcceptRole
CANCEL_ROLE = QtGui.QDialogButtonBox.RejectRole
APPLY_ROLE = QtGui.QDialogButtonBox.ApplyRole


def _(string):
    if type(string) == unicode:
        string = string.encode('utf-8', 'replace')
    return string


class MessageDialog(QtGui.QDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent, QtCore.Qt.Dialog | QtCore.Qt.FramelessWindowHint)

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        self.setModal(True)

        self.ui.buttonBox.clicked.connect(self._button_click)
        self.ui.plain_text_detail.setVisible(False)
        self._filename = None
        self.retval = None
        if parent:
            parent._close_window = False
        self.parent = parent
        self._close = False
        self.ui.buttonBox.setFocus()
        scanner_timer = QtCore.QTimer(self)
        scanner_timer.timeout.connect(self._timer)
        scanner_timer.start(1000)
        self._tempo_ocioso = 0

    def _timer(self):
        time_cda = 5
        if self._tempo_ocioso >= time_cda:
            self._tempo_ocioso = 0
            if self.parent:
                self.close()
        else:
            self._tempo_ocioso += 1

    def _button_click(self, b):
        ret_cmd = False
        print self.ui.buttonBox.buttonRole(b), self._filename
        if self._filename and self.ui.buttonBox.buttonRole(b) == APPLY_ROLE:
            if os.path.exists(self._filename):
                dirapp = os.path.join(root_path, 'qtedit', 'edit.py')
                if not os.path.exists(dirapp):
                    dirapp = os.path.join(root_path, 'qtedit', 'edit.pyc')
                cmd = 'python %s %s&' % (dirapp, self._filename)
                traced.info('Executar editor de texto: %s' % cmd)
                ret_cmd = os.system(cmd)
            else:
                traced.warning('Arquivo não encontrado: %s' % self._filename)
            self.close()
        return ret_cmd

    def set_caption(self, text):
        self.ui.label_caption.setText(text)
        return text

    def set_info_text(self, text):
        text = self.trUtf8(text)
        msg = '<h3><font color="white">%s</font></h3>' % text.replace('\n', '<br></br>')
        self.ui.label_info.setText(msg)
        return _(text)

    def set_detail(self, text):
        if text:
            self.ui.plain_text_detail.setVisible(True)
            self.ui.plain_text_detail.setPlainText(self.trUtf8(text))
        else:
            self.ui.plain_text_detail.setVisible(False)

    def set_file(self, filename):
        self.set_detail(filename)
        if not config.get('usarcodificado'):
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.NoButton)
            self.ui.buttonBox.addButton(u'&Abrir', APPLY_ROLE)
            b = self.ui.buttonBox.addButton(u'&Ok', YES_ROLE)
            self._filename = filename
            b.setFocus()

    def set_type(self, type):
        str_type = MSG_NAMES[type]
        if type == MSG_INFO:
            self.ui.label_icon.setPixmap(QtGui.QPixmap(os.path.join(ROOT_PATH, 'img', 'dialog-information.svg')))
            self.set_caption(u'Informação')
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Ok)

        elif type == MSG_ERROR:
            self.ui.label_icon.setPixmap(QtGui.QPixmap(os.path.join(ROOT_PATH, 'img', 'dialog-error.svg')))
            self.set_caption('Erro')
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel)

        elif type == MSG_QUESTION:
            self.ui.label_icon.setPixmap(QtGui.QPixmap(os.path.join(ROOT_PATH, 'img', 'dialog-warning.svg')))
            self.set_caption(u'Questão')
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.NoButton)
            b = self.ui.buttonBox.addButton(u'&Sim', YES_ROLE)
            self.ui.buttonBox.addButton(u'&Não', NO_ROLE)
            b.setFocus()

        elif type == MSG_ALL:
            self.ui.label_icon.setPixmap(QtGui.QPixmap("img/dialog-information.svg"))
            self.set_caption(u'Informação')
            self.ui.buttonBox.setStandardButtons(
                QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel | QtGui.QDialogButtonBox.Yes | QtGui.QDialogButtonBox.No | QtGui.QDialogButtonBox.Open)

        elif type == MSG_NONE:
            self.set_caption(u'Informação: Pressione a Tecla SAIR/ESC')
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.NoButton)


    def done(self, ev):
#        self._timer.stop()
        if self.parent:
            self.parent._close_window = True
            self.parent.activateWindow()
        self._close = True
        try:
            self.retval = self.ui.buttonBox.buttonRole(self.ui.buttonBox.focusWidget())
        except:
            self.retval = self.ui.buttonBox.NoButton
            pass
        return super(MessageDialog, self).done(ev)

    def keyPressEvent(self, ev):
        if not ev.key():
            return
        if config.get('usarcodificado'):
            if ev.key() in [config['bt_cima'], config['bt_direita']]:
                self.ui.buttonBox.focusNextChild()
            elif ev.key() in [config['bt_baixo'], config['bt_esquerda']]:
                self.ui.buttonBox.focusPreviousChild()
            elif ev.key() == config['bt_entrar']:
                bt_focus = self.ui.buttonBox.focusWidget()
                bt_focus.emit(QtCore.SIGNAL("clicked()"))
            elif ev.key() == config['bt_sair']:
                self.reject()
                self.retval = self.ui.buttonBox.NoRole
        else:
            if ev.key() == QtCore.Qt.Key_Escape.__int__():
                self.reject()
                self.retval = self.ui.buttonBox.NoRole


def message_dlg(parent=None, type=MSG_INFO, text='', detail='', file=''):
    qtTranslator = QtCore.QTranslator()
    msg_box = MessageDialog(parent=parent)
    msg_box.set_type(type)
    text = _(text)
    if type == MSG_ERROR and not detail:
        msg_box.set_info_text(text)
    else:
        msg_box.set_info_text(text)
    if detail:
        msg_box.set_detail(detail)
    if file:
        msg_box.set_file(file)
    if parent:
        parent._message_box = msg_box

    msg_box.exec_()

    return msg_box.retval

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    qtTranslator = QtCore.QTranslator()
    print '----', os.path.join(root_path,  'qt_pt.qm')
    if qtTranslator.load(os.path.join(root_path,  'qt_pt.qm')):
        print('INSTALADO LN')
    msg = '<center><h2><font color="red">A Quantidade informada está incorreta.</font></h2> ' \
          '<center><h2><font color="white">Pressione <font color="yellow">[Sair]</font> para digitar a quantidade ' \
          'correta.</font></h2>'

    print '>>>', message_dlg(None, MSG_QUESTION, msg, file=__file__)

    application = app
