#!/usr/bin/python
# -*- encoding: utf-8 -*-

__autor__ = 'Dener Rodrigues'

import os
import commands
import sys
from time import sleep
from sigpdv.tools.log_update import log_update as log
from SigUtils import __version__ as sigutils_version
from sigpdv import get_version
from sigpdv import root_path
from sigpdv.tools.config import config
from sigpdv.ui.dlpPwd import password_dlg
from sigpdv.ui.messagebox import YES_ROLE, NO_ROLE, MSG_ERROR, MSG_QUESTION, MSG_INFO, MSG_NONE, \
    message_dlg


class SigFrenteUp(object):
    def __init__(self, parent=None, qt=None, frente_comanda='F'):
        self.frente_comanda = frente_comanda
        version = get_version()

        if not version:
            version = get_version().replace('3.1 - SVN-', '')

        self.version = version.replace('.', '')
        self.sig_utils_version = '.'.join(['%02d' % int(v) for v in sigutils_version.split('.')])
        self.new_version = self.version
        self.new_sig_utils_version = self.sig_utils_version
        self.parent = parent
        self._log = ''
        self.progress = 0

        if not parent:
            self.do_checks_version()

    def set_log(self, msg):
        log.info(msg)
        if self.parent:
            self._log += '\n' + msg
            self.parent.ui.text_log.setText(self._log)
            self.parent.QtGui.QApplication.processEvents()
            self.parent.QtGui.QApplication.processEvents()

    def set_progress(self, value):
        """
        Atualiza a barra de pogressão.
        """
        if self.parent:
            self.progress += value
            self.parent.ui.progressBar.setValue(self.progress + value)

    @staticmethod
    def get_cmd(cmd):
        return commands.getoutput(cmd)

    def get_ping(self, host):
        self.set_log('\nVerificando conexao com %s ...\n' % host)

        if os.path.exists('/tmp/ping.log'):
            os.remove('/tmp/ping.log')

        ping = 'ping -c 3 -W 3 %s > /tmp/ping.log' % host
        self.get_cmd(ping)

        if self.parent and os.path.exists('/tmp/ping.log'):
            file_ping = open('/tmp/ping.log')
            lines_ping = file_ping.readlines()

            if lines_ping:
                for i in lines_ping:
                    log.info(i)
                    for x in i.split(','):
                        if x == ' 0 received':
                            msg = '\nFalha ao obter conexão com download.sigsolucoes.net.br\n'
                            self.set_log(msg)
                            return message_dlg(self.parent, MSG_INFO, msg)

            os.remove('/tmp/ping.log')

        return True

    def do_checks_version(self):
        """
        Verifica se as versões do SigFrente e SigUtils estão atualizadas.
        """
        res = False

        try:
            up = self.get_download('A')
            if not up:
                return False

            file_update = open(up)
            lines_update = file_update.readlines()

            if lines_update:
                for i in lines_update:
                    if i[:9] == 'SigUtils=':
                        self.new_sig_utils_version = '.'.join(['%02d' % int(v)
                                                               for v in i[9:].replace('\n', '').split('.')])
                    elif i[:13] == 'SigFrente=3.1':
                        self.new_version = i[14:].replace('\n', '')

            if self.new_sig_utils_version >= self.sig_utils_version:
                if not self.do_update('U'):
                    return message_dlg(self.parent, MSG_ERROR, 'Falha ao atualizar sistema!')

            if self.version[7:] != self.new_version:
                if not self.parent:
                    if self.frente_comanda == 'F':
                        msg = 'Versão 3.1.%s disponível! \n' \
                              'Para atualizar o sistema chame o Supervisor e ' \
                              'caminhe para o Menu principal.' % self.new_version
                        message_dlg(self.parent, MSG_INFO, msg)
                        log.info(msg)
                else:
                    if self.frente_comanda == 'F':
                        if self.do_update('F'):
                            res = True
                            self._del_package_old()
                        else:
                            msg = 'Falha ao atualizar sistema!'
                            log.info(msg)
                            sleep(2)
                            message_dlg(self.parent, MSG_ERROR, msg)

                    elif self.frente_comanda == 'C':
                        msg = u'Versão 3.1.%s disponível! \n' \
                              u'Deseja atualizar o sistema?' % self.new_version

                        if message_dlg(self.parent, MSG_QUESTION, msg) == YES_ROLE:
                            self.parent._atualiza_versao = True
                            log.info(msg)

                            if self.parent._operador.supope_operador_supervisor == 'O':
                                dlgpwd = password_dlg(self.parent, msg_info=u'Atualização de versão')

                                if not dlgpwd:
                                    return False

                            if self.do_update('F'):
                                res = True
                                self._del_package_old()
                        else:
                            res = True
                            self.parent.ui.stackedWidget.setCurrentIndex(1)
                            self.parent.setControlFocus(self.parent._CONTROL_FOCUS,
                                                        self.parent.ui.edt_comanda)

            else:
                msg = 'O Sistema já está atualizado! \n Última versão: 3.1.%s' % self.new_version

                if self.parent:
                    self.set_progress(80)
                    message_dlg(self.parent, MSG_INFO, msg)

                    if self.frente_comanda == 'C':
                        res = True
                        self.parent.ui.stackedWidget.setCurrentIndex(1)
                        self.parent.setControlFocus(self.parent._CONTROL_FOCUS,
                                                    self.parent.ui.edt_comanda)

                log.info(msg)
        except Exception as err:
            log.error(str(err))

        return res

    def get_download(self, download='F'):
        """
        Dependendo de qual parâmetro é utilizado, realiza o download do arquivo de versão,
        pacote SigFrente e SigUtils.

        :param 'F' = SigFrente
        :param 'U' = SigUtils
        :param 'A' = Arquivo de versão
        """
        if self.frente_comanda == 'C' and download == 'F':
            self.parent.show_info_proccess(True, 'Atualização em andamento, aguarde ...')
            self.parent.QtGui.QApplication.processEvents()

        res = None
        link_sig = 'download.sigsolucoes.net.br/produtos/releases/sigfrente/3.1/versoes/'

        if not config['link_sig'] or config['link_sig'] != link_sig:
            config.set('link_sig', link_sig)
            config.save()

        tries = config['tentativas_atualiza']
        cmd = 'wget -c --tries=%s -o /tmp/wget.log %s' % (tries, link_sig)

        try:
            if download == 'A':
                res = self.get_down_arq(cmd)
            elif download == 'U':
                link_sigutils = 'download.sigsolucoes.net.br/produtos/releases/sigutils/'
                cmd = 'wget -c --tries=%s -o /tmp/wget.log %s' % (tries, link_sigutils)
                res = self.get_down_utils(cmd)
            else:
                if self.parent:
                    if download == 'F':
                        res = self.get_down_frente(cmd)
                elif self.frente_comanda == 'C':
                    res = self.get_down_frente(cmd)
        except Exception as err:
            res = False
            log.error(str(err))

        return res

    def get_down_arq(self, cmd=None):
        res = False

        if cmd:
            try:
                if os.path.exists('/tmp/update'):
                    os.remove('/tmp/update')

                self.set_progress(5)

                self.set_log('Baixando arquivo de atualizacao...')

                if os.path.exists('/tmp/wget.log'):
                    os.remove('/tmp/wget.log')

                self.get_cmd(cmd + 'update -O /tmp/update')

                file_wget = open('/tmp/wget.log')
                lines_wget = file_wget.readlines()

                if lines_wget:
                    [log.info(line) for line in lines_wget]

                erros = ["wget: não foi possível resolver endereço de máquina ", " Desistindo"]

                fail = False
                for i in lines_wget[-2].split('.'):
                    if i in erros:
                        fail = True

                if line.replace("“", "-").split("-")[0] in erros or fail is True:
                    msg = '%s \n\nVerifique sua conexão com a Internet !!!' % line
                    self.set_log(msg)
                    os.remove('/tmp/update')

                    if self.parent:
                        res = message_dlg(self.parent, MSG_ERROR, msg)
                    else:
                        res = False
                else:
                    if os.path.exists('/tmp/wget.log'):
                        os.remove('/tmp/wget.log')
                    self.set_progress(5)
                    if os.path.exists('/tmp/update'):
                        if open('/tmp/update').readlines():
                            if os.path.getsize('/tmp/update') > 0:
                                self.set_log(
                                    ' Movendo /tmp/update para %s/update/download/' % root_path)
                                self.get_cmd('mv /tmp/update %s/update/download/update' % root_path)
                                self.set_progress(5)
                            else:
                                msg = 'Falha ao baixar arquivo de atualizacao...'
                                self.set_log(msg)
                                os.remove('/tmp/update')
                                return message_dlg(self.parent, MSG_ERROR, msg)
                    if os.path.exists('%s/update/download/update' % root_path):
                        res = '%s/update/download/update' % root_path
            except Exception as err:
                res = False
                log.error(str(err))

        return res

    def get_down_frente(self, cmd=None):
        res = False

        if cmd:
            try:
                if os.path.exists('/tmp/sigpdv-3.1.%s.deb' % str(self.version)):
                    os.remove('/tmp/sigpdv-3.1.%s.deb' % str(self.version))

                self.set_progress(5)

                self.set_log('\nBaixando versao 3.1.%s ...' % self.new_version)

                if os.path.exists('/tmp/wget.log'):
                    os.remove('/tmp/wget.log')

                self.get_cmd(cmd + '3.1.%s/sigpdv-3.1.%s.deb -O /tmp/sigpdv-3.1.%s.deb' %
                             (self.new_version, self.new_version, self.new_version))

                file_wget = open('/tmp/wget.log')
                lines_wget = file_wget.readlines()

                if lines_wget:
                    [log.info(line) for line in lines_wget]

                erros = ["wget: não foi possível resolver endereço de máquina ", " Desistindo"]

                fail = False

                for i in lines_wget[-2].split('.'):
                    if i in erros:
                        fail = True

                if line.replace("“", "-").split("-")[0] in erros or fail is True:
                    msg = '%s \n\nVerifique sua conexão com a Internet !!!' % line
                    self.set_log(msg)
                    os.remove('/tmp/sigpdv-3.1.%s.deb' % self.new_version)
                    if self.parent:
                        res = message_dlg(self.parent, MSG_ERROR, msg)
                    else:
                        res = False
                else:
                    if os.path.exists('/tmp/wget.log'):
                        os.remove('/tmp/wget.log')

                    if os.path.exists('/tmp/sigpdv-3.1.%s.deb' % self.new_version):
                        if os.path.getsize('/tmp/sigpdv-3.1.%s.deb' % self.new_version) > 0:
                            self.set_progress(25)
                            self.set_log(
                                ' Movendo /tmp/sigpdv-3.1.%s.deb para %s/update/download/' % (
                                    self.new_version, root_path))
                            self.get_cmd(
                                'mv /tmp/sigpdv-3.1.%s.deb %s/update/download/sigpdv-3.1.%s.deb' % (
                                    self.new_version, root_path, self.new_version))
                            self.set_progress(10)
                        else:
                            self.set_log('Falha ao baixar sigpdv-3.1.%s.deb...' % self.new_version)
                            os.remove('/tmp/sigpdv-3.1.%s.deb' % self.new_version)

                    if os.path.exists('%s/update/download/sigpdv-3.1.%s.deb' %
                                      (root_path, self.new_version)):
                        res = '%s/update/download/sigpdv-3.1.%s.deb' % (root_path, self.new_version)
            except Exception as err:
                res = False
                log.error(str(err))

        return res

    def get_down_utils(self, cmd=None):
        res = False

        if cmd:
            try:
                if os.path.exists('/tmp/sigutils-%s.tar.gz' % self.sig_utils_version):
                    os.remove('/tmp/sigutils-%s.tar.gz' % self.sig_utils_version)

                self.get_cmd('%s%s/%s' % (cmd,
                                         self.new_sig_utils_version,
                                         'sigutils-%s.tar.gz -O /tmp/sigutils-%s.tar.gz' %
                             (self.new_sig_utils_version, self.new_sig_utils_version)))

                if os.path.exists('/tmp/sigutils-%s.tar.gz' % self.new_sig_utils_version):
                    if os.path.getsize('/tmp/sigutils-%s.tar.gz' % self.new_sig_utils_version) > 0:
                        self.get_cmd(
                            'mv /tmp/sigutils-%s.tar.gz %s/update/download/sigutils-%s.tar.gz' % (
                                self.new_sig_utils_version, root_path, self.new_sig_utils_version))
                    else:
                        os.remove('/tmp/sigutils-%s.tar.gz' % self.new_sig_utils_version)

                if os.path.exists('%s/update/download/update' % root_path):
                    res = '%s/update/download/sigutils-%s.tar.gz' % (root_path,
                                                                  self.new_sig_utils_version)
            except Exception as err:
                res = False
                log.error(str(err))

        return res

    def do_update(self, update='F'):
        """
        Atualiza SigFrente e SigUtils.
        :param 'F' = SigFrente
        :param 'U' = SigUtils
        """
        res = False

        if update == 'F' and self._frente_update():
            res = True

        elif update == 'U': # SigUtils
            self._utils_update()
            res = True

        return res

    def _frente_update(self):
        """
        Instala o pacote do SigFrente.
        """
        res = False

        if self.frente_comanda == 'C':
            self.parent.show_info_proccess(True, 'Atualização em andamento, aguarde ...')

        download_frente = self.get_download('F')

        if download_frente:
            dir = '%s/update/download/' % root_path
            cmd = 'dpkg -i %ssigpdv-3.1.%s.deb' % (dir, str(self.new_version))
            self.set_log('\nInstalando atualizacao ... \n %s' % cmd)
            users = ['caixa', 'supersig', 'sig']
            password = 'sig'

            if not self.get_cmd('whoami') in users:
                self.set_log('\nUsuario do sistema operacional nao e caixa/supersig/sig!')
                res = False
            else:
                res = True
                try:
                    self.set_log(self.get_cmd('echo %s | sudo -S %s' % (password, cmd)))
                    self.set_progress(20)

                    for i in self._log.split('\n'):
                        if i == 'Erros foram encontrados durante o processamento de:':
                            res = False
                except Exception as err:
                    res = False
                    self.set_log('\n' + str(err))

        return res

    def _utils_update(self):
        """
        Instala o pacote do SigUtils.
        """
        download_sig_utils = self.get_download('U')
        if download_sig_utils:
            senha = 'sig'
            cmd = 'tar -xzf %s' % download_sig_utils
            self.get_cmd('echo %s | sudo -S %s' % (senha, cmd))
            cmd = 'mv sigutils-%s update/download/sigutils' % self.new_sig_utils_version
            self.get_cmd('echo %s | sudo -S %s' % (senha, cmd))
            dir = '%s/update/download/sigutils/' % root_path
            self.get_cmd('cd %s; echo %s | sudo -S %s' % (dir, senha, 'python setup.py install'))

    def _del_package_old(self):
        """
        Apaga os pacotes antigos baixados na pasta sigpdv/update/download/.
        """
        dir = root_path + '/update/download/'
        versions = self.get_cmd('ls %ssigpdv-3.1*' % dir).split('\n')
        if versions:
            self.set_log('\nDeletando pacotes antigos ...\n')
            count = 0
            for x in versions:
                if not x == dir + str(self.version) + '.deb' \
                        and not x == dir + str(self.new_version) + '.deb':
                    count += 1
                    self.set_log(os.remove(str(x)))
            if count > 0:
                self.set_log('%s pacotes deletados\n' % count)


def start():
    exec_update = SigFrenteUp()


if __name__ == "__main__":
    try:
        start()
    except Exception as err:
        log.error(str(err))
        sys.exit(-1)
